package com.brilliance.ccbank.api;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import com.brilliance.ccbank.common.CommonUtil;
import com.brilliance.eibs.core.model.IArgumentDef;
import com.brilliance.eibs.core.model.IFieldDef;
import com.brilliance.eibs.core.model.ISubfieldDef;
import com.brilliance.eibs.core.model.impl.ArgumentDef;

public class ArgMapApi
{

  /***
   * 设置基础组件的CALL 入参
   * **/
  public static List<IArgumentDef> getCallArgs(List<ArgMapping> argMappings)
  {
    List<IArgumentDef> list = new ArrayList<IArgumentDef>();
    List<ArgMapping> topArgs = getTopArgMappings(argMappings);
    for (ArgMapping argMapping : topArgs)
    {
      IArgumentDef argumentDef = new ArgumentDef();
      argumentDef.setValue(CommonUtil.elStr(argMapping.getArgCode()));
      list.add(argumentDef);
    }
    return list;
  }

  /**
   * 设置Call节点对应配置中形式参数的Field
   * **/
  public static List<IFieldDef> getCallInnerArgs(List<ArgMapping> argMappings)
  {
    List<IFieldDef> list = new ArrayList<IFieldDef>();
    List<ArgMapping> topArgs = getTopArgMappings(argMappings);
    int i = 0;
    for (ArgMapping argMapping : topArgs)
    {
      IFieldDef fieldDef = CommonUtil.getFieldDef(argMapping.getArgCode(), null, "${this[" + i + "]}", null, null);
      list.add(fieldDef);
      i++;
    }
    return list;
  }

  public static List<ArgMapping> getTopArgMappings(List<ArgMapping> argMappings)
  {
    List<ArgMapping> list = new ArrayList<ArgMapping>();
    for (ArgMapping argMapping : argMappings)
    {
      if (argMapping.getPntArgCode() == null)
        list.add(argMapping);
    }
    ArgComparator argComparator = new ArgComparator();
    Collections.sort(list, argComparator);
    return list;
  }

  /**
   * 获取基础组件输入参数映射信息
   * ***/
  public static List<IFieldDef> getFieldDefsByArgMapping(List<ArgMapping> argMappings)
  {
    List<IFieldDef> list = new ArrayList<IFieldDef>();

    for (ArgMapping argMapping : argMappings)
    {
      if (argMapping.getPntArgCode() == null)
      {
        IFieldDef fieldDef = CommonUtil.getFieldDef(argMapping.getArgCode(), null, argMapping.getRealExp(), null, null);
        list.add(fieldDef);
        //处理循环
        if (argMapping.getCycFlg() == 1)
        {
          fieldDef.setSubfields(getSubfields(argMapping.getArgCode(), fieldDef, argMappings));
        }
      }
    }
    return list;
  }

  public static List<ArgMapping> getSubArgMappings(String argCode, List<ArgMapping> argMappings)
  {
    List<ArgMapping> list = new ArrayList<ArgMapping>();
    for (ArgMapping argMapping : argMappings)
    {
      if (argMapping.getPntArgCode() != null && argMapping.getPntArgCode().equals(argCode))
      {
        list.add(argMapping);
      }
    }
    return list;
  }

  public static List<ISubfieldDef> getSubfields(String argCode, IFieldDef fieldDef, List<ArgMapping> argMappings)
  {
    List<ISubfieldDef> subfields = new LinkedList<ISubfieldDef>();

    List<ArgMapping> subArgMappings = getSubArgMappings(argCode, argMappings);

    for (ArgMapping argMapping : subArgMappings)
    {
      ISubfieldDef subfield = CommonUtil.getSubFieldDef(argMapping.getArgCode(), null, argMapping.getRealExp(), null, null);
      subfields.add(subfield);
      if (argMapping.getCycFlg() == 1)
      {
        subfield.setSubfields(getSubfields(argMapping.getArgCode(), subfield, argMappings));
      }
    }
    return subfields;
  }
}
