package com.brilliance.ccbank.srcipt.reader;

import java.util.LinkedList;
import java.util.List;

import com.brilliance.ccbank.srcipt.core.FlwMapping;
import com.brilliance.ccbank.srcipt.core.ScriptType;
import com.brilliance.ccbank.srcipt.parser.TokenParser;

/**
 * 脚本阅读器
 * @author weicong
 *
 */
public class DbScriptReader
    extends ScriptReader
{
  List<String>     scripts;
  List<FlwMapping> orginScript;

  public List<String> getScripts()
  {
    return scripts;
  }

  public DbScriptReader( List<FlwMapping> script, String charset )
  {
    this.orginScript = script;
    scripts = new LinkedList<String>();
    handleScript(script);
    System.out.println(getFineScripts());
  }

  public void handleScript(List<FlwMapping> script)
  {
    scripts.clear();
    //处理头
    FlwMapping tmp = null;
    for (FlwMapping fm : script)
    {
      if (-1 == fm.getSeqId())
      {
        tmp = fm;
        break;
      }
    }
    if (tmp == null)
      throw new IllegalArgumentException("未发现脚本启动命令行");
    addLast(tmp, tmp.getBegin());
    //处理核心
    handleCoreScript(script, tmp);
  }

  private void handleCoreScript(List<FlwMapping> script, FlwMapping cur)
  {
    boolean isIf = true;
    for (FlwMapping fm : script)
    {
      if (fm.getSeqId() == cur.getNextSeqId())
      {
        if (TokenParser.BC.equals(fm.getCompTyp()))
        {
          addLast(fm, fm.getCall());
        }
        else if (TokenParser.CC.equals(fm.getCompTyp()))
        {
          if (fm.getRetVal() != null ){
            if(isIf){
              isIf=false;
              addLast(fm, fm.getIf());
            }
            else
              addLast(fm, fm.getElseIf());
          }else
            addLast(fm, fm.getElse());
        }else if(ScriptType.END.TYPE_CODE.equals(fm.getSeqId().toString())){
          addLast(fm, fm.getEnd());
        }
        handleCoreScript(script, fm);
      }
    }
  }

  /**
   * 编辑脚本，并将已完成元素剔除
   * @param fm
   * @param str
   * @param script
   */
  private void addLast(FlwMapping fm, String str)
  {
    scripts.add(scripts.size(), str);
  }

  @Override
  public String readScriptLine()
      throws Exception
  {
    while ((scriptline = readLine()) != null)
    {
      return scriptline;
    }
    return null;
  }

  public String readLine() throws Exception
  {
    if(scripts.size()>0){
      scriptline=scripts.get(0);
      scripts.remove(0);
      return scriptline.trim();
    }
    return null;
  }

  public String getFineScripts()
  {
    StringBuffer sb=new StringBuffer();
    for(String s:scripts){
      sb.append(s).append(System.getProperty("line.separator"));
    }
    return sb.toString();
  }
}
