package com.brilliance.ext.util;

import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;


import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class EncryUtil {
    static Logger logger = LoggerFactory.getLogger("EncryUtil");

    /**
     * @param args
     * @throws BadPaddingException
     * @throws IllegalBlockSizeException
     * @throws NoSuchPaddingException
     * @throws InvalidKeySpecException
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeyException
     */
    public static void main(String[] args) throws Exception{
        if (args.length != 2){
            throw new NullPointerException("parameter not satisfied,the lack of the plaintext password");
        }
        String way=args[0];
        String coderaw = args[1];
        if("D".equals(way)){
            byte[] data = Base64.decodeBase64(coderaw);
            String code=new String(DESDecodeCoder.decrypt(data, DESCoder.generateKey()));
            logger.info("After decryption==" + code.trim());
        }else{
            String code =
                    Base64.encodeBase64URLSafeString(DESEncodeCoder.encrypt((null == coderaw
                            ? ""
                            : coderaw).getBytes(), DESEncodeCoder.generateKey()));
            logger.info("After encryption==" + code.trim());
        }
    }

    static abstract class DESCoder{
        protected static final String KEY_ALGORITHM = "DES";
        protected static final int KEY_SIZE_64 = 64;
        protected static final int KEY_SIZE_56 = 56;
        protected static final String keyword = "www.brilliance.com.cn";
        protected static byte[] generateKey() throws NoSuchAlgorithmException {
            return generateKey(keyword, KEY_SIZE_56);
        }

        protected static byte[] generateKey(String strKey, int keySize)
                throws NoSuchAlgorithmException {
            return generateKey(strKey, keySize, KEY_ALGORITHM);
        }

        protected static byte[] generateKey(String strKey, int keySize, String algorithm)
                throws NoSuchAlgorithmException {
            KeyGenerator kg = KeyGenerator.getInstance(algorithm);
            kg.init(keySize, getSecureRandom(strKey));
            SecretKey secretKey = kg.generateKey();
            return secretKey.getEncoded();
        }
        
        private static SecureRandom getSecureRandom(String strKey) throws NoSuchAlgorithmException {
            if (strKey == null) return null;
            SecureRandom secureRandom = SecureRandom.getInstance("SHA1PRNG");
            secureRandom.setSeed(strKey.getBytes());
            return secureRandom;
        }
    }
    
    static class DESDecodeCoder extends DESCoder{
        public static byte[] decrypt(byte[] data, byte[] key) throws Exception{
            return decrypt(data, key, KEY_ALGORITHM);
        }

        private static byte[] decrypt(byte[] data, byte[] key, String algorithm) throws Exception{
            Key k = new SecretKeySpec(key, algorithm);
            Cipher cipher = Cipher.getInstance(algorithm);
            cipher.init(Cipher.DECRYPT_MODE, k);
            return cipher.doFinal(data);
        }
    }
    
    static class DESEncodeCoder extends DESCoder{
        public static byte[] encrypt(byte[] data, byte[] key) throws InvalidKeyException,
                NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException,
                IllegalBlockSizeException, BadPaddingException {
            return encrypt(data, key, KEY_ALGORITHM);
        }

        public static byte[] encrypt(byte[] data, byte[] key, String algorithm)
                throws InvalidKeyException, NoSuchAlgorithmException, InvalidKeySpecException,
                NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException {
            Key k = new SecretKeySpec(key, algorithm);
            Cipher cipher = Cipher.getInstance(k.getAlgorithm());
            cipher.init(Cipher.ENCRYPT_MODE, k);
            return cipher.doFinal(data);
        }
    }
}
