package com.brilliance.swift;

import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.io.IOUtils;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@SuppressWarnings("rawtypes")
public class MTUnPackMessage extends AbstractMTPackMessage {

    public static Map<String, String> getSwfMsgInfo(String pth) {
        String msg = getSwfMsg(pth);
        return getSwfMsgInfoViaMsg(msg);
    }

    public static String getSwfMsg(String pth) {
        BufferedReader br = null;
        try {
            br = new BufferedReader(new FileReader(pth));
            StringBuilder sb = new StringBuilder();
            char[] buf = new char[1024];
            int len = -1;
            while ((len = br.read(buf)) > 0) {
                sb.append(buf, 0, len);
            }
            return sb.toString();
        } catch (Exception e) {
            log.error("getSwfMsg error", e);
            return null;
        } finally {
            IOUtils.closeQuietly(br);
        }
    }

    public static boolean isContainFieldIn4(String pth, String field) {
        String msg = getSwfMsg(pth);
        if (msg == null)
            return false;
        if ((msg != null) && (msg.trim().length() > 0)) {
            Map heads = anlyMsgHead(msg);
            String hd4 = heads.containsKey(Integer.valueOf(-4)) ? (String) heads
                    .get(Integer.valueOf(-4)) : (String) heads.get(Integer
                    .valueOf(4));
            if (hd4 != null) {
                List<String[]> kv = msgTxtAnly(hd4);
                for (String[] strArr : kv) {
                    if (strArr[0].matches(field)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public static String getFieldValIn4(String pth, String field) {
        String msg = getSwfMsg(pth);
        if (msg == null)
            return null;
        if ((msg != null) && (msg.trim().length() > 0)) {
            Map heads = anlyMsgHead(msg);
            String hd4 = heads.containsKey(Integer.valueOf(-4)) ? (String) heads
                    .get(Integer.valueOf(-4)) : (String) heads.get(Integer
                    .valueOf(4));
            if (hd4 != null) {
                List<String[]> kv = msgTxtAnly(hd4);
                for (String[] strArr : kv) {
                    if (strArr[0].matches(field)) {
                        return strArr[1];
                    }
                }
            }
        }
        return null;
    }

    public static Map<String, String> getFieldsForMsg(String msg) {
        Map<String, String> cache = new HashMap<String, String>();
        if (msg == null)
            return cache;
        if ((msg != null) && (msg.trim().length() > 0)) {
            Map heads = anlyMsgHead(msg);
            String hd4 = heads.containsKey(Integer.valueOf(-4)) ? (String) heads
                    .get(Integer.valueOf(-4)) : (String) heads.get(Integer
                    .valueOf(4));
            if (hd4 != null) {

                List<String[]> kv = msgTxtAnly(hd4);
                for (String[] strArr : kv) {
                    cache.put(strArr[0], strArr[1]);
                }
                return cache;
            }
        }
        return cache;
    }

    public static Map<String, List<String>> getFieldsForMsgV2(String msg) {
        Map<String, List<String>> cache = new HashMap<String, List<String>>();
        if (msg == null)
            return cache;
        if ((msg != null) && (msg.trim().length() > 0)) {
            Map heads = anlyMsgHead(msg);
            String hd4 = heads.containsKey(Integer.valueOf(-4)) ? (String) heads
                    .get(Integer.valueOf(-4)) : (String) heads.get(Integer
                    .valueOf(4));
            if (hd4 != null) {

                List<String[]> kv = msgTxtAnly(hd4);
                for (String[] strArr : kv) {
                    List<String> tmp = cache.get(strArr[0]);
                    if (null == tmp) {
                        tmp = new ArrayList<String>();
                        cache.put(strArr[0], tmp);
                    }
                    tmp.add(strArr[1]);
                }
                return cache;
            }
        }
        return cache;
    }

    public static Map<String, String> getFieldsIn4(String pth) {
        String msg = getSwfMsg(pth);
        return getFieldsForMsg(msg);
    }

    public static Map<String, List<String>> getFieldsIn4V2(String pth) {
        String msg = getSwfMsg(pth);
        return getFieldsForMsgV2(msg);
    }

    @SuppressWarnings("unchecked")
    public static Map<String, String> getSwfMsgInfoViaMsg(String msg) {
        Map infMap = new HashMap();
        if ((msg != null) && (msg.trim().length() > 0)) {
            Map heads = anlyMsgHead(msg);
            String hd4 = (String) heads.get(Integer.valueOf(4));
            boolean isNACK = false;
            if ((hd4 != null)
                    && ((hd4.contains("{451:0}")) || (hd4.contains("{451:1}")))) {
                isNACK = true;
                if (hd4.contains("{451:0}"))
                    infMap.put("mty", "ACK");
                else {
                    infMap.put("mty", "NCK");
                }
                int errIdx = hd4.indexOf("{405:");
                if (errIdx > -1) {
                    int idx2 = hd4.indexOf("}", errIdx);
                    if (idx2 > -1)
                        infMap.put("errcod", hd4.substring(errIdx + 5, idx2));

                }

            }

            String hd2 = heads.containsKey(Integer.valueOf(-2)) ? (String) heads
                    .get(Integer.valueOf(-2)) : (String) heads.get(Integer
                    .valueOf(2));
            String hd1 = heads.containsKey(Integer.valueOf(-1)) ? (String) heads
                    .get(Integer.valueOf(-1)) : (String) heads.get(Integer
                    .valueOf(1));
            String hd3 = heads.containsKey(Integer.valueOf(-3)) ? (String) heads
                    .get(Integer.valueOf(-3)) : (String) heads.get(Integer
                    .valueOf(3));
            hd4 = heads.containsKey(Integer.valueOf(-4)) ? (String) heads
                    .get(Integer.valueOf(-4)) : (String) heads.get(Integer
                    .valueOf(4));
            String hd5 = heads.containsKey(Integer.valueOf(-5)) ? (String) heads
                    .get(Integer.valueOf(-5)) : (String) heads.get(Integer
                    .valueOf(5));
            if (hd5 != null)
                infMap.put("trl", hd5);
            String msgTyp = null;
            String direct = null;
            if (hd2 != null) {
                if (hd2.length() >= 4) {
                    direct = hd2.substring(0, 1);
                    infMap.put("direct", direct);
                    msgTyp = hd2.substring(1, 4);
                    if ("I".equals(direct)) {
                        if (hd2.length() >= 16) {
                            String rcvbic = hd2.substring(4, 16);
                            infMap.put("rcvbic", AbstractMTPackMessage.getBicByLogicalTerminalAddress(rcvbic));
                        }
                        if (hd2.length() >= 17)
                            infMap.put("prt", hd2.substring(16, 17));
                        if (hd2.length() >= 18)
                            infMap.put("tns", hd2.substring(17, 18));
                        if (hd2.length() >= 21)
                            infMap.put("dyt", hd2.substring(18, 21));
                    } else if (("O".equals(direct)) && (hd2.length() >= 26)) {
                        String sndbic = hd2.substring(14, 26);
                        infMap.put("sndbic", AbstractMTPackMessage.getBicByLogicalTerminalAddress(sndbic));
                        if (hd2.length() >= 36) {
                            infMap.put(
                                    "mir",
                                    hd2.substring(8, 14) + " "
                                            + hd2.substring(14, 36));
                            if (hd2.length() == 47)
                                infMap.put("prt", hd2.substring(46));
                        }
                    }
                }
            }

            if (hd1 != null) {
                if (hd1.length() >= 15) {
                    String bnk1 = hd1.substring(3, 15);
                    if ("O".equals(direct))
                        infMap.put("rcvbic", AbstractMTPackMessage.getBicByLogicalTerminalAddress(bnk1));
                    else if ("I".equals(direct)) {
                        infMap.put("sndbic", AbstractMTPackMessage.getBicByLogicalTerminalAddress(bnk1));
                    }
                    if ((hd2 != null) && (hd2.length() >= 14))
                        infMap.put("mor",
                                hd2.substring(8, 14) + " " + hd1.substring(3));
                }
            }
            int idx;
            if (hd3 != null) {
                if (hd3.contains("{119:COV}")) {
                    msgTyp = msgTyp + "COV";
                }

                Matcher m = Pattern.compile("\\{108:\\w+\\}").matcher(hd3);
                if (m.find()) {
                    String gp = m.group();
                    idx = gp.indexOf(":");
                    infMap.put("sndnum", gp.substring(idx + 1, gp.length() - 1));
                }
            }
            infMap.put("std", "MT");
            if (!isNACK)
                infMap.put("mty", msgTyp);
            else
                infMap.put("emty", msgTyp);
            if (hd4 != null) {
                List<String[]> kv = msgTxtAnly(hd4);
                for (String[] strArr : kv) {
                    if (strArr[0].endsWith("20")) {
                        if (!infMap.containsKey("ref")) {
                            if (strArr[1].length() > 16)
                                infMap.put("ref", strArr[1].substring(0, 16)
                                        .trim());
                            else
                                infMap.put("ref", strArr[1].trim());
                        }
                    }
                    if (strArr[0].endsWith("21")) {
                        if (!infMap.containsKey("tscref")) {
                            if (strArr[1].length() > 16)
                                infMap.put("tscref", strArr[1].substring(0, 16));
                            else
                                infMap.put("tscref", strArr[1]);
                        }
                    }
                    strArr[0].endsWith("22C");

                    if (strArr[0].endsWith("32A")) {
                        if (strArr[1].matches("\\d{6}[A-Z]{3}\\d{1,15},\\d*")) {
                            String valDat = strArr[1].substring(0, 6);
                            String cur = strArr[1].substring(6, 9);
                            String amt = strArr[1].substring(9);
                            infMap.put("vludat", valDat);
                            infMap.put("amt", amt.replace(",", "."));
                            infMap.put("cur", cur);
                        }
                    }

                    if (strArr[0].endsWith("32B")) {
                        if (strArr[1].matches("\\[A-Z]{3}\\d{1,15},\\d*")) {
                            String cur = strArr[1].substring(0, 3);
                            String amt = strArr[1].substring(3);

                            infMap.put("itdamt", amt.replace(",", "."));
                            infMap.put("itdcur", cur);
                        }
                    }

                    if (strArr[0].endsWith("30")) {
                        String valDat = strArr[1].substring(0, 6);
                        infMap.put("itdvludat", valDat);
                    }

                    if (strArr[0].matches("62[FM]")) {
                        if ((strArr[1] != null)
                                && (strArr[1]
                                .matches("[DC]\\d{6}[A-Z]{3}\\d{1,15},\\d*"))) {
                            String valDat = strArr[1].substring(1, 7);
                            String cur = strArr[1].substring(7, 10);
                            String amt = strArr[1].substring(10).replace(",",
                                    ".");
                            infMap.put("vludat", valDat);
                            infMap.put("amt", amt.replace(",", "."));
                            infMap.put("cur", cur);
                        } else {
                            return infMap;
                        }
                    }
                    if (strArr[0].matches("25")) {
                        String act = strArr[1];
                        infMap.put("act25", act);
                    }
                    if (("O".equals(infMap.get("direct")))
                            && (strArr[0].matches("54A"))
                            && (strArr[1] != null)) {
                        String actbic = strArr[1];
                        int secondLine = getLinePos(actbic);
                        if (secondLine > -1) {
                            actbic = actbic.substring(secondLine);
                            actbic = actbic.replaceAll("\\r|\\n", "");
                        }
                        infMap.put("actbic", actbic);
                    }
                    if (("O".equals(infMap.get("direct")))
                            && (strArr[0].matches("53A"))
                            && (strArr[1] != null)) {
                        String actbic = strArr[1];
                        int secondLine = getLinePos(actbic);
                        if (secondLine > -1) {
                            actbic = actbic.substring(secondLine);
                            actbic = actbic.replaceAll("\\r|\\n", "");
                        }
                        infMap.put("t53bic", actbic);
                    }
                    if (("I".equals(direct)) && (strArr[0].matches("53A"))
                            && (strArr[1] != null)) {
                        String actbic = strArr[1];
                        int secondLine = getLinePos(actbic);
                        if (secondLine > -1) {
                            actbic = actbic.substring(secondLine);
                            actbic = actbic.replaceAll("\\r|\\n", "");
                        }
                        infMap.put("actbic", actbic);
                    }
                    if ((strArr[0].matches("50[AFK]")) && (strArr[1] != null)) {
                        int index = getLinePos(strArr[1]);
                        if ((index > -1) && (strArr[1].startsWith("/")))
                            infMap.put("act", strArr[1].substring(1, index));
                        if (strArr[0].endsWith("K") || strArr[0].endsWith("F")) {
                            if (index > -1) {
                                String nam = strArr[1].substring(index)
                                        .replaceAll("^[\\r\\n]{1,2}", "");
                                infMap.put("ordnamadr", nam);
                            } else {
                                infMap.put("ordnamadr", strArr[1]);
                            }
                        }
                        if (strArr[0].endsWith("A")) {
                            if (index > -1) {
                                String nam = strArr[1].substring(index)
                                        .replaceAll("^[\\r\\n]{1,2}", "");
                                infMap.put("ordnamadr", nam);
                            } else {
                                infMap.put("ordnamadr", strArr[1]);
                            }
                        }
                    }
                    if ((strArr[0].matches("57A")) && (strArr[1] != null)) {
                        String corbic = strArr[1];
                        infMap.put("corbic", corbic);
                    }
                    if ((strArr[0].matches("59")) && (strArr[1] != null)) {
                        int index = getLinePos(strArr[1]);
                        String nam = strArr[1];
                        if ((index > -1) && (strArr[1].startsWith("/"))) {
//							String act = strArr[1].substring(1, index);
                            String act = strArr[1].substring(1, index).replaceAll("[- ]", "");
                            infMap.put("benact", act);
                            nam = strArr[1].substring(index).replaceAll(
                                    "^[\\r\\n]{1,2}", "");
                        }
                        infMap.put("bennamadr", nam);
                    }

                    if ((strArr[0].matches("59[AF]")) && (strArr[1] != null)) {
                        int index = getLinePos(strArr[1]);
                        String nam = strArr[1];
                        if ((index > -1) && (strArr[1].startsWith("/"))) {
                            String act = strArr[1].substring(1, index).replaceAll("[- ]", "");
                            infMap.put("benact", act);
                            nam = strArr[1].substring(index).replaceAll(
                                    "^[\\r\\n]{1,2}", "");
                        }
                        infMap.put("bennamadr", nam);
                    }

                    if (strArr[0].matches("71A")) {
                        String dtlchg = strArr[1].substring(0, 3);
                        infMap.put("dtlchg", dtlchg);
                    }

                    if ((strArr[0].endsWith("22C"))
                            && (!infMap.containsKey("comref"))) {
                        if (strArr[1].length() > 16)
                            infMap.put("comref", strArr[1].substring(0, 16));
                        else {
                            infMap.put("comref", strArr[1]);
                        }
                    }
                    if (strArr[0].endsWith("33B")) {
                        String cur = strArr[1].substring(0, 3);
                        String amt = strArr[1].substring(3);
                        infMap.put("sellamt", amt.replace(",", "."));
                        infMap.put("sellcur", cur);
                    }

                    if ((strArr[0].endsWith("17R"))
                            && (!infMap.containsKey("BorL")))
                        infMap.put("BorL", strArr[1]);

                    if (strArr[0].endsWith("34E")) {
                        String tagN = strArr[1].substring(0, 1);
                        String interest = "";
                        if ("N".equals(tagN))
                            interest = strArr[1].substring(4);
                        else
                            interest = strArr[1].substring(3);
                        infMap.put("interest", interest.replace(",", "."));
                    }

                    if ((strArr[0].endsWith("30V"))
                            && (!infMap.containsKey("m3xValDate")))
                        infMap.put("m3xValDate", strArr[1]);

                    if (strArr[0].endsWith("32B")) {
                        String cur = strArr[1].substring(0, 3);
                        String amt = strArr[1].substring(3);
                        infMap.put("buyamt", amt.replace(",", "."));
                        infMap.put("buycur", cur);
                    }

                    if (strArr[0].endsWith("79")) {
                        String addinfo = strArr[1];
                        int secondLine = getLinePos(addinfo);
                        if (secondLine > -1) {
                            addinfo = addinfo.substring(secondLine);
                            addinfo = addinfo.replaceAll("\\r|\\n", "");
                        }
                        infMap.put("addinfo", addinfo);
                    }
                    if ("950".equals(msgTyp)) {
                        if ((strArr[0].endsWith("28C")) && (strArr[1] != null)) {
                            infMap.put("bilnum", strArr[1]);
                        }

                        if ((strArr[0].endsWith("60F")) && (strArr[1] != null)) {
                            infMap.put("begamtf", strArr[1]);
                        }
                        if ((strArr[0].endsWith("60M")) && (strArr[1] != null)) {
                            infMap.put("begamtm", strArr[1]);
                        }

                        if ((strArr[0].endsWith("62F")) && (strArr[1] != null)) {
                            infMap.put("endamtf", strArr[1]);
                        }
                        if ((strArr[0].endsWith("62M")) && (strArr[1] != null)) {
                            infMap.put("endamtm", strArr[1]);
                        }

                        if ((strArr[0].endsWith("64")) && (strArr[1] != null)) {
                            infMap.put("valamt", strArr[1]);
                        }
                    }
                }
            }
        }
        return infMap;
    }

    @SuppressWarnings("unchecked")
    public static Map<Integer, String> anlyMsgHead(String msg) {
        Map headSet = new HashMap();
        String reg = "\\{[\\d|S]:";
        Matcher m = Pattern.compile(reg).matcher(msg);
        int lstEnd = -1;
        int lstHead = -1;
        while (m.find()) {
            if ((lstEnd > 0) && (lstHead > 0)) {
                String val = msg.substring(lstEnd, m.start() - 1);
                if (!headSet.containsKey(Integer.valueOf(lstHead)))
                    headSet.put(Integer.valueOf(lstHead), val);
                else {
                    headSet.put(Integer.valueOf(-lstHead), val);
                }
            }
            String g = m.group();
            char c = g.charAt(1);
            lstHead = c;
            if ((c >= '0') && (c <= '9'))
                lstHead -= 48;
            lstEnd = m.end();
        }
        if ((lstEnd > 0) && (lstHead > 0)) {
            if (!headSet.containsKey(Integer.valueOf(lstHead)))
                headSet.put(Integer.valueOf(lstHead), msg.substring(lstEnd));
            else
                headSet.put(Integer.valueOf(-lstHead), msg.substring(lstEnd));
        }
        return headSet;
    }

    private static int getLinePos(String str) {
        if (str == null)
            return -1;
        int idxr = str.indexOf("\r");
        int idxn = str.indexOf("\n");
        if ((idxr > -1) && (idxn > -1)) {
            return idxr < idxn ? idxr : idxn;
        }
        return idxr <= idxn ? idxn : idxr;
    }

    @SuppressWarnings("unchecked")
    public static List<String[]> msgTxtAnly(String mt) {
        String[] tags = mt.split("((\\r\\n)|(\\n\\r)|(\\r)|(\\n)):");
        List list = new ArrayList();
        for (String tv : tags) {
            if (tv.trim().length() > 0) {
                int i = tv.indexOf(':');
                if (i > 0) {
                    String[] arr = new String[2];
                    arr[0] = tv.substring(0, i);
                    arr[1] = tv.substring(i + 1);
                    list.add(arr);
                }
            }
        }
        if (list.size() > 0) {
            String[] lstOne = (String[]) list.get(list.size() - 1);
            int i = lstOne[1].lastIndexOf("\r\n");
            if (i < 0) {
                i = lstOne[1].lastIndexOf("\r");
            }
            if (i < 0) {
                i = lstOne[1].lastIndexOf("\n");
            }
            if (i > -1) {
                lstOne[1] = lstOne[1].substring(0, i);
                list.set(list.size() - 1, lstOne);
            }
        }
        return list;
    }

    public static String[] getTwoLayerFiledVal(String content) {
        String[] layer = new String[2];
        if (StringUtil.isEmpty(content))
            return layer;
        if (content.startsWith("/")) {
            String[] tmp = content.split(SwtBufferedWriter.NEW_LINE);
            layer[0] = tmp[0].substring(1);
            if (content.length() > (tmp[0].length() + 2)) {
                layer[1] = content.substring(tmp[0].length() + 2);
            }
        } else {
            layer[1] = content;
        }
        return layer;
    }

    public static String[] get50FTwoLayerFiledVal(String content) {
        String[] layer = new String[2];
        if (StringUtil.isEmpty(content))
            return layer;

        String[] tmp = content.split(SwtBufferedWriter.NEW_LINE);
//			layer[0]=tmp[0].substring(1);
        layer[0] = tmp[0];
        if (content.length() > (tmp[0].length() + 2)) {
            layer[1] = content.substring(tmp[0].length() + 2);
        }
        return layer;
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
//		String filepath = "F:/brilliance/jkpt-pj/mnbw/swift/swtout/SDBC/00001428.SF2";
        String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift返回/950.sf2";
//		String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift返回/103_02.sf2";
//		String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift_send/103.sf2";

//		String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift返回/202COV.sf2";
//		String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift返回/202.sf2";
//		String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift返回/950.sf2";
//		String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift返回/103(103+202).sf2";
//		String filepath = "F:/brilliance/jkpt-pj/mnbw/data/rcvmsg/0000/00000011.sf2";
//		 String filepath="F:/brilliance/jkpt-pj/mnbw/swift/swtin/700.sf2";
//		String filepath = "F:/brilliance/jkpt-pj/task/南洋商业银行/接口配置/mnbw/swift返回/NCK.sf2";

//		String filepath = "F:\\brilliance\\mnbw\\swift_send\\012.sf2";
        Map<String, String> tmp = getSwfMsgInfo(filepath);
        Map<String, String> tmp1 = getFieldsIn4(filepath);

//		Map<String, String> tmp=getFieldsIn4(filepath);
        System.out.println(tmp);
        System.out.println(tmp1);
        String amt = tmp.get("benact");
        String bennamadr = tmp.get("bennamadr");
        System.out.println(amt);
//		String[] act = getTwoLayerFiledVal(amt);
        String[] bennamadr1 = get50FTwoLayerFiledVal(bennamadr);

        System.out.println("act:" + amt);
        System.out.println("bennamadr1:" + bennamadr);

//		String amt=tmp.get("amt");
//		String a="34362.00";
//		System.out.println(amt.toString());
//		double value1 = Double.valueOf(amt.toString());
//		double value2 = Double.valueOf(a.toString());
//		String b=Double.toString(value1);
//		Double.toString(value2);
//		System.out.println(value1);
//		System.out.println(value2);
//		if(value1==value2) {System.out.println("1");}

//		String pyr=tmp.get("50K");
//		String[]PYRADRBLKS= getTwoLayerFiledVal(pyr);
//		System.out.println(PYRADRBLKS[0]);
//		System.out.println(CommonFunctionUtils.toJson(tmp));
        //		System.out.println(CommonFunctionUtils.toJson(getSwfMsgInfo(filepath)));
//		System.out.println(isContainFieldIn4(filepath,"52[A]"));
//		System.out.println(getFieldValIn4(filepath,"72"));
//		System.out.println(getTwoLayerFiledVal(tmp.get("59"))[1]);
//		System.out.println(getTwoLayerFiledVal("/bicxxxxx"
//				+ "adresssdkflsd")[1]);
    }

}
