package com.brilliance.eibs.bean;

import com.brilliance.eibs.core.model.IMethod;
import com.brilliance.eibs.util.StringUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * 装载接口原型的源类
 * 
 * @author yangrui
 * 
 */
public class InterfaceBean extends Bean {

    /**
     * 
     */
    private static final long serialVersionUID = 1L;
    private boolean impl;
    private List<Method> methods = new ArrayList<Method>();

    public List<Method> getMethods() {
        return methods;
    }

    public void setMethods(List<Method> methods) {
        this.methods = methods;
    }

    public boolean isImpl() {
        return impl;
    }

    public void setImpl(boolean impl) {
        this.impl = impl;
    }

    /**
     * 接口中的方法原型
     * 
     * @author hujun
     * 
     */
    public class Method implements IMethod {

        private static final long serialVersionUID = 1L;
        private String returnType;
        private String name;
        private List<Property> params = new ArrayList<Property>();
        private String impl;
        private String targetNamespace;

        public String getTargetNamespace() {
            return targetNamespace;
        }

        public void setTargetNamespace(String targetNamespace) {
            this.targetNamespace = targetNamespace;
        }

        private String interfaceName;
        private String transactionName;

        public String getReturnType() {
            return returnType;
        }

        public void setReturnType(String returnType) {
            this.returnType = returnType;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public List<Property> getParams() {
            return params;
        }

        public void setParams(List<Property> params) {
            this.params = params;
        }

        public String getImpl() {
            return impl;
        }

        public void setImpl(String impl) {
            this.impl = impl;
        }

        public String getinterfaceName() {
            return interfaceName;
        }

        public void setinterfaceName(String interfaceName) {
            this.interfaceName = interfaceName;
        }

        public String gettransactionName() {
            return transactionName;
        }

        public void settransactionName(String transactionName) {
            this.transactionName = transactionName;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == this) return true;
            if (!(obj instanceof Method)) return false;
            Method method = (Method) obj;
            return method.getName().equals(name) && method.getReturnType().equals(returnType)
                    && method.getImpl().equals(impl) && method.getParams().equals(params);
        }

    }

    public void execute() {
        if (packageName.length() != 0) sb.append("package " + packageName + ";\n\n");
        sb.append("import javax.jws.WebService;\n\n");
        String interfaceNamespace = this.getTagetNameSpace();
        if (StringUtil.isEmpty(interfaceNamespace))
            sb.append("@WebService\n");
        else {
            /**
             * import javax.xml.ws.Action import javax.jws.WebMethod; import javax.jws.WebParam;
             * import javax.jws.WebResult;
             **/
            sb.append("import javax.xml.ws.Action;\n");
            sb.append("import javax.jws.WebMethod;\n");
            sb.append("import javax.jws.WebParam;\n");
            sb.append("import javax.jws.WebResult;\n");
            sb.append("import javax.jws.soap.SOAPBinding;\n");
            sb.append(" import javax.xml.bind.annotation.XmlSeeAlso;\n");
            sb.append("@WebService(targetNamespace =\"" + interfaceNamespace + "\", name=\""
                    + this.getClassName() + "\")\n");
            // sb.append("@XmlSeeAlso({ObjectFactory.class})\n");
            sb.append("@SOAPBinding(parameterStyle = SOAPBinding.ParameterStyle.BARE)\n");
        }

        sb.append("public interface " + className + " { \n");
        String returnType = null;
        String name = null;
        Method method = null;
        for (int i = 0; i < methods.size(); i++) {
            method = methods.get(i);
            returnType = method.getReturnType();
            name = method.getName();
            String bname =
                    name.substring(0, 1).toUpperCase() + name.substring(1, name.length() - 1);
            String methodtargetNamespace = method.getTargetNamespace();
            if (!StringUtil.isEmpty(methodtargetNamespace)) {
                sb.append("@WebResult(name = \"" + bname + "Response\", targetNamespace =\""
                        + methodtargetNamespace + "\", partName = \"" + bname + "Response\")\n");
                sb.append("@Action(input = \"" + this.getTagetNameSpace() + "/"
                        + this.getClassName() + "/" + method.getName() + "Request\", output = \""
                        + this.getTagetNameSpace() + "/" + this.getClassName() + "/"
                        + method.getName() + "Response\")\n");
                sb.append(" @WebMethod\n");
            }
            sb.append("\tpublic ");
            if (null == returnType || "".equals(returnType)) {
                sb.append("void ");
            } else {
                sb.append(returnType + " ");
            }
            sb.append(name + "(");
            List<Property> params = method.getParams();
            Property pro = null;
            for (int j = 0; j < params.size(); j++) {
                pro = params.get(j);
                String header = pro.getHeader();
                if (!StringUtil.isEmpty(header)) {
                    String bproname = pro.getName().substring(0, 1).toUpperCase()
                            + pro.getName().substring(1, pro.getName().length());
                    // @WebParam(partName = "RequestHead", name = "RequestHead", targetNamespace =
                    // "http://cctv.com/ESB/schema/MessageHead/0.1", header = true)
                    sb.append("@WebParam(partName =\"" + bproname + "\", name =\"" + bproname
                            + "\", targetNamespace =\"" + methodtargetNamespace + "\", header ="
                            + header + ")");
                }

                sb.append(pro.getType() + " ");
                sb.append(pro.getName());
                if (j != params.size() - 1) sb.append(", ");
            }
            sb.append(");\n");
        }
        sb.append("}\n");
    }

    /**
     * 输出接口类的的实现类
     */
    public String executeImpl() {
        StringBuffer sb = new StringBuffer("");
        if (packageName.length() != 0) sb.append("package " + packageName + ";\n\n");
        sb.append("import javax.jws.WebService;\n\n");
        sb.append("import " + ImplName + ";\n\n");

        sb.append("@javax.jws.WebService(\n");
        int index = this.getImplName().lastIndexOf(".");
        String interfaceName = "";
        if (index == -1) {
            index = 0;
            interfaceName = this.getImplName().substring(index, this.getImplName().length());
        }

        else {
            interfaceName = this.getImplName().substring(index + 1, this.getImplName().length());
        }
        String service = interfaceName + "Service";
        // serviceName = "TaskReceiveService",
        String serviceName = this.getServiceName();
        if (StringUtil.isEmpty(serviceName)) {
            sb.append("serviceName = \"" + service + "\",\n");
            sb.append("portName = \"" + service + "HttpPort" + "\",\n");
        } else {
            sb.append("serviceName = \"" + serviceName + "\",\n");
            sb.append("portName = \"" + serviceName + "HttpPort" + "\",\n");
        }

        if (this.getTagetNameSpace() != null)
            sb.append("targetNamespace =\"" + this.getTagetNameSpace() + "\",\n");
        if (this.getWsdlLocation() != null)
            sb.append("wsdlLocation = \"" + this.getWsdlLocation() + "\",\n");
        sb.append("endpointInterface =\"" + this.getImplName() + "\")");
        sb.append("\npublic class " + className + " implements " + ImplName + " { \n");
        String returnType = null;
        String name = null;
        Method method = null;
        for (int i = 0; i < methods.size(); i++) {
            method = methods.get(i);
            returnType = method.getReturnType();
            name = method.getName();
            sb.append("\tpublic ");
            if (null == returnType || "".equals(returnType)) {
                sb.append("void ");
            } else {
                sb.append(returnType + " ");
            }
            sb.append(name + "(");
            List<Property> params = method.getParams();
            Property pro = null;
            for (int j = 0; j < params.size(); j++) {
                pro = params.get(j);
                sb.append(pro.getType() + " ");
                sb.append(pro.getName());
                if (j != params.size() - 1) sb.append(", ");
            }
            sb.append("){\n\t\t"
                    + "com.brilliance.eibs.core.service.ResultMsg  msg = new com.brilliance.eibs.main.Client().call(");
            sb.append("\"" + method.getinterfaceName() + "\"" + ", " + "\""
                    + method.gettransactionName() + "\"");
            for (int k = 0; k < params.size(); k++) {
                pro = params.get(k);
                sb.append(", " + pro.getName());
            }

            sb.append(");" + "\n");
            if (!StringUtil.isEmpty(returnType)) {
                sb.append("\t" + "return (");
                sb.append(chg2TypPack(returnType));
                sb.append(") msg.getContent();");
            }
            sb.append("\n\t}" + "\n");
        }
        sb.append("}\n");
        return sb.toString();
    }

    private String chg2TypPack(String s) {
        if ("int".startsWith(s)) return s.replace("int", "Integer");
        if ("short".startsWith(s)) return s.replace("short", "Short");
        if ("long".startsWith(s)) return s.replace("long", "Long");
        if ("byte".startsWith(s)) return s.replace("byte", "Byte");
        if ("float".startsWith(s)) return s.replace("float", "Float");
        if ("double".startsWith(s)) return s.replace("double", "Double");
        if ("char".startsWith(s)) return s.replace("char", "Character");
        if ("boolean".startsWith(s)) return s.replace("boolean", "Boolean");
        return s;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) return true;
        if (!(obj instanceof InterfaceBean)) return false;
        InterfaceBean bean = (InterfaceBean) obj;
        return super.equals(obj) && bean.isImpl() == impl && bean.getMethods().equals(methods);
    }

}
