package com.brilliance.eibs.bean;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.impl.ClassDef;
import com.brilliance.eibs.core.model.impl.ClassListDef;
import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.util.ClassPathUpdater;
import com.brilliance.eibs.util.Serializable2FileUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;

import javax.tools.Diagnostic;
import javax.tools.DiagnosticCollector;
import javax.tools.JavaFileObject;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class ParseConfig
{
  private Logger             logger;
  public static final String buildpath = ClassPathUpdater.addClasspath("classes");
  public static final String separator = "/";

  public void setLogger(Logger logger)
  {
    this.logger = logger;
  }

  /**
   * 解析XML文件，得到类名包名，把所有属性放入容器
   * 
   * @author yangrui
   */
  public List<Bean> parse(String fileName)
  {
    Element rootNode = getXMLConfigRoot(fileName);
    List<Bean> list = parseXml(rootNode);
    return list;
  }

  private List<Bean> getBeansFromClassList(ClassListDef classListDef)
  {
    List<Bean> list = new ArrayList<Bean>();
    List<ClassDef> classlist = classListDef.getclassList();
    String classname = null;
    for (int i = 0; i < classlist.size(); i++)
    {
      classname = classlist.get(i).getclassname();
      Object beanObject = Serializable2FileUtil.readObject("javabean_" + classname);
      if (beanObject != null)
      {
        if ("class".equals(classlist.get(i).getclasstype()) && (null == classlist.get(i).getimpl() || classlist.get(i).getimpl().isEmpty()))
        {
          JavaBean bean = (JavaBean) beanObject;
          list.add(bean);
        }
        else
        {
          InterfaceBean interfacebean = (InterfaceBean) beanObject;
          list.add(interfacebean);
        }

      }
      else
        throw new InterfaceException("00010", "serializable file " + " javabean_" + classname + " not exists");
    }
    return list;
  }

  /**
   * 序列化文件存在时，先读序列化文件，不存在解析指定class节点
   * @param filename 序列化文件名
   * @param xml 配置文件
   */
  private List<Bean> checkserialize(String fileName, Element rootNode)
  {
    ClassListDef classList = null;
    List<Bean> list = new ArrayList<Bean>();
    try
    {
      classList = (ClassListDef) Serializable2FileUtil.readObject(fileName);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    List<ClassDef> classlist = classList.getclassList();
    String classname = null;
    for (int i = 0; i < classlist.size(); i++)
    {
      classname = classlist.get(i).getclassname();
      Object beanObject = null;
      try
      {
        beanObject = Serializable2FileUtil.readObject("javabean_" + classname);
      }
      catch (Exception e1)
      {
        e1.printStackTrace();
      }
      if (beanObject != null)
      {
        if ("class".equals(classlist.get(i).getclasstype()) && (null == classlist.get(i).getimpl() || classlist.get(i).getimpl().isEmpty()))
        {
          try
          {
            JavaBean bean = (JavaBean) beanObject;
            list.add(bean);
          }
          catch (Exception e)
          {
            e.printStackTrace();
          }
        }
        else
        {
          try
          {
            InterfaceBean interfacebean = (InterfaceBean) beanObject;
            list.add(interfacebean);
          }
          catch (Exception e)
          {
            e.printStackTrace();
          }
        }

      }
      else
      {
        String type = classlist.get(i).getclasstype();
        String impl = classlist.get(i).getimpl();
        if ("class".equals(type) && (null == impl || impl.isEmpty()))
        {
          logger.debug(classname + "不存在");
          JavaBean bean = parsejavabean(rootNode, classname);
          list.add(bean);
        }
        else if ("interface".equals(type))
        {
          logger.debug(classname + "不存在");
          InterfaceBean interfacebean = parseInterface(rootNode, classname);
          list.add(interfacebean);
        }
        else if ("class".equals(type) && !(null == impl || impl.isEmpty()))
        {
          logger.debug(classname + "不存在");
          InterfaceBean interfacebean = parseInterfaceimpl(rootNode, classname, impl);
          list.add(interfacebean);
        }
      }
    }
    return list;
  }

  /**
   * 根据路径获取配置类
   * 
   * @param fileName
   *            文件路径(相对编译目录的路径)
   * @return XMLConfiguration
   */
  private Element getXMLConfigRoot(String fileName)
  {
    SAXReader saxReader = new SAXReader();
    try
    {
      Document doc = null;
      File file = new File(fileName);
      if (!file.exists())
        throw new InterfaceException("00004", "can not find generatebean file:[" + fileName + "]");
      doc = saxReader.read(file);
      return doc.getRootElement();
    }
    catch (DocumentException e)
    {
      throw new InterfaceException("00004", "document " + fileName + " read error");
    }
  }

  /**
   * 如果接口以及实现类文件不存在，解析XML得到该CLASS配置文件，并对其序列化
   * @param classname  序列化文件不存在类名
   * @param xml 配置文件
   */
  private JavaBean parsejavabean(Element beansNode, String classname)
  {
    JavaBean bean = new JavaBean();
    List<?> classes = beansNode.elements("class");
    for (int i = 0; i < classes.size(); i++)
    {
      Element class1 = (Element) classes.get(i);
      if (classname.equals(class1.attributeValue("name")))
      {
        bean.setPackageName(class1.attributeValue("package"));
        bean.setClassName(class1.attributeValue("name"));
        List<?> argument = class1.elements("argument");
        for (int j = 0; j < argument.size(); j++)
        {
          Element argumentNode = (Element) argument.get(j);
          Property prop = new Property();
          prop.setName(argumentNode.attributeValue("name"));
          prop.setType(argumentNode.attributeValue("type"));
          bean.getProps().add(prop);
        }
        try
        {
          Serializable2FileUtil.initObject(bean, "javabean_" + classname);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
      }
    }
    return bean;
  }

  /**
   * 如果接口文件不存在，解析XML得到该CLASS配置文件，并对其序列化
   * @param classname  序列化文件不存在类名
   * @param xml 配置文件
   */
  private InterfaceBean parseInterface(Element beans, String classname)
  {
    InterfaceBean interfacebean = new InterfaceBean();
    List<?> classes = beans.elements("class");
    for (int i = 0; i < classes.size(); i++)
    {
      Element class1 = (Element) classes.get(i);
      if (classname.equals(class1.attributeValue("name")))
      {
        interfacebean.setPackageName(class1.attributeValue("package"));
        interfacebean.setClassName(class1.attributeValue("name"));
        List<?> methods = class1.elements("method");
        for (int j = 0; j < methods.size(); j++)
        {
          InterfaceBean.Method method = new InterfaceBean().new Method();
          Element methodNode = (Element) methods.get(j);
          method.setName(methodNode.attributeValue("name"));
          method.setReturnType(methodNode.attributeValue("return"));
          List<?> arguments = methodNode.elements("argument");
          for (int k = 0; k < arguments.size(); k++)
          {
            Element argumentNode = (Element) arguments.get(k);
            Property prop = new Property();
            prop.setName(argumentNode.attributeValue("name"));
            prop.setType(argumentNode.attributeValue("type"));
            prop.setHeader(argumentNode.attributeValue("head"));
            method.getParams().add(prop);
          }
          interfacebean.getMethods().add(method);
        }
        try
        {
          Serializable2FileUtil.initObject(interfacebean, "javabean_" + classname);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
      }
    }
    return interfacebean;
  }

  /**
   * 如果实现类文件不存在，解析XML得到该CLASS配置文件，并对其序列化
   * @param classname  序列化文件不存在类名
   * @param xml 配置文件
   */
  private InterfaceBean parseInterfaceimpl(Element beans, String classname, String impl)
  {
    InterfaceBean interfacebean = new InterfaceBean();
    List<?> classes = beans.elements("class");
    for (int i = 0; i < classes.size(); i++)
    {
      Element class1 = (Element) classes.get(i);
      if (classname.equals(class1.attributeValue("name")))
      {
        interfacebean.setPackageName(class1.attributeValue("package"));
        interfacebean.setClassName(class1.attributeValue("name"));

        interfacebean.setImplName(impl);
        List<?> methods = class1.elements("method");
        for (int j = 0; j < methods.size(); j++)
        {
          InterfaceBean.Method method = new InterfaceBean().new Method();
          Element methodNode = (Element) methods.get(j);
          method.setName(methodNode.attributeValue("name"));
          method.setReturnType(methodNode.attributeValue("return"));
          List<?> arguments = methodNode.elements("argument");
          for (int k = 0; k < arguments.size(); k++)
          {
            Element argumentNode = (Element) arguments.get(k);
            Property prop = new Property();
            prop.setName(argumentNode.attributeValue("name"));
            prop.setType(argumentNode.attributeValue("type"));
            method.getParams().add(prop);
          }
          Element callNode = methodNode.element("call");
          method.setinterfaceName(callNode.attributeValue("interfaceName"));
          method.settransactionName(callNode.attributeValue("transactionName"));

          interfacebean.getMethods().add(method);
        }
        try
        {
          Serializable2FileUtil.initObject(interfacebean, "javabean_" + classname);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
      }
    }
    return interfacebean;

  }

  /**
   * 初次部署时解析整个文件，并得到序列化文件
   * 
   * @param doc
   */
  public List<Bean> parseXml(Element beans)
  {
    List<Bean> list = new ArrayList<Bean>();
    ClassListDef classList = new ClassListDef();

    List<ClassDef> classdeflist = new ArrayList<ClassDef>();
    List<?> classes = beans.elements("class");

    for (int i = 0; i < classes.size(); i++)
    {
      JavaBean bean = new JavaBean();
      InterfaceBean interfacebean = new InterfaceBean();
      ClassDef classdef = new ClassDef();
      //classList.getclassList();     
      Element classes1 = (Element) classes.get(i);
      String classname = classes1.attributeValue("name");
      String type = classes1.attributeValue("type");
      classdef.setclassname(classname);
      classdef.setclasstype(type);
      String impl = classes1.attributeValue("impl");
      if ("class".equals(type) && StringUtil.isEmpty(impl))
      {
        bean.setPackageName(classes1.attributeValue("package"));
        bean.setClassName(classes1.attributeValue("name"));
        List<?> argument = classes1.elements("argument");
        for (int j = 0; j < argument.size(); j++)
        {
          Element argumentNode = (Element) argument.get(j);
          Property prop = new Property();
          prop.setName(argumentNode.attributeValue("name"));
          prop.setType(argumentNode.attributeValue("type"));
          bean.getProps().add(prop);
        }
        try
        {
          Serializable2FileUtil.initObject(bean, "javabean_" + classname);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
        list.add(bean);
      }

      else if ("interface".equals(type))
      {
        interfacebean.setPackageName(classes1.attributeValue("package"));
        interfacebean.setClassName(classes1.attributeValue("name"));
        interfacebean.setTagetNameSpace(classes1.attributeValue("targetNamespace"));

        // interfacebean.setWsdlLocation(classes1.attributeValue("wsdlLocation"));
        List<?> methods = classes1.elements("method");
        for (int j = 0; j < methods.size(); j++)
        {
          InterfaceBean.Method method = new InterfaceBean().new Method();
          Element methodNode = (Element) methods.get(j);
          method.setName(methodNode.attributeValue("name"));
          method.setReturnType(methodNode.attributeValue("return"));
          method.setTargetNamespace(methodNode.attributeValue("targetNamespace"));
          List<?> arguments = methodNode.elements("argument");
          for (int k = 0; k < arguments.size(); k++)
          {
            Element argumentNode = (Element) arguments.get(k);
            Property prop = new Property();
            prop.setName(argumentNode.attributeValue("name"));
            prop.setType(argumentNode.attributeValue("type"));
            prop.setHeader(argumentNode.attributeValue("head"));
            method.getParams().add(prop);
          }
          interfacebean.getMethods().add(method);
        }
        try
        {
          Serializable2FileUtil.initObject(interfacebean, "javabean_" + classname);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
        list.add(interfacebean);
      }
      else if ("class".equals(type) && !StringUtil.isEmpty(impl))
      {
        interfacebean.setPackageName(classes1.attributeValue("package"));
        interfacebean.setClassName(classes1.attributeValue("name"));
        interfacebean.setTagetNameSpace(classes1.attributeValue("targetNamespace"));
        interfacebean.setWsdlLocation(classes1.attributeValue("wsdlLocation"));
        interfacebean.setServiceName(classes1.attributeValue("serviceName"));
        classdef.setimpl(impl);
        interfacebean.setImplName(impl);
        List<?> methods = classes1.elements("method");
        for (int j = 0; j < methods.size(); j++)
        {
          InterfaceBean.Method method = new InterfaceBean().new Method();
          Element methodNode = (Element) methods.get(j);
          method.setName(methodNode.attributeValue("name"));
          method.setReturnType(methodNode.attributeValue("return"));
          List<?> arguments = methodNode.elements("argument");
          for (int k = 0; k < arguments.size(); k++)
          {
            Element argumentNode = (Element) arguments.get(k);
            Property prop = new Property();
            prop.setName(argumentNode.attributeValue("name"));
            prop.setType(argumentNode.attributeValue("type"));
            method.getParams().add(prop);
          }
          Element callNode = methodNode.element("call");
          method.setinterfaceName(callNode.attributeValue("interfaceName"));
          method.settransactionName(callNode.attributeValue("transactionName"));

          interfacebean.getMethods().add(method);
        }

        try
        {
          Serializable2FileUtil.initObject(interfacebean, "javabean_" + classname);
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
        list.add(interfacebean);
      }
      classdeflist.add(classdef);
    }
    for (int n = 0; n < classdeflist.size(); n++)
    {
      logger.debug(classdeflist.get(n).getclassname());
    }
    classList.setclassList(classdeflist);
    for (int m = 0; m < classList.getclassList().size(); m++)
    {
      logger.debug(classList.getclassList().get(m).getclassname());
    }
    try
    {
      Serializable2FileUtil.initObject(classList, "javabean_GenerateBeans");
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    return list;
  }

  /**
   * 批量对beans进行处理 生成javabean以及class文件
   * 
   * @param path
   *            编译生成路径
   */
  public void generateClass( String path)
  {
    try
    {
      logger.debug("buildpath=" + path);
      DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<JavaFileObject>();
      DynamicCompiler dynamicCompilerUtil = new DynamicCompiler();
      boolean compilerResult = dynamicCompilerUtil.compiler("UTF-8", System.getProperty("java.class.path"), path, "", path,
          diagnostics);
      if (compilerResult)
      {
        logger.info("----编译成功-----");
      }
      else
      {
        logger.info("+++++编译失败++++++");
        for (Diagnostic<?> diagnostic : diagnostics.getDiagnostics())
        {
          logger.debug("diagnostic=" + diagnostic.getMessage(null));
        }
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

  }
  
  
  public void generateJavaBeanAndClass(List<Bean> beans, String path)
  {
    for (Iterator<Bean> iterator = beans.iterator(); iterator.hasNext();)
    {
      makeJavaFileAndComplie(path, iterator.next());
    }
    try
    {
     // String classpath = this.getClass().getClassLoader().getResource("").getFile() + path;
      String classpath = CommonFunctionUtils.getCommonURL("").getFile() + path;
      classpath = CommonFunctionUtils.decodeURL(classpath);
      logger.debug("buildpath=" + classpath);
      File fp = new File(buildpath);
      // 创建目录   
      if (!fp.exists())
      {
        fp.mkdirs();// 目录不存在的情况下，创建目录。   
      }
      DiagnosticCollector<JavaFileObject> diagnostics = new DiagnosticCollector<JavaFileObject>();
      DynamicCompiler dynamicCompilerUtil = new DynamicCompiler();
      boolean compilerResult = dynamicCompilerUtil.compiler("UTF-8", System.getProperty("java.class.path"), buildpath, "", buildpath,
          diagnostics);
      if (compilerResult)
      {
        logger.info("----编译成功-----");
      }
      else
      {
        logger.info("+++++编译失败++++++");
        for (Diagnostic<?> diagnostic : diagnostics.getDiagnostics())
        {
          logger.debug("diagnostic=" + diagnostic.getMessage(null));
        }
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

  }

  /**
   * 单个生成javabean及其class文件
   * 
   * @param path
   *            编译生成路径
   * @param bean
   *            需要生成的javabean的配置类
   */
  private void makeJavaFileAndComplie(String path, Bean bean)
  {
    //String separeator=System.getProperty("file.separator");
  //  String classpath = this.getClass().getClassLoader().getResource("").getFile() + path;
    String classpath = CommonFunctionUtils.getCommonURL("").getFile() + path;
    classpath = CommonFunctionUtils.decodeURL(classpath);
    logger.debug("classpath=" + classpath);
    String packagePath = bean.getPackageName().replace(".", separator);
    logger.debug("packagePath=" + packagePath);
    logger.debug("classes file pah=" + buildpath + separator + packagePath);
    File file = new File(buildpath + separator + packagePath);
    boolean ismk = file.mkdirs();
    logger.debug("ismk=" + ismk);
    file = new File(buildpath + separator + packagePath + separator + bean.getClassName() + ".java");
    logger.debug(buildpath + separator + packagePath + separator + bean.getClassName() + ".java");
    logger.debug("get classes file successed..........");
    PrintWriter pw = null;
    if (null == bean.getImplName())
    {
      try
      {
        pw = new PrintWriter(file);
        bean.execute();
        pw.print(bean.getSb().toString());
      }
      catch (FileNotFoundException e)
      {
        logger.error("fileNotFound bean  :", e);
      }
      finally
      {
        if (null != pw)
          pw.close();
      }
    }
    else
    {
      if (bean instanceof InterfaceBean)
      {
        {
          file = new File(buildpath + separator + packagePath);
          boolean intmk = file.mkdirs();
          logger.debug("intmk=" + intmk);
          String ifilepath = buildpath + separator + packagePath + separator + bean.getClassName() + ".java";
          file = new File(ifilepath);
          logger.debug("ifilepath=" + ifilepath);
          logger.debug("interfaceBean excute...................");
        }
        try
        {
          pw = new PrintWriter(file);
          String sb = ((InterfaceBean) bean).executeImpl();
          pw.print(sb);
          logger.debug("Impl excute....................");
        }
        catch (FileNotFoundException e)
        {
          logger.error("filenodfound of Imp:", e);
        }
        finally
        {
          if (null != pw)
            pw.close();
        }
      }
    }

  }

  public boolean equals(Object obj)
  {
    return EqualsBuilder.reflectionEquals(this, obj);
  }

  public int hashCode()
  {
    return HashCodeBuilder.reflectionHashCode(this);
  }

}
