package com.brilliance.eibs.coder.impl;

import com.brilliance.eibs.coder.AsymmetryCoder;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.DHParameterSpec;
import java.security.*;
import java.security.spec.*;
import java.util.HashMap;
import java.util.Map;

/**
 * @author hujun
 *
 * 该类测试未通过
 * 考虑可能是由于JDK版本的问题
 * 单独从Oracle官网下载对应JDK版本的Java Cryptography Extension (JCE) Unlimited Strength Jurisdiction Policy Files文件
 */
public class ELGamalCoder
    extends AsymmetryCoder
{

  public static final String KEY_ALGORITHM = "ELGamal";

  // 160-16384 8的倍数 默认1024
  private static final int   KEY_SIZE      = 256;

  public static Map<String, Object> initKey(int keySize)
      throws NoSuchAlgorithmException, InvalidParameterSpecException, InvalidAlgorithmParameterException
  {
    return generateKey(null, keySize);
  }

  public static Map<String, Object> initKey()
      throws NoSuchAlgorithmException, InvalidParameterSpecException, InvalidAlgorithmParameterException
  {
    return initKey(KEY_SIZE);
  }

  public static Map<String, Object> generateKey(String strKey)
      throws NoSuchAlgorithmException, InvalidParameterSpecException, InvalidAlgorithmParameterException
  {
    return generateKey(strKey, KEY_SIZE);
  }

  public static Map<String, Object> generateKey(String strKey, int keySize)
      throws NoSuchAlgorithmException, InvalidParameterSpecException, InvalidAlgorithmParameterException
  {
    AlgorithmParameterGenerator apg = AlgorithmParameterGenerator.getInstance(KEY_ALGORITHM);
    apg.init(KEY_SIZE);
    AlgorithmParameters paras = apg.generateParameters();
    AlgorithmParameterSpec elParams = paras.getParameterSpec(DHParameterSpec.class);
    KeyPairGenerator kpg = KeyPairGenerator.getInstance(KEY_ALGORITHM);
    SecureRandom secureRandom = null;
    if (strKey == null)
      secureRandom = new SecureRandom();
    else
    {
      secureRandom = SecureRandom.getInstance("SHA1PRNG");
      secureRandom.setSeed(strKey.getBytes());
    }
    kpg.initialize(elParams, secureRandom);
    KeyPair keyPair = kpg.genKeyPair();
    PublicKey publicKey = keyPair.getPublic();
    PrivateKey privateKey = keyPair.getPrivate();
    Map<String, Object> keyMap = new HashMap<String, Object>(2);
    keyMap.put(PUBLIC_KEY, publicKey);
    keyMap.put(PRIVATE_KEY, privateKey);
    return keyMap;
  }

  public static byte[] decryptByPrivateKey(byte[] data, byte[] key)
      throws NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, InvalidKeyException, IllegalBlockSizeException,
      BadPaddingException
  {
    PKCS8EncodedKeySpec pkcs8KeySpec = new PKCS8EncodedKeySpec(key);
    KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
    PrivateKey privateKey = keyFactory.generatePrivate(pkcs8KeySpec);
    Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
    cipher.init(Cipher.DECRYPT_MODE, privateKey);
    return cipher.doFinal(data);
  }

  public static byte[] encryptByPublicKey(byte[] data, byte[] key)
      throws NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, InvalidKeyException, IllegalBlockSizeException,
      BadPaddingException
  {
    X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(key);
    KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
    PublicKey publicKey = keyFactory.generatePublic(x509KeySpec);
    Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
    cipher.init(Cipher.ENCRYPT_MODE, publicKey);
    return cipher.doFinal(data);
  }
}