package com.brilliance.eibs.coder.impl;

import com.brilliance.eibs.coder.Coder;
import org.bouncycastle.util.encoders.Hex;

import javax.crypto.KeyGenerator;
import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

/**
 * @author hujun
 *
 */
public class MACCoder extends Coder
{
  public static final String HMACMD5    = "HmacMD5";
  public static final String HMACMD2    = "HmacMD2";
  public static final String HMACMD4    = "HmacMD4";
  public static final String HMACSHA1   = "HmacSHA1";
  public static final String HMACSHA256 = "HmacSHA256";
  public static final String HMACSHA384 = "HmacSHA384";
  public static final String HMACSHA512 = "HmacSHA512";
  public static final String HMACSHA224 = "HmacSHA224";

  public static byte[] initKey()
      throws NoSuchAlgorithmException
  {
    return initKey(HMACMD5);
  }

  public static byte[] initKey(String algorithm)
      throws NoSuchAlgorithmException
  {
    KeyGenerator keyGenerator = KeyGenerator.getInstance(algorithm);
    SecretKey secretKey = keyGenerator.generateKey();
    return secretKey.getEncoded();
  }

  public static byte[] generateKey(String strKey)
      throws NoSuchAlgorithmException
  {
    return generateKey(strKey, HMACMD5);
  }

  public static byte[] generateKey(String strKey, String algorithm)
      throws NoSuchAlgorithmException
  {
    KeyGenerator keyGenerator = KeyGenerator.getInstance(algorithm);
    SecureRandom secureRandom = SecureRandom.getInstance("SHA1PRNG");
    secureRandom.setSeed(strKey.getBytes());
    keyGenerator.init(secureRandom);
    SecretKey secretKey = keyGenerator.generateKey();
    return secretKey.getEncoded();
  }

  public static String generateKeyHex(String strKey)
      throws NoSuchAlgorithmException
  {
    return generateKeyHex(strKey, HMACMD5);
  }

  public static String generateKeyHex(String strKey, String algorithm)
      throws NoSuchAlgorithmException
  {
    byte[] b = generateKey(strKey, algorithm);
    return new String(Hex.encode(b));
  }

  public static String initKeyHex()
      throws NoSuchAlgorithmException
  {
    return initKeyHex(HMACMD5);
  }

  public static String initKeyHex(String algorithm)
      throws NoSuchAlgorithmException
  {
    byte[] b = initKey(algorithm);
    return new String(Hex.encode(b));
  }

  public static byte[] encode(byte[] data, byte[] key)
      throws NoSuchAlgorithmException, InvalidKeyException
  {
    return encode(data, key, HMACMD5);
  }

  public static byte[] encode(byte[] data, byte[] key, String algorithm)
      throws NoSuchAlgorithmException, InvalidKeyException
  {
    SecretKey secretKey = new SecretKeySpec(key, algorithm);
    Mac mac = Mac.getInstance(secretKey.getAlgorithm());
    mac.init(secretKey);
    return mac.doFinal(data);
  }

  public static String encodeHex(byte[] data, byte[] key)
      throws NoSuchAlgorithmException, InvalidKeyException
  {
    return encodeHex(data, key, HMACMD5);
  }

  public static String encodeHex(byte[] data, byte[] key, String algorithm)
      throws NoSuchAlgorithmException, InvalidKeyException
  {
    byte[] b = encode(data, key, algorithm);
    return new String(Hex.encode(b));
  }
}
