package com.brilliance.eibs.core.model.converter;

import com.brilliance.eibs.core.model.*;
import com.brilliance.eibs.factory.BeanFactory;
import com.brilliance.eibs.factory.FactoryManager;
import com.brilliance.eibs.factory.parser.impl.XmlParser;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import com.thoughtworks.xstream.XStream;
import org.apache.commons.io.FileUtils;
import org.dom4j.*;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class BeBean2Xml {
	final String MAIN_PATH = "cfg/main";
	final String ETC_PATH = "cfg/etc";
	final String HEAD = "<root xmlns=\"http://www.brilliance.com.cn/interface\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.brilliance.com.cn/interface ../etc/schema/interface.xsd\">";
	final String SERVICEHEAD = "<root xmlns=\"http://www.brilliance.com.cn/service\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.brilliance.com.cn/service ../etc/schema/service.xsd\">";

	XmlParser xmlParser = new XmlParser(LogUtil.getLogger("services"));

	public static void main(String[] args) {

	}

	public boolean servicedef2XmlFile(List<IServiceDef> serviceDefs) throws DocumentException {
		XStream xstream = new XStream();
		String xml;
		xstream.autodetectAnnotations(true);
		xstream.setMode(XStream.NO_REFERENCES);
		xstream.alias("services", LinkedList.class);
		xml = xstream.toXML(serviceDefs);
		StringBuilder sb = new StringBuilder(xml);
		sb.insert(sb.indexOf("<services"), SERVICEHEAD + System.getProperty("line.separator"));
		sb.insert(sb.length(), System.getProperty("line.separator") + "</root>");
		xml = sb.toString();
		FormatXml2File(DocumentHelper.parseText(xml.replaceAll("\\s+", " ")), ETC_PATH + "/services.xml");
		return true;
	}

	public void bean2xml(String storedir, String interfacenam) {
		// 存储根路设置为为空，则默认为存储在cgf/maig目录下
		if (StringUtil.isEmpty(storedir))
			storedir = MAIN_PATH;
		if (!StringUtil.isEmpty(storedir) && !storedir.endsWith("/"))
			storedir += "/";

		BeanFactory beanFactory = FactoryManager.getBeanFactory();
		List<IInterfaceDef> temp = new ArrayList<IInterfaceDef>();
		if (interfacenam != null && interfacenam != "")
			temp.add(beanFactory.getInterface(interfacenam));
		else
			temp.addAll(beanFactory.getInterfaceMap().values());

		for (IInterfaceDef ifd : temp) {
			try {
				String xmlstr = bean2xmlStr(ifd);
				FormatXml2File(DocumentHelper.parseText(xmlstr.replaceAll("\\s+", " ")), storedir + ifd.getId() + ".xml");
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	/**
	 * 直接通过接口对象转换为接口配置文件，该文件保存在\etc\main目录下
	 * 该方法可配合getIInterfaceDef方法使用，通过xml获取接口对象
	 * 
	 * @param ifd
	 * @throws Exception
	 */
	public void bean2xmlfile(IInterfaceDef ifd) throws Exception {
		try {
			FormatXml2File(DocumentHelper.parseText(bean2xmlStr(ifd).replaceAll("\\s+", " ")), MAIN_PATH + "/" + ifd.getId() + ".xml");
		} catch (DocumentException e) {
			throw new Exception(e);
		}
	}

	private String bean2xmlStr(IInterfaceDef ifd) throws Exception {
		XStream xstream = new XStream();
		String xml;
		xstream.autodetectAnnotations(true);
		xstream.setMode(XStream.NO_REFERENCES);
		xml = xstream.toXML(ifd);
		Document doc = null;
		try {
			doc = DocumentHelper.parseText(xml);
			// 提示：目前未能利用xstream完全转换ifDef对象成为标准接口配置，因此利用Dom4j进行弥补
			if (doc.selectNodes("//if[not(@*)]").size() > 0) {
				xml = IfMakeUp(doc);
			}
		} catch (DocumentException e) {
			throw new Exception(e);
		}
		// 最外层添加root元素
		// Document document = DocumentHelper.createDocument();
		// document.addElement("root")
		// .addAttribute("xmlns", "\"http://www.brilliance.com.cn/interface\"")
		// .addAttribute("xmlns:xsi",
		// "\"http://www.w3.org/2001/XMLSchema-instance\"")
		// .addAttribute("xsi:schemaLocation",
		// "\"http://www.brilliance.com.cn/interface ../etc/schema/interface.xsd\"");
		StringBuilder sb = new StringBuilder(xml);
		sb.insert(sb.indexOf("<interface"), HEAD + System.getProperty("line.separator"));
		sb.insert(sb.length(), System.getProperty("line.separator") + "</root>");

		// 增加connections和filters配置
		return ConnectionsAndFiltersMakeUp(sb.toString(), ifd);
	}

	private String ConnectionsAndFiltersMakeUp(String xml, IInterfaceDef ifd) throws Exception {
		try {
			Document doc = DocumentHelper.parseText(xml);
			Element root = doc.getRootElement();
			if (ifd.getConnections().size() > 0) {
				Element connectionsnode = DocumentHelper.createElement(new QName("connections", root.getNamespace()));
				for (IConnectionDef c : ifd.getConnections()) {
					Element connectionnode = DocumentHelper.createElement(new QName("connection", root.getNamespace()));
					connectionnode.addAttribute("id", c.getId());
					connectionnode.addAttribute("class", c.getExecuteClass());
					if (c.isPooled())
						connectionnode.addAttribute("pooled", "true");
					for (IPropertyDef p : c.getPropertyMap().values()) {
						Element propertynode = DocumentHelper.createElement(new QName("property", root.getNamespace()));
						propertynode.addAttribute("name", p.getName());
						propertynode.addAttribute("value", p.getValue());
						connectionnode.add(propertynode);
					}
					connectionsnode.add(connectionnode);
				}
				root.add(connectionsnode);
			}

			if (ifd.getFilters().size() > 0) {
				Element filtersnode = DocumentHelper.createElement(new QName("filters", root.getNamespace()));
				for (IFilterDef f : ifd.getFilters()) {
					Element filternode = DocumentHelper.createElement(new QName("filter", root.getNamespace()));
					filternode.addAttribute("class", f.getExecuteClass());
					filternode.addAttribute("id", f.getId());
					Map<String, String> pm = f.getCommonsPropertyMap();
					for (String key : pm.keySet()) {
						Element parameternode = DocumentHelper.createElement(new QName("parameter", root.getNamespace()));
						parameternode.addAttribute("name", key);
						parameternode.addAttribute("value", pm.get(key));
						filternode.add(parameternode);
					}
					filtersnode.add(filternode);
				}
				root.add(filtersnode);
			}
			return doc.asXML();
		} catch (DocumentException e) {
			throw new Exception(e);
		}
	}

	/**
	 * 弥补措施：目前未能利用xstream完全转换ifDef对象成为标准接口配置，因此利用Dom4j进行弥补
	 * 
	 * @param doc
	 */
	@SuppressWarnings("unchecked")
	private String IfMakeUp(Document doc) {
		List<Element> ifnodes = doc.selectNodes("//if[not(@*)]");
		List<Element> cachedelnode = new LinkedList<Element>();
		for (Element element : ifnodes) {
			List<Element> entrynodes = element.selectNodes("./entry");
			Element elsenode = (Element) element.selectSingleNode("./else");
			// 第一个Entry是if节点，剩余的是elseif节点
			boolean flag = true;
			int elseifnum = 0;
			List<Element> iftjnodes = null;// 与if同层的所有节点，包括if节点自身
			for (Element entrynode : entrynodes) {
				++elseifnum;
				String condition = entrynode.selectSingleNode("./string").getText();
				List<Element> listnodes = entrynode.selectNodes("./list/*");
				// 缓存待删除节点
				cachedelnode.add(entrynode);
				// if的处理
				if (flag) {
					flag = false;
					element.addAttribute("condition", condition);
					for (Element e : listnodes)
						element.add(e.detach());
				} else {
					// 插入elseif节点，紧跟在if节点后，后续节点后移
					iftjnodes = element.getParent().elements();
					Element elseifnode = DocumentHelper.createElement("elseif");
					elseifnode.addAttribute("condition", condition);
					for (Element e : listnodes)
						elseifnode.add(e.detach());
					iftjnodes.add(iftjnodes.indexOf(element) + 1, elseifnode);
				}
			}

			// 最后处理else
			if (elsenode != null) {
				Attribute attribute = elsenode.attribute("class");
				if (attribute != null)
					elsenode.remove(attribute);
				iftjnodes = element.getParent().elements();
				iftjnodes.add(iftjnodes.indexOf(element) + elseifnum, (Element) elsenode.detach());
			}
		}
		clearNodes(cachedelnode);
		return doc.asXML();
	}

	/**
	 * 移除当前节点及其子节点
	 * 
	 * @param curnodes
	 */
	private void clearNodes(List<Element> curnodes) {
		for (Element curnode : curnodes) {
			curnode.getParent().remove(curnode);
		}
	}

	private void FormatXml2File(Document doc, String path) {
		FileWriter fileWriter;
		try {
			String root = BeBean2Xml.class.getResource("/").getPath();
			File tmp = new File(root + path);
			if (!tmp.exists()) {
				if (!tmp.getParentFile().exists()) {
					if (tmp.getParentFile().mkdirs())
						tmp.createNewFile();
					else
						throw new IOException();
				}
			} else {
				tmp.delete();
				tmp.createNewFile();
			}
			fileWriter = new FileWriter(tmp);
			OutputFormat xmlFormat = new OutputFormat();
			xmlFormat.setEncoding("UTF-8");
			xmlFormat.setIndent(true);
			xmlFormat.setNewlines(true);
			xmlFormat.setNewLineAfterDeclaration(false);
			XMLWriter xmlWriter = new XMLWriter(fileWriter, xmlFormat);
			xmlWriter.write(doc);
			xmlWriter.close();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

	public IInterfaceDef getInterfaceDef(String interfaceid) {
		File file = null;
		String filename = "";
		try {
			// 首先在BeBean2Xml.class.getResource("/").getPath()目录下查找,便可直接执行
			filename = BeBean2Xml.class.getResource("/").getPath() + MAIN_PATH + "/" + interfaceid + ".xml";
			file = new File(filename);
			if (file.exists()) {
				return null;
			}
			// 其次在System.getProperty("user.dir")\cfg\main目录下搜索名为interfaceid.xml的文件
			filename = System.getProperty("user.dir") + "/" + MAIN_PATH + "/" + interfaceid + ".xml";
			file = new File(filename);
			if (file.exists()) {
				// 需去掉root的命名空间
				String filecontent = FileUtils.readFileToString(file, "UTF-8");
				filecontent = filecontent.replace(HEAD, "<root>");
				return null;
			}

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		return null;
	}

}
