package com.brilliance.eibs.core.model.impl;

import com.brilliance.eibs.core.model.IArgumentDef;
import com.brilliance.eibs.core.model.IFieldDef;
import com.brilliance.eibs.core.model.ISubfieldDef;
import com.brilliance.eibs.core.model.converter.NullStringConverter;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

@XStreamAlias("field")
@XStreamConverter(NullStringConverter.class)
public class FieldDef
    extends AbsDef
    implements IFieldDef, Serializable
{
  private static final long  serialVersionUID = 1L;
  @XStreamAsAttribute
  private String             id;
  @XStreamAsAttribute
  private String             tag;                                             //数据标签
  @XStreamAsAttribute
  private String             etag;                                            //EMF
  @XStreamAsAttribute
  private String             type;                                            //数据类型
  @XStreamAsAttribute
  private String             size;                                            //数据长度
  @XStreamAsAttribute
  private String             varSize;                                         //变长数据长度
  @XStreamAsAttribute
  private String             value;                                           //值
  @XStreamAsAttribute
  private String             description;                                     //描述
  @XStreamAsAttribute
  private String             scope;                                           //生命周期
  @XStreamImplicit
  private List<Object>       convertorsAndValidators;                         //转换器和校验器
  @XStreamAsAttribute
  private boolean            required;                                        //是否可以为空
  @XStreamAsAttribute
  private String             codetable;                                       //码表
  @XStreamAsAttribute
  private String             validator;                                       //校验正则式
  @XStreamAsAttribute
  private String             fill;                                            //补充策略
  @XStreamImplicit
  private List<IArgumentDef> arguments        = new ArrayList<IArgumentDef>();
  @XStreamAsAttribute
  private String             direct;

  @XStreamAsAttribute
  private String             group;
  //------------added by hujun on 2015/4/24 -----------
  @XStreamAsAttribute
  private String             ref;                                             //本地外部文件属性
  
  //------------added by weicong on 2016/3/14 -----------
  @XStreamAsAttribute
  private String             split;                                             //filed支持分割符属性
  @XStreamImplicit
  private List<ISubfieldDef> subfields        = new ArrayList<ISubfieldDef>();//subfield子节点


  public List<ISubfieldDef> getSubfields()
  {
    return subfields;
  }

  public void setSubfields(List<ISubfieldDef> subfields)
  {
    this.subfields = subfields;
  }
  
  public String getRef()
  {
    return ref;
  }

  public void setRef(String ref)
  {
    this.ref = ref;
  }

  //---------------------------------------------------

  public String getId()
  {
    return id;
  }

  public void setId(String id)
  {
    this.id = id;
  }

  public String getTag()
  {
    return tag;
  }

  public void setTag(String tag)
  {
    this.tag = tag;
  }

  public String getEtag()
  {
    return etag;
  }

  public void setEtag(String etag)
  {
    this.etag = etag;
  }

  public String getValue()
  {
    return value;
  }

  public void setValue(String value)
  {
    this.value = value;
  }

  public String getType()
  {
    return type;
  }

  public void setType(String type)
  {
    this.type = type;
  }

  public String getFill()
  {
    return fill;
  }

  public void setFill(String fill)
  {
    this.fill = fill;
  }

  public String getSize()
  {
    return size;
  }

  public void setSize(String size)
  {
    this.size = size;
  }

  public String getVarSize()
  {
    return varSize;
  }

  public void setVarSize(String varSize)
  {
    this.varSize = varSize;
  }

  public String getScope()
  {
    return scope;
  }

  public void setScope(String scope)
  {
    this.scope = scope;
  }

  public boolean isRequired()
  {
    return required;
  }

  public void setRequired(boolean required)
  {
    this.required = required;
  }

  public String getCodetable()
  {
    return codetable;
  }

  public void setCodetable(String codetable)
  {
    this.codetable = codetable;
  }

  public String getGroup()
  {
    return group;
  }

  public void setGroup(String group)
  {
    this.group = group;
  }

  public String getDescription()
  {
    return description;
  }

  public void setDescription(String description)
  {
    this.description = description;
  }

  public List<Object> getConvertorsAndValidators()
  {
    return convertorsAndValidators;
  }

  public void setConvertorsAndValidators(List<Object> convertorsAndValidators)
  {
    this.convertorsAndValidators = convertorsAndValidators;
  }

  public boolean equals(Object obj)
  {
    boolean b = EqualsBuilder.reflectionEquals(this, obj);
    return b;
  }

  public int hashCode()
  {
    return HashCodeBuilder.reflectionHashCode(this);
  }

  @Override
  public void setArguments(List<IArgumentDef> arguments)
  {
    this.arguments = arguments;
  }

  @Override
  public List<IArgumentDef> getAuguments()
  {
    return this.arguments;
  }

  @Override
  public String getUnique()
  {
    return "field:" + id;
  }

  public String getValidator()
  {
    return validator;
  }

  public void setValidator(String validator)
  {
    this.validator = validator;
  }

  public String getSplit()
  {
    return split;
  }

  public void setSplit(String split)
  {
    this.split = split;
  }

  public String getDirect()
  {
    return direct;
  }

  public void setDirect(String direct)
  {
    this.direct = direct;
  }

}
