package com.brilliance.eibs.core.model.impl;

import com.brilliance.eibs.core.model.IAbsDef;
import com.brilliance.eibs.core.model.IFilterDef;
import com.brilliance.eibs.core.model.converter.NullStringConverter;
import com.brilliance.eibs.util.PropertyUtil;
import com.brilliance.eibs.util.StringUtil;
import com.thoughtworks.xstream.annotations.*;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @see IFilterDef
 */
@XStreamAlias("filter")
@XStreamConverter(NullStringConverter.class)
public class FilterDef
    extends AbsDef
    implements IFilterDef
{
  private static final long   serialVersionUID   = 1L;
  @XStreamOmitField
  private String              id;
  @XStreamAsAttribute
  private String              ref;
  @XStreamAsAttribute
  private String              type;
  @XStreamAsAttribute
  private String              via;
  @XStreamAsAttribute
  private String              scope;
  @XStreamOmitField
  private String              executeClass;
  @XStreamOmitField
  private Map<String, String> commonsPropertyMap = new HashMap<String, String>();
  @XStreamImplicit
  private List<IAbsDef>       subList            = new ArrayList<IAbsDef>();
  @XStreamAsAttribute
  private String              encoding;
  private String              importfile;
  @XStreamAsAttribute
  private String              tag;
  @XStreamAsAttribute
  private String              times;
  @XStreamOmitField
  private String              errorDescription;
  @XStreamOmitField
  private String              level;

  public String getRef()
  {
    return ref;
  }

  public void setRef(String ref)
  {
    this.ref = ref;
  }

  public String getId()
  {
    return this.id;
  }

  public void setId(String id)
  {
    this.id = id;
  }

  public String getType()
  {
    return type;
  }

  public void setType(String type)
  {
    this.type = type;
  }

  public String getExecuteClass()
  {
    return executeClass;
  }

  public void setExecuteClass(String executeClass)
  {
    this.executeClass = executeClass;
  }

  public List<IAbsDef> getSubList()
  {
    return subList;
  }

  public void setSubList(List<IAbsDef> subList)
  {
    this.subList = subList;
  }

  public String getEncoding()
  {
    if (StringUtil.isEmpty(encoding))
      this.encoding = PropertyUtil.getProperty("sys.properties", "SYS_ENCODING");
    return encoding;
  }

  public void setEncoding(String encoding)
  {
    this.encoding = encoding;
  }

  public String getImportfile()
  {
    return importfile;
  }

  public void setImportfile(String importfile)
  {
    this.importfile = importfile;
  }

  public Map<String, String> getCommonsPropertyMap()
  {
    return commonsPropertyMap;
  }

  public String getCommonsProperty(String key)
  {
    String value = commonsPropertyMap.get(key);
    return value;
  }

  public void addCommonsProperty(String key, String value)
  {
    this.commonsPropertyMap.put(key, value);
  }

  public void addSub(IAbsDef def)
  {
    if (def != null)
      this.subList.add(def);
  }

  public String getScope()
  {
    return scope;
  }

  public void setScope(String scope)
  {
    this.scope = scope;
  }

  public void combine(IFilterDef filterDef)
  {
    executeClass = filterDef.getExecuteClass();
    id = filterDef.getId();
    commonsPropertyMap = filterDef.getCommonsPropertyMap();

  }

  public boolean equals(Object obj)
  {
    boolean b = EqualsBuilder.reflectionEquals(this, obj);
    return b;
  }

  public int hashCode()
  {
    return HashCodeBuilder.reflectionHashCode(this);
  }

  @Override
  public String getTimes()
  {
    if (StringUtil.isEmpty(times))
      times = "1";
    return times;
  }

  @Override
  public void setTimes(String times)
  {
    this.times = times;
  }

  public String getTag()
  {
    return tag;
  }

  public void setTag(String tag)
  {
    this.tag = tag;
  }

  public String getVia()
  {
    return via;
  }

  public void setVia(String via)
  {
    this.via = via;
  }

  @Override
  public String getErrorDescription()
  {
    return errorDescription;
  }

  @Override
  public void setErrorDescription(String description)
  {
    this.errorDescription = description;
  }

  @Override
  public String getUnique()
  {
    return "filter:" + ref;
  }

  public String getLevel()
  {
    return level;
  }

  public void setLevel(String level)
  {
    this.level = level;
  }

}
