package com.brilliance.eibs.core.service;

import com.brilliance.eibs.core.exception.ErrorCodes;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.frame.IIdentity;
import com.brilliance.eibs.core.model.*;
import com.brilliance.eibs.core.service.instance.GenericResource;
import com.brilliance.eibs.core.service.instance.IExecutableInstance;
import com.brilliance.eibs.core.service.instance.IResource;
import com.brilliance.eibs.core.service.instance.plugin.ConnectionAdaptor;
import com.brilliance.eibs.core.service.instance.plugin.JdbcExecutor;
import com.brilliance.eibs.core.service.instance.plugin.JotmHelper;
import com.brilliance.eibs.core.service.instance.plugin.TibcoProxy;
import com.brilliance.eibs.el.AbstractEL;
import com.brilliance.eibs.el.Sys;
import com.brilliance.eibs.jedis.JedisManager;
import com.brilliance.eibs.util.Constants;
import com.brilliance.eibs.util.StaticInfomation;
import com.brilliance.eibs.util.StringUtil;

import java.sql.SQLException;
import java.util.*;
import java.util.Map.Entry;

/*
 * 上下文对象类,隶属于接口实例,负责传递变量/处理返回消息
 */
public class Context {

	private Map<String, Object> variableMap = new HashMap<String, Object>();

	private Map<String, Object> inputMap = new HashMap<String, Object>();
	private Map<String, Object> outputMap = new HashMap<String, Object>();

	private Map<String, Object> globalMap = new HashMap<String, Object>();
	private Map<String, Object> transactionMap = new HashMap<String, Object>();
	private Map<String, Object> stepMap = new HashMap<String, Object>();
	private Map<String, Object> transitionMap = new HashMap<String, Object>();

	private Map<String, Map<String, Object>> foreachMap = new HashMap<String, Map<String, Object>>();

	private Map<String, ConnectionAdaptor> connectionAdaptors = new HashMap<String, ConnectionAdaptor>();

	private Map<String, Map<String, String>> codetableMap = new HashMap<String, Map<String, String>>();
	private Map<String, String> dynamicCodetableMap = new HashMap<String, String>();

	private Map<String, IModuleDef> modulesMap = new HashMap<String, IModuleDef>();

	private Map<String, ICounterDef> counterMap = new HashMap<String, ICounterDef>();

	private JedisManager jedisManager = new JedisManager();

	private String logLevel;

	// private BeanFactory beanFactory;
	private ResourceBundle resb;

	private Object currentInstance; // 当前实例

	private IResource resource; // 上下文资源

	private ResultMsg resultMsg; // 返回消息

	private int query_timeout = 0; // 查询超时

	private AbstractEL elParser;

	private ContextFactory factory;

	private IInterfaceDef interfaceDef;

	private ErrorCodes errorCodes;

	private JotmHelper jotmHelper;
	private int rollbackFlag = 0;

	private IIdentity identity;

	public IIdentity getIdentity() {
		return identity;
	}

	public void setIdentity(IIdentity identity) {
		this.identity = identity;
	}

	public JotmHelper getJotmHelper() {
		return jotmHelper;
	}

	public int getRollbackFlag() {
		return rollbackFlag;
	}

	public void setRollbackFlag(int rollbackFlag) {
		this.rollbackFlag = rollbackFlag;
	}

	public void setJotmHelper(JotmHelper jotmHelper) {
		this.jotmHelper = jotmHelper;
	}

	public AbstractEL getElParser() {
		return elParser;
	}

	public void setElParser(AbstractEL elParser) {
		this.elParser = elParser;
	}

	public ContextFactory getFactory() {
		if (factory == null) {
			this.factory = new ContextFactory(this);
		}
		return this.factory;
	}

	public void setFactory(ContextFactory factory) {
		this.factory = factory;
	}

	public int getQuery_timeout() {
		return query_timeout;
	}

	public void setQuery_timeout(int query_timeout) {
		this.query_timeout = query_timeout;
	}

	public ResultMsg getResultMsg() {
		return resultMsg;
	}

	public void setResultMsg(ResultMsg resultMsg) {
		this.resultMsg = resultMsg;
	}

	public void closeConnectionAdaptors() throws SQLException {
		for (Entry<String, ConnectionAdaptor> entry : connectionAdaptors.entrySet()) {
			ConnectionAdaptor connection = entry.getValue();
			if (null != connection)
				connection.close();
		}
	}

	public void closeJotm() {
		if (jotmHelper != null) {
			jotmHelper.stopTMService();
		}
	}

	public void rollbackConnectionAdaptors() throws SQLException {
		for (Entry<String, ConnectionAdaptor> entry : connectionAdaptors.entrySet()) {
			ConnectionAdaptor connection = entry.getValue();
			if (null != connection)
				connection.rollback();
		}
	}

	/**
	 * 添加foreach变量
	 * 
	 * @param absDef
	 * @param key
	 * @param value
	 */
	public void addForeachVariable(IAbsDef absDef, String key, Object value) {
		IAbsDef parentDef = absDef.getParentDef();
		while (parentDef != null && !(parentDef instanceof IForeachDef))
			parentDef = parentDef.getParentDef();
		if (parentDef instanceof IForeachDef) {
			IForeachDef foreachDef = (IForeachDef) parentDef;
			String tag = foreachDef.getTag();
			if (!foreachMap.containsKey(tag)) {
				foreachMap.put(tag, new HashMap<String, Object>());
			}
			foreachMap.get(tag).put(key, value);
		}
	}

	public ResourceBundle getResb() {
		return resb;
	}

	public void setResb(ResourceBundle resb) {
		this.resb = resb;
	}

	/**
	 * 循环每执行一次都对scope="foreach"的变量进行一次清空，按照foreach的tag属性区分不同的foreach
	 * 
	 * @param tag
	 */
	public void removeForeachMap(String tag) {
		foreachMap.remove(tag);
	}

	/**
	 * 获取当前的所有foreach变量，子foreach可以访问父foreach的变量
	 * 
	 * @return
	 */
	public List<Map<String, Object>> getCurrentInstanceForeachMaps() {
		if (currentInstance == null)
			return null;
		IExecutableInstance instance = (IExecutableInstance) currentInstance;
		IAbsDef absDef = (IAbsDef) instance.getIDef();
		IAbsDef parentDef = absDef.getParentDef();
		List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
		while (parentDef != null) {
			if (parentDef instanceof IForeachDef) {
				IForeachDef foreachDef = (IForeachDef) parentDef;
				String tag = foreachDef.getTag();
				Map<String, Object> map = foreachMap.get(tag);
				if (map != null)
					list.add(map);
			}
			parentDef = parentDef.getParentDef();
		}
		return list;
	}

	/**
	 * 增加变量到MAP
	 * 
	 * @param scope
	 *            生命周期 null或"" 表示最小生命周期，只在当前filter或connection有效 step 表示在当前step有效
	 *            transaction 表示在当前整个交易有效 this
	 *            表示将变量放到当前内存的Object中，可以由filter直接进行处理
	 * @param key
	 * @param value
	 */
	public void addVariable(String scope, String key, Object value) {
		if (!StringUtil.isEmpty(scope)) {
			scope = scope.toLowerCase().trim();
			if ("global".equals(scope)) {
				globalMap.put(key, value);
			} else if ("transaction".equals(scope)) {
				transactionMap.put(key, value);
			} else if ("step".equals(scope)) {
				stepMap.put(key, value);
			}
		} else {
			Object obj = this.getVariable("__scope");
			if (obj == null) {
				transitionMap.put(key, value);
			} else {
				String defaultScope = (String) obj;
				if (defaultScope.equals("transition")) {
					transitionMap.put(key, value);
				} else if (defaultScope.equals("step")) {
					stepMap.put(key, value);
				} else if (defaultScope.equals("transaction")) {
					transactionMap.put(key, value);
				}
				//
			}
		}

	}

	/**
	 * 从MAP中获取变量 先从transition------>this----->step--->transaction----->global
	 */

	public Object getVariable(String key) {
		// if (variablesMap.containsKey(key))
		// return variablesMap.get(key);
		if (transitionMap.containsKey(key)) {
			return transitionMap.get(key);
		} else if (stepMap.containsKey(key)) {
			return stepMap.get(key);
		} else if (transactionMap.containsKey(key)) {
			return transactionMap.get(key);
		} else if (globalMap.containsKey(key)) {
			return globalMap.get(key);
		}
		return null;
	}

	public void remove(String scope, String key) {
		if (!StringUtil.isEmpty(scope)) {
			scope = scope.toLowerCase().trim();
			if ("global".equals(scope)) {
				globalMap.remove(key);
			} else if ("transaction".equals(scope)) {
				transactionMap.remove(key);
			} else if ("step".equals(scope)) {
				stepMap.remove(key);
			}
		}
	}

	public IResource getResource() {
		if (this.resource == null) {
			this.resource = new GenericResource();
		}
		return resource;
	}

	public Object getObject() {
		return getResource().getObject();
	}

	public void setObject(Object obj) {
		getResource().setObject(obj);
	}

	public Map<String, ConnectionAdaptor> getConnectionAdaptors() {
		return connectionAdaptors;
	}

	public void setConnections(Map<String, ConnectionAdaptor> connectionAdaptors) {
		this.connectionAdaptors = connectionAdaptors;
	}

	public IInterfaceDef getInterfaceDef() {
		return interfaceDef;
	}

	public void setInterfaceDef(IInterfaceDef interfaceDef) {
		this.interfaceDef = interfaceDef;
	}

	public ErrorCodes getErrorCodes() {
		return errorCodes;
	}

	public void setErrorCodes(ErrorCodes errorCodes) {
		this.errorCodes = errorCodes;
	}

	public void setResource(IResource resource) {
		this.resource = resource;
	}

	public Map<String, Object> getGlobalMap() {
		return globalMap;
	}

	public Map<String, Object> getTransactionMap() {
		return transactionMap;
	}

	public Map<String, Object> getStepMap() {
		return stepMap;
	}

	public Map<String, Object> getTransitionMap() {
		return transitionMap;
	}

	public void setGlobalMap(Map<String, Object> globalMap) {
		this.globalMap = globalMap;
	}

	public void setTransactionMap(Map<String, Object> transactionMap) {
		this.transactionMap = transactionMap;
	}

	public void setStepMap(Map<String, Object> stepMap) {
		this.stepMap = stepMap;
	}

	public Object getCurrentInstance() {
		return currentInstance;
	}

	public void setCurrentInstance(Object currentInstance) {
		this.currentInstance = currentInstance;
		if (currentInstance instanceof IExecutableInstance) {
			IExecutableInstance instance = (IExecutableInstance) currentInstance;
			String via = instance.getIDef().getVia();
			if (!StringUtil.isEmpty(via))
				setObject(elParser.getExPression(this, via, null));
		}
	}

	public void setTransitionMap(Map<String, Object> transitionMap) {
		this.transitionMap = transitionMap;
	}

	public void addCodeTable(String tableName, Map<String, String> table) {
		codetableMap.put(tableName, table);
	}

	public String getCodeTableValue(String tableName, String key) {
		if (StaticInfomation.getDynamicCodeTabe() == null || StaticInfomation.getDynamicCodeTabe().size() == 0) {
			StaticInfomation.setDynamicCodeTabe(codetableMap);
			// codetableMap.clear();
		}
		Map<String, String> map = StaticInfomation.getDynamicCodeTabe().get(tableName);
		// Map<String, String> map = codetableMap.get(tableName);
		String value = null;
		if (map == null) {
			String sql = dynamicCodetableMap.get(tableName);
			if (StringUtil.isEmpty(sql)) {
				throw new InterfaceException("00201", "codetable [" + tableName + "] sql is null.");
			}
			Map<String, String> dynamicMap = getDynamicMap(sql);
			value = dynamicMap.get(key);
			StaticInfomation.getDynamicCodeTabe().put(tableName, dynamicMap);
		} else {
			value = map.get(key);
		}
		// *****modified by hujun on 2014/5/12
		// if (value == null)
		// throw new InterfaceException("00202", "key [" + key +
		// "] not exists is codetable [" +
		// tableName + "]");
		return value;
	}

	public Map<String, String> getDynamicMap(String sql) {
		try {
			JdbcExecutor excutor = new JdbcExecutor(this, getResource().getConnection());
			return excutor.executeQueryForCodeTables(sql);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public Map<String, String> getDynamicCodetableMap() {
		return dynamicCodetableMap;
	}

	public void setDynamicCodetableMap(Map<String, String> dynamicCodetableMap) {
		this.dynamicCodetableMap = dynamicCodetableMap;
	}

	@Override
	public Context clone() {
		Context context = new Context();
		for (Entry<String, Object> globalEntry : this.globalMap.entrySet()) {
			context.addVariable("global", globalEntry.getKey(), globalEntry.getValue());
		}
		for (Entry<String, Object> transactionEntry : this.transactionMap.entrySet()) {
			context.addVariable("transaction", transactionEntry.getKey(), transactionEntry.getValue());
		}
		for (Entry<String, Object> stepEntry : this.stepMap.entrySet()) {
			context.addVariable("step", stepEntry.getKey(), stepEntry.getValue());
		}
		for (Entry<String, Object> transitionEntry : this.transitionMap.entrySet()) {
			context.addVariable("transitionMap", transitionEntry.getKey(), transitionEntry.getValue());
		}
		context.setInterfaceDef(interfaceDef);
		context.setErrorCodes(errorCodes);
		return context;
	}
	
	public void copy(Context context) {
	    this.getTransitionMap().putAll(context.getTransitionMap());
	    this.getStepMap().putAll(context.getStepMap());
	    this.getTransactionMap().putAll(context.getTransactionMap());
	    this.getGlobalMap().putAll(context.getGlobalMap());
	    this.getCurrentInstanceForeachMaps().addAll(context.getCurrentInstanceForeachMaps());
	}
	

	public void setCodetableMap(Map<String, Map<String, String>> codeTables) {
		codetableMap = codeTables;
	}

	public Map<String, ICounterDef> getCounterMap() {
		return counterMap;
	}

	public void setCounterMap(Map<String, ICounterDef> counterMap) {
		this.counterMap = counterMap;
	}

	public Map<String, IModuleDef> getModulesMap() {
		return modulesMap;
	}

	public void setModulesMap(Map<String, IModuleDef> modulesMap) {
		this.modulesMap = modulesMap;
	}

	public Map<String, Object> getVariableMap() {
		// Map<String, Object> variableMap = new HashMap<String, Object>();
		variableMap.clear();
		Map<String, Object> transitionMap = getTransitionMap();
		Map<String, Object> stepMap = getStepMap();
		Map<String, Object> transactionMap = getTransactionMap();
		Map<String, Object> globalMap = getGlobalMap();
		List<Map<String, Object>> foreachMaps = getCurrentInstanceForeachMaps();
		variableMap.putAll(transactionMap);
		variableMap.putAll(stepMap);
		if (foreachMaps != null) {
			for (int i = foreachMaps.size() - 1; i >= 0; i--) {
				variableMap.putAll(foreachMaps.get(i));
			}
		}
		variableMap.putAll(transitionMap);
		variableMap.putAll(globalMap);
		variableMap.put("this", getResource().getObject());
		variableMap.put("super", getResource());
		variableMap.put("system", new Sys());
		return variableMap;
	}

	public String getLogLevel() {
		return logLevel;
	}

	public void setLogLevel(String logLevel) {
		this.logLevel = logLevel;
	}

	public void closeAllConnection() throws Exception {
		for (String key : connectionAdaptors.keySet()) {
			connectionAdaptors.get(key).close();
		}
	}

	public void rollbackAllConnection() throws Exception {
		for (String key : connectionAdaptors.keySet()) {
			connectionAdaptors.get(key).rollback();
		}
	}

	public Map<String, Object> getInputMap() {
		return inputMap;
	}

	public Map<String, Object> getOutputMap() {
		return outputMap;
	}

	public void setInputMap(Map<String, Object> inputMap) {
		this.inputMap = inputMap;
	}

	public void setOutputMap(Map<String, Object> outputMap) {
		this.outputMap = outputMap;
	}

	public void closeJedis() {
		jedisManager.closeJedis();
	}

	public void closeTibco() throws Exception {
		TibcoProxy tibcoproxy = (TibcoProxy) (getResource().getResourceParameter(Constants.TIBCO_PROXY));
		if (tibcoproxy != null)
			tibcoproxy.close();
	}

	public JedisManager getJedisManager() {
		return jedisManager;
	}

	public void setJedisManager(JedisManager jedisManager) {
		this.jedisManager = jedisManager;
	}

}
