package com.brilliance.eibs.core.service.instance.impl;

import com.brilliance.eibs.bean.ParseConfig;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IConnectionDef;
import com.brilliance.eibs.core.model.IExecutableDef;
import com.brilliance.eibs.core.model.IPropertyDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.instance.IConnectionInstance;
import com.brilliance.eibs.el.AbstractEL;
import com.brilliance.eibs.util.DateUtil;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import com.brilliance.eibs.util.XmlUtil;
import org.dom4j.Document;
import org.slf4j.Logger;

import java.util.Map;

/**
 * 连接的基础抽象类，其中完成了部分基础功能
 *
 * @author hujun
 */
public class AbsConnection implements IConnectionInstance {

    private static final int CLIENT_CONNECT_TIMEOUT = 0;
    private static final int CLIENT_RECEIVE_TIMEOUT = 0;

    private static final int CLIENT_SO_TIMEOUT = 0;
    private static final int DEFAULT_TIMEOUT = 0;

    protected int connectTimeout = CLIENT_CONNECT_TIMEOUT;
    protected int receiveTimeout = CLIENT_RECEIVE_TIMEOUT;
    protected int soTimeout = CLIENT_SO_TIMEOUT;
    protected int default_timeout = DEFAULT_TIMEOUT;
    protected IConnectionDef connectionDef;
    protected Context context;

    protected Logger logger;
    protected AbstractEL elParser;

    @Override
    public void setContext(Context context) {
        this.context = context;
        this.logger = LogUtil.getLogger(context);
    }

    @Override
    public void setIDef(IExecutableDef executableDef) {
        this.connectionDef = (IConnectionDef) executableDef;
    }

    @Override
    public IExecutableDef getIDef() {
        return connectionDef;
    }

    @Override
    public void execute(Context context) {
        // context.addVariable("transaction", "__encoding",
        // elParser.getExPression(context, connectionDef.getEncoding(),
        // null).toString());
    }

    /**
     * 返回classname
     *
     * @return
     */
    protected String returnClassName() {
        return this.getClass().getName();
    }

    /**
     * 获取连接的type操作类型(小写)
     *
     * @return
     * @throws InterfaceException
     */
    protected String getType() throws InterfaceException {
        String type = connectionDef.getType();
        // logger.debug("Type attribute of " + returnClassName() + " is " + type
        // + ".");
        if (StringUtil.isEmpty(type)) {
            logger.error("Type attribute of " + returnClassName() + " is null.");
            throw new InterfaceException("00302");
        }
        return type.toLowerCase();
    }

    /**
     * 获取制定名称的连接属性
     *
     * @param name
     * @param canBeEmpty 是否允许为空
     * @return
     * @throws InterfaceException
     */
    protected IPropertyDef getProperty(String name, boolean canBeEmpty) throws InterfaceException {
        IPropertyDef prop = connectionDef.getProperty(name);
        if (!canBeEmpty && prop == null) {
            logger.error(returnClassName() + " lack " + name + " property.");
            throw new InterfaceException("00301");
        }
        return prop;
    }

    /**
     * 获取连接属性
     *
     * @return
     */
    protected Map<String, IPropertyDef> getPropertyMap() {
        return connectionDef.getPropertyMap();
    }

    /**
     * 获取制定名称的连接属性,不能为空
     *
     * @param name
     * @return
     */
    protected IPropertyDef getProperty(String name) {
        return getProperty(name, false);
    }

    /**
     * 返回制定名称的连接属性的value，不能为空
     *
     * @param name
     * @return
     * @throws InterfaceException
     */
    protected String getPropertyValue(String name) {
        return getPropertyValue(getProperty(name));
    }

    /**
     * 返回制定名称的连接属性的value
     *
     * @param name
     * @param canBeEmpty 是否允许为空
     * @return
     */
    protected String getPropertyValue(String name, boolean canBeEmpty) {
        return getPropertyValue(getProperty(name, canBeEmpty));
    }

    public boolean getPropertyValueBoolean(String name, boolean flg) {
        String value = getPropertyValue(name, true);
        if (StringUtil.isEmpty(value)) {
            return flg;
        } else {
            return Boolean.valueOf(value);
        }
    }

    public int getPropertyValue(String name, int val) {
        String value = getPropertyValue(name, true);
        if (StringUtil.isEmpty(value)) {
            return val;
        } else {
            return Integer.parseInt(value);
        }
    }

    public String getPropertyValue(String name, String val) {
        String value = getPropertyValue(name, true);
        if (StringUtil.isEmpty(value)) {
            return val;
        } else {
            return value;
        }
    }

    /**
     * 返回制定连接属性的value
     *
     * @param prop
     * @return
     */
    protected String getPropertyValue(IPropertyDef prop) {
        if (prop == null)
            return "";
        String value = elParser.getExPression(context, prop.getValue(), null) + "";
        return value;
    }

    protected Object getPropertyValueOfObject(IPropertyDef prop) {
        if (prop == null)
            return null;
        Object value = elParser.getExPression(context, prop.getValue(), null);
        return value;
    }

    /**
     * 返回制定名称的连接属性的value 直接返回整形
     *
     * @param name
     * @return
     */
    protected int getPropertyValueInt(String name) {
        return Integer.valueOf(getPropertyValue(name));
    }

    /**
     * 返回制定名称的连接属性的value 直接返回整形
     *
     * @param canBeEmpty 是否允许为空
     * @param name
     * @return
     */
    protected int getPropertyValueInt(String name, boolean canBeEmpty) {
        String str = getPropertyValue(name, canBeEmpty);
        if (StringUtil.isEmpty(str))
            return 0;
        return Integer.valueOf(str);
    }

    /**
     * 将参数加入到上下文
     *
     * @param scope
     * @param name
     */
    protected void addProp2Context(String scope, String name) {
        String nam = getPropertyValue(name);
        context.addVariable(scope, name, nam);
    }

    /**
     * 将参数加入到上下文,作用域默认最小transition
     *
     * @param scope
     * @param name
     */
    protected void addProp2Context(String name) {
        addProp2Context(null, name);
    }

    /**
     * 设置超时时间，包括连接超时，返回超时
     */
    protected void setTimeOut() {
        setConnectTimeOut();
        setReceiveTimeOut();
        setSoTimeOut();
        setDefaultTimeOut();
    }

    /**
     * 获取连接超时时间，如果为空，则使用默认时间
     */
    protected void setConnectTimeOut() {
        int connectTimeout = getPropertyValueInt(CONNECT_TIMEOUT_SYMBOL, true);
        if (connectTimeout != 0) {
            this.connectTimeout = connectTimeout;
        }
        logger.debug("Connect timeout = " + this.connectTimeout);
    }

    /**
     * 获取返回超时时间，如果为空，则使用默认时间
     */
    protected void setReceiveTimeOut() {
        int receiveTimeout = getPropertyValueInt(RECEIVE_TIMEOUT_SYMBOL, true);
        if (receiveTimeout != 0) {
            this.receiveTimeout = receiveTimeout;
        }
        logger.debug("Receive timeout = " + this.receiveTimeout);
    }

    /**
     * 获取读超时时间，如果为空，则使用默认时间
     */
    protected void setSoTimeOut() {
        int soTimeout = getPropertyValueInt(SO_TIMEOUT_SYMBOL, true);
        if (soTimeout != 0) {
            this.soTimeout = soTimeout;
        }
        logger.debug("soTimeout timeout = " + this.soTimeout);
    }

    /**
     * 获取默认超时时间
     */
    protected void setDefaultTimeOut() {
        int default_timeout = getPropertyValueInt(DEFAULT_TIMEOUT_SYMBOL, true);
        if (default_timeout != 0) {
            this.default_timeout = default_timeout;
        }
        logger.debug("soTimeout timeout = " + this.soTimeout);
    }

    public void GenerateBeans(String configpath, String generatepath) {
        ParseConfig config = new ParseConfig();
        config.setLogger(logger);
        config.generateJavaBeanAndClass(config.parse(configpath), generatepath);
    }

    /**
     * 按照指定格式格式化当前日期
     *
     * @param pattern
     * @return
     */
    public String formatNow(String pattern) {
        return DateUtil.formatNow(pattern);
    }

    public String formatNow() {
        return DateUtil.formatNow(DateUtil.defaultFormate);
    }

    public String getenv(String name) {
        return System.getenv(name);
    }

    public String getExtXmlValue(String filename, String xpath) {
        Document doc = XmlUtil.getDocument(filename);
        return doc.selectSingleNode(xpath).getText();
    }

    @Override
    public void setElPaser(AbstractEL apacheEL) {
        this.elParser = apacheEL;
    }

}
