package com.brilliance.eibs.core.service.instance.plugin;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IConnectionDef;
import com.brilliance.eibs.core.model.IPropertyDef;
import com.brilliance.eibs.util.PropertyUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.dbcp.BasicDataSource;
import org.slf4j.Logger;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TimeZone;

public class DBConnPool {
	public static Map<String, DataSourceAdaptor> dbPool = new HashMap<String, DataSourceAdaptor>();
	private static Object lock = new Object();

	public static Map<String, IConnectionDef> appMap = new HashMap<String, IConnectionDef>();

	public static void add(IConnectionDef connectionDefintiion, Logger logger) {
		synchronized (DBConnPool.class) {
			if (!dbPool.containsKey(connectionDefintiion.getId())) {
				DataSource ds = initDS(connectionDefintiion, logger);
				dbPool.put(connectionDefintiion.getId(), new DataSourceAdaptor(ds, connectionDefintiion));
			}
		}
	}

	public static Connection getConnection(String appNo, Logger logger) {
		if (appMap.containsKey(appNo)) {
			IConnectionDef connectionDef = (IConnectionDef) appMap.get(appNo);
			try {
				return getConnection(connectionDef, logger);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	private static DataSource isExistedAndRemove(IConnectionDef connectionDefintiion) {
		if (connectionDefintiion == null)
			return null;
		DataSourceAdaptor adaptor = (DataSourceAdaptor) dbPool.get(connectionDefintiion.getId());
		DataSource dataSource = null;
		if ((adaptor != null) && ((dataSource = adaptor.getDataSource()) != null))
			return dataSource;

		String id = connectionDefintiion.getId();
		synchronized (DBConnPool.class) {
			for (Iterator it = dbPool.entrySet().iterator(); it.hasNext();) {
				Map.Entry entry = (Map.Entry) it.next();
				String key = (String) entry.getKey();
				if (id.equals(key)) {
					it.remove();
					break;
				}
			}
		}
		return null;
	}

	public static boolean isMybatisConnection(IConnectionDef connectionDefintiion) {
		boolean isMybatis = false;
		if (connectionDefintiion == null)
			return isMybatis;
		IPropertyDef typeDef = connectionDefintiion.getProperty("type");
		if ((typeDef != null) && ("mybatis".equals(typeDef.getValue())))
			isMybatis = true;
		return isMybatis;
	}

	public static Connection getConnection(IConnectionDef connectionDefintiion, Logger logger) throws SQLException, ClassNotFoundException {
		if (connectionDefintiion.isPooled()) {
			DataSource ds = isExistedAndRemove(connectionDefintiion);
			if (ds != null) {
				return ds.getConnection();
			}
			synchronized (lock) {
				DataSourceAdaptor adaptor = (DataSourceAdaptor) dbPool.get(connectionDefintiion.getId());
				if ((adaptor != null) && ((ds = adaptor.getDataSource()) != null)) {
					return ds.getConnection();
				}
				ds = initDS(connectionDefintiion, logger);
				DataSourceAdaptor dsa = null;
				try {
					dsa = new DataSourceAdaptor(ds, connectionDefintiion);

				} catch (Exception e) {
					throw new InterfaceException("01502", e);
				}
				logger.info("创建数据库连接池:"+connectionDefintiion.getId());
				dbPool.put(connectionDefintiion.getId(), dsa);
			}
			return ds.getConnection();
		}else{
			Class.forName(connectionDefintiion.getProperty("driverClassName").getValue());
			return DriverManager.getConnection(connectionDefintiion.getProperty("url").getValue(), connectionDefintiion.getProperty("username")
					.getValue(), connectionDefintiion.getProperty("password").getValue());
		}
	}

	private static BasicDataSource initDS(IConnectionDef connectionDefintiion, Logger logger) {
		BasicDataSource ds = new BasicDataSource();
		ds.setDriverClassName(connectionDefintiion.getProperty("driverClassName").getValue());
		logger.info("database driverClassName=" + ds.getDriverClassName());
		ds.setUsername(connectionDefintiion.getProperty("username").getValue());
		logger.info("database username=" + ds.getUsername());
		ds.setPassword(connectionDefintiion.getProperty("password").getValue());
		ds.setUrl(connectionDefintiion.getProperty("url").getValue());
		logger.info("database url=" + ds.getUrl());
		ds.setInitialSize(Integer.valueOf(connectionDefintiion.getPoolSetting().getProperty("initialSize")).intValue());

		ds.setMaxActive(Integer.valueOf(connectionDefintiion.getPoolSetting().getProperty("maxActive")).intValue());

		ds.setMaxIdle(Integer.valueOf(connectionDefintiion.getPoolSetting().getProperty("maxIdle")).intValue());
		ds.setMinIdle(Integer.valueOf(connectionDefintiion.getPoolSetting().getProperty("minIdle")).intValue());
		ds.setLogAbandoned(Boolean.valueOf(connectionDefintiion.getPoolSetting().getProperty("logAbandoned")).booleanValue());

		ds.setRemoveAbandoned(Boolean.valueOf(connectionDefintiion.getPoolSetting().getProperty("removeAbandoned")).booleanValue());
		ds.setRemoveAbandonedTimeout(Integer.valueOf(connectionDefintiion.getPoolSetting().getProperty("removeAbandonedTimeout")).intValue());

		ds.setMaxWait(Long.valueOf(connectionDefintiion.getPoolSetting().getProperty("maxWait")).longValue());

		String testWhileIdle = connectionDefintiion.getPoolSetting().getProperty("testWhileIdle");
		String testOnBorrow = connectionDefintiion.getPoolSetting().getProperty("testOnBorrow");
		String testOnReturn = connectionDefintiion.getPoolSetting().getProperty("testOnReturn");
		String validatquery = connectionDefintiion.getPoolSetting().getProperty("validationQuery");
		String validationQueryTimeout = connectionDefintiion.getPoolSetting().getProperty("validationQueryTimeout");
		String timeBetweenEvictionRunsMillis = connectionDefintiion.getPoolSetting().getProperty("timeBetweenEvictionRunsMillis");
		String numTestsPerEvictionRun = connectionDefintiion.getPoolSetting().getProperty("numTestsPerEvictionRun");
		String minEvictableIdleTimeMillis = connectionDefintiion.getPoolSetting().getProperty("minEvictableIdleTimeMillis");

		if (!StringUtil.isEmpty(testWhileIdle)) {
			ds.setTestWhileIdle(Boolean.valueOf(testWhileIdle).booleanValue());
		}
		if (!StringUtil.isEmpty(testOnBorrow)) {
			ds.setTestOnBorrow(Boolean.valueOf(testOnBorrow).booleanValue());
		}
		if (!StringUtil.isEmpty(testOnReturn)) {
			ds.setTestOnReturn(Boolean.valueOf(testOnReturn).booleanValue());
		}
		if (!StringUtil.isEmpty(validatquery))
			ds.setValidationQuery(validatquery);
		if (!StringUtil.isEmpty(validationQueryTimeout))
			ds.setValidationQueryTimeout(Integer.parseInt(validationQueryTimeout));
		if (!StringUtil.isEmpty(timeBetweenEvictionRunsMillis))
			ds.setTimeBetweenEvictionRunsMillis(Long.parseLong(timeBetweenEvictionRunsMillis));
		if (!StringUtil.isEmpty(numTestsPerEvictionRun))
			ds.setNumTestsPerEvictionRun(Integer.parseInt(numTestsPerEvictionRun));
		if (!StringUtil.isEmpty(minEvictableIdleTimeMillis))
			ds.setMinEvictableIdleTimeMillis(Long.parseLong(minEvictableIdleTimeMillis));

		String zoneValue = PropertyUtil.getProperty("sys.properties", "SET_TIME_ZONE");
		if (StringUtil.isEmpty(zoneValue)) {
			zoneValue = "false";
		}
		Boolean setZone = Boolean.valueOf(zoneValue);
		if (setZone)
			TimeZone.setDefault(TimeZone.getTimeZone("GMT+08"));
		return ds;
	}

	public static class DataSourceAdaptor {
		private DataSource dataSource;
		private IConnectionDef connectionDef;

		public DataSourceAdaptor(DataSource dataSource, IConnectionDef connectionDef) {
			this.dataSource = dataSource;
			this.connectionDef = connectionDef;
			if (DBConnPool.isMybatisConnection(connectionDef)) {
			}
		}

		public DataSource getDataSource() {
			return this.dataSource;
		}

	}
}
