package com.brilliance.eibs.core.service.instance.plugin;

import com.brilliance.eibs.core.model.IConnectionDef;
import com.brilliance.eibs.util.StringUtil;
import org.slf4j.Logger;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;
import redis.clients.jedis.JedisSentinelPool;
import redis.clients.util.Pool;

import java.util.*;

public class JSPool {
    
    @SuppressWarnings("rawtypes")
    private static Map<String, Pool> jsPool = new HashMap<String, Pool>();

    public static Jedis getConnection(IConnectionDef connectionDefintiion, Logger logger) {

        String id = connectionDefintiion.getId();
        if (connectionDefintiion.isPooled()) {
            @SuppressWarnings("rawtypes")
            Pool js = jsPool.get(id);
            if (null != js) return (Jedis)js.getResource();

            // 保证在初次创建连接池的时候，不会因为多线程导致重复创建
            synchronized (JSPool.class) {
                js = jsPool.get(id);
                if (null != js) return (Jedis)js.getResource();
                js = initPool(connectionDefintiion, logger);
                jsPool.put(id, js);
            }

            return (Jedis)js.getResource();
        }
        return new Jedis(connectionDefintiion.getProperty("ip").getValue(),
                Integer.valueOf(connectionDefintiion.getProperty("port").getValue()));
    }


    @SuppressWarnings("rawtypes")
    private static Pool initPool(IConnectionDef connectionDefintiion, Logger logger) {
        logger.info("###########创建连接池##########");
        JedisPoolConfig config = new JedisPoolConfig();
        config.setMaxTotal(
                Integer.valueOf(connectionDefintiion.getPoolSetting().getProperty("maxActive")));
        config.setMaxIdle(
                Integer.valueOf(connectionDefintiion.getPoolSetting().getProperty("maxIdle")));
        config.setMaxWaitMillis(
                Long.valueOf(connectionDefintiion.getPoolSetting().getProperty("maxWait")));
        String numTestsPerEvictionRun =
                connectionDefintiion.getPoolSetting().getProperty("numTestsPerEvictionRun");
        if (!StringUtil.isEmpty(numTestsPerEvictionRun))
            config.setNumTestsPerEvictionRun(Integer.valueOf(numTestsPerEvictionRun));
        String timeBetweenEvictionRunsMillis =
                connectionDefintiion.getPoolSetting().getProperty("timeBetweenEvictionRunsMillis");
        if (!StringUtil.isEmpty(timeBetweenEvictionRunsMillis))
            config.setTimeBetweenEvictionRunsMillis(Long.valueOf(timeBetweenEvictionRunsMillis));
        config.setTestOnBorrow(
                Boolean.valueOf(connectionDefintiion.getPoolSetting().getProperty("testOnBorrow")));
        config.setTestOnReturn(
                Boolean.valueOf(connectionDefintiion.getPoolSetting().getProperty("testOnReturn")));
        boolean isSentinel =
                Boolean.valueOf(connectionDefintiion.getPoolSetting().getProperty("isSentinel"));

        if (isSentinel) {
            String masterName = connectionDefintiion.getPoolSetting().getProperty("masterName");
            String s_sentinels = connectionDefintiion.getPoolSetting().getProperty("sentinels");
            Set<String> sentinels = null;
            if (!StringUtil.isEmpty(s_sentinels)) {
                String[] sentinelsArray = s_sentinels.split(",");
                sentinels = new HashSet<String>(Arrays.asList(sentinelsArray));
            }
            String s_timeout = connectionDefintiion.getPoolSetting().getProperty("timeout");
            int timeout = StringUtil.isEmpty(s_timeout) ? 0 : Integer.valueOf(s_timeout);
            return new JedisSentinelPool(masterName, sentinels, config, timeout);
        } else
            return new JedisPool(config, connectionDefintiion.getProperty("ip").getValue(),
                    Integer.valueOf(connectionDefintiion.getProperty("port").getValue()));
    }

}

