package com.brilliance.eibs.core.service.instance.plugin;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import sun.net.TelnetInputStream;
import sun.net.TelnetOutputStream;
import sun.net.ftp.FtpClient;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * Java自带的API对FTP的操作
 * 
 * @Title:Ftp.java
 * @author: xxx
 */
public class JdkFtpExecutor {
	/**
	 * 本地文件名
	 */
	private String localfilename;
	/**
	 * 远程文件名
	 */
	private String remotefilename;
	/**
	 * FTP客户端
	 */
	private FtpClient ftpClient;
	protected Logger logger;

	/**
	 * 服务器连接
	 * 
	 * @param ip
	 *            服务器IP
	 * @param port
	 *            服务器端口
	 * @param user
	 *            用户名
	 * @param password
	 *            密码
	 * @param path
	 *            服务器路径
	 * @author xxx
	 * @date 2012-7-11
	 */
	public void connectServer(Logger logger, String ip, int port, String user, String password) {
		this.logger = logger;
		try {
			/* ******连接服务器的两种方法****** */
			// 第二种方法
			// ftpClient = new FtpClient(ip);
			// ftpClient.login(user, password);
			// 设置成2进制传输
			// ftpClient.binary();
			logger.info("login success!");

		} catch (Exception ex) {
			ex.printStackTrace();
			throw new RuntimeException(ex);
		}
	}

	// 把远程系统上的目录切换到参数path所指定的目录
	public void cd(String remotePath) throws IOException {
		if (remotePath.length() != 0) {
			// ftpClient.cd(remotePath);
			// ftpClient.binary();
		}
	}

	/**
	 * 关闭连接
	 * 
	 * @author xxx
	 * @throws IOException
	 * @date 2012-7-11
	 */
	public void closeConnect() throws IOException {
		// ftpClient.closeServer();
		logger.info("disconnect success");
	}

	/**
	 * 上传文件
	 * 
	 * @param localFile
	 *            本地文件
	 * @param remoteFile
	 *            远程文件
	 * @author xxx
	 * @throws IOException
	 * @date 2012-7-11
	 */
	public void upload(String localFile, String remoteFile) throws IOException {
		this.localfilename = localFile;
		this.remotefilename = remoteFile;
		TelnetOutputStream os = null;
		FileInputStream is = null;

		// 将远程文件加入输出流中
		// os = ftpClient.put(this.remotefilename);
		// 获取本地文件的输入流
		File file_in = new File(this.localfilename);
		is = new FileInputStream(file_in);
		// 创建一个缓冲区
		byte[] bytes = new byte[1024];
		int c;
		while ((c = is.read(bytes)) != -1) {
			os.write(bytes, 0, c);
		}
		logger.info("upload success");

		IOUtils.closeQuietly(os);
		IOUtils.closeQuietly(is);

	}

	/**
	 * 下载文件
	 * 
	 * @param remoteFile
	 *            远程文件路径(服务器端)
	 * @param localFile
	 *            本地文件路径(客户端)
	 * @author xxx
	 * @date 2012-7-11
	 */
	public void download(String remoteFile, String localFile) {
		TelnetInputStream is = null;
		FileOutputStream os = null;
		try {
			// 获取远程机器上的文件filename，借助TelnetInputStream把该文件传送到本地。
			// is = ftpClient.get(remoteFile);
			File file_in = new File(localFile);
			if (!file_in.getParentFile().exists())
				file_in.getParentFile().mkdirs();
			os = new FileOutputStream(file_in);
			byte[] bytes = new byte[1024];
			int c;
			while ((c = is.read(bytes)) != -1) {
				os.write(bytes, 0, c);
			}
			logger.info("download success");
		} catch (IOException ex) {
			logger.info("not download");
			ex.printStackTrace();
			throw new RuntimeException(ex);
		} finally {
			IOUtils.closeQuietly(os);
			IOUtils.closeQuietly(is);
		}
	}

	public static void main(String agrs[]) {

		JdkFtpExecutor fu = new JdkFtpExecutor();
		/*
		 * 使用默认的端口号、用户名、密码以及根目录连接FTP服务器
		 */
		fu.connectServer(null, "192.168.1.8", 21, "manager", "28259");

		try {
			fu.cd("/testgecy1/");
			fu.download("1.txt", "d:/wo1/111.txt");
			fu.closeConnect();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}