package com.brilliance.eibs.core.service.instance.plugin;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.main.Client;
import com.brilliance.eibs.util.DateUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.saaj.SAAJInInterceptor;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.phase.AbstractPhaseInterceptor;
import org.apache.cxf.phase.Phase;
import org.w3c.dom.Document;

import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringWriter;
import java.text.ParseException;

public class MessageInterceptor extends AbstractPhaseInterceptor<SoapMessage> {

	private String phase;
	private Context context;
	private String interfaceName;
	private String transactionName;

	private SAAJInInterceptor saa = new SAAJInInterceptor();

	public MessageInterceptor(Context context, String interfaceName, String transName, String phase) {
		super(phase);
		this.context = context;
		this.phase = phase;
		this.interfaceName = interfaceName;
		this.transactionName = transName;
		getAfter().add(SAAJInInterceptor.class.getName());
	}

	@Override
	public void handleMessage(SoapMessage message) throws Fault {
		SOAPMessage soapMessage = message.getContent(SOAPMessage.class);
		if (soapMessage == null) {
			saa.handleMessage(message);
			soapMessage = message.getContent(SOAPMessage.class);
		}
		if (soapMessage != null) {
			try {
				Document doc = soapMessage.getSOAPPart().getEnvelope().getOwnerDocument();
				StringWriter output = new StringWriter();
				TransformerFactory.newInstance().newTransformer().transform(new DOMSource(doc), new StreamResult(output));

				if (!StringUtil.isEmpty(interfaceName) && !StringUtil.isEmpty(transactionName)) {
					Client.scall(interfaceName, transactionName, new Object[] { output.toString() });
				}
			} catch (SOAPException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (TransformerConfigurationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (TransformerException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (TransformerFactoryConfigurationError e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (phase.equals(Phase.PRE_INVOKE)) {
			try {
				if (!DateUtil.isDateValid((String) context.getVariable("cronExpression"))) {
					throw new Fault(new SOAPException("time is invalid"));
				}
			} catch (ParseException e) {
				throw new InterfaceException("00004", "cronExpression config error.");
			}
			// List<Header> headers = message.getHeaders();
			// if (null == headers || headers.size() < 1)
			// {
			// throw new Fault(new SOAPException("SOAP消息头格式不正确！"));
			// }
			// for (Header header : headers)
			// {
			// // 取出SOAP的Header元素
			// Element element = (Element) header.getObject();
			// logger.info("ELEMENT =" + element.getNodeName() + "," +
			// element.getTextContent());
			// }
		} else if (phase.equals(Phase.POST_INVOKE)) {
			// TODO
		}
	}
}
