package com.brilliance.eibs.core.service.instance.plugin;

import com.brilliance.eibs.core.model.IFieldDef;

import java.util.List;

/**
 * xml报文，变长报文解析和组装模板，完成逻辑处理，具体实习交由具体类完成
 * @author weicong
 *
 */
public class MsgFormTemplate
{
  public Object getFieldValue(IFieldDef fieldDef, IMsgParseFilter callback)
  {
    if (fieldDef.getSubfields().size() > 0)
      return callback.extensionSubfieldParseMsg(fieldDef);
    return callback.parseMsg(fieldDef);
  }

  public void packField(IFieldDef fieldDef, Object fieldValue, IMsgAssembleFilter callback)
  {
    if (fieldDef.getSubfields().size() > 0)
    {
      if (fieldValue instanceof List)
        callback.extensionSubfieldAssembleMsg(fieldDef, fieldValue);
    }
    else
      callback.assembelMsg(fieldDef, fieldValue);
  }

  /**
   * 报文解析接口
   * @author weicong
   *
   */
  public interface IMsgParseFilter
  {
    /**
     * 原有各类报文的接口解析方式
     * @param fieldDef
     * @return
     */
    public Object parseMsg(IFieldDef fieldDef);

    /**
     * 扩展对各类报文含有subfield节点的接口解析方式
     * @param fieldDef
     * @return
     */
    public Object extensionSubfieldParseMsg(IFieldDef fieldDef);
  }

  /**
   * 报文组装接口
   * @author weicong
   *
   */
  public interface IMsgAssembleFilter
  {
    /**
     * 原有各类报文的接口组装方式
     * @param fieldDef
     * @param fieldValue 字段值
     */
    public void assembelMsg(IFieldDef fieldDef, Object fieldValue);

    /**
     * 扩展对各类报文的接口含有subfield节点的组装方式
     * @param fieldDef
     * @param fieldValue 字段值
     */
    public void extensionSubfieldAssembleMsg(IFieldDef fieldDef, Object fieldValue);
  }

}
