package com.brilliance.eibs.core.service.instance.plugin;

import com.brilliance.eibs.util.StringUtil;
import org.slf4j.Logger;

import javax.jms.*;

public class TibcoProxy extends AbsCloseConnection implements MessageListener {

	private Logger log;
	private QueueConnection connection;
	private String name;

	private QueueSession session;
	// private Queue queue;
	private Destination destination;
	private boolean useTopic;
	// private boolean flag = false;
	private String msg;

	public TibcoProxy(QueueConnection connection, String name, Logger log) {
		// super();
		this.connection = connection;
		this.name = name;
		this.log = log;
	}

	public void init() throws JMSException {
		/* create the session */
		session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
		/* create the destination */
		if (useTopic)
			destination = session.createTopic(name);
		else
			destination = session.createQueue(name);
	}

	public void send(String message) throws JMSException {
		send(message, null, null);
	}

	public void send(String message, String name, String value) throws JMSException {
		// if (null == session ||null==destination)
		if (null == session && null == destination)
			throw new UnsupportedOperationException("Must be initialized");
		MessageProducer msgProducer = session.createProducer(null);
		TextMessage msg = session.createTextMessage(message);
		if (!StringUtil.isEmpty(name)) {
			log.debug("send property " + name + " = " + value);
			msg.setStringProperty(name, value);
		}
		msgProducer.send(destination, msg);
	}

	public String receive() throws JMSException {
		return receive(0, null);
	}

	public String receive(long timeout, String selector) throws JMSException {
		if (null == session && null == connection && null == destination)
			throw new UnsupportedOperationException("Must be initialized");
		MessageConsumer msgConsumer = session.createConsumer(destination, selector);
		connection.start();
		Message message = null;
		if (timeout == 0)
			message = msgConsumer.receive();
		else
			message = msgConsumer.receive(timeout);
		log.debug("selector = " + selector);
		return null == message ? null : ((TextMessage) message).getText();

	}

	public String receivewithtimeout(Long timeout) throws JMSException {
		return receive(timeout, null);
	}

	public QueueConnection getQueueConnection() {
		return connection;
	}

	public String getmsg() {
		return msg;
	}

	@Override
	public void close() throws Exception {
		connection.close();
	}

	@Override
	public void onMessage(Message message) {
		// TODO Auto-generated method stub
	}

}
