package com.brilliance.eibs.core.threadpool;

import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.util.StringUtil;

import java.util.Map;
import java.util.concurrent.*;

public class ThreadPoolFactory {
	private static String DEFAULT_THREAD_NAME = "eibs";

	private static Map<String, ExecutorService> excutorServiceMap = new ConcurrentHashMap<String, ExecutorService>();

	public static Map<String, ExecutorService> getExcutorInfo() {
		return excutorServiceMap;

	}

	public static ExecutorService getSingleExecutor(String name) {
		return getFixedExecutor(1, name);
	}

	public synchronized static ExecutorService getFixedExecutor(int threads, String name) {
		name = StringUtil.isEmpty(name) ? DEFAULT_THREAD_NAME : name;
		if (!excutorServiceMap.containsKey(name)) {
			ExecutorService es = new ThreadPoolExecutor(threads, threads, 0, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>(),
					new NamedThreadFactory(name, true), new AbortPolicyWithReport(name));
			excutorServiceMap.put(name, es);
		}
		return excutorServiceMap.get(name);
	}

	public synchronized static void shutdown(String name) {
		ExecutorService es = excutorServiceMap.get(name);
		if (es != null) {
			es.shutdown();
			excutorServiceMap.remove(name);
		}
	}

	public static Executor getCachedThreadPool(int corePoolSize, int maximumPoolSize, String name, Context context) {
		return new ThreadPoolExecutor(corePoolSize, maximumPoolSize, 60 * 1000, TimeUnit.MILLISECONDS, new LinkedBlockingQueue<Runnable>(),
				new NamedThreadFactory(name, true), new AbortPolicyWithReport(name));
	}

}