package com.brilliance.eibs.el;

import com.brilliance.eibs.core.model.IFunctionDef;
import com.brilliance.eibs.factory.BeanFactory.FunctionKey;
import com.brilliance.eibs.factory.parser.impl.FunctionParser;
import org.apache.commons.beanutils.MethodUtils;

import java.util.Iterator;
import java.util.Map;

/**
 * @author hujun
 *
 *         自定义函数搜索器，用于查找自定义函数
 */
public class CustomFunctionFinder {

    private Map<FunctionKey, IFunctionDef> functionMap;
    private IFunctionDef func;

    public CustomFunctionFinder(Map<FunctionKey, IFunctionDef> functionMap) {
        this.functionMap = functionMap;
    }

    /**
     * @param name
     * @param objs
     * @return
     * 
     *         判断是否有满足的自定义函数
     */
    public boolean findFunction(String name, Object[] objs) {
        Class<?>[] parameterTypes = getParameterTypes(objs);
        func = functionMap.get(FunctionParser.buildFunctionKey(name, parameterTypes));
        if (null != func || null != (func = depthMatchFunc(name, parameterTypes))) return true;
        return false;
    }

    // /**
    // * @param finder
    // * @param objs
    // * @return
    // * @throws Exception
    // *
    // * 返回自定义函数运行结果
    // */
    // public Object returnValue(CustomFunctionFinder finder, Object[] objs)
    // throws Exception
    // {
    // return FunctionInstanceFactory.createInstance(finder.func, finder.logger).execute(objs);
    // }

    public IFunctionDef getFunc() {
        return func;
    }

    /**
     * @param params
     * @return
     * 
     *         获得参数的类型数组
     */
    private Class<?>[] getParameterTypes(Object[] params) {
        if (params == null) return new Class[0];
        Class<?>[] parameterTypes = new Class[params.length];
        for (int i = 0; i < params.length; i++)
            parameterTypes[i] = params[i] == null ? Object.class : params[i].getClass();
        return parameterTypes;
    }

    /**
     * @param name
     * @param parameterTypes
     * @return
     * 
     *         深度匹配自定义函数，主要是满足包装类型与基本类型，继承实现关系的参数调用
     */
    private IFunctionDef depthMatchFunc(String name, Class<?>[] parameterTypes) {
        Iterator<Map.Entry<FunctionKey, IFunctionDef>> iter = functionMap.entrySet().iterator();
        while (iter.hasNext()) {
            boolean match = true;
            Map.Entry<FunctionKey, IFunctionDef> entry = iter.next();
            FunctionKey key = entry.getKey();
            Class<?>[] types = key.getParameterTypes();
            if (!name.equals(key.getName()) || parameterTypes.length != types.length) continue;
            for (int i = 0; i < parameterTypes.length; i++) {
                if (!isAssignmentCompatible(types[i], parameterTypes[i])) {
                    match = false;
                    break;
                }
            }
            if (match) return entry.getValue();
        }
        return null;
    }

    private boolean isAssignmentCompatible(Class<?> parameterType, Class<?> parameterization) {
        if (parameterType.isArray() && parameterization.isArray())
            return isAssignmentCompatible(parameterType.getComponentType(),
                    parameterization.getComponentType());
        if (parameterType == parameterization) return true;
        if (parameterType.isAssignableFrom(parameterization)) return true;
        if (parameterization.isPrimitive()
                && parameterType == MethodUtils.getPrimitiveWrapper(parameterization))
            return true;
        if (parameterType == MethodUtils.getPrimitiveType(parameterization)) return true;
        return false;
    }

}
