package com.brilliance.eibs.extension.mina.handle;

import com.brilliance.eibs.util.LogUtil;
import org.apache.mina.core.service.IoHandlerAdapter;
import org.apache.mina.core.session.IdleStatus;
import org.apache.mina.core.session.IoSession;
import org.slf4j.Logger;

/**
 * @author hujun
 * <p>
 * mina nio 框架中需要实现的抽象类
 */
public abstract class NioProcessHandler extends IoHandlerAdapter {
    Logger logger = LogUtil.getLogger("mina");

    @Override
    public void sessionCreated(IoSession session) throws Exception {
        super.sessionCreated(session);
        session.setAttribute("REMOTE_ADDRESS", session.getRemoteAddress());
        logger.info("创建客户端连接,远程地址" + session.getAttribute("REMOTE_ADDRESS"));
    }

    @Override
    public void exceptionCaught(IoSession session, Throwable cause) throws Exception {
        super.exceptionCaught(session, cause);
        logger.error("监听通讯异常,远程地址", cause);
        session.closeOnFlush();
    }

    @Override
    public void messageReceived(IoSession session, Object message) throws Exception {
        process(session, message);
    }

    protected abstract void process(IoSession session, Object message);

    @Override
    public void sessionIdle(IoSession session, IdleStatus status) throws Exception {
        Object msg = session.getAttribute(session.getId());
        if (msg != null) {
            logger.error("渠道接收消息超时");
        }
        session.closeOnFlush();
    }

    @Override
    public void sessionClosed(IoSession session) throws Exception {
        super.sessionClosed(session);
        logger.info("客户端连接关闭" + session.getAttribute("REMOTE_ADDRESS"));
    }

}
