package com.brilliance.eibs.factory.parser;

import com.brilliance.eibs.core.model.IInterfaceDef;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.model.ITaskDef;
import com.brilliance.eibs.core.model.ITransactionDef;
import com.brilliance.eibs.factory.BeanFactory;
import org.slf4j.Logger;

import java.util.List;

/**
 * @author hujun
 *
 *         用于抽象所有配置解析器的接口，用于增加兼容性及拓展性 支持多种配置形式
 *
 */
public interface ConfigParser
{

  /**
   * 主配置解析到内存
   * 
   * @param beanFactory
   */
  boolean mainFileToMemo(BeanFactory beanFactory);

  boolean mainFileToMemo(BeanFactory beanFactory, String interfaceid, String transactionid);

  /**
   * 获取指定交易定义
   * 
   * @param interfaceid
   * @param transId
   * @return
   */
  ITransactionDef getTransactionDef(String interfaceid, String transid);

  /**
   * 从接口interfaceDef定义的所有交易中，获取与transid指定的id相同的交易
   * 
   * @param interfaceDef
   * @param transid
   * @return
   */
  ITransactionDef getTransactionDef(IInterfaceDef interfaceDef, String transid);

  /**
   * 获取服务配置定义
   * 
   * @param filename
   * @return
   */
  List<IServiceDef> getServiceDefs(String filename);


  /**
   * 获取定时任务Task信息
   * 
   * @param filename
   * @return
   */
  List<ITaskDef> getTaskDefs(String filename);

  IServiceDef getServiceDef(String id);

  void setLogger(Logger logger);

  Logger getLogger();

}
