package com.brilliance.eibs.listener;

import com.brilliance.eibs.bean.ParseConfig;
import com.brilliance.eibs.core.model.IInterfaceDef;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.model.ITaskDef;
import com.brilliance.eibs.factory.BeanFactory;
import com.brilliance.eibs.factory.FactoryManager;
import com.brilliance.eibs.factory.parser.impl.XmlParser;
import com.brilliance.eibs.listener.helpers.impl.FilesWatchdog;
import com.brilliance.eibs.main.Server;
import com.brilliance.eibs.util.Constants;
import com.brilliance.eibs.util.FileUtil;
import com.brilliance.eibs.util.Serializable2FileUtil;
import com.brilliance.eibs.util.XmlUtil;
import org.apache.commons.io.FileUtils;
import org.dom4j.Document;
import org.dom4j.Element;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 热部署线程监听线程
 * 
 * @author xiaoyuanzhen
 *
 */
public class HotDeployListener extends FilesWatchdog {
    protected XmlParser xmlParser;
    protected FileFilter xmlFileFilter = new FileFilter() {

        @Override
        public boolean accept(File f) {
            if (f.isFile() && f.getName().endsWith(".xml")) return true;
            return false;
        }
    }; // xml文件过滤器

    public HotDeployListener(XmlParser xmlParser) {
        super(xmlParser.getCfgPath(), xmlParser.getLogger());
        this.xmlParser = xmlParser;
    }

    @Override
    protected void doOnChange(Map<String, List<String>> changedFileMap) {
        boolean mainFileDeployedDown = false;// 标识主配置的热部署是否已经完成
        boolean serviceChanged = false;
        // support文件夹文件被修改
        if (changedFileMap.containsKey(Constants.ETC_PATH)) {
            List<String> filenames = changedFileMap.get(Constants.ETC_PATH);
            if (filenames.contains("commons.xml") || filenames.contains("functions.xml")
                    || filenames.contains("codetables.xml")) {// 重启所有服务
                Server.closeAllServers();
                Server.startServers();
                updateMap(pathnames);
                return;
            } else if (filenames.contains("services.xml")) {
                serviceChanged = true;
            } else {
                // 主配置文件重新初始化到新内存
                changeMainFiles();
                mainFileDeployedDown = true;
            }
        }
        if (changedFileMap.containsKey(Constants.BEANS_PATH)) {// 重新生成beans
            changeBeanFiles();
        }
        // services.xml文件被修改
        if (serviceChanged) {// 操作服务
            changeServicesCfg();
        }
        // main文件夹文件被修改
        if (changedFileMap.containsKey(XmlParser.MAIN_PATH)) {
            if (!mainFileDeployedDown) changeMainFiles();
            // 切换内存
            FactoryManager.switchNext();
        }
        updateMap(pathnames);
        if (!Serializable2FileUtil.hotDeploy && isAlive()) {
            interrupt();
            Server.listener = null;
        }
    }

    /**
     * 更新bean目录文件
     * 
     * @return
     */
    protected boolean changeBeanFiles() {
        try {
            File file = FileUtils.toFile(XmlUtil.getUrl(Constants.BEANS_PATH));
            if (file == null) return true;
            File[] beanFiles = file.listFiles(xmlFileFilter);
            for (File beanFile : beanFiles) {
                String filename = beanFile.getName();
                long l = lastModifMap.get(filename);
                if (beanFile.lastModified() > l) {
                    lastModifMap.put(filename, beanFile.lastModified());
                    Document doc = XmlUtil.getDocument(Constants.BEANS_PATH, filename);
                    // 重新生成beans
                    ParseConfig config = new ParseConfig();
                    config.setLogger(xmlParser.getLogger());
                    config.generateJavaBeanAndClass(config.parseXml(doc.getRootElement()), "");
                }
            }
        } catch (Exception e) {
            log.info("generatebeans files have some error", e);
            return false;
        }
        return true;
    }

    /**
     * 重新部署主配置
     * 
     * @return
     */
    protected void changeMainFiles() {

        Collection<File> mainFiles = XmlUtil.getAllFile(XmlParser.MAIN_PATH);
        BeanFactory beanFactory = FactoryManager.getNextBeanFactory();
        for (File mainFile : mainFiles) {
            if (mainFile.isFile()) {
                IInterfaceDef interfaceDef = null;
                String filename = FileUtil.getFileName(mainFile.getName());
                // 新增 modifieb by gechengyang TODO
                filename = XmlUtil.getRelativeName(mainFile, XmlParser.MAIN_PATH);
                try {
                    Document doc =
                            XmlUtil.getDocument(XmlParser.getInterfaceRelativePath(filename));
                    Element interfaceNode = doc.getRootElement().element("interface");
                    if (interfaceNode == null) continue;
                    String type = interfaceNode.attributeValue("type");
                    if (!"client".equals(type)) {
                        interfaceDef = xmlParser.getInterfaceDef(filename);
                    }
                } catch (Exception e) {
                    log.error("configuration file [" + filename
                            + "] has some error,hot deploy failed.", e);
                    continue;
                }
                IInterfaceDef curDef = FactoryManager.getBeanFactory().getInterface(filename);
                if (interfaceDef != null && !interfaceDef.equals(curDef)) {
                    beanFactory.addInterface(filename, interfaceDef);
                    log.info("Hot deploy interface [" + filename + "] success.");
                }
            }
        }
    }



    /**
     * 修改services.xml文件
     * 
     */
    @SuppressWarnings("unchecked")
    protected boolean changeServicesCfg() {
        try {
            Document doc = XmlUtil.getDocument(Constants.ETC_PATH + "/services.xml");
            List<IServiceDef> oldServices =
                    (List<IServiceDef>) Serializable2FileUtil.readObject("services");
            List<IServiceDef> newServices = xmlParser.getServiceDefs(doc);
            if (!newServices.equals(oldServices)) {
                compareAndDealServices(oldServices, newServices);
                Serializable2FileUtil.deleteSerialFile("services");
                Serializable2FileUtil.writeObject(newServices, "services");
            }
            List<ITaskDef> oldTaskDefs = (List<ITaskDef>) Serializable2FileUtil.readObject("tasks");
            List<ITaskDef> newTaskDefs = xmlParser.getTaskDefs(doc);
            if (!newTaskDefs.equals(oldTaskDefs)) {
                compareAndDealTasks(oldTaskDefs, newTaskDefs);
                Serializable2FileUtil.deleteSerialFile("tasks");
                Serializable2FileUtil.writeObject(newTaskDefs, "tasks");
            }
        } catch (Exception e) {
            log.info("service hot deploy error.", e);
            return false;
        }
        return true;
    }


    private boolean compareAndDealServices(List<IServiceDef> oldTasks, List<IServiceDef> newTasks) {
        boolean isChanged = false;
        List<IServiceDef> startNew = new ArrayList<IServiceDef>();
        if (oldTasks != null) {
            for (IServiceDef newServiceDef : newTasks) {
                int i;
                int oldSize = oldTasks.size();
                for (i = 0; i < oldSize; i++)// 遍历旧服务
                {
                    IServiceDef oldServiceDef = oldTasks.get(i);
                    if (oldServiceDef.getId().equals(newServiceDef.getId()))// 存在相同id的新服务
                    {
                        if (!oldServiceDef.equals(newServiceDef))// 比较新旧服务
                        {
                            isChanged = true;
                            // update old service
                            if (oldServiceDef.getState().equals("on"))
                                Server.closeServer(oldServiceDef.getId());
                            if (newServiceDef.getState().equals("on"))
                                Server.startServer(newServiceDef);
                        }
                        // remove old service,means it has bean handled
                        oldTasks.remove(i);
                        break;
                    }
                }
                if (i == oldSize) {
                    isChanged = true;
                    if (newServiceDef.getState().equals("on")) {
                        startNew.add(newServiceDef);
                    }
                }
            }
        }
        // remain tasks that not exist in new configuration file should be closed
        if (oldTasks.size() > 0) {
            isChanged = true;
            for (IServiceDef serviceDef : oldTasks) {
                Server.closeServer(serviceDef.getId());
            }
        }
        // start new services
        if (startNew.size() > 0) {
            isChanged = true;
            for (IServiceDef serviceDef : startNew) {
                Server.startServer(serviceDef);
            }
        }
        return isChanged;
    }

    private boolean compareAndDealTasks(List<ITaskDef> oldTasks, List<ITaskDef> newTasks) {
        boolean isChanged = false;
        List<ITaskDef> startNew = new ArrayList<ITaskDef>();
        if (oldTasks != null) {
            for (ITaskDef newTaskDef : newTasks) {
                int i;
                int oldSize = oldTasks.size();
                for (i = 0; i < oldSize; i++)// 遍历旧任务
                {
                    ITaskDef oldServiceDef = oldTasks.get(i);
                    if (oldServiceDef.getId().equals(newTaskDef.getId()))// 存在相同id的新任务
                    {
                        if (!oldServiceDef.equals(newTaskDef))// 比较新旧任务
                        {
                            isChanged = true;
                            // update old service
                            if (oldServiceDef.getState().equals("on"))
                                Server.closeTask(oldServiceDef.getId());
                            if (newTaskDef.getState().equals("on")) Server.startTask(newTaskDef);
                        }
                        // remove old service,means it has bean handled
                        oldTasks.remove(i);
                        break;
                    }
                }
                if (i == oldSize) {
                    isChanged = true;
                    if (newTaskDef.getState().equals("on")) {
                        startNew.add(newTaskDef);
                    }
                }
            }
        }
        // remain services that not exist in new configuration file should be closed
        if (oldTasks.size() > 0) {
            isChanged = true;
            for (ITaskDef serviceDef : oldTasks) {
                Server.closeTask(serviceDef.getId());
            }
        }
        // start new services
        if (startNew.size() > 0) {
            isChanged = true;
            for (ITaskDef serviceDef : startNew) {
                Server.startTask(serviceDef);
            }
        }
        return isChanged;
    }

}
