package com.brilliance.eibs.main;

import com.brilliance.eibs.core.exception.SysCode;
import com.brilliance.eibs.core.frame.ISocketProxy;
import com.brilliance.eibs.core.model.IInterfaceDef;
import com.brilliance.eibs.core.model.ITransactionDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.ResultMsg;
import com.brilliance.eibs.core.service.instance.IResource;
import com.brilliance.eibs.core.service.instance.ITransactionInstance;
import com.brilliance.eibs.core.service.instance.TransactionInstance;
import com.brilliance.eibs.el.AbstractEL;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.el.ApacheJexl;
import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.factory.BeanFactory;
import com.brilliance.eibs.factory.FactoryManager;
import com.brilliance.eibs.factory.parser.ConfigParser;
import com.brilliance.eibs.factory.parser.ParserFactory;
import com.brilliance.eibs.util.*;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.MDC;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Date;
import java.util.Map;

/**
 * 调用交易接口API
 * 
 * @author hujun
 * 
 */
public class Client {

	public static String VERSION = "2.0";

	/**
	 * 提供client的静态调用方式
	 * 
	 * @param interfaceName
	 *            接口名
	 * @param transactionName
	 *            交易名
	 * @return ResultMsg
	 */
	public static ResultMsg scall(String interfaceName, String transactionName) {
		return new Client().call(interfaceName, transactionName);
	}

	/**
	 * 提供client的静态调用方式
	 * 
	 * @param interfaceName
	 *            接口名
	 * @param transactionName
	 *            交易名
	 * @param objs
	 *            传入的参数
	 * @return ResultMsg
	 */
	public static ResultMsg scall(String interfaceName, String transactionName, Object[] objs) {
		return new Client().call(interfaceName, transactionName, objs);
	}

	/**
	 * 调用交易接口
	 * 
	 * @param interfaceName
	 *            接口名
	 * @param transactionName
	 *            交易名
	 * @param objs
	 *            参数
	 * @return ResultMsg
	 */
	public ResultMsg call(String interfaceName, String transactionName, String pre, boolean merge, Object... objs) {
		Logger logger = LogUtil.getTrnLog(interfaceName, transactionName, pre, merge);
		return call(interfaceName, transactionName, objs, logger);
	}

	public ResultMsg call(String interfaceName, String transactionName, Object... objs) {
		return call(interfaceName, transactionName, null, false, objs);
	}

	/**
	 * 
	 * @param interfaceName
	 *            接口名
	 * @param transactionName
	 *            交易名
	 * @param objs
	 *            参数
	 * @param logger
	 *            日志打印器
	 * @return
	 */
	public ResultMsg call(String interfaceName, String transactionName, Context fcontext, Object[] objs, Logger logger) {
		Context context = null;// 上下文
		context = new Context();// 创建上下文
		context.getResource().setExecutor(fcontext.getResource().getExecutor());
		context.getResource().setCntConnection(fcontext.getResource().getCntConnection());
		return call(interfaceName, transactionName, objs, logger, context);
	}

	public ResultMsg call(String interfaceName, String transactionName, Object[] objs, Logger logger) {
		Context context = null;// 上下文
		context = new Context();// 创建上下文
		return call(interfaceName, transactionName, objs, logger, context);
	}

	private class Detect implements Runnable {

		private String parentThreadName;
		private long delay = 20000;
		private Logger log;
		private Date startTime;

		private Detect(String parentThreadName, Date startTime, Logger log) {
			this.parentThreadName = parentThreadName;
			this.log = log;
			this.startTime = startTime;
		}

		private Detect(String parentThreadName, Date startTime, long delay, Logger log) {
			this(parentThreadName, startTime, log);
			this.delay = delay;
		}

		@Override
		public void run() {
			try {
				log.debug(parentThreadName + " detect thread startup.");
				Thread.sleep(delay);
				log.error(parentThreadName + " start on " + startTime + ", after " + delay / 1000 + " seconds timeout.");
			} catch (InterruptedException e) {
				log.error(parentThreadName + " detect thread interrupted error.");
			}
		}
	}

	public static void initInterfaceDef(String interfaceName, String transactionName, Logger logger) {
		ConfigParser parser = ParserFactory.getInstance(logger);

		BeanFactory beanFactory = FactoryManager.getBeanFactory();

		IInterfaceDef interfaceDef = beanFactory.getInterface(interfaceName);
		ITransactionDef transDef = null;
		if (interfaceDef != null) // 内存中存在，则使用内存中的接口定义获取交易定义，并进行其它初始化
		{
			// 获取交易实例
			transDef = parser.getTransactionDef(interfaceDef, transactionName);
		} else
		// 内存中不存在，则使用配置解析器读取配置进行解析
		{
			// 创建交易实例
			transDef = parser.getTransactionDef(interfaceName, transactionName);
			interfaceDef = transDef.getInterfaceDef();
			beanFactory.addInterface(interfaceName, interfaceDef);
		}
	}

	public ResultMsg call(Context context, String interfaceName, String transactionName, Object[] objs) {
		Logger logger = (Logger) context.getVariable(Constants.Log_VAR);
		if (logger == null) {
			logger = LogUtil.getTrnLog(interfaceName, transactionName);
		}
		long currentTime = System.currentTimeMillis();
		Date startTime = new Date(currentTime);
		final String threadName = interfaceName + "_" + transactionName + "_" + currentTime;

		ResultMsg msg = null;// 返回对象
		ConfigParser parser = null;// 配置解析器
		String detail = "";
		boolean reload = false;
		try {
			context.addVariable("transaction", Constants.Log_VAR, logger);
			context.addVariable("transaction", Constants.DATE_VAR, CommonFunctionUtils.format(new Date(), "yyyyMMdd"));

			BeanFactory beanFactory = FactoryManager.getBeanFactory();
			// context.setBeanFactory(beanFactory);//设置上下文内存

			parser = ParserFactory.getInstance(logger);
			if (interfaceName.startsWith("__"))
				parser = ParserFactory.getInstance(logger, "xml");
			// 从内存中查找，看是否存在已经解析完的接口定义
			IInterfaceDef interfaceDef = beanFactory.getInterface(interfaceName);
			ITransactionDef transDef = null;
			if (interfaceDef != null) // 内存中存在，则使用内存中的接口定义获取交易定义，并进行其它初始化
			{
				// 获取交易实例
				transDef = parser.getTransactionDef(interfaceDef, transactionName);
			} else
			// 内存中不存在，则使用配置解析器读取配置进行解析
			{
				// 创建交易实例
				transDef = parser.getTransactionDef(interfaceName, transactionName);
				interfaceDef = transDef.getInterfaceDef();
				if (interfaceDef.getType().equals("server"))
					beanFactory.addInterface(interfaceName, interfaceDef);
			}
			reload = transDef.isReload();
			context.addVariable("transaction", Constants.Log_LEVEL, transDef.getLogLevel());

			long timeout = transDef.getTimeout();
			if (timeout > 0) {
				Thread.currentThread().setName(threadName);
				Thread t = new Thread(new Detect(threadName, startTime, timeout, logger));
				t.setName(threadName + " detect thread");
				t.setDaemon(true);
				t.start();
			}

			String iELversion = interfaceDef.getVersion();
			String tELVersion = transDef.getVersion();
			if (iELversion.equals(VERSION) && StringUtil.isEmpty(tELVersion)) {
				transDef.setVersion(VERSION);
				tELVersion = VERSION;
			}

			// 接口级别的LOG日志，暂时先只处理Transaction节点的日志级别.
			String trasLogLevel = transDef.getLogLevel();
			context.setLogLevel(trasLogLevel);

			// 将接口和交易名信息放入内存
			context.getResource().setInterfaceName(interfaceName);
			context.addVariable("transaction", "interfaceName", interfaceName);
			context.getResource().setTransactionName(transactionName);
			context.addVariable("transaction", "transactionName", transactionName);

			context.addVariable("transaction", "__scope", interfaceDef.getDefaultScope());

			// 设置输入
			setCallInput(context);

			// 表达式解析器
			AbstractEL apacheEL = null;
			if (tELVersion.equals(VERSION)) {
				apacheEL = new ApacheJexl(context);
				// apacheEL.setFuncMap(interfaceDef.getFuncMap());
				apacheEL.setFunctionMap(interfaceDef.getFuncMap(), context);
				apacheEL.dealElExpression(context, "");
			} else {
				apacheEL = new ApacheEL(context);
				apacheEL.setFunctionMap(interfaceDef.getFunctionMap());
			}
			context.setElParser(apacheEL);// 指定表达式解析器

			msg = new ResultMsg(true);// 返回对象状态初始化为true,表明调用成功

			context.setErrorCodes(interfaceDef.getErrorCodes());
			context.setCodetableMap(interfaceDef.getCodetableMap());
			context.setDynamicCodetableMap(interfaceDef.getDynamicCodetableMap());
			context.setModulesMap(interfaceDef.getModuleMap());
			// 初始化计数器信息
			context.setCounterMap(interfaceDef.getCountermap());

			// 初始化语言环境
			LocaleUtil.initLanguage(context, interfaceDef.getLanguage(), logger);
			context.setInterfaceDef(interfaceDef);
			if (null != transDef) {
				// 将参数数组放入到上下文中
				context.getResource().setObject(objs);
				context.getResource().setTransArgs(objs);
				detail = transDef.getDetail();
				if (!StringUtil.isEmpty(detail)) {
					detail = (String) apacheEL.getExPression(context, detail, null);
					// TODO此处将info改为Stringbuffer进行输出
					// Logger detailLogger = LogUtil.getLogger(detail);
					// context.getResource().setDetailLog(detailLogger);
				}
				context.getResource().setLogbuffer(new StringBuffer());
				context.getResource().setErrorcode(IResource.ERRORCODE_NORMAL);
				context.getResource().setDescription("操作正常");
				context.setResultMsg(msg);

				// 执行交易实例
				execute(transDef, context);
			}
		} catch (Exception e) {
			logger.error("transaction error:", e);
			LocaleUtil.initLanguage(context, "zh", logger);
			// 获取异常对应的错误码和错误描述
			SysCode sysCode = ExceptionUtil.getErrorCode(context, e);
			String errorMsg = sysCode.getErrorText();
			String errorId = sysCode.getId();
			logger.error("[error:" + errorId + "] " + errorMsg);
			msg.setSuccess(false);// 返回对象状态置为false，表明调用过程出错
			msg.setCauseException(e);// 保存引起出错的异常
			msg.setContent(errorId);// 设置返回的错误码

			// 清空输入输出设置
			context.getInputMap().clear();
			context.getOutputMap().clear();
		} catch (Throwable e) {
			logger.error("transaction throwable error:", e);
		} finally {
			logger.debug("transaction finally.");
			FileOutputStream fos = null;
			try {
				// 关闭所有未关闭的数据库连接
				context.closeAllConnection();
				// 关闭所有Jotm
				context.closeJotm();
				// 关闭所有未关闭的jedis连接
				context.closeJedis();
				if (reload) {
					logger.info("transaction reload.");
					new reloadThread(context, interfaceName, transactionName, objs, null).start();
				}
				// 关闭tibico
				// context.closeTibco();
				// 交易详细信息打印
				StringBuffer sb = context.getResource().getLogbuffer();
				if (!StringUtil.isEmpty(detail)) {
					File file = new File(detail);
					if (!file.getParentFile().exists()) {
						file.getParentFile().mkdirs();
					}
					if (!file.exists())
						file.createNewFile();
					fos = new FileOutputStream(file, true);
					String logmsg = sb.toString();
					logmsg = logmsg.replace("\\r\\n", "\r\n").replace("\\r", "\r").replace("\\n", "\n");
					IOUtils.write(logmsg, fos);
				}
			} catch (SQLException e) {
				logger.error("sql connection close error.", e);
			} catch (IOException e) {
				// TODO 抛出IO异常不一定是logbuf错误，
				logger.error("logbuf file read or write error", e);
			} catch (Exception e) {
				logger.error("Close connection error", e);
			} finally {
				if (fos != null){
                    IOUtils.closeQuietly(fos);
                }
			}
            MDC.remove(LogUtil.LOG_FILE_NAME_SYMBOL);
            logger.debug("transaction finally done.");
		}
		return msg;
	}

	private class reloadThread extends Thread {
		private Context context;
		private String interfaceName;
		private String transactionName;
		private Object[] objs;
		private Logger logger;

		private reloadThread(Context context, String interfaceName, String transactionName, Object[] objs, Logger logger) {
			this.context = context;
			this.interfaceName = interfaceName;
			this.transactionName = transactionName;
			this.objs = objs;
			this.logger = logger;
		}

		@Override
		public void run() {
			ISocketProxy socketProxy = context.getResource().getSocketProxy();
			if (null != socketProxy)
				socketProxy.close();
			if (null != logger)
				new Client().call(interfaceName, transactionName, objs, logger, context);
			else
				new Client().call(context, interfaceName, transactionName, objs);
		}

	}

	public ResultMsg call(Logger logger, String interfaceName, String transactionName, Object[] objs, Context context) {
		return call(interfaceName, transactionName, objs, logger, context);
	}

	public ResultMsg call(String interfaceName, String transactionName, Object[] objs, Logger logger, Context context) {
		return call(interfaceName, transactionName, objs, logger, context, null, false);
	}

	public ResultMsg call(String interfaceName, String transactionName, Object[] objs, Logger logger, Context context, String pre, boolean merge) {
		if (logger == null) {
			logger = LogUtil.getTrnLog(interfaceName, transactionName, pre, merge);
		}
		long currentTime = System.currentTimeMillis();
		Date startTime = new Date(currentTime);
		final String threadName = interfaceName + "_" + transactionName + "_" + currentTime;

		ResultMsg msg = null;// 返回对象
		ConfigParser parser = null;// 配置解析器
		String detail = "";
		boolean reload = false;
		try {
			context.addVariable("transaction", Constants.Log_VAR, logger);
			context.addVariable("transaction", Constants.DATE_VAR, CommonFunctionUtils.format(new Date(), "yyyyMMdd"));

			BeanFactory beanFactory = FactoryManager.getBeanFactory();
			// context.setBeanFactory(beanFactory);//设置上下文内存

			parser = ParserFactory.getInstance(logger);
			if (interfaceName.startsWith("__"))
				parser = ParserFactory.getInstance(logger, "xml");
			// 从内存中查找，看是否存在已经解析完的接口定义
			IInterfaceDef interfaceDef = beanFactory.getInterface(interfaceName);
			ITransactionDef transDef = null;
			if (interfaceDef != null) // 内存中存在，则使用内存中的接口定义获取交易定义，并进行其它初始化
			{
				// 获取交易实例
				transDef = parser.getTransactionDef(interfaceDef, transactionName);
			} else
			// 内存中不存在，则使用配置解析器读取配置进行解析
			{
				// 创建交易实例
				transDef = parser.getTransactionDef(interfaceName, transactionName);
				interfaceDef = transDef.getInterfaceDef();
				if (interfaceDef.getType().equals("server"))
					beanFactory.addInterface(interfaceName, interfaceDef);
			}
			reload = transDef.isReload();

			context.addVariable("transaction", Constants.Log_LEVEL, transDef.getLogLevel());

			long timeout = transDef.getTimeout();
			if (timeout > 0) {
				Thread.currentThread().setName(threadName);
				Thread t = new Thread(new Detect(threadName, startTime, timeout, logger));
				t.setName(threadName + " detect thread");
				t.setDaemon(true);
				t.start();
			}

			String iELversion = interfaceDef.getVersion();
			String tELVersion = transDef.getVersion();
			if (iELversion.equals(VERSION) && StringUtil.isEmpty(tELVersion)) {
				transDef.setVersion(VERSION);
				tELVersion = VERSION;
			}

			// 接口级别的LOG日志，暂时先只处理Transaction节点的日志级别.
			String trasLogLevel = transDef.getLogLevel();
			context.setLogLevel(trasLogLevel);

			// 将接口和交易名信息放入内存
			context.getResource().setInterfaceName(interfaceName);
			context.addVariable("transaction", "interfaceName", interfaceName);
			context.getResource().setTransactionName(transactionName);
			context.addVariable("transaction", "transactionName", transactionName);

			context.addVariable("transaction", "__scope", interfaceDef.getDefaultScope());

			// 设置输入
			setCallInput(context);

			// 表达式解析器
			AbstractEL apacheEL = null;
			if (tELVersion.equals(VERSION)) {
				apacheEL = new ApacheJexl(context);
				// apacheEL.setFuncMap(interfaceDef.getFuncMap());
				apacheEL.setFunctionMap(interfaceDef.getFuncMap(), context);
				apacheEL.dealElExpression(context, "");
			} else {
				apacheEL = new ApacheEL(context);
				apacheEL.setFunctionMap(interfaceDef.getFunctionMap());
			}
			context.setElParser(apacheEL);// 指定表达式解析器

			msg = new ResultMsg(true);// 返回对象状态初始化为true,表明调用成功

			context.setErrorCodes(interfaceDef.getErrorCodes());
			context.setCodetableMap(interfaceDef.getCodetableMap());
			context.setDynamicCodetableMap(interfaceDef.getDynamicCodetableMap());
			context.setModulesMap(interfaceDef.getModuleMap());
			// 初始化计数器信息
			context.setCounterMap(interfaceDef.getCountermap());

			// 初始化语言环境
			LocaleUtil.initLanguage(context, interfaceDef.getLanguage(), logger);
			context.setInterfaceDef(interfaceDef);
			if (null != transDef) {
				// 将参数数组放入到上下文中
				context.getResource().setObject(objs);
				context.getResource().setTransArgs(objs);
				detail = transDef.getDetail();
				if (!StringUtil.isEmpty(detail)) {
					detail = (String) apacheEL.getExPression(context, detail, null);
					// TODO此处将info改为Stringbuffer进行输出
					// Logger detailLogger = LogUtil.getLogger(detail);
					// context.getResource().setDetailLog(detailLogger);
				}
				context.getResource().setLogbuffer(new StringBuffer());
				context.getResource().setErrorcode(IResource.ERRORCODE_NORMAL);
				context.getResource().setDescription("操作正常");
				context.setResultMsg(msg);

				// 执行交易实例
				execute(transDef, context);
			}
		} catch (Exception e) {
			logger.error("transaction error:", e);
			LocaleUtil.initLanguage(context, "zh", logger);
			// 获取异常对应的错误码和错误描述
			SysCode sysCode = ExceptionUtil.getErrorCode(context, e);
			String errorMsg = sysCode.getErrorText();
			String errorId = sysCode.getId();
			logger.error("[error:" + errorId + "] " + errorMsg);
			msg.setSuccess(false);// 返回对象状态置为false，表明调用过程出错
			msg.setCauseException(e);// 保存引起出错的异常
			msg.setContent(errorId);// 设置返回的错误码
			// 清空输入输出设置
			context.getInputMap().clear();
			context.getOutputMap().clear();

			try {
				context.rollbackAllConnection();
			} catch (Exception e1) {
				logger.error("transaction rollback error:", e);
			}
		} catch (Throwable e) {
			logger.error("transaction throwable error:", e);

			try {
				context.rollbackAllConnection();
			} catch (Exception e1) {
				logger.error("transaction rollback error:", e);
			}
		} finally {
			// logger.debug("transaction finally.");
			FileOutputStream fos = null;
			try {
				// 关闭所有未关闭的数据库连接
				context.closeAllConnection();
				// 关闭所有Jotm
				context.closeJotm();
				// 关闭所有未关闭的jedis连接
				context.closeJedis();
				// 关闭tibico
				// context.closeTibco();
				if (reload) {
					logger.info("transaction reload.");
					new reloadThread(context, interfaceName, transactionName, objs, logger).start();
				}
				// 交易详细信息打印
				StringBuffer sb = context.getResource().getLogbuffer();
				if (!StringUtil.isEmpty(detail)) {
					File file = new File(detail);
					if (!file.getParentFile().exists()) {
						file.getParentFile().mkdirs();
					}
					if (!file.exists())
						file.createNewFile();
					fos = new FileOutputStream(file, true);
					String logmsg = sb.toString();
					logmsg = logmsg.replace("\\r\\n", "\r\n").replace("\\r", "\r").replace("\\n", "\n");
					IOUtils.write(logmsg, fos);
				}
			} catch (SQLException e) {
				logger.error("sql connection close error.", e);
			} catch (IOException e) {
				// TODO 抛出IO异常不一定是logbuf错误，
				logger.error("logbuf file read or write error", e);
			} catch (Exception e) {
				logger.error("Close connection error", e);
			} finally {
				if (fos != null) {
                    IOUtils.closeQuietly(fos);
                }
			}
            MDC.remove(LogUtil.LOG_FILE_NAME_SYMBOL);
            logger.debug("transaction finally done.");
		}
		return msg;
	}

	/**
	 * 执行交易实例
	 * 
	 * @param transDef
	 * @param context
	 */
	public void execute(ITransactionDef transDef, Context context) {
		ITransactionInstance instance = null;

		instance = new TransactionInstance(transDef, context);
		instance.execute();
		/*
		 * if (transDef.isSingleton() == true) {
		 * TransactionInstance.getInstance(transDef, context); } else { instance
		 * = new TransactionInstance(transDef, context); instance.execute(); }
		 */
	}

	public static void main(String[] args) {
		Client client = new Client();
		ResultMsg resultMsg = null;
		if (args.length < 2)
			System.exit(1);
		else if (args.length == 2)
			resultMsg = client.call(args[0], args[1]);
		else if (args.length > 2) {
			Object[] objects = new Object[args.length - 2];
			for (int i = 0; i < objects.length; i++) {
				objects[i] = args[i + 2];
			}
			resultMsg = client.call(args[0], args[1], objects);

		}
		if (resultMsg == null) // 接口平台自身错误
			System.exit(1);
		else {
			// FIXME
			if (resultMsg.isToException()) // 直接进入异常之后
				System.exit(1);
			if (resultMsg.isIgnoredException()) // 接口平台ignore异常错误
				System.exit(1);
			else if (resultMsg.isSuccess()) // 调用成功
				System.exit(0);
			else {
				if (resultMsg.getContent().toString().matches("\\d{5}")) // 接口平台自身错误
					System.exit(1);
				else
					// 其它系统错误或业务错误(validator抛出)
					System.exit(2);
			}
		}
	}

	public void setCallInput(Context context) {
		Map<String, Object> map = context.getInputMap();
		for (String key : map.keySet()) {
			context.addVariable("transaction", key, map.get(key));
		}

		context.getInputMap().clear();
	}
}
