package com.brilliance.eibs.main.client;

import com.brilliance.eibs.core.frame.IIdentity;
import com.brilliance.eibs.core.frame.ISocketProxy;
import com.brilliance.eibs.core.frame.ISocketProxyBuilder;
import com.brilliance.eibs.core.frame.SocketProxyFactry;
import com.brilliance.eibs.core.model.ITransactionDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.core.service.instance.plugin.MinaSocketProxy;
import com.brilliance.eibs.extension.core.IClient;
import com.brilliance.eibs.main.Client;
import com.brilliance.eibs.util.LogUtil;
import org.apache.mina.core.session.IoSession;

import java.io.IOException;

/**
 * @author hujun
 */
public class MinaClient extends Client implements IClient {

    private IoSession session;
    private Object message;
    private IIdentity identity;
    private String charset;
    private int timeout;

    public MinaClient(IoSession session, Object message, String charset, int timeout, IIdentity identity) {
        this.session = session;
        this.message = message;
        this.identity = identity;
        this.charset = charset;
        this.timeout = timeout;
    }

    @Override
    public void execute(ITransactionDef transDef, final Context context) {
        ISocketProxy socketProxy = null;
        try {
            socketProxy = SocketProxyFactry.getProxy(new ISocketProxyBuilder() {
                @Override
                public ISocketProxy buildSocketProxy() throws IOException {
                    return new MinaSocketProxy(session, message, charset, timeout, MinaClient.this);
                }
            });
        } catch (IOException e) {
            LogUtil.getLogger(context).error(e.getMessage(), e);
        }

        context.setIdentity(identity);
        context.getResource().setSocketProxy(socketProxy);
        super.execute(transDef, context);
    }

    @Override
    public void init() {
        // TODO Auto-generated method stub

    }

    @Override
    public void connect() {

    }

    @Override
    public void close() {
        this.session.closeNow();

    }

    @Override
    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }

    @Override
    public IoSession getSession() {
        return this.session;
    }
}
