package com.brilliance.eibs.monitor.nms;

import com.brilliance.eibs.util.ClassPathUpdater;
import com.brilliance.eibs.util.DateUtil;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;

import java.io.*;
import java.nio.channels.FileChannel;
import java.text.ParseException;
import java.util.*;

public class LogFileSearcher
{

  private String[] files;
  private String   starttime;
  private String   endtime;

  public LogFileSearcher( String[] files )
  {
    this.files = files;
  }

  public void setTime(String starttime, String endtime)
  {
    this.starttime = starttime;
    this.endtime = endtime;
    vaildateTime();
  }

  private void vaildateTime()
  {
    if (StringUtil.isEmpty(starttime) || StringUtil.isEmpty(endtime))
      return;
    if (Integer.valueOf(starttime) > Integer.valueOf(endtime))
      throw new IllegalArgumentException("Start time can not be later than the end of time.");
  }

  private boolean match(String[] arr, String targetValue)
  {
    for (String s : arr)
    {
      if (s.equals(targetValue))
        return true;
    }
    return false;
  }

  public LogFilePageDto query(int page, int records)
  {
    File root = new File(ClassPathUpdater.addClasspath(LogUtil.LOG_PATH));
    if (!root.exists())
      return null;
    File[] dirs = root.listFiles(new FileFilter()
    {

      @Override
      public boolean accept(File pathname)
      {
        if (!pathname.isDirectory())
          return false;
        if (StringUtil.isEmpty(starttime) && StringUtil.isEmpty(endtime))
          return true;
        int current = Integer.valueOf(pathname.getName());
        if (StringUtil.isEmpty(starttime) && !StringUtil.isEmpty(endtime) && current <= Integer.valueOf(endtime))
          return true;
        if (StringUtil.isEmpty(endtime) && !StringUtil.isEmpty(starttime) && current >= Integer.valueOf(starttime))
          return true;
        if (!StringUtil.isEmpty(starttime) && !StringUtil.isEmpty(endtime) && current <= Integer.valueOf(endtime)
            && current >= Integer.valueOf(starttime))
          return true;
        return false;
      }
    });

    List<LogFileDto> dtos = new ArrayList<LogFileDto>();

    for (File f : dirs)
    {
      if (!f.isDirectory())
        continue;

      File[] results = f.listFiles(new FilenameFilter()
      {
        @Override
        public boolean accept(File dir, String name)
        {
          if (null == files || files.length == 0)
            return true;
          return match(files, name);
        }
      });

      for (File result : results)
      {
        String fileName = result.getName();
        String dirName = f.getName();
        Date date = null;
        try
        {
          date = DateUtil.parseDate(dirName, "yyyyMMdd");
        }
        catch (ParseException e1)
        {}
        LogFileDto dto = new LogFileDto(fileName, DateUtil.format(date, "yyyy/MM/dd"));
        FileChannel fc = null;
        try
        {
          FileInputStream fis = null;
          try
          {
            fis = new FileInputStream(result);
            fc = fis.getChannel();
            dto.setSize(fc.size());
          }
          catch (IOException e)
          {}
        }
        finally
        {
          if (null != fc)
            try
            {
              fc.close();
            }
            catch (IOException e)
            {}
        }
        dto.setFile(LogUtil.LOG_PATH + File.separator + dirName + File.separator + fileName);
        dtos.add(dto);
      }
    }

    Collections.sort(dtos, new Comparator<LogFileDto>()
    {
      @Override
      public int compare(LogFileDto o1, LogFileDto o2)
      {
        return o2.getFile().compareTo(o1.getFile());
      }
    });

    int size = dtos.size();
    LogFilePageDto pageDto = new LogFilePageDto(size / records + 1, page);

    int tmpRecords = page * records > size ? size - (page - 1) * records : records;
    pageDto.setRecords(size);
    pageDto.setRows(dtos.subList((page - 1) * records, (page - 1) * records + tmpRecords));

    return pageDto;

  }
}
