package com.brilliance.eibs.quartz;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IArgumentDef;
import com.brilliance.eibs.core.model.IPropertyDef;
import com.brilliance.eibs.core.model.ITaskDef;
import com.brilliance.eibs.core.service.ResultMsg;
import com.brilliance.eibs.el.CommonFunctionUtils;
import com.brilliance.eibs.main.Client;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 时间表达式任务
 * 
 * @author xiaoyuanzhen
 *
 */
public class CronTask extends Thread {
	private ITaskDef taskDef; // 定时任务定义
	private static Object lock = new Object();
	private Object[] args = null;
	private static ExecutorService executorService = Executors.newCachedThreadPool();
	private boolean needStop = false;

	public CronTask(ITaskDef taskDef) {
		synchronized (lock) {
			this.taskDef = taskDef;
		}
	}

	public void setStop() {
		this.needStop = true;
	}

	public CronTask(ITaskDef taskDef, Object[] args) {
		synchronized (lock) {
			this.taskDef = taskDef;
			this.args = args;
		}
	}

	@Override
	public void run() {

		synchronized (this) {
			try {
				startTask();
			} catch (ParseException e) {
				e.printStackTrace();
			}

		}
	}

	/**
	 * 启动任务
	 * 
	 * @throws ParseException
	 */
	public void startTask() throws ParseException {
		// Thread.currentThread().setName();
		String expression = taskDef.getProperty("expression").getValue();
		CronExpression cron = new CronExpression(expression);
		Date nextValidDate = cron.getNextValidTimeAfter(new Date());
		while (!isInterrupted() && (!needStop)) {
			if (nextValidDate == null)
				break;
			try {
				long nextValidTime = nextValidDate.getTime();
				while (System.currentTimeMillis() < nextValidTime)
					Thread.sleep(1000);
				taskwork();
			} catch (InterruptedException e) {
				System.out.println("interrupted!");
				break;
			}
			nextValidDate = cron.getNextValidTimeAfter(new Date());
		}
		System.out.println("task finished.");
	}

	/**
	 * 任务处理内容
	 */
	private void taskwork() {
		String id = taskDef.getId();
		StopServerHandle s = null;
		boolean done = false;
		if (id.startsWith("_")) {
			// if (!Server.stopServermap.containsKey(id)) {
			long startTime = System.currentTimeMillis();
			s = new StopServerHandle(this, startTime, done);
			executorService.execute(s);
			// Server.stopServermap.put(taskDef.getId(), s);
			// }
		}

		String interfaceName = taskDef.getProperty("interfacename").getValue();
		String transName = taskDef.getProperty("transname").getValue();
		IPropertyDef argsProp = taskDef.getProperty("args");
		Object[] objs = null;
		if (argsProp != null) {
			List<IArgumentDef> arguments = taskDef.getProperty("args").getArguments();
			int size = arguments.size();
			objs = new Object[size];
			for (int i = 0; i < size; i++) {
				IArgumentDef argument = arguments.get(i);
				objs[i] = argument.getValue();
			}
		} else {
			objs = this.args;
		}
		String other = taskDef.getOtherprocess();
		if (other == null || other.trim() == "") {
			String dependence = taskDef.getDepencency();
			if (!StringUtil.isEmpty(dependence)) {
				if (constainsNoCompleteTask(taskDef)) {
					System.out.println("任务ID:[" + taskDef.getId() + "]有未完成的依赖任务,请检查！");
					return;
				}
			}

			// 如果当前任务已经执行且continueExcute为false，则不再执行当前任务
			if (!needContinueExcute(taskDef)) {
				System.out.println("任务ID:[" + taskDef.getId() + "]已经执行,且属性continueExcute为false，不再执行！");
				return;
			}

			ResultMsg resultMsg = new Client().call(interfaceName, transName, objs);
			if (resultMsg.isSuccess()) {
				writeTaskId(taskDef);
			}
		} else
			//
			// executeShell(other.trim()+" " + interfaceName + " " + transName);
			executeShell(other.trim());
		System.out.println(interfaceName + "," + transName + "," + objs);
		if (s != null) {
			s.setStartTime(System.currentTimeMillis());
			s.setTotalSleepTime(0);
		}

		if (id.startsWith("_") && s != null) {
			s.setStop();
		}
	}

	public synchronized static boolean needContinueExcute(ITaskDef taskDef) {
		boolean continueExcute = Boolean.valueOf(taskDef.getContinueExcute());
		if (continueExcute)
			return true;
		String filePath = getPath(taskDef);
		generateCompletedTaskFile(filePath);
		try {
			List<String> list = FileUtils.readLines(new File(filePath));
			if (list.contains(taskDef.getId()))
				return false;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return true;
	}

	public synchronized static boolean constainsNoCompleteTask(ITaskDef taskDef) {
		String filePath = getPath(taskDef);
		generateCompletedTaskFile(filePath);
		try {
			List<String> list = FileUtils.readLines(new File(filePath));
			String[] depencys = taskDef.getDepencency().split(",");
			for (int i = 0; i < depencys.length; i++) {
				if (!list.contains(depencys[i])) {
					return true;
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return false;
	}

	public synchronized static void writeTaskId(ITaskDef taskDef) {
		List<String> list = new ArrayList<String>();
		list.add(taskDef.getId());
		try {
			String filePath = getPath(taskDef);
			generateCompletedTaskFile(filePath);
			List<String> readList = FileUtils.readLines(new File(filePath));
			if (!readList.contains(taskDef.getId()))
				FileUtils.writeLines(new File(filePath), list, true);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static String getPath(ITaskDef taskDef) {
		Date date = CommonFunctionUtils.dateAdd(new Date(), 4, Integer.parseInt(taskDef.getDate()));
		String filePath = CommonFunctionUtils.getCommonURL("").getPath() + "/taskRecord/completed" + CommonFunctionUtils.format(date, "yyyyMMdd")
				+ ".txt";
		return filePath;
	}

	private static void generateCompletedTaskFile(String path) {
		File file = new File(path);
		if (!file.getParentFile().exists()) {
			file.getParentFile().mkdirs();
		}
		if (!file.exists())
			try {
				file.createNewFile();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	}

	public int executeShell(String shellCommand) {
		int success = 0;
		StringBuffer stringBuffer = new StringBuffer();
		// 格式化日期时间，记录日志时使用
		DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss ");
		InputStream in = null;
		InputStream errorin = null;
		try {
			stringBuffer.append(dateFormat.format(new Date())).append("准备执行Shell命令 ").append(shellCommand).append(" \r\n");
			Process pid = null;
			String[] cmdarray = shellCommand.split("\\ ");// 空格
			for (String str : cmdarray) {
				System.out.println("every command parameter : \n" + str);
			}
			Runtime runtime = Runtime.getRuntime();
			pid = runtime.exec(cmdarray);
			// 执行Shell命令
			if (pid != null) {
				stringBuffer.append("进程号：").append(pid.toString()).append("\r\n");
				System.out.println("test shell bufferedread............");
				in = pid.getInputStream();
				errorin = pid.getErrorStream();
				stringBuffer.append(loadStream(in));
				stringBuffer.append(loadStream(errorin));
				int a = pid.waitFor();
				System.out.println("pid flag=" + a);
			} else {
				stringBuffer.append("没有pid\r\n");
			}
		} catch (Exception ioe) {
			ioe.printStackTrace();
			success = 1;
			stringBuffer.append("执行Shell命令时发生异常：\r\n").append(ioe.getMessage()).append("\r\n");
			throw new InterfaceException("00801", "read file error.", ioe);
		} finally {
			System.out.println("shell mssage:" + stringBuffer.toString());
			IOUtils.closeQuietly(errorin);
			IOUtils.closeQuietly(in);
		}
		System.out.println("******end*******");
		return success;
	}

	public String loadStream(InputStream in) {
		int ptr = 0;
		in = new BufferedInputStream(in);
		StringBuffer buffer = new StringBuffer();
		try {
			while ((ptr = in.read()) != -1) {
				buffer.append((char) ptr);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return buffer.toString();
	}

	public ITaskDef getTaskDef() {
		return taskDef;
	}

	public void setTaskDef(ITaskDef taskDef) {
		this.taskDef = taskDef;
	}

}