package com.brilliance.eibs.server;

import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.service.Context;
import com.brilliance.eibs.el.ApacheEL;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import org.slf4j.Logger;

/**
 * 服务抽象类，提供服务通用的基本方法
 *
 * @author xiaoyuanzhen
 */
public abstract class AbsServer extends Thread implements IServerInstance {
    protected Object[] args;
    protected Context context; // 上下文对象
    protected ApacheEL elParser; // 表达式处理对象
    protected Logger logger; // 日志对象
    protected IServiceDef serviceDef; // 服务定义结构

    protected int receiveTimeout; // 接收消息超时时间
    protected int connectTimeout; // 连接超时时间

    public AbsServer(Context context, IServiceDef serviceDef, ApacheEL elParser) {
        this.context = context;
        this.elParser = elParser;
        this.logger = LogUtil.getLogger(context);
        this.serviceDef = serviceDef;
    }

    /**
     * 获取服务属性值
     *
     * @param name
     * @return
     */
    public String getPropertyValue(String name) {
        return serviceDef.getPropertyMap().get(name);
    }

    public boolean getPropertyValue(String name, boolean flg) {
        String value = serviceDef.getPropertyMap().get(name);
        if (StringUtil.isEmpty(value)) {
            return flg;
        } else {
            return Boolean.valueOf(value);
        }
    }

    public int getPropertyValue(String name, int val) {
        String value = serviceDef.getPropertyMap().get(name);
        if (StringUtil.isEmpty(value)) {
            return val;
        } else {
            return Integer.parseInt(value);
        }
    }

    public String getPropertyValue(String name, String val) {
        String value = serviceDef.getPropertyMap().get(name);
        if (StringUtil.isEmpty(value)) {
            return val;
        } else {
            return value;
        }
    }

    /**
     * 获取服务必配属性值，如果不存在，产生异常
     *
     * @param name
     * @return
     */
    public String getRequiredPropertyValue(String name) {
        String value = getPropertyValue(name);
        if (value == null) {
            logger.error("service [" + serviceDef.getId() + "] lack " + name + " property.");
            throw new InterfaceException("00301");
        }
        return value;
    }

    public Object[] getArgs() {
        return args;
    }

    public void setArgs(Object[] args) {
        this.args = args;
    }

}
