package com.brilliance.eibs.server.manager;

import com.brilliance.eibs.core.model.IServiceDef;
import com.brilliance.eibs.core.model.ITaskDef;
import com.brilliance.eibs.factory.FactoryManager;
import com.brilliance.eibs.factory.parser.ConfigParser;
import com.brilliance.eibs.factory.parser.ParserFactory;
import com.brilliance.eibs.main.Server;
import com.brilliance.eibs.quartz.CronTask;
import com.brilliance.eibs.server.IServerInstance;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.Serializable2FileUtil;
import org.slf4j.Logger;

import java.util.List;

public class ServerUtil {
    private static Logger logger = LogUtil.getLogger("services");
    private static ConfigParser parser = ParserFactory.getInstance(logger);
    private static boolean initFlag = false;

    /**
     * 服务调用入口
     *
     * @param serviceId 服务ID/TaskId
     * @param type      值为"service"或者"task"
     * @param args      服务相关参数
     **/
    public static synchronized void start(final String serviceId, String type, Object[] args) {
        if (type.equals("service")) {
            dealService(parser, serviceId, args);
        } else {
            dealTask(parser, serviceId, args);
        }
        if (!initFlag) {
            Server.initBeanFactoryMemory(parser, FactoryManager.getBeanFactory());
            initFlag = true;
        }
    }

    /**
     * 服务关闭
     *
     * @param serviceId 服务ID/TaskId
     * @param type      值为"service"或者"task"
     **/
    public static synchronized void close(final String serviceId, String type) {
        if (type.equals("service")) {
            Server.closeServer(serviceId);
        } else {
            Server.closeTask(serviceId);
        }
        if (!initFlag) {
            Server.initBeanFactoryMemory(parser, FactoryManager.getBeanFactory());
            initFlag = true;
        }
    }

    public static void dealService(ConfigParser parser, String serviceId, Object[] args) {
        if (Server.servers.containsKey(serviceId)) {
            logger.info("service id[" + serviceId + "] has been started");
            return;
        }
        logger.info("service id[" + serviceId + "] will be started");
        Server.dealBDHotDeploy();
        // Server.dealHotDeploy();
        logger.info("before to start service....");
        List<IServiceDef> serviceDefs = parser.getServiceDefs("services");// 获取所有服务定义
        Serializable2FileUtil.writeObject(serviceDefs, "services");// 保存为序列化文件，用于热部署
        try {
            for (IServiceDef serviceDef : serviceDefs)// 启动services
            {
                if (serviceId.equals(serviceDef.getId())) {
                    if (serviceDef.getState().equals("on") && !Server.servers.containsKey(serviceId)) {
                        Server.startServer(serviceDef, args);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            removeServer(serviceId);
        }
    }

    public static void dealTask(ConfigParser parser, String serviceId, Object[] args) {
        if (Server.taskMap.containsKey(serviceId)) {
            logger.info("task id[" + serviceId + "] has been started");
            return;
        }
        logger.info("task id[" + serviceId + "] will be started");
        Server.dealBDHotDeploy();
        // Server.dealHotDeploy();
        logger.info("before to start task....");
        try {
            List<ITaskDef> taskDefs = parser.getTaskDefs("services");
            Serializable2FileUtil.writeObject(taskDefs, "tasks");
            for (ITaskDef taskDef : taskDefs)// 启动tasks
            {
                if (taskDef.getState().equals("on") && !Server.taskMap.containsKey(taskDef.getId())) {
                    Server.startTask(taskDef, args);
                }
            }
            // 初始化内存
            // Server.initBeanFactoryMemory(xmlParser,
            // FactoryManager.getBeanFactory());
        } catch (Exception e) {
            e.printStackTrace();
            removeTask(serviceId);
        }
    }

    public static void removeServer(String serviceId) {
        IServerInstance serverInstance = Server.servers.get(serviceId);
        Server.servers.remove(serviceId);
        if (serverInstance != null)
            serverInstance = null;
    }

    public static void removeTask(String serviceId) {
        CronTask task = Server.taskMap.get(serviceId);
        Server.taskMap.remove(serviceId);
        if (task != null)
            task = null;
    }

    public static void main(String[] args) {
        // Service
        /*
         * ServerUtil.start("beijing", "service", new Object[]{"AAA"});
         * ServerUtil.start("beijing1", "service", new Object[]{"AAA"});
         */
        // Task
        ServerUtil.start("swiftrecievetask", "task", new Object[]{"AAA"});
        // ServerUtil.close("swiftrecievetask", "task");
    }
}
