package com.brilliance.eibs.server.socket.impl;

import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolCodecFactory;
import org.apache.mina.filter.codec.ProtocolDecoder;
import org.apache.mina.filter.codec.ProtocolEncoder;
import org.apache.mina.filter.codec.prefixedstring.PrefixedStringCodecFactory;
import org.apache.mina.filter.codec.serialization.ObjectSerializationCodecFactory;
import org.apache.mina.filter.codec.textline.LineDelimiter;
import org.apache.mina.filter.codec.textline.TextLineCodecFactory;

import javax.resource.NotSupportedException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.nio.charset.Charset;
import java.util.Map;

@SuppressWarnings("rawtypes")
public class BeProtocolCodecFactoryWrapper implements ProtocolCodecFactory {
	private ProtocolEncoder encoder;
	private ProtocolDecoder decoder;
	private Map<String, String> props;

	public BeProtocolCodecFactoryWrapper(Map<String, String> props)
			throws Exception {
		this.props = props;
		// 先获取工厂
		String factroy = getRequiredPropertyValue("factory");
		ProtocolCodecFactory instance = initFactory(factroy);
		// 赋值编解码对象
		this.encoder = instance.getEncoder(null);
		this.decoder = instance.getDecoder(null);

	}

	private ProtocolCodecFactory initFactory(String factroy) throws Exception {
		ProtocolCodecFactory instance = null;
		Charset charset = Charset.forName(getPropertyValue("charset",
				Charset.defaultCharset().name()));
		System.out.println("平台Mina目前使用的字符集："+charset);
		if ("TextLineCodecFactory".equals(factroy)) {
			LineDelimiter encodingDelimiter = new LineDelimiter(
					getPropertyValue("encodingDelimiter", "\n"));
			LineDelimiter decodingDelimiter = new LineDelimiter(
					getPropertyValue("decodingDelimiter", ""));
			instance = new TextLineCodecFactory(charset, encodingDelimiter,
					decodingDelimiter);
		}else if("PrefixedStringCodecFactory".equals(factroy)){
			instance=new PrefixedStringCodecFactory(charset);
		}else if("ObjectSerializationCodecFactory".equals(factroy)){
			instance=new ObjectSerializationCodecFactory();
		}
		else {
			throw new NotSupportedException("mina不支持的编解码工厂:" + factroy);
		}
		// set方法赋值
		try {
			Class cl = instance.getClass();
			for (; cl != null; cl = cl.getSuperclass()) {
				Method[] methods = cl.getDeclaredMethods();
				for (int i = 0; i < methods.length; i++) {
					Method method = methods[i];
					if (Modifier.isStatic(method.getModifiers()))
						continue;
					String name = method.getName();
					if (!name.startsWith("setDecoder") && !name.startsWith("setEncoder"))
						continue;
					Class[] paramTypes = method.getParameterTypes();
					if (paramTypes.length != 1)
						continue;
					if (!method.getReturnType().equals(void.class))
						continue;
					if (findGetter(methods, name, paramTypes[0]) == null)
						continue;
					method.setAccessible(true);
					name = name.substring(10);
					int j = 0;
					for (; j < name.length()
							&& Character.isUpperCase(name.charAt(j)); j++) {
					}
					if (j == 1)
						name = name.substring(0, j).toLowerCase()
								+ name.substring(j);
					else if (j > 1)
						name = name.substring(0, j - 1).toLowerCase()
								+ name.substring(j - 1);
					if(isNotEmpty(name)){
						method.invoke(instance, new Object[]{getParamArg(paramTypes[0],props.get(name))});
					}
				}
			}
		} catch (Exception e) {
			throw new IllegalArgumentException("编解码工厂配置异常:" + e.getMessage(),e);
		}

		return instance;
	}

	private Method findGetter(Method[] methods, String setterName, Class arg) {
		String getterName = "get" + setterName.substring(3);
		for (int i = 0; i < methods.length; i++) {
			Method method = methods[i];
			if (!method.getName().equals(getterName))
				continue;
			if (!method.getReturnType().equals(arg))
				continue;
			Class[] params = method.getParameterTypes();
			if (params.length == 0)
				return method;
		}
		return null;
	}

	protected static Object getParamArg(Class cl,String val) {
        if (String.class.equals(cl))
            return val;
        else if (int.class.equals(cl))
            return Integer.valueOf(val);
        else if (boolean.class.equals(cl))
            return Boolean.FALSE;
        else if (byte.class.equals(cl))
            return Byte.valueOf(val);
        else if (short.class.equals(cl))
            return Short.valueOf(val);
        else if (long.class.equals(cl))
            return Long.valueOf(val);
        else if (float.class.equals(cl))
            return Double.valueOf(val);
        else if (double.class.equals(cl))
            return Double.valueOf(val);
        else
            throw new UnsupportedOperationException();
    }
	
	private String getPropertyValue(String key, String defaultValue) {
		String value = props.get(key);
		if (value == null || value.length() == 0) {
			return defaultValue;
		}
		return value;
	}

	private String getRequiredPropertyValue(String key) {
		String value = props.get(key);
		if (value == null || value.equals("")) {
			throw new IllegalArgumentException("属性[" + key + "]不能为空");
		}
		return value;
	}

	private boolean isNotEmpty(String key){
		String value = props.get(key);
		if (value == null || value.length() == 0) {
			return false;
		}
		return true;
	}
	
	@Override
	public ProtocolEncoder getEncoder(IoSession session) throws Exception {
		// TODO Auto-generated method stub
		return encoder;
	}

	@Override
	public ProtocolDecoder getDecoder(IoSession session) throws Exception {
		// TODO Auto-generated method stub
		return decoder;
	}
	
}