package com.brilliance.eibs.server.socket.impl;

import com.brilliance.eibs.core.debug.model.DebugIdentity;
import com.brilliance.eibs.core.exception.InterfaceException;
import com.brilliance.eibs.core.threadpool.NamedThreadFactory;
import com.brilliance.eibs.extension.mina.handle.NioProcessHandler;
import com.brilliance.eibs.extension.mina.server.MinaNioServer;
import com.brilliance.eibs.main.client.MinaClient;
import com.brilliance.eibs.server.IServerInstance;
import com.brilliance.eibs.server.socket.IServerMode;
import org.apache.mina.core.session.IoSession;
import org.apache.mina.filter.codec.ProtocolCodecFilter;
import org.apache.mina.filter.executor.ExecutorFilter;
import org.slf4j.Logger;

import java.net.InetSocketAddress;
import java.nio.charset.Charset;
import java.util.Map;
import java.util.concurrent.Executors;

/**
 * @author hujun
 * 
 *         mina nio 实现socket服务 201908 weicong 修改
 */
public class MinaNioMode implements IServerMode {

	private String trustips;// 信任的IP列表
	private int port;
	private int queueLen;
	private String interfaceName;
	private String transName;
	private int timeout;
	private String serverId;
	private String charset = Charset.defaultCharset().name();
	private Logger logger;

	private MinaNioServer server;

	private Map<String, String> props;

	private MinaNioMode(Builder builder) {
		this.trustips = builder.trustips;
		this.port = builder.port;
		this.queueLen = builder.queueLen;
		this.interfaceName = builder.interfaceName;
		this.transName = builder.transName;
		this.timeout = builder.timeout;
		this.serverId = builder.serverId;
		this.logger = builder.logger;
		this.props = builder.props;
	}

	public static class Builder {
		private String trustips; // 信任的IP列表
		private int port;
		private int queueLen;
		private String interfaceName;
		private String transName;
		private int timeout;
		private String serverId;
		private Map<String, String> props;
		private Logger logger;

		public Builder(int port, int queueLen) {
			this.port = port;
			this.queueLen = queueLen;
		}

		public Builder setTrustips(String trustips) {
			this.trustips = trustips;
			return this;
		}

		public Builder setInterfaceName(String interfaceName) {
			this.interfaceName = interfaceName;
			return this;
		}

		public Builder setTimeout(int timeout) {
			this.timeout = timeout;
			return this;
		}

		public Builder setTransName(String transName) {
			this.transName = transName;
			return this;
		}

		public Builder setServerId(String serverId) {
			this.serverId = serverId;
			return this;
		}

		public Builder setLogger(Logger logger) {
			this.logger = logger;
			return this;
		}

		public MinaNioMode build() {
			return new MinaNioMode(this);
		}

		public Builder setProps(Map<String, String> props) {
			this.props = props;
			return this;
		}

	}

	@Override
	public void startup() {
		if (this.props.containsKey("charset")) {
			charset = this.props.get("charset");
		}
		int processNum = IServerInstance.DEFAULT_IO_THREADS;
		int readBufferSize = 2048;
		server = new MinaNioServer.Builder(processNum, port)
				.setBacklog(queueLen).setIdleTimeMillis(timeout)
				.setReadBufferSize(readBufferSize).build();
		server.init();
		if (!"*".equals(trustips))
			server.trusts(trustips.split(","));
		try {
			server.addFilter("codec", new ProtocolCodecFilter(
					new BeProtocolCodecFactoryWrapper(this.props)));

			server.addFilter(
					"threadPool",
					new ExecutorFilter(Executors
							.newCachedThreadPool(new NamedThreadFactory(
									"MinaServerWorker", true))));

			server.loadPloyHandler(new NioProcessHandler() {
				@Override
				protected void process(IoSession session, Object message) {
					InetSocketAddress isa = (InetSocketAddress) session
							.getRemoteAddress();
					new MinaClient(session, message, charset, timeout,
							new DebugIdentity(
									isa.getAddress().getHostAddress(),
									interfaceName, transName)).call(
							interfaceName, transName, new Object[] { null });
				}

			});

			server.start();
		} catch (Exception e) {
			throw new InterfaceException("02301", e);
		}
		logger.info(IServerInstance.LOG_FLAG + "MinaNioSocketServer started."
				+ IServerInstance.LOG_FLAG);

	}

	@Override
	public void close() {
		server.close();
		logger.info("MinaNioSocketServer[id:" + serverId + "] is closed.");
	}
}
