package com.brilliance.eibs.util;

import org.apache.commons.io.FileUtils;
import org.apache.tools.tar.TarEntry;
import org.apache.tools.tar.TarInputStream;
import org.apache.tools.tar.TarOutputStream;
import org.apache.tools.zip.ZipEntry;
import org.apache.tools.zip.ZipFile;
import org.apache.tools.zip.ZipOutputStream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.zip.*;

/**
 * 压缩解压缩工具类
 * 
 * @author xiaoyuanzhen
 *
 */
public class CompressUtil {

	private static Logger logger = LoggerFactory.getLogger(CompressUtil.class);

	// 增加压缩和解压缩的方法
	// 处理文件和目录
	/**
	 * @param sourceDirectory
	 * @param zipFile
	 * 
	 * @param pathStyle
	 *            是否要目录
	 * @throws IOException
	 */
	public static void zipFile(String sourceDirectory, String zipFile, boolean pathStyle) throws IOException {
		CompressUtil.zipFile(sourceDirectory, zipFile, null, pathStyle, false);
	}

	public static void zipFile(String[] paths, String zipFile, boolean pathStyle) throws IOException {
		Collection<File> files = new ArrayList<File>();
		for (String path : paths) {
			File f = new File(path);
			files.add(f);
		}
		CompressUtil.zipFile(files, zipFile, null, pathStyle, false);

	}

	public static void zipFile(String sourceDirectory, String zipFile, boolean pathStyle, boolean iscontaincurrent) throws IOException {
		CompressUtil.zipFile(sourceDirectory, zipFile, null, pathStyle, iscontaincurrent);
	}

	/***
	 * 压缩Zip
	 * 
	 * @param data
	 * @return
	 */
	public static byte[] zip(byte[] data) throws IOException {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		java.util.zip.ZipOutputStream zos = new java.util.zip.ZipOutputStream(bos);
		java.util.zip.ZipEntry entry = new java.util.zip.ZipEntry("zip");
		entry.setSize(data.length);
		zos.putNextEntry(entry);
		zos.write(data);
		zos.close();
		byte[] zipBytes = bos.toByteArray();
		bos.close();
		return zipBytes;
	}

	/***
	 * 解压Zip
	 * 
	 * @param data
	 * @return
	 * @throws IOException
	 */
	public static byte[] unZip(byte[] data) throws IOException {
		byte[] b = null;
		ByteArrayInputStream bis = new ByteArrayInputStream(data);
		ZipInputStream zip = new ZipInputStream(bis);
		while (zip.getNextEntry() != null) {
			byte[] buf = new byte[1024];
			int num = -1;
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			while ((num = zip.read(buf, 0, buf.length)) != -1) {
				baos.write(buf, 0, num);
			}
			b = baos.toByteArray();
			baos.flush();
			baos.close();
		}
		zip.close();
		bis.close();
		return b;
	}

	/***
	 * 压缩GZip
	 * 
	 * @param data
	 * @return
	 * @throws IOException
	 */
	public static byte[] gZip(byte[] data) throws IOException {
		byte[] b = null;
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		GZIPOutputStream gzip = new GZIPOutputStream(bos);
		gzip.write(data);
		gzip.finish();
		gzip.close();
		b = bos.toByteArray();
		bos.close();
		return b;
	}

	/***
	 * 解压GZip
	 * 
	 * @param data
	 * @return
	 * @throws IOException
	 */
	public static byte[] unGZip(byte[] data) throws IOException {
		byte[] b = null;
		ByteArrayInputStream bis = new ByteArrayInputStream(data);
		GZIPInputStream gzip = new GZIPInputStream(bis);
		byte[] buf = new byte[1024];
		int num = -1;
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		while ((num = gzip.read(buf, 0, buf.length)) != -1) {
			baos.write(buf, 0, num);
		}
		b = baos.toByteArray();
		baos.flush();
		baos.close();
		gzip.close();
		bis.close();
		return b;
	}

	private static void zipFile(Collection<File> files, String sourceDirectory, String zipFile, FilenameFilter filter, boolean pathStyle,
			boolean include) throws IOException {
		byte b[] = new byte[512];
		// 压缩文件的保存路径 zipFile
		// 压缩文件目录 sourceDirectory
		if (!StringUtil.isEmpty(sourceDirectory))
			files = CompressUtil.allFile(sourceDirectory);
		FileOutputStream fileOutputStream = new FileOutputStream(zipFile);
		// 使用输出流检查
		CheckedOutputStream cs = new CheckedOutputStream(fileOutputStream, new CRC32());
		// 声明输出zip流
		ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(cs));
		Iterator<?> iterator = files.iterator();

		String path = null;
		String directory = null;
		if (!StringUtil.isEmpty(sourceDirectory)) {
			File sourceDirectory_f = getFile(sourceDirectory);
			path = sourceDirectory_f.getPath();
			directory = sourceDirectory_f.getName();
		}
		while (iterator.hasNext()) {
			File next = (File) iterator.next();
			InputStream in = new FileInputStream(next);
			String fileName = null;
			fileName = next.getPath();
			if (filter != null) {
				if (!filter.accept(next, fileName)) {
					continue;
				}
			}
			if (pathStyle) {
				if (!StringUtil.isEmpty(sourceDirectory)) {
					// TODO 压缩待压缩目录名
					if (include) {
						fileName = fileName.substring(fileName.indexOf(path) + path.length() - directory.length() - 1, fileName.length());
					} else {
						fileName = fileName.substring(fileName.indexOf(path) + path.length(), fileName.length());
					}
					fileName = fileName.substring(fileName.indexOf(File.separator) + 1);
				} else
					fileName = next.getParent().substring(next.getParent().lastIndexOf(File.separator) + 1) + File.separator + next.getName();
			} else {
				fileName = next.getName();
				fileName = fileName.substring(fileName.indexOf(File.separator) + 1);
			}
			logger.debug("ziping " + fileName);
			ZipEntry e = new ZipEntry(fileName);
			out.putNextEntry(e);
			int len = 0;
			while ((len = in.read(b)) != -1) {
				out.write(b, 0, len);
			}
			in.close();
			out.closeEntry();
		}
		out.close();
		cs.close();
		// fileOutputStream.close();

	}

	public static void zipFile(Collection<File> files, String zipFile, FilenameFilter filter, boolean pathStyle, boolean include) throws IOException {
		zipFile(files, null, zipFile, filter, pathStyle, include);
	}

	public static void zipFile(String sourceDirectory, String zipFile, FilenameFilter filter, boolean pathStyle, boolean include) throws IOException {
		zipFile(null, sourceDirectory, zipFile, filter, pathStyle, include);
	}

	public static void unzipFile(String zipFileName, String targetDirectory) throws IOException {
		ZipFile zipFile;

		zipFile = new ZipFile(zipFileName);

		java.util.Enumeration<?> e = zipFile.getEntries();
		ZipEntry zipEntry = null;

		File f1 = new File(targetDirectory + File.separator);
		if (!f1.exists())
			f1.mkdir();

		while (e.hasMoreElements()) {
			zipEntry = (ZipEntry) e.nextElement();
			logger.debug("unziping " + zipEntry.getName());
			if (zipEntry.isDirectory()) {
				String name = zipEntry.getName();
				name = name.substring(0, name.length() - 1);
				logger.debug("输出路径：" + targetDirectory + File.separator + name);
				File f = new File(targetDirectory + File.separator + name);
				// f.mkdir();
				FileUtils.touch(f);
				logger.debug("创建目录：" + targetDirectory + File.separator + name);
			} else {
				File f = new File(targetDirectory + File.separator + zipEntry.getName());
				FileUtils.touch(f);
				InputStream in = zipFile.getInputStream(zipEntry);
				FileOutputStream out = new FileOutputStream(f);
				int c;
				byte[] by = new byte[1024];
				while ((c = in.read(by)) != -1) {
					out.write(by, 0, c);
				}
				in.close();
				out.close();
			}
		}
		zipFile.close();
	}

	/**
	 * @param sourceDirectory
	 * @param tarFile
	 * @param filter
	 *            文件名过滤器
	 * @param pathStyle
	 *            是否带目录
	 * @param include
	 *            是否创建待压缩目录
	 * @throws IOException
	 */
	public static void tarFile(String sourceDirectory, String tarFile, FilenameFilter filter, boolean pathStyle, boolean include) throws IOException {

		Collection<File> files = null;

		files = CompressUtil.allFile(sourceDirectory);

		// 只能列出所有文件，不包括目录，所以导致空目录不能压缩进去
		// 写出了列出所有文件和目录的方法，但压缩时出错,因为输入流读写不支持目录
		// 所以对子目录，主要是空目录的压缩是个问题,已经解决

		File target = getFile(tarFile);
		if (target.getParentFile() != null)
			FileUtils.forceMkdir(target.getParentFile());

		// target.compareTo(new File(sourceDirectory));//TODO
		// 目标文件不能在sourceDirectory或其子目录 之下,要解决些问题

		FileOutputStream fileOutputStream = new FileOutputStream(target);
		// 使用输出流检查
		CheckedOutputStream cs = new CheckedOutputStream(fileOutputStream, new CRC32());
		// 声明输出tar流
		TarOutputStream out = new TarOutputStream(new BufferedOutputStream(cs));
		out.setLongFileMode(TarOutputStream.LONGFILE_GNU);// TODO
		// 没有解决中文长文件名的问题
		Iterator<File> iterator = files.iterator();

		File sourceDirectory_f = getFile(sourceDirectory);
		String path = sourceDirectory_f.getPath();
		String directory = sourceDirectory_f.getName();
		byte b[] = new byte[512];

		while (iterator.hasNext()) {
			File next = iterator.next();

			String fileName = null;
			fileName = next.getPath();

			if (filter != null) {

				if (!filter.accept(next, fileName)) {
					continue;
				}
			}
			if (pathStyle) {
				// include 包含待压缩目录名
				if (include) {

					fileName = fileName.substring(fileName.indexOf(path) + path.length() - directory.length() - 1, fileName.length());
				} else {
					fileName = fileName.substring(fileName.indexOf(path) + path.length(), fileName.length());
				}
			} else
				fileName = next.getName();

			fileName = fileName.substring(fileName.indexOf(File.separator) + 1);
			logger.debug("taring " + fileName);

			if (!next.isDirectory()) {
				InputStream in = new FileInputStream(next);// 输入流不能支持目录读,碰到目录应该如何解决？
				TarEntry e = new TarEntry(fileName);
				e.setSize(next.length());
				out.putNextEntry(e);
				int len = 0;
				while ((len = in.read(b)) != -1) {
					out.write(b, 0, len);
				}
				in.close();
			} else {
				if (next.list().length == 0) {
					// 加入空目录
					TarEntry ex = new TarEntry(fileName);

					ex.setSize(0);
					ex.setMode(TarEntry.DEFAULT_DIR_MODE);
					out.putNextEntry(ex);
				}
			}
			out.closeEntry();

		}
		out.close();
	}

	public static void tarFile(String sourceDirectory, String tarFile, boolean pathStyle) throws IOException {
		CompressUtil.tarFile(sourceDirectory, tarFile, null, pathStyle, false);
	}

	public static void tarFile(String sourceDirectory, String tarFile, boolean pathStyle, boolean iscontainCurrent) throws IOException {
		CompressUtil.tarFile(sourceDirectory, tarFile, null, pathStyle, iscontainCurrent);
	}

	public static void untarFile(String tarFileName, String targetDirectory) throws IOException {
		CompressUtil.untarFile(tarFileName, targetDirectory, null);
	}

	public static void untarFile(String tarFileName, String targetDirectory, String case_convert) throws IOException {
		File tarFile;
		tarFile = new File(tarFileName);
		FileInputStream inputStream = new FileInputStream(tarFile);
		TarInputStream tarInputStream = new TarInputStream(inputStream);
		TarEntry nextEntry = tarInputStream.getNextEntry();
		File target = getFile(targetDirectory);
		if (!target.exists())
			target.mkdir();
		try {
			while (nextEntry != null) {
				File file = getFile(nextEntry.getName());
				// 1适应不同操作系统下文件分隔符对压缩包的影响
				String rtl_convert = StringUtil.convertRightToLeftSlash(file.getPath());
				// 2转换大小写
				if (!StringUtil.isEmpty(case_convert)) {
					if (case_convert.equals("upper"))
						rtl_convert = rtl_convert.toUpperCase();
					if (case_convert.equals("lower"))
						rtl_convert = rtl_convert.toLowerCase();
				}
				File destPath = getFile(targetDirectory + "/" + rtl_convert);// 抽象路径和绝对路径是有区别的,path是抽象路径，保留了原来的路径间隔符
				logger.debug("untaring entry : " + nextEntry.getName());
				logger.debug("untaring destPath : " + destPath.getAbsolutePath());
				if (!StringUtil.isEmpty(nextEntry.getName())) {
					if (nextEntry.getMode() == TarEntry.DEFAULT_DIR_MODE) {
						// 空目录条目认出是目录
						destPath.mkdir();
					} else {
						FileUtils.touch(destPath);
						FileOutputStream fout = new FileOutputStream(destPath);
						tarInputStream.copyEntryContents(fout);
						fout.close();
					}
				}
				nextEntry = tarInputStream.getNextEntry();
			}
		} finally {
			tarInputStream.close();
		}
	}

	private static Collection<File> allFile(String path) throws FileNotFoundException {
		File file;
		file = getFile(path);
		Collection<File> list = null;
		if (file.isDirectory()) {
			list = FileUtils.listFiles(file, null, true);// 空目录被忽略
		} else {
			list = new ArrayList<File>();
			list.add(file);
		}
		return list;
	}

	public static File getFile(String path) {
		return new File(path);
	}

	public static void main(String[] args) throws IOException {
		// untarFile("G:/test.tar", "G:/test", "upper");
		tarFile("G:/logs", "G:/logs1.tar", false);
	}
}
