package com.brilliance.eibs.util;

import java.io.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class IniUtil
{

  private final static Map<String, Map<String, Object>> iniFile = new HashMap<String, Map<String, Object>>();

  final public static synchronized void setValue(String section, String key, Object value)
  {
    Map<String, Object> sectionMap = iniFile.get(section);

    if (sectionMap == null)
    {
      sectionMap = new HashMap<String, Object>();
      iniFile.put(section, sectionMap);
    }
    sectionMap.put(key, value);
  }

  final public static synchronized Object getValue(String section, String key)
  {
    Object obj = null;
    Map<String, Object> item = iniFile.get(section);
    if (item != null)
    {
      obj = item.get(key);
    }
    return obj;
  }

  /**
   * 加载INI文件
   * **/
  final public static synchronized void load(String path)
      throws IOException
  {
    FileReader fr = new FileReader(path);
    BufferedReader br = new BufferedReader(fr);
    String str = null;
    String section = null;
    while ((str = br.readLine()) != null)
    {
      if (StringUtil.isEmpty(str))
        continue;
      if (str.startsWith("["))
      {
        Map<String, Object> itemMap = new HashMap<String, Object>();
        section = str.substring(1, str.length() - 1);
        System.out.println(section);
        iniFile.put(section.trim(), itemMap);
      }
      else
      {
        Map<String, Object> itemMap = iniFile.get(section);
        String key = str.substring(0, str.indexOf("="));
        String value = str.substring(str.indexOf("=") + 1);
        itemMap.put(key, value);
        System.out.println(itemMap);
      }
    }
  }

  final public static synchronized void initINIFile(List<String> list)
      throws IOException
  {
    String section = null;
    for (String str : list)
    {
      if (StringUtil.isEmpty(str))
        continue;
      if (str.startsWith("["))
      {
        Map<String, Object> itemMap = new HashMap<String, Object>();
        section = str.substring(1, str.length() - 1);
        System.out.println(section);
        iniFile.put(section.trim(), itemMap);
      }
      else
      {
        Map<String, Object> itemMap = iniFile.get(section);
        String key = str.substring(0, str.indexOf("="));
        String value = str.substring(str.indexOf("=") + 1);
        itemMap.put(key, value);
        System.out.println(itemMap);
      }
    }
  }

  final public static synchronized void write(String name)
      throws IOException
  {
    String lineSeparator = System.getProperty("line.separator", "\n");
    StringBuilder sb = new StringBuilder("");
    for (String section : iniFile.keySet())
    {
      sb.append("[").append(section).append("]").append(lineSeparator);
      Map<String, Object> map = iniFile.get(section);
      Set<String> keySet = map.keySet();
      for (String key : keySet)
      {
        sb.append(key).append("=").append(map.get(key)).append(lineSeparator);
      }

    }

    File file = new File(name);
    if (!file.exists())
    {
      file.createNewFile();

    }
    try
    {

      OutputStream os = new FileOutputStream(file);
      os.write(sb.toString().getBytes());
      os.flush();
      os.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }

  }

  public static void main(String[] args)
      throws Exception
  {
    load("f:/1.ini");
    write("F:/2.ini");
  }
  
  public static Map<String, Map<String, Object>> initINIFileX(List<String> list)
      throws IOException
  {
    Map<String, Map<String, Object>> iniFile = new HashMap<String, Map<String, Object>>();
    String section = null;
    for (String str : list)
    {
      if (StringUtil.isEmpty(str))
        continue;
      if (str.startsWith("["))
      {
        Map<String, Object> itemMap = new HashMap<String, Object>();
        section = str.substring(1, str.length() - 1);
        //System.out.println(section);
        iniFile.put(section.trim(), itemMap);
      }
      else
      {
        Map<String, Object> itemMap = iniFile.get(section);
        String key = str.substring(0, str.indexOf("="));
        String value = str.substring(str.indexOf("=") + 1);
        itemMap.put(key.trim(), value.trim());
        //  System.out.println(itemMap);
      }
    }
    return iniFile;
  }
}
