package com.brilliance.eibs.util;

import com.brilliance.eibs.core.service.Context;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.*;

/**
 * 国际化工具类
 * 
 * @author xiaoyuanzhen
 *
 */
public class LocaleUtil {

	private static Map<String, Object> map = new HashMap<String, Object>();

	/**
	 * 获取国际化资源对象
	 * 
	 * @param country
	 *            传入的国家，如zh/en
	 * @return
	 */
	public static ResourceBundle getLocaleByLanguage(String country) {
		String language = null;
		if (StringUtil.isEmpty(country)) {
			country = "en";
			language = "US";
		}
		country = country.trim();
		if (country.equals("zh")) {
			country = "zh";
			language = "CN";
		} else if (country.equals("en")) {
			country = "en";
			language = "US";
		} else {
			country = "en";
			language = "US";
		}
		Locale locale = new Locale(country, language);
		ResourceBundle resb = ResourceBundle.getBundle("cfg.locale.interface", locale);

		return resb;
	}

	/**
	 * 从上下文获取资源初始化国际化对象
	 * 
	 * @param context
	 * @param language
	 */
	public static void initLanguage(Context context, String language, Logger logger) {
		if (map.size() != 0) {
			for (String key : map.keySet()) {
				context.addVariable("global", key, map.get(key));
			}
			return;
		}
		URL url = Thread.currentThread().getContextClassLoader().getResource("");
		// logger.info("cfg/etc/interface is "+url);
		if (url == null) {
			InputStream is = null;
			try {
				Properties propertie = new Properties();
				is = new FileInputStream(new File("cfg/locale/interface.properties"));
				propertie.load(is);
				Set<Object> keys = propertie.keySet();
				for (Object key : keys) {
					context.addVariable("global", key.toString(), propertie.get(key));
				}
			} catch (Exception e) {
				logger.error("Read Locale Properties Error:", e);
			} finally {
				try {
					is.close();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			return;
		}

		ResourceBundle resb = context.getResb();
		if (resb == null) {
			resb = LocaleUtil.getLocaleByLanguage(language);
			context.setResb(resb);
		}
		Enumeration<String> enumeration = resb.getKeys();
		while (enumeration.hasMoreElements()) {
			String key = enumeration.nextElement();
			String value = resb.getString(key);
			context.addVariable("global", key, value);
		}
	}

	public static void main(String[] args) {
		LocaleUtil.getLocaleByLanguage("en");
	}

}
