package com.brilliance.eibs.util.logging.log4j;


import com.brilliance.eibs.util.Constants;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import com.brilliance.eibs.util.logging.AbstractLoggingFrame;
import com.brilliance.eibs.util.logging.LoggingFrame;
import org.apache.log4j.LogManager;
import org.apache.log4j.MDC;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.xml.DOMConfigurator;

import java.net.URL;

/**
 * log4j适配
 */
public class Log4JLoggingFrame extends AbstractLoggingFrame {

    public Log4JLoggingFrame(ClassLoader classLoader) {
        super(classLoader);
    }

    @Override
    protected String getConfigPath() {
        String logMode = LogUtil.LOG_MODE;
        if (!StringUtil.isEmpty(logMode) && "BD".equals(logMode)) {
            return "log4j.xml";
        } else if (!StringUtil.isEmpty(logMode) && "no".equals(logMode)) {
            return "log4j.properties";
        } else {
            return Constants.CFG_PATH + "/log4j.properties";
        }
    }

    @Override
    protected void loadConfiguration(String location) {
        try {
            URL url = getURL(location);
            if (url.toString().endsWith("xml")) {
                DOMConfigurator.configure(url);
            } else {
                PropertyConfigurator.configure(url);
            }
        } catch (Exception ex) {
            throw new IllegalStateException("Could not initialize Log4J logging from " + location, ex);
        }
    }

    @Override
    protected String[] getStandardConfigLocations() {
        return new String[]{"log4j.properties", "log4j.xml"};
    }


    @Override
    public void cleanUp() {
        super.cleanUp();
        markAsUninitialized();
    }

    @Override
    public void beforeInitialize() {
        if (isAlreadyInitialized()) {
            return;
        }
        super.beforeInitialize();
    }

    @Override
    public void initialize() {
        if (isAlreadyInitialized()) {
            return;
        }
        super.initialize();
        markAsInitialized();
    }

    @Override
    protected void reinitialize(String configLocation) {
        LogManager.resetConfiguration();
    }


    private boolean isAlreadyInitialized() {
        return MDC.get(LoggingFrame.class.getName()) != null;
    }

    private void markAsInitialized() {
        MDC.put(LoggingFrame.class.getName(), new Object());
    }

    private void markAsUninitialized() {
        MDC.remove(LoggingFrame.class.getName());
    }

}
