package com.brilliance.eibs.util.logging.log4j;

import com.brilliance.eibs.util.StringUtil;
import org.apache.log4j.FileAppender;
import org.apache.log4j.RollingFileAppender;
import org.apache.log4j.spi.LoggingEvent;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 移植接口平台2.0针对Log4j日志框架的扩展，日志按照接口名+交易名方式输出到各自的文件
 */
public class MyRollingFileAppender extends RollingFileAppender {
    private Map<String, FileAppender> appenderCache = new HashMap<>();
    private Map<FileAppender, String> appenderId = new HashMap<FileAppender, String>();


    @Override
    public synchronized void doAppend(LoggingEvent event) {
        String name = event.getLoggerName();
        String key = name.indexOf("/") == -1 ? name : name.substring(name.indexOf("/") + 1);
        FileAppender appender;
        appender = appenderCache.get(key);
        if (appender == null) {
            try {
                appender = createAppender(name, key);
            } catch (IOException e) {
                // 因为该类属于log4j，顾不知道如何处理异常，直接抛出
                throw new IllegalArgumentException("创建log4j的appender失败[" + e.getMessage() + "]", e);
            }
        }
        if (null != appender) {
            //如果隔日了，则重新创建该appender
            if (!name.equals(appenderId.get(appender))) {
                appender = reCreateApppender(name, key, appender);
            }
            appender.doAppend(event);
        }

    }

    private FileAppender reCreateApppender(String name, String key, FileAppender appender) {
        appenderCache.remove(key);
        appenderId.remove(appender);
        appender.close();
        try {
            return createAppender(name, key);
        } catch (IOException e) {
            // 因为该类属于log4j，顾不知道如何处理异常，直接抛出
            throw new IllegalArgumentException("创建log4j的appender失败[" + e.getMessage() + "]", e);
        }
    }

    private FileAppender createAppender(String name, String key) throws IOException {
        String directory = StringUtil.isEmpty(getFile()) ?
                System.getProperty("root.path") + "log" : getFile();
        RollingFileAppender result = new RollingFileAppender(getLayout(),
                directory + File.separator + name + ".log");
        result.setName(getName());
        result.setLayout(getLayout());
        result.setAppend(getAppend());
        result.setBufferedIO(getBufferedIO());
        result.setBufferSize(getBufferSize());
        result.setEncoding(getEncoding());
        result.setErrorHandler(getErrorHandler());
        result.setImmediateFlush(getImmediateFlush());
        result.setThreshold(getThreshold());
        result.setMaxBackupIndex(getMaxBackupIndex());
        result.setMaximumFileSize(getMaximumFileSize());
        appenderCache.put(key, result);
        appenderId.put(result, name);
        return result;
    }

}
