package com.brilliance.eibs.util.logging.log4j2;

import com.brilliance.eibs.util.logging.AbstractLoggingFrame;
import com.brilliance.eibs.util.logging.LoggingFrame;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.Logger;
import org.apache.logging.log4j.core.LoggerContext;
import org.apache.logging.log4j.core.config.ConfigurationFactory;
import org.apache.logging.log4j.core.config.ConfigurationSource;
import org.apache.logging.log4j.core.filter.AbstractFilter;
import org.apache.logging.log4j.message.Message;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

/**
 * log4j2适配
 */
public class Log4J2LoggingFrame extends AbstractLoggingFrame {

    protected static final String FILE_PROTOCOL = "file";

    private static final Filter FILTER = new AbstractFilter() {
        @Override
        public Result filter(LogEvent event) {
            return Result.DENY;
        }

        @Override
        public Result filter(Logger logger, Level level, Marker marker, Message msg, Throwable t) {
            return Result.DENY;
        }

        @Override
        public Result filter(Logger logger, Level level, Marker marker, Object msg, Throwable t) {
            return Result.DENY;
        }

        @Override
        public Result filter(Logger logger, Level level, Marker marker, String msg, Object... params) {
            return Result.DENY;
        }
    };

    public Log4J2LoggingFrame(ClassLoader classLoader) {
        super(classLoader);
    }

    @Override
    protected String getConfigPath() {
        return "cfg/log4j2.xml";
    }

    @Override
    protected void loadConfiguration(String location) {
        try {
            LoggerContext ctx = getLoggerContext();
            URL url = getURL(location);
            ConfigurationSource source = getConfigurationSource(url);
            ctx.start(ConfigurationFactory.getInstance().getConfiguration(ctx, source));
        } catch (Exception ex) {
            throw new IllegalStateException("Could not initialize Log4J2 logging from " + location, ex);
        }
    }

    @Override
    protected String[] getStandardConfigLocations() {
        return new String[]{"log4j2.properties", "log4j2.xml"};
    }


    @Override
    public void cleanUp() {
        super.cleanUp();
        LoggerContext loggerContext = getLoggerContext();
        markAsUninitialized(loggerContext);
        loggerContext.getConfiguration().removeFilter(FILTER);
    }

    @Override
    public void beforeInitialize() {
        LoggerContext loggerContext = getLoggerContext();
        if (isAlreadyInitialized(loggerContext)) {
            return;
        }
        super.beforeInitialize();
        loggerContext.getConfiguration().addFilter(FILTER);
    }

    @Override
    public void initialize() {
        LoggerContext loggerContext = getLoggerContext();
        if (isAlreadyInitialized(loggerContext)) {
            return;
        }
        loggerContext.getConfiguration().removeFilter(FILTER);
        super.initialize();
        markAsInitialized(loggerContext);
    }

    @Override
    protected void reinitialize(String configLocation) {
        getLoggerContext().reconfigure();
    }

    private ConfigurationSource getConfigurationSource(URL url) throws IOException {
        InputStream stream = url.openStream();
        if (FILE_PROTOCOL.equals(url.getProtocol())) {
            return new ConfigurationSource(stream, getFile(url));
        }
        return new ConfigurationSource(stream, url);
    }

    private LoggerContext getLoggerContext() {
        return (LoggerContext) LogManager.getContext(false);
    }

    private boolean isAlreadyInitialized(LoggerContext loggerContext) {
        return LoggingFrame.class.getName().equals(loggerContext.getExternalContext());
    }

    private void markAsInitialized(LoggerContext loggerContext) {
        loggerContext.setExternalContext(LoggingFrame.class.getName());
    }

    private void markAsUninitialized(LoggerContext loggerContext) {
        loggerContext.setExternalContext(null);
    }


}
