package com.brilliance.eibs.util.logging.logback;

import ch.qos.logback.classic.spi.LoggingEvent;
import ch.qos.logback.core.AppenderBase;
import ch.qos.logback.core.encoder.Encoder;
import com.brilliance.eibs.util.DateUtil;
import com.brilliance.eibs.util.LogUtil;
import com.brilliance.eibs.util.StringUtil;
import org.apache.commons.io.FileUtils;
import org.slf4j.MDC;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.Map;

/**
 * @Description
 * @Author s_guodong
 * @Date 2022/6/27
 */
public class LogbackAppender extends AppenderBase<LoggingEvent> {

    private String rootDir;

    private Encoder<LoggingEvent> encoder;

    public void setRootDir(String rootDir) {
        this.rootDir = rootDir;
    }

    public void setEncoder(Encoder<LoggingEvent> encoder) {
        this.encoder = encoder;
    }

    @Override
    protected void append(LoggingEvent event) {
        Map<String, String> copyOfContextMap = MDC.getCopyOfContextMap();
        String loggerName;
        if (copyOfContextMap != null && StringUtil.isNotEmpty(copyOfContextMap.get(LogUtil.LOG_FILE_NAME_SYMBOL))) {
            loggerName = copyOfContextMap.get(LogUtil.LOG_FILE_NAME_SYMBOL);
        } else {
            loggerName = "services";
            MDC.put(LogUtil.LOG_FILE_NAME_SYMBOL, loggerName);
        }
        String date = DateUtil.format(new Date(event.getTimeStamp()), "yyyyMMdd");
        if (LogUtil.SERVICE.equals(loggerName)) {
            loggerName = LogUtil.SERVICE + "_" + date;
        } else {
            loggerName = date + "/" + loggerName;
        }
        final byte[] bytes = encoder.encode(event);
        try {
            String filePath = loggerName + LogUtil.LOG_SUFFIX;
            FileUtils.writeByteArrayToFile(new File(this.rootDir, filePath), bytes, true);
        } catch (IOException e) {
            this.addError("Failed to output log to [" + loggerName + "] file, cause by " + e.getLocalizedMessage());
        }
    }
}
