package com.brilliance.eibs.util.mbeans;

import java.lang.management.*;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class JVMMonitorUtil
{

  //JVM堆内存信息
  public static Map<String, String[]> getJVMHeapMemory()
  {
    MemoryMXBean mm = (MemoryMXBean) ManagementFactory.getMemoryMXBean();

    Map<String, String[]> JVMHeapMemory = new LinkedHashMap<String, String[]>();

    long initHeapMemory = mm.getHeapMemoryUsage().getInit();
    long usedHeapMemory = mm.getHeapMemoryUsage().getUsed();
    long totalHeapMemory = mm.getHeapMemoryUsage().getCommitted();
    DecimalFormat formater = new DecimalFormat("#.##");
    String usage = formater.format((double) 100 * usedHeapMemory / totalHeapMemory);
    long freeHeapMemory = Runtime.getRuntime().freeMemory();
    long maxHeapMemory = mm.getHeapMemoryUsage().getMax();

    String pathBuf = "/JVMHeapMemory/";

    JVMHeapMemory.put(pathBuf + "InitHeapMemory", TypeFormat.sizeFormat(initHeapMemory));
    JVMHeapMemory.put(pathBuf + "UsedHeapMemory", TypeFormat.sizeFormat(usedHeapMemory));
    JVMHeapMemory.put(pathBuf + "TotalHeapMemory", TypeFormat.sizeFormat(totalHeapMemory));
    JVMHeapMemory.put(pathBuf + "HeapMemoryUsage", new String[]{usage, "%"});
    JVMHeapMemory.put(pathBuf + "FreeHeapMemory", TypeFormat.sizeFormat(freeHeapMemory));
    JVMHeapMemory.put(pathBuf + "MaxHeapMemory", TypeFormat.sizeFormat(maxHeapMemory));

    return JVMHeapMemory;
  }

  //JVM非堆内存信息
  public static Map<String, String[]> getJVMNonHeapMemory()
  {
    MemoryMXBean mm = (MemoryMXBean) ManagementFactory.getMemoryMXBean();

    Map<String, String[]> JVMNonHeapMemory = new LinkedHashMap<String, String[]>();

    long initNonHeapMemory = mm.getNonHeapMemoryUsage().getInit();
    long usedNonHeapMemory = mm.getNonHeapMemoryUsage().getUsed();
    long totalNonHeapMemory = mm.getNonHeapMemoryUsage().getCommitted();
    DecimalFormat formater = new DecimalFormat("#.##");
    String usage = formater.format((double) 100 * usedNonHeapMemory / totalNonHeapMemory);
    long maxNonHeapMemory = mm.getNonHeapMemoryUsage().getMax();

    String pathBuf = "/JVMNonHeapMemory/";

    JVMNonHeapMemory.put(pathBuf + "InitNonHeapMemory", TypeFormat.sizeFormat(initNonHeapMemory));
    JVMNonHeapMemory.put(pathBuf + "UsedNonHeapMemory", TypeFormat.sizeFormat(usedNonHeapMemory));
    JVMNonHeapMemory.put(pathBuf + "TotalNonHeapMemory", TypeFormat.sizeFormat(totalNonHeapMemory));
    JVMNonHeapMemory.put(pathBuf + "NonHeapMemoryUsage", new String[]{usage, "%"});
    JVMNonHeapMemory.put(pathBuf + "MaxNonHeapMemory", TypeFormat.sizeFormat(maxNonHeapMemory));

    return JVMNonHeapMemory;
  }

  //JVM内存池信息
  public static Map<String, String[]> getJVMMemoryPool()
  {
    List<MemoryPoolMXBean> mpmList = ManagementFactory.getMemoryPoolMXBeans();

    Map<String, String[]> JVMMemoryPool = new LinkedHashMap<String, String[]>();

    for (int i = 0; i < mpmList.size(); i++)
    {
      MemoryPoolMXBean mpm = mpmList.get(i);
      String pathBuf = "/JVMMemoryPool/" + String.valueOf(i + 1) + "/";

      JVMMemoryPool.put(pathBuf + "Name", new String[]{String.valueOf(mpm.getName()), "*"});
      JVMMemoryPool.put(pathBuf + "Type", new String[]{String.valueOf(mpm.getType()), "*"});
      JVMMemoryPool.put(pathBuf + "Init", TypeFormat.sizeFormat(mpm.getUsage().getInit()));
      JVMMemoryPool.put(pathBuf + "Total", TypeFormat.sizeFormat(mpm.getUsage().getCommitted()));
      JVMMemoryPool.put(pathBuf + "Used", TypeFormat.sizeFormat(mpm.getUsage().getUsed()));
      JVMMemoryPool.put(pathBuf + "Max", TypeFormat.sizeFormat(mpm.getUsage().getMax()));
    }

    return JVMMemoryPool;
  }

  //JVM线程信息
  public static Map<String, String[]> getJVMThread()
  {
    ThreadMXBean tm = (ThreadMXBean) ManagementFactory.getThreadMXBean();

    Map<String, String[]> JVMThread = new LinkedHashMap<String, String[]>();

    int threadCount = tm.getThreadCount();
    int peakThreadCount = tm.getPeakThreadCount();
    int daemonThreadCount = tm.getDaemonThreadCount();
    long totalStartedThreadCount = tm.getTotalStartedThreadCount();
    long currentThreadCpuTime = tm.getCurrentThreadCpuTime();
    long currentThreadUserTime = tm.getCurrentThreadUserTime();

    String pathBuf = "/JVMThread/";

    JVMThread.put(pathBuf + "ThreadCount", new String[]{String.valueOf(threadCount), "*"});
    JVMThread.put(pathBuf + "PeakThread", new String[]{String.valueOf(peakThreadCount), "*"});
    JVMThread.put(pathBuf + "DaemonThread", new String[]{String.valueOf(daemonThreadCount), "*"});
    JVMThread.put(pathBuf + "TotalStartedThread", new String[]{String.valueOf(totalStartedThreadCount), "*"});
    JVMThread.put(pathBuf + "TotalCpuTime", TypeFormat.timeFormat(currentThreadCpuTime));
    JVMThread.put(pathBuf + "TotalUserTime", TypeFormat.timeFormat(currentThreadUserTime));

    long[] allThread = tm.getAllThreadIds();

    for (long id : allThread)
    {
      String strBuf = "/JVMThread/" + String.valueOf(id) + "/";

      if (tm.getThreadInfo(id) != null)
      {
        ThreadInfo threadInfo = tm.getThreadInfo(id);

        JVMThread.put(strBuf + "ThreadId", new String[]{String.valueOf(threadInfo.getThreadId()), "*"});
        JVMThread.put(strBuf + "ThreadName", new String[]{threadInfo.getThreadName(), "*"});
        JVMThread.put(strBuf + "ThreadState", new String[]{String.valueOf(threadInfo.getThreadState()), "*"});
        JVMThread.put(strBuf + "BlockedCount", new String[]{String.valueOf(threadInfo.getBlockedCount()), "*"});
        JVMThread.put(strBuf + "BlockedTime", new String[]{String.valueOf(threadInfo.getBlockedTime()), "*"});
        JVMThread.put(strBuf + "LockOwnerId", new String[]{String.valueOf(threadInfo.getLockOwnerId()), "*"});
        JVMThread.put(strBuf + "LockOwnerName", new String[]{String.valueOf(threadInfo.getLockOwnerName()), "*"});
        JVMThread.put(strBuf + "WaitedCount", new String[]{String.valueOf(threadInfo.getWaitedCount()), "*"});
        JVMThread.put(strBuf + "WaitedTime", new String[]{String.valueOf(threadInfo.getWaitedTime()), "*"});
      }

      JVMThread.put(strBuf + "ThreadCpuTime", TypeFormat.timeFormat(tm.getThreadCpuTime(id)));
      JVMThread.put(strBuf + "ThreadUserTime", TypeFormat.timeFormat(tm.getThreadUserTime(id)));
    }

    return JVMThread;
  }

  //JVM运行信息
  public static Map<String, String[]> getJVMRuntime()
  {
    RuntimeMXBean rmb = (RuntimeMXBean) ManagementFactory.getRuntimeMXBean();

    Map<String, String[]> JVMRuntime = new LinkedHashMap<String, String[]>();

    String pathBuf = "/JVMRuntime/";

    JVMRuntime.put(pathBuf + "BootClassPath", new String[]{rmb.getBootClassPath(), "*"});
    JVMRuntime.put(pathBuf + "ClassPath", new String[]{rmb.getClassPath(), "*"});
    JVMRuntime.put(pathBuf + "LibraryPath", new String[]{rmb.getLibraryPath(), "*"});
    JVMRuntime.put(pathBuf + "ManagementSpecVer", new String[]{rmb.getManagementSpecVersion(), "*"});
    JVMRuntime.put(pathBuf + "RunningJVMName", new String[]{rmb.getName(), "*"});
    JVMRuntime.put(pathBuf + "SpecName", new String[]{rmb.getSpecName(), "*"});
    JVMRuntime.put(pathBuf + "SpecVendor", new String[]{rmb.getSpecVendor(), "*"});
    JVMRuntime.put(pathBuf + "SpecVersion", new String[]{rmb.getSpecVersion(), "*"});
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    JVMRuntime.put(pathBuf + "StartTime", new String[]{sdf.format(rmb.getStartTime()), "*"});
    JVMRuntime.put(pathBuf + "VmName", new String[]{rmb.getVmName(), "*"});
    JVMRuntime.put(pathBuf + "VmVendor", new String[]{rmb.getVmVendor(), "*"});
    JVMRuntime.put(pathBuf + "VmVersion", new String[]{rmb.getVmVersion(), "*"});
    return JVMRuntime;
  }

  public static Map<String, String[]> getJVMInformation()
  {
    Map<String, String[]> JVMInformation = new LinkedHashMap<String, String[]>();

    JVMInformation.putAll(getJVMHeapMemory());
    JVMInformation.putAll(getJVMNonHeapMemory());
    JVMInformation.putAll(getJVMThread());
    JVMInformation.putAll(getJVMMemoryPool());
    JVMInformation.putAll(getJVMRuntime());

    return JVMInformation;
  }

  public static void main(String args[])
  {
    Map<String, String[]> JVMInformation = getJVMInformation();

    for (String s : JVMInformation.keySet())
    {
      System.out.println(s + "\t" + JVMInformation.get(s)[0] + "\t" + JVMInformation.get(s)[1] + "\t");
    }
  }

}
