package com.brilliance.eibs.util.mbeans;

import org.apache.commons.modeler.ManagedBean;
import org.apache.commons.modeler.Registry;

import javax.management.*;

public class MBeanUtils {
	private static Registry registry = createRegistry();

	private static MBeanServer mserver = createServer();

	public static void registerMbeans() {
		try {
			// 注册服务管理MBean
			createMBean(ServerMBean.serverMBean);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public static Registry createRegistry() {
		if (registry == null) {
			registry = Registry.getRegistry(null, null);
			ClassLoader cl = MBeanUtils.class.getClassLoader();
			registry.loadDescriptors("com.brilliance.eibs.util.mbeans", cl);
		}
		return (registry);

	}

	public static synchronized MBeanServer createServer() {
		if (mserver == null) {
			mserver = Registry.getRegistry(null, null).getMBeanServer();
		}
		return (mserver);
	}

	static String createManagedName(Object component) {

		String className = component.getClass().getName();
		int period = className.lastIndexOf('.');
		if (period >= 0)
			className = className.substring(period + 1);
		return (className);

	}

	private static ObjectName createObjectName(String domain, Object obj) throws MalformedObjectNameException {
		String type = createManagedName(obj);
		return new ObjectName(domain + ":type=" + type);
	}

	public static DynamicMBean createMBean(ServerMBean server) throws Exception {
		String mname = createManagedName(server);
		ManagedBean managed = registry.findManagedBean(mname);
		if (managed == null) {
			Exception e = new Exception("ManagedBean is not found with " + mname);
			throw new MBeanException(e);
		}
		String domain = managed.getDomain();
		if (domain == null)
			domain = mserver.getDefaultDomain();
		DynamicMBean mbean = managed.createMBean(server);
		ObjectName oname = createObjectName(domain, server);
		if (mserver.isRegistered(oname)) {
			mserver.unregisterMBean(oname);
		}
		mserver.registerMBean(mbean, oname);
		return (mbean);
	}

	public static void destroyMBean(ServerMBean server) throws Exception {
		String mname = createManagedName(server);
		ManagedBean managed = registry.findManagedBean(mname);
		if (managed == null) {
			return;
		}
		String domain = managed.getDomain();
		if (domain == null)
			domain = mserver.getDefaultDomain();
		ObjectName oname = createObjectName(domain, server);
		if (mserver.isRegistered(oname))
			mserver.unregisterMBean(oname);
	}
}
