/*
 * Create on 2012-06-22
 */
package com.brilliance.ext.util;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.IOUtils;

import com.brilliance.swift.MTUnPackMessage;

/**
 * 电报码转换
 * 
 * @author Jason.Hoo (latest modification by $Author: hujianxin78728 $)
 * @version $Revision: 813 $ $Date: 2012-12-11 23:54:33 +0800 (Tue, 11 Dec 2012)
 *          $
 */
public class TelexCodeUtils {
	static Map<String, String> m = new HashMap<String, String>();
	static Pattern pattern = Pattern.compile("(\\d{4} )");
	static String LineSeparator = MTUnPackMessage.getNewLine();

	static {
		init0();
		init1();
		init2();
		init3();
	}

	static void put(String value, String key) {
		m.put(key, value);
		m.put(value, key);
	}

	static void init0() {
		put("坰", "0990");
		put("坳", "0991");
		put("型", "0992");
		put("垓", "0993");
		put("坬", "0994");
		put("垠", "0995");
		put("垢", "0996");
		put("垣", "0997");
		put("垃", "0998");
		put("坯", "0999");
		put("垛", "1000");
		put("坨", "1001");
		put("埃", "1002");
		put("埋", "1003");
		put("城", "1004");
		put("埒", "1005");
		put("垮", "1006");
		put("埏", "1007");
		put("域", "1008");
		put("埠", "1009");
		put("垧", "1010");
		put("埭", "1011");
		put("垡", "1012");
		put("执", "1013");
		put("培", "1014");
		put("基", "1015");
		put("堂", "1016");
		put("坚", "1017");
		put("堆", "1018");
		put("垩", "1019");
		put("埤", "1020");
		put("埴", "1021");
		put("埵", "1022");
		put("埸", "1023");
		put("堃", "1024");
		put("堙", "1025");
		put("堞", "1026");
		put("堡", "1027");
		put("堠", "1028");
		put("堤", "1029");
		put("堪", "1030");
		put("尧", "1031");
		put("报", "1032");
		put("埔", "1033");
		put("场", "1034");
		put("堵", "1035");
		put("埝", "1036");
		put("堰", "1037");
		put("垾", "1038");
		put("塍", "1039");
		put("块", "1040");
		put("茔", "1041");
		put("塌", "1042");
		put("塑", "1043");
		put("塔", "1044");
		put("墓", "1045");
		put("塚", "1046");
		put("塘", "1048");
		put("塞", "1049");
		put("填", "1050");
		put("坞", "1051");
		put("垲", "1052");
		put("埙", "1053");
		put("埘", "1054");
		put("堼", "1055");
		put("坝", "1056");
		put("尘", "1057");
		put("堑", "1058");
		put("塾", "1060");
		put("塂", "1061");
		put("墀", "1062");
		put("墁", "1063");
		put("境", "1064");
		put("墅", "1065");
		put("嘱", "0928");
		put("叻", "0930");
		put("囗", "0931");
		put("回", "0932");
		put("囚", "0933");
		put("四", "0934");
		put("囱", "0935");
		put("因", "0936");
		put("囤", "0937");
		put("困", "0938");
		put("囫", "0939");
		put("囷", "0940");
		put("囹", "0941");
		put("固", "0942");
		put("囿", "0943");
		put("圃", "0944");
		put("圄", "0945");
		put("圈", "0946");
		put("圉", "0947");
		put("国", "0948");
		put("囵", "0950");
		put("圐", "0951");
		put("圙", "0952");
		put("围", "0953");
		put("园", "0954");
		put("圆", "0955");
		put("图", "0956");
		put("团", "0957");
		put("圜", "0958");
		put("土", "0960");
		put("在", "0961");
		put("圩", "0962");
		put("圳", "1101");
		put("士", "1102");
		put("壬", "1103");
		put("壮", "1104");
		put("壹", "1105");
		put("壶", "1106");
		put("寿", "1108");
		put("壸", "1109");
		put("椴", "1110");
		put("夂", "1111");
		put("夆", "1112");
		put("簖", "1113");
		put("夊", "1114");
		put("夏", "1115");
		put("砘", "1116");
		put("跺", "1118");
		put("夕", "1119");
		put("外", "1120");
		put("夙", "1121");
		put("多", "1122");
		put("夜", "1123");
		put("够", "1124");
		put("梦", "1125");
		put("夤", "1126");
		put("夥", "1127");
		put("婀", "1128");
		put("大", "1129");
		put("夭", "1130");
		put("天", "1131");
		put("太", "1132");
		put("夫", "1133");
		put("夬", "1134");
		put("央", "1135");
		put("失", "1136");
		put("夯", "1137");
		put("夷", "1138");
		put("夸", "1139");
		put("夹", "1140");
		put("奄", "1141");
		put("奇", "1142");
		put("奈", "1143");
		put("奉", "1144");
		put("奎", "1145");
		put("奏", "1146");
		put("奂", "1147");
		put("契", "1148");
		put("奔", "1149");
		put("奕", "1150");
		put("奓", "1151");
		put("套", "1152");
		put("奚", "1153");
		put("锇", "1154");
		put("奘", "1155");
		put("奠", "1156");
		put("奢", "1158");
		put("奥", "1159");
		put("奁", "1160");
		put("夺", "1161");
		put("奖", "1162");
		put("奭", "1163");
		put("奋", "1164");
		put("奀", "1165");
		put("女", "1166");
		put("奴", "1167");
		put("奶", "1168");
		put("奸", "1169");
		put("好", "1170");
		put("妁", "1171");
		put("如", "1172");
		put("妃", "1173");
		put("妄", "1174");
		put("妊", "1175");
		put("妒", "1176");
		put("妍", "1177");
		put("妓", "1178");
		put("妖", "1179");
		put("妗", "1180");
		put("妙", "1181");
		put("妆", "1182");
		put("妣", "1183");
		put("妤", "1184");
		put("妥", "1185");
		put("妨", "1186");
		put("妯", "1187");
		put("妹", "1188");
		put("妻", "1189");
		put("妾", "1190");
		put("姆", "1191");
		put("姊", "1192");
		put("始", "1193");
		put("姗", "1194");
		put("姐", "1195");
		put("姑", "1196");
		put("姒", "1197");
		put("姓", "1198");
		put("妲", "1199");
		put("妮", "1200");
		put("委", "1201");
		put("姚", "1202");
		put("姜", "1203");
		put("墉", "1066");
		put("垫", "1067");
		put("墄", "1068");
		put("埫", "1069");
		put("坠", "1071");
		put("增", "1073");
		put("墟", "1074");
		put("墨", "1075");
		put("墩", "1076");
		put("堕", "1077");
		put("塄", "1080");
		put("塆", "1081");
		put("墈", "1082");
		put("垦", "1083");
		put("壁", "1084");
		put("壅", "1085");
		put("坛", "1086");
		put("壑", "1089");
		put("压", "1090");
		put("壕", "1091");
		put("垒", "1093");
		put("圹", "1094");
		put("垄", "1096");
		put("垆", "1097");
		put("壤", "1099");
		put("婊", "1239");
		put("婕", "1240");
		put("婚", "1241");
		put("婢", "1242");
		put("妇", "1244");
		put("婪", "1245");
		put("娅", "1246");
		put("她", "1247");
		put("妳", "1248");
		put("婼", "1249");
		put("婷", "1250");
		put("婺", "1251");
		put("媒", "1252");
		put("媚", "1253");
		put("媛", "1254");
		put("娲", "1257");
		put("媖", "1260");
		put("媳", "1261");
		put("媵", "1262");
		put("媸", "1263");
		put("媪", "1264");
		put("妈", "1265");
		put("媾", "1266");
		put("嫁", "1268");
		put("嫂", "1269");
		put("嫄", "1270");
		put("嫉", "1271");
		put("嫌", "1273");
		put("媲", "1274");
		put("嫖", "1276");
		put("妪", "1277");
		put("嫠", "1278");
		put("嫡", "1279");
		put("嫣", "1280");
		put("嫦", "1281");
		put("嫩", "1282");
		put("嫪", "1283");
		put("嫘", "1284");
		put("嫚", "1285");
		put("嫜", "1286");
		put("嫫", "1287");
		put("妫", "1289");
		put("娆", "1290");
		put("嬉", "1291");
		put("婵", "1292");
		put("娇", "1293");
		put("媭", "1294");
		put("嬖", "1295");
		put("嬗", "1296");
		put("嫱", "1297");
		put("嬛", "1298");
		put("嬴", "1299");
		put("嫔", "1300");
		put("嬷", "1301");
		put("嬲", "1303");
		put("婴", "1305");
		put("婶", "1306");
		put("孀", "1307");
		put("娈", "1309");
		put("妞", "1310");
		put("子", "1311");
		put("孑", "1312");
		put("孔", "1313");
		put("孕", "1314");
		put("孓", "1315");
		put("字", "1316");
		put("存", "1317");
		put("孚", "1318");
		put("孛", "1319");
		put("孜", "1320");
		put("孝", "1321");
		put("孟", "1322");
		put("季", "1323");
		put("孤", "1324");
		put("孥", "1325");
		put("孩", "1326");
		put("孙", "1327");
		put("孰", "1328");
		put("孱", "1329");
		put("孳", "1330");
		put("学", "1331");
		put("孺", "1332");
		put("孪", "1334");
		put("孵", "1335");
		put("宀", "1336");
		put("宁", "1337");
		put("它", "1338");
		put("宄", "1339");
		put("宅", "1341");
		put("姝", "1204");
		put("姣", "1207");
		put("姥", "1209");
		put("姨", "1210");
		put("侄", "1212");
		put("姬", "1213");
		put("姻", "1215");
		put("娃", "1216");
		put("姿", "1217");
		put("威", "1218");
		put("娉", "1219");
		put("姮", "1220");
		put("娌", "1222");
		put("娑", "1223");
		put("娘", "1224");
		put("娱", "1225");
		put("娜", "1226");
		put("娟", "1227");
		put("娠", "1228");
		put("娣", "1229");
		put("娥", "1230");
		put("娩", "1231");
		put("娓", "1232");
		put("娼", "1233");
		put("姹", "1234");
		put("娶", "1235");
		put("娄", "1236");
		put("婆", "1237");
		put("婉", "1238");
		put("寅", "1377");
		put("密", "1378");
		put("寇", "1379");
		put("富", "1381");
		put("寐", "1382");
		put("寒", "1383");
		put("寓", "1384");
		put("寞", "1389");
		put("察", "1390");
		put("寡", "1391");
		put("寝", "1392");
		put("寤", "1393");
		put("寥", "1394");
		put("实", "1395");
		put("寨", "1396");
		put("审", "1399");
		put("写", "1400");
		put("宽", "1401");
		put("寮", "1402");
		put("寰", "1403");
		put("宠", "1404");
		put("宝", "1405");
		put("宧", "1406");
		put("寸", "1407");
		put("寺", "1408");
		put("封", "1409");
		put("射", "1410");
		put("尅", "1411");
		put("将", "1412");
		put("专", "1413");
		put("尉", "1414");
		put("尊", "1415");
		put("寻", "1416");
		put("对", "1417");
		put("导", "1418");
		put("蒽", "1419");
		put("小", "1420");
		put("少", "1421");
		put("尔", "1422");
		put("尖", "1423");
		put("尚", "1424");
		put("尕", "1427");
		put("尢", "1428");
		put("尤", "1429");
		put("尪", "1431");
		put("就", "1432");
		put("尰", "1433");
		put("尴", "1434");
		put("尬", "1435");
		put("铒", "1436");
		put("尸", "1437");
		put("尹", "1438");
		put("尺", "1439");
		put("尻", "1440");
		put("尼", "1441");
		put("尾", "1442");
		put("尿", "1443");
		put("局", "1444");
		put("屁", "1445");
		put("居", "1446");
		put("届", "1447");
		put("屈", "1448");
		put("屉", "1449");
		put("屋", "1450");
		put("屎", "1452");
		put("屐", "1453");
		put("屑", "1454");
		put("展", "1455");
		put("屏", "1456");
		put("屝", "1457");
		put("屠", "1458");
		put("屡", "1459");
		put("屣", "1460");
		put("层", "1461");
		put("履", "1462");
		put("屦", "1464");
		put("属", "1466");
		put("砩", "1468");
		put("", "1469");
		put("屯", "1470");
		put("岳", "1471");
		put("山", "1472");
		put("屹", "1473");
		put("屺", "1474");
		put("峁", "1475");
		put("岌", "1476");
		put("岐", "1477");
		put("岑", "1478");
		put("岔", "1479");
		put("宇", "1342");
		put("守", "1343");
		put("安", "1344");
		put("宋", "1345");
		put("完", "1346");
		put("宏", "1347");
		put("宓", "1348");
		put("宕", "1349");
		put("宗", "1350");
		put("官", "1351");
		put("宙", "1352");
		put("定", "1353");
		put("宛", "1354");
		put("宜", "1355");
		put("客", "1356");
		put("宣", "1357");
		put("室", "1358");
		put("宥", "1359");
		put("宦", "1360");
		put("宬", "1361");
		put("宫", "1362");
		put("宰", "1363");
		put("害", "1364");
		put("宴", "1365");
		put("宵", "1366");
		put("家", "1367");
		put("宸", "1368");
		put("容", "1369");
		put("宿", "1372");
		put("寂", "1374");
		put("寄", "1376");
		put("崚", "1515");
		put("崧", "1516");
		put("崦", "1517");
		put("嵇", "1518");
		put("崆", "1519");
		put("嵋", "1520");
		put("嵫", "1521");
		put("嵊", "1522");
		put("嵌", "1523");
		put("峧", "1524");
		put("嵎", "1525");
		put("岚", "1526");
		put("崀", "1527");
		put("", "1528");
		put("嵩", "1529");
		put("嵝", "1530");
		put("嵬", "1531");
		put("嵯", "1532");
		put("崌", "1533");
		put("嶂", "1534");
		put("崭", "1535");
		put("岖", "1536");
		put("嶒", "1537");
		put("嵚", "1538");
		put("嵘", "1539");
		put("峤", "1540");
		put("崤", "1541");
		put("峄", "1542");
		put("嶙", "1543");
		put("嶷", "1544");
		put("岭", "1545");
		put("屿", "1546");
		put("崾", "1547");
		put("巉", "1548");
		put("嵛", "1549");
		put("巍", "1550");
		put("峦", "1551");
		put("嶅", "1552");
		put("巅", "1553");
		put("巖", "1554");
		put("崂", "1555");
		put("巛", "1556");
		put("川", "1557");
		put("州", "1558");
		put("巡", "1559");
		put("巢", "1560");
		put("镄", "1561");
		put("工", "1562");
		put("左", "1563");
		put("巧", "1564");
		put("巨", "1565");
		put("巫", "1566");
		put("差", "1567");
		put("酚", "1568");
		put("己", "1569");
		put("巳", "1570");
		put("已", "1571");
		put("巴", "1572");
		put("巷", "1574");
		put("巽", "1575");
		put("钆", "1576");
		put("巾", "1577");
		put("币", "1578");
		put("市", "1579");
		put("布", "1580");
		put("帆", "1581");
		put("帋", "1582");
		put("帑", "1583");
		put("帕", "1584");
		put("希", "1585");
		put("帖", "1586");
		put("帗", "1587");
		put("帘", "1588");
		put("帙", "1589");
		put("帚", "1590");
		put("帛", "1591");
		put("帔", "1592");
		put("帝", "1593");
		put("帡", "1594");
		put("帅", "1596");
		put("师", "1597");
		put("席", "1598");
		put("帨", "1599");
		put("帐", "1600");
		put("带", "1601");
		put("帷", "1602");
		put("常", "1603");
		put("帽", "1604");
		put("帏", "1605");
		put("幄", "1606");
		put("幅", "1607");
		put("帧", "1608");
		put("幌", "1610");
		put("幔", "1611");
		put("幕", "1612");
		put("帼", "1613");
		put("帻", "1614");
		put("帜", "1615");
		put("幡", "1616");
		put("幢", "1617");
		put("峿", "1480");
		put("冈", "1481");
		put("岢", "1482");
		put("岈", "1483");
		put("岩", "1484");
		put("岫", "1485");
		put("岱", "1486");
		put("岵", "1487");
		put("一", "0001");
		put("丁", "0002");
		put("七", "0003");
		put("丈", "0004");
		put("三", "0005");
		put("上", "0006");
		put("下", "0007");
		put("不", "0008");
		put("丐", "0009");
		put("丑", "0010");
		put("且", "0011");
		put("丕", "0012");
		put("世", "0013");
		put("丙", "0014");
		put("丞", "0015");
		put("丢", "0016");
		put("並", "0017");
		put("丏", "0018");
		put("丨", "0019");
		put("个", "0020");
		put("丫", "0021");
		put("中", "0022");
		put("丰", "0023");
		put("串", "0025");
		put("锕", "0026");
		put("丶", "0027");
		put("锿", "0028");
		put("丸", "0029");
		put("丹", "0030");
		put("主", "0031");
		put("砹", "0032");
		put("丿", "0033");
		put("乂", "0034");
		put("乃", "0035");
		put("久", "0036");
		put("之", "0037");
		put("乍", "0038");
		put("乎", "0039");
		put("乏", "0040");
		put("乖", "0041");
		put("乘", "0042");
		put("桉", "0043");
		put("乙", "0044");
		put("乜", "0045");
		put("九", "0046");
		put("乞", "0047");
		put("也", "0048");
		put("乳", "0050");
		put("乾", "0051");
		put("乱", "0052");
		put("凼", "0053");
		put("亅", "0054");
		put("了", "0055");
		put("予", "0056");
		put("事", "0057");
		put("苯", "0058");
		put("二", "0059");
		put("于", "0060");
		put("云", "0061");
		put("互", "0062");
		put("五", "0063");
		put("井", "0064");
		put("些", "0067");
		put("亚", "0068");
		put("亟", "0069");
		put("吡", "0070");
		put("亠", "0071");
		put("亡", "0072");
		put("亢", "0073");
		put("交", "0074");
		put("亥", "0075");
		put("亦", "0076");
		put("亨", "0077");
		put("享", "0078");
		put("京", "0079");
		put("亭", "0080");
		put("亮", "0081");
		put("亳", "0082");
		put("亶", "0083");
		put("醭", "0085");
		put("人", "0086");
		put("什", "0087");
		put("仁", "0088");
		put("仃", "0089");
		put("仄", "0090");
		put("仆", "0091");
		put("仇", "0092");
		put("今", "0093");
		put("介", "0094");
		put("仍", "0095");
		put("佣", "0096");
		put("佬", "0097");
		put("仔", "0098");
		put("仕", "0099");
		put("伸", "0135");
		put("伺", "0136");
		put("伻", "0137");
		put("似", "0138");
		put("倮", "0139");
		put("佃", "0140");
		put("但", "0141");
		put("位", "0143");
		put("低", "0144");
		put("住", "0145");
		put("佐", "0146");
		put("佑", "0147");
		put("何", "0149");
		put("佗", "0150");
		put("余", "0151");
		put("佘", "0152");
		put("佚", "0153");
		put("佛", "0154");
		put("作", "0155");
		put("佞", "0156");
		put("佟", "0157");
		put("佺", "0158");
		put("佩", "0160");
		put("伢", "0161");
		put("佯", "0162");
		put("佳", "0163");
		put("佶", "0165");
		put("佝", "0166");
		put("佻", "0167");
		put("佾", "0168");
		put("使", "0169");
		put("侃", "0170");
		put("来", "0171");
		put("侈", "0172");
		put("例", "0173");
		put("侍", "0174");
		put("侏", "0175");
		put("侑", "0176");
		put("侔", "0177");
		put("仑", "0178");
		put("侗", "0179");
		put("供", "0180");
		put("依", "0181");
		put("伕", "0183");
		put("佰", "0184");
		put("侮", "0185");
		put("侯", "0186");
		put("侵", "0187");
		put("侣", "0188");
		put("便", "0189");
		put("促", "0191");
		put("俄", "0192");
		put("俊", "0193");
		put("俎", "0194");
		put("俏", "0195");
		put("俐", "0196");
		put("俑", "0197");
		put("俗", "0198");
		put("俘", "0199");
		put("俚", "0200");
		put("佡", "0201");
		put("保", "0202");
		put("俟", "0203");
		put("侠", "0204");
		put("俞", "0205");
		put("俫", "0206");
		put("信", "0207");
		put("修", "0208");
		put("俶", "0209");
		put("伥", "0210");
		put("働", "0211");
		put("倌", "0212");
		put("倬", "0213");
		put("俯", "0214");
		put("俱", "0215");
		put("俳", "0216");
		put("俵", "0217");
		put("俸", "0218");
		put("俺", "0219");
		put("俾", "0220");
		put("仓", "0221");
		put("倍", "0223");
		put("俩", "0224");
		put("倏", "0225");
		put("们", "0226");
		put("倒", "0227");
		put("倘", "0229");
		put("候", "0230");
		put("倚", "0231");
		put("倜", "0232");
		put("借", "0234");
		put("倡", "0235");
		put("值", "0237");
		put("他", "0100");
		put("仗", "0101");
		put("付", "0102");
		put("仙", "0103");
		put("仝", "0104");
		put("仞", "0105");
		put("仡", "0106");
		put("仟", "0107");
		put("代", "0108");
		put("令", "0109");
		put("以", "0110");
		put("仰", "0111");
		put("仲", "0112");
		put("仳", "0113");
		put("仵", "0114");
		put("件", "0115");
		put("价", "0116");
		put("任", "0117");
		put("份", "0118");
		put("仿", "0119");
		put("企", "0120");
		put("伉", "0121");
		put("伊", "0122");
		put("伋", "0123");
		put("伍", "0124");
		put("伎", "0125");
		put("伏", "0126");
		put("伐", "0127");
		put("休", "0128");
		put("伙", "0129");
		put("伯", "0130");
		put("估", "0131");
		put("你", "0132");
		put("伴", "0133");
		put("伶", "0134");
		put("催", "0275");
		put("傲", "0277");
		put("传", "0278");
		put("伛", "0279");
		put("债", "0280");
		put("伤", "0281");
		put("倾", "0282");
		put("偻", "0283");
		put("仅", "0284");
		put("佥", "0286");
		put("僳", "0287");
		put("像", "0288");
		put("僦", "0291");
		put("偾", "0292");
		put("侨", "0294");
		put("僖", "0296");
		put("僚", "0297");
		put("伪", "0298");
		put("侥", "0299");
		put("僧", "0300");
		put("僭", "0301");
		put("僮", "0302");
		put("僵", "0304");
		put("僻", "0306");
		put("仪", "0308");
		put("侬", "0309");
		put("亿", "0310");
		put("儆", "0311");
		put("侩", "0312");
		put("俭", "0313");
		put("傤", "0314");
		put("僰", "0316");
		put("儋", "0317");
		put("儇", "0318");
		put("傧", "0319");
		put("儒", "0320");
		put("俦", "0321");
		put("侪", "0322");
		put("偿", "0326");
		put("优", "0327");
		put("储", "0328");
		put("俪", "0330");
		put("傥", "0331");
		put("俨", "0332");
		put("儿", "0334");
		put("兀", "0335");
		put("允", "0336");
		put("元", "0337");
		put("兄", "0338");
		put("充", "0339");
		put("兆", "0340");
		put("先", "0341");
		put("光", "0342");
		put("克", "0344");
		put("兑", "0345");
		put("免", "0346");
		put("兔", "0347");
		put("兕", "0349");
		put("兖", "0350");
		put("兜", "0351");
		put("兢", "0352");
		put("入", "0354");
		put("内", "0355");
		put("全", "0356");
		put("两", "0357");
		put("钚", "0359");
		put("八", "0360");
		put("公", "0361");
		put("六", "0362");
		put("兮", "0363");
		put("共", "0364");
		put("兵", "0365");
		put("其", "0366");
		put("具", "0367");
		put("典", "0368");
		put("兼", "0369");
		put("冀", "0370");
		put("糍", "0371");
		put("冂", "0372");
		put("冉", "0373");
		put("册", "0374");
		put("再", "0375");
		put("倥", "0238");
		put("倦", "0239");
		put("倨", "0240");
		put("倩", "0241");
		put("倪", "0242");
		put("伦", "0243");
		put("倭", "0244");
		put("偈", "0245");
		put("偎", "0246");
		put("傻", "0247");
		put("偃", "0249");
		put("假", "0250");
		put("伟", "0251");
		put("偏", "0252");
		put("偕", "0253");
		put("做", "0254");
		put("停", "0255");
		put("健", "0256");
		put("侧", "0258");
		put("侦", "0259");
		put("偶", "0260");
		put("仫", "0261");
		put("偷", "0262");
		put("伧", "0263");
		put("傀", "0264");
		put("傅", "0265");
		put("傍", "0266");
		put("傑", "0267");
		put("傒", "0268");
		put("傈", "0269");
		put("伞", "0270");
		put("备", "0271");
		put("凛", "0412");
		put("凝", "0413");
		put("决", "0414");
		put("几", "0415");
		put("凡", "0416");
		put("凭", "0417");
		put("凯", "0418");
		put("凳", "0419");
		put("凰", "0420");
		put("茌", "0421");
		put("凵", "0422");
		put("凶", "0423");
		put("凸", "0424");
		put("凹", "0425");
		put("出", "0427");
		put("函", "0428");
		put("菪", "0429");
		put("刀", "0430");
		put("刁", "0431");
		put("刃", "0432");
		put("分", "0433");
		put("切", "0434");
		put("刈", "0435");
		put("刊", "0436");
		put("刎", "0437");
		put("刑", "0438");
		put("划", "0439");
		put("刖", "0440");
		put("列", "0441");
		put("刨", "0442");
		put("初", "0443");
		put("删", "0444");
		put("判", "0445");
		put("别", "0446");
		put("利", "0448");
		put("剁", "0449");
		put("刮", "0450");
		put("到", "0451");
		put("刲", "0452");
		put("刳", "0453");
		put("制", "0455");
		put("刷", "0456");
		put("券", "0457");
		put("刹", "0458");
		put("刺", "0459");
		put("剃", "0461");
		put("刭", "0462");
		put("则", "0463");
		put("削", "0465");
		put("刻", "0466");
		put("前", "0467");
		put("剜", "0468");
		put("剌", "0469");
		put("剡", "0470");
		put("剔", "0471");
		put("剖", "0472");
		put("刬", "0473");
		put("刚", "0474");
		put("剥", "0475");
		put("剩", "0476");
		put("剪", "0477");
		put("剐", "0478");
		put("副", "0479");
		put("割", "0480");
		put("剀", "0481");
		put("创", "0482");
		put("剽", "0484");
		put("剿", "0485");
		put("剧", "0489");
		put("劈", "0490");
		put("刘", "0491");
		put("刽", "0492");
		put("剑", "0494");
		put("剂", "0495");
		put("劓", "0496");
		put("劘", "0497");
		put("剅", "0499");
		put("力", "0500");
		put("功", "0501");
		put("加", "0502");
		put("劣", "0503");
		put("助", "0504");
		put("努", "0505");
		put("劫", "0506");
		put("劬", "0507");
		put("劭", "0508");
		put("効", "0509");
		put("劻", "0510");
		put("劾", "0511");
		put("劼", "0512");
		put("劲", "0513");
		put("冏", "0376");
		put("冇", "0377");
		put("冒", "0379");
		put("冕", "0380");
		put("镩", "0381");
		put("冖", "0382");
		put("冗", "0383");
		put("冠", "0385");
		put("冢", "0386");
		put("冤", "0387");
		put("冥", "0388");
		put("幂", "0389");
		put("耖", "0390");
		put("冫", "0391");
		put("冬", "0392");
		put("冰", "0393");
		put("冲", "0394");
		put("冱", "0395");
		put("冶", "0396");
		put("冷", "0397");
		put("冮", "0398");
		put("冽", "0399");
		put("况", "0400");
		put("凄", "0401");
		put("准", "0402");
		put("净", "0403");
		put("凉", "0404");
		put("冼", "0405");
		put("凋", "0406");
		put("凌", "0407");
		put("冻", "0408");
		put("凑", "0410");
		put("匏", "0549");
		put("匐", "0550");
		put("勾", "0551");
		put("匕", "0552");
		put("化", "0553");
		put("北", "0554");
		put("匙", "0555");
		put("碲", "0556");
		put("匚", "0557");
		put("匜", "0558");
		put("匝", "0559");
		put("匠", "0561");
		put("匡", "0562");
		put("匣", "0563");
		put("匪", "0564");
		put("汇", "0565");
		put("匮", "0566");
		put("匦", "0567");
		put("啶", "0570");
		put("匸", "0571");
		put("匹", "0572");
		put("匾", "0573");
		put("匿", "0574");
		put("区", "0575");
		put("铥", "0576");
		put("十", "0577");
		put("千", "0578");
		put("廿", "0579");
		put("卅", "0580");
		put("升", "0581");
		put("午", "0582");
		put("卉", "0583");
		put("半", "0584");
		put("卑", "0585");
		put("卒", "0586");
		put("卓", "0587");
		put("协", "0588");
		put("南", "0589");
		put("博", "0590");
		put("卜", "0592");
		put("卞", "0593");
		put("占", "0594");
		put("卡", "0595");
		put("卣", "0596");
		put("卦", "0597");
		put("氡", "0598");
		put("卩", "0599");
		put("卯", "0602");
		put("印", "0603");
		put("危", "0604");
		put("却", "0606");
		put("卵", "0607");
		put("卷", "0608");
		put("卸", "0609");
		put("卺", "0610");
		put("卮", "0612");
		put("即", "0613");
		put("卿", "0615");
		put("胨", "0616");
		put("厂", "0617");
		put("厄", "0618");
		put("厘", "0622");
		put("厍", "0623");
		put("厚", "0624");
		put("厝", "0625");
		put("原", "0626");
		put("厕", "0627");
		put("厥", "0628");
		put("厌", "0630");
		put("厮", "0631");
		put("厉", "0632");
		put("厦", "0633");
		put("胴", "0634");
		put("厶", "0635");
		put("去", "0637");
		put("叁", "0638");
		put("参", "0639");
		put("塅", "0641");
		put("又", "0642");
		put("叉", "0643");
		put("及", "0644");
		put("友", "0645");
		put("反", "0646");
		put("叔", "0647");
		put("取", "0648");
		put("受", "0649");
		put("叙", "0650");
		put("叛", "0651");
		put("勃", "0514");
		put("勐", "0515");
		put("勇", "0516");
		put("勉", "0517");
		put("勍", "0518");
		put("勒", "0519");
		put("动", "0520");
		put("勖", "0521");
		put("勘", "0522");
		put("务", "0523");
		put("胜", "0524");
		put("劳", "0525");
		put("募", "0527");
		put("势", "0528");
		put("勤", "0530");
		put("勚", "0532");
		put("勰", "0533");
		put("勳", "0534");
		put("劢", "0535");
		put("励", "0536");
		put("勷", "0537");
		put("劝", "0538");
		put("锝", "0539");
		put("勹", "0540");
		put("勺", "0541");
		put("匀", "0542");
		put("勿", "0543");
		put("包", "0545");
		put("匈", "0546");
		put("匊", "0547");
		put("匍", "0548");
		put("吓", "0687");
		put("君", "0689");
		put("吝", "0690");
		put("吞", "0691");
		put("吟", "0692");
		put("吠", "0693");
		put("否", "0694");
		put("吩", "0695");
		put("咐", "0696");
		put("吆", "0697");
		put("含", "0698");
		put("吭", "0699");
		put("吮", "0700");
		put("呈", "0701");
		put("吴", "0702");
		put("吵", "0703");
		put("呐", "0704");
		put("吸", "0705");
		put("吹", "0706");
		put("告", "0707");
		put("吻", "0708");
		put("吼", "0709");
		put("吾", "0710");
		put("呀", "0711");
		put("吕", "0712");
		put("呃", "0713");
		put("呆", "0714");
		put("咂", "0715");
		put("呢", "0716");
		put("呦", "0717");
		put("呋", "0718");
		put("周", "0719");
		put("咒", "0720");
		put("吧", "0721");
		put("呱", "0722");
		put("味", "0724");
		put("呵", "0725");
		put("呶", "0726");
		put("呷", "0727");
		put("呻", "0728");
		put("呼", "0729");
		put("命", "0730");
		put("咀", "0731");
		put("咄", "0732");
		put("咆", "0733");
		put("咕", "0734");
		put("和", "0735");
		put("咎", "0736");
		put("咏", "0737");
		put("咋", "0738");
		put("咖", "0739");
		put("哎", "0740");
		put("咤", "0741");
		put("响", "0742");
		put("咪", "0743");
		put("咧", "0744");
		put("咨", "0745");
		put("咫", "0746");
		put("咬", "0747");
		put("咯", "0748");
		put("咱", "0749");
		put("咳", "0750");
		put("咸", "0752");
		put("咽", "0754");
		put("哀", "0755");
		put("品", "0756");
		put("哂", "0757");
		put("哄", "0758");
		put("啊", "0759");
		put("哇", "0760");
		put("哈", "0761");
		put("哉", "0762");
		put("哪", "0763");
		put("咿", "0764");
		put("员", "0765");
		put("哥", "0766");
		put("唔", "0767");
		put("哦", "0768");
		put("哩", "0769");
		put("哭", "0770");
		put("哮", "0771");
		put("哲", "0772");
		put("哺", "0773");
		put("哼", "0774");
		put("哽", "0775");
		put("哿", "0776");
		put("唁", "0777");
		put("唆", "0778");
		put("唉", "0780");
		put("唐", "0781");
		put("唇", "0782");
		put("哨", "0783");
		put("啦", "0784");
		put("唏", "0785");
		put("售", "0786");
		put("唯", "0787");
		put("唱", "0788");
		put("唳", "0789");
		put("叟", "0652");
		put("丛", "0654");
		put("叠", "0655");
		put("口", "0656");
		put("古", "0657");
		put("句", "0658");
		put("另", "0659");
		put("叨", "0660");
		put("叩", "0661");
		put("只", "0662");
		put("叫", "0663");
		put("召", "0664");
		put("叭", "0665");
		put("叮", "0666");
		put("叱", "0667");
		put("可", "0668");
		put("台", "0669");
		put("史", "0670");
		put("右", "0671");
		put("叵", "0672");
		put("叶", "0673");
		put("司", "0674");
		put("吁", "0675");
		put("吃", "0676");
		put("各", "0677");
		put("合", "0678");
		put("吉", "0679");
		put("吊", "0680");
		put("同", "0681");
		put("名", "0682");
		put("后", "0683");
		put("吏", "0684");
		put("吐", "0685");
		put("向", "0686");
		put("唧", "0825");
		put("喧", "0826");
		put("喻", "0827");
		put("丧", "0828");
		put("乔", "0829");
		put("单", "0830");
		put("啾", "0831");
		put("喹", "0832");
		put("嗅", "0833");
		put("吗", "0834");
		put("啬", "0835");
		put("嗑", "0836");
		put("嗓", "0837");
		put("嗔", "0838");
		put("呜", "0839");
		put("嗪", "0840");
		put("嗜", "0841");
		put("嗟", "0842");
		put("嗣", "0843");
		put("嗤", "0844");
		put("喔", "0845");
		put("喟", "0846");
		put("哟", "0847");
		put("嗡", "0849");
		put("呛", "0850");
		put("嗉", "0851");
		put("嗒", "0852");
		put("喽", "0853");
		put("叹", "0855");
		put("嘈", "0856");
		put("嘉", "0857");
		put("嘏", "0858");
		put("嘛", "0859");
		put("唛", "0860");
		put("啧", "0862");
		put("尝", "0863");
		put("嘘", "0864");
		put("噻", "0865");
		put("嚎", "0866");
		put("嘎", "0867");
		put("呕", "0868");
		put("嗷", "0869");
		put("嗽", "0870");
		put("嗾", "0871");
		put("嘬", "0872");
		put("哗", "0873");
		put("哔", "0874");
		put("叽", "0875");
		put("啸", "0876");
		put("嘲", "0877");
		put("嘴", "0878");
		put("哓", "0879");
		put("嘶", "0880");
		put("嗥", "0881");
		put("嘹", "0882");
		put("嘻", "0883");
		put("嘿", "0884");
		put("噌", "0886");
		put("噍", "0887");
		put("噎", "0888");
		put("噢", "0889");
		put("噤", "0891");
		put("器", "0892");
		put("噩", "0893");
		put("噪", "0894");
		put("噫", "0895");
		put("噬", "0896");
		put("嗳", "0897");
		put("哙", "0898");
		put("喷", "0899");
		put("噶", "0900");
		put("哝", "0901");
		put("哕", "0902");
		put("吨", "0903");
		put("噱", "0904");
		put("咛", "0905");
		put("嚅", "0906");
		put("嚏", "0908");
		put("啮", "0909");
		put("嚚", "0910");
		put("冁", "0912");
		put("严", "0917");
		put("咙", "0918");
		put("嘤", "0919");
		put("嚼", "0920");
		put("啭", "0921");
		put("嗫", "0922");
		put("嚣", "0923");
		put("嚷", "0924");
		put("囊", "0926");
		put("呓", "0927");
		put("唾", "0790");
		put("啁", "0791");
		put("啄", "0793");
		put("商", "0794");
		put("问", "0795");
		put("启", "0796");
		put("啖", "0797");
		put("啥", "0798");
		put("啜", "0799");
		put("哑", "0800");
		put("唬", "0801");
		put("啤", "0802");
		put("啡", "0803");
		put("啻", "0804");
		put("啼", "0805");
		put("喁", "0806");
		put("喀", "0807");
		put("喂", "0808");
		put("喃", "0809");
		put("善", "0810");
		put("喆", "0811");
		put("喇", "0812");
		put("喈", "0813");
		put("喉", "0814");
		put("喊", "0815");
		put("喏", "0816");
		put("喋", "0818");
		put("喑", "0819");
		put("喘", "0820");
		put("喙", "0821");
		put("唤", "0822");
		put("喜", "0823");
		put("喝", "0824");
		put("圬", "0963");
		put("圭", "0964");
		put("圮", "0965");
		put("地", "0966");
		put("圻", "0967");
		put("址", "0968");
		put("圾", "0969");
		put("坟", "0970");
		put("均", "0971");
		put("坊", "0972");
		put("坍", "0973");
		put("坎", "0974");
		put("坏", "0975");
		put("坐", "0976");
		put("坑", "0977");
		put("坂", "0978");
		put("坌", "0979");
		put("坡", "0980");
		put("坤", "0981");
		put("坦", "0982");
		put("坭", "0983");
		put("坩", "0984");
		put("坷", "0985");
		put("坼", "0986");
		put("垂", "0987");
		put("坪", "0988");
		put("坫", "0989");
		put("淙", "3222");
		put("泪", "3223");
		put("淝", "3224");
		put("淡", "3225");
		put("淤", "3226");
		put("淦", "3227");
		put("浄", "3228");
		put("沦", "3229");
		put("淫", "3230");
		put("淬", "3231");
		put("淮", "3232");
		put("浛", "3233");
		put("深", "3234");
		put("菏", "3235");
		put("混", "3236");
		put("清", "3237");
		put("淹", "3238");
		put("浅", "3239");
		put("添", "3240");
		put("涎", "3241");
		put("涪", "3242");
		put("涴", "3243");
		put("淀", "3244");
		put("淄", "3245");
		put("涠", "3246");
		put("淞", "3247");
		put("浯", "3248");
		put("涭", "3249");
		put("滧", "3250");
		put("涣", "3251");
		put("渚", "3252");
		put("減", "3253");
		put("渝", "3254");
		put("渠", "3255");
		put("渡", "3256");
		put("渣", "3257");
		put("渤", "3258");
		put("渥", "3259");
		put("涡", "3260");
		put("测", "3261");
		put("渭", "3262");
		put("港", "3263");
		put("渴", "3265");
		put("游", "3266");
		put("渺", "3267");
		put("浑", "3268");
		put("湃", "3269");
		put("湄", "3270");
		put("湍", "3273");
		put("泥", "3136");
		put("注", "3137");
		put("泫", "3138");
		put("泯", "3139");
		put("泮", "3140");
		put("泰", "3141");
		put("泱", "3142");
		put("泳", "3144");
		put("沭", "3145");
		put("泐", "3147");
		put("泖", "3148");
		put("洄", "3150");
		put("浐", "3151");
		put("洋", "3152");
		put("浕", "3153");
		put("洎", "3154");
		put("洒", "3155");
		put("洗", "3156");
		put("洛", "3157");
		put("浉", "3158");
		put("洞", "3159");
		put("津", "3160");
		put("洧", "3161");
		put("汧", "3162");
		put("洪", "3163");
		put("洫", "3164");
		put("洮", "3165");
		put("洲", "3166");
		put("洱", "3167");
		put("洳", "3168");
		put("洵", "3169");
		put("浈", "3170");
		put("溲", "3309");
		put("溶", "3310");
		put("溷", "3311");
		put("溺", "3312");
		put("溽", "3314");
		put("滁", "3315");
		put("滂", "3316");
		put("滃", "3317");
		put("沧", "3318");
		put("灭", "3319");
		put("滋", "3320");
		put("涤", "3321");
		put("荥", "3322");
		put("滑", "3323");
		put("滓", "3324");
		put("滔", "3325");
		put("滕", "3326");
		put("溘", "3328");
		put("滇", "3329");
		put("滘", "3330");
		put("滞", "3333");
		put("渗", "3334");
		put("滴", "3336");
		put("沪", "3337");
		put("浒", "3338");
		put("洑", "3339");
		put("滚", "3340");
		put("满", "3341");
		put("渔", "3342");
		put("漂", "3343");
		put("漆", "3344");
		put("漏", "3345");
		put("溉", "3346");
		put("漓", "3347");
		put("演", "3348");
		put("漠", "3351");
		put("汉", "3352");
		put("涟", "3353");
		put("漪", "3354");
		put("漫", "3355");
		put("渍", "3356");
		put("淌", "3358");
		put("漱", "3359");
		put("涨", "3360");
		put("漳", "3361");
		put("渐", "3362");
		put("漾", "3363");
		put("浆", "3364");
		put("滹", "3367");
		put("漉", "3369");
		put("溎", "3370");
		put("沤", "3371");
		put("漦", "3372");
		put("漩", "3373");
		put("漯", "3374");
		put("溆", "3375");
		put("漶", "3376");
		put("溇", "3377");
		put("漕", "3378");
		put("颍", "3379");
		put("泼", "3380");
		put("洁", "3381");
		put("潘", "3382");
		put("潜", "3383");
		put("潟", "3384");
		put("潢", "3385");
		put("涧", "3386");
		put("润", "3387");
		put("潦", "3388");
		put("潭", "3389");
		put("潮", "3390");
		put("溃", "3391");
		put("潼", "3392");
		put("潴", "3393");
		put("潸", "3394");
		put("潺", "3395");
		put("涩", "3396");
		put("澄", "3397");
		put("浇", "3398");
		put("涝", "3399");
		put("澈", "3400");
		put("澔", "3401");
		put("澌", "3402");
		put("澎", "3403");
		put("潖", "3404");
		put("澛", "3405");
		put("潞", "3406");
		put("浔", "3410");
		put("潵", "3411");
		put("湎", "3274");
		put("湖", "3275");
		put("湘", "3276");
		put("湛", "3277");
		put("湧", "3279");
		put("湫", "3280");
		put("湮", "3281");
		put("汤", "3282");
		put("湲", "3283");
		put("濉", "3285");
		put("湓", "3288");
		put("湔", "3289");
		put("湜", "3290");
		put("源", "3293");
		put("準", "3294");
		put("溵", "3295");
		put("溜", "3296");
		put("沟", "3297");
		put("溟", "3298");
		put("溠", "3299");
		put("溢", "3300");
		put("溥", "3302");
		put("溧", "3303");
		put("溪", "3305");
		put("温", "3306");
		put("溯", "3307");
		put("溱", "3308");
		put("涛", "3447");
		put("滥", "3448");
		put("濬", "3449");
		put("濮", "3450");
		put("濯", "3451");
		put("潍", "3452");
		put("滨", "3453");
		put("泻", "3454");
		put("濞", "3456");
		put("溅", "3457");
		put("泺", "3458");
		put("滤", "3459");
		put("渎", "3460");
		put("浏", "3461");
		put("瀑", "3462");
		put("濒", "3464");
		put("瀚", "3466");
		put("瀛", "3467");
		put("沥", "3468");
		put("潇", "3469");
		put("濑", "3471");
		put("泸", "3472");
		put("瀣", "3474");
		put("瀼", "3475");
		put("瀍", "3477");
		put("泷", "3478");
		put("潋", "3481");
		put("澜", "3482");
		put("溁", "3484");
		put("瀹", "3485");
		put("灌", "3487");
		put("沣", "3488");
		put("滠", "3490");
		put("滩", "3492");
		put("灏", "3493");
		put("湾", "3494");
		put("滟", "3495");
		put("灞", "3496");
		put("滦", "3497");
		put("炸", "3498");
		put("火", "3499");
		put("灰", "3500");
		put("灶", "3501");
		put("灸", "3502");
		put("灿", "3503");
		put("灼", "3504");
		put("灾", "3506");
		put("炊", "3507");
		put("炎", "3508");
		put("炒", "3509");
		put("炕", "3510");
		put("炙", "3511");
		put("炘", "3512");
		put("炞", "3513");
		put("炫", "3514");
		put("炬", "3515");
		put("炭", "3516");
		put("炮", "3517");
		put("炯", "3518");
		put("炱", "3520");
		put("炳", "3521");
		put("炷", "3522");
		put("畑", "3524");
		put("烈", "3525");
		put("乌", "3527");
		put("烙", "3529");
		put("烘", "3530");
		put("烜", "3531");
		put("烝", "3532");
		put("烟", "3533");
		put("烹", "3534");
		put("烺", "3535");
		put("烽", "3536");
		put("焌", "3537");
		put("焙", "3538");
		put("焚", "3539");
		put("焜", "3540");
		put("焦", "3542");
		put("焰", "3543");
		put("然", "3544");
		put("焠", "3546");
		put("焉", "3547");
		put("煅", "3548");
		put("焊", "3549");
		put("澍", "3412");
		put("澂", "3413");
		put("沩", "3415");
		put("澡", "3416");
		put("泽", "3419");
		put("浍", "3420");
		put("澳", "3421");
		put("澹", "3422");
		put("激", "3423");
		put("浊", "3424");
		put("濂", "3425");
		put("浓", "3426");
		put("渑", "3427");
		put("滪", "3429");
		put("澧", "3430");
		put("澴", "3433");
		put("澶", "3436");
		put("澼", "3437");
		put("泞", "3438");
		put("湿", "3440");
		put("济", "3444");
		put("濠", "3445");
		put("濡", "3446");
		put("颎", "3585");
		put("熹", "3588");
		put("炽", "3589");
		put("烫", "3594");
		put("燃", "3595");
		put("灯", "3597");
		put("燎", "3598");
		put("烧", "3599");
		put("燔", "3600");
		put("燕", "3601");
		put("营", "3602");
		put("燠", "3603");
		put("燥", "3604");
		put("燧", "3606");
		put("烛", "3608");
		put("燮", "3610");
		put("燹", "3611");
		put("烬", "3612");
		put("燿", "3613");
		put("焘", "3614");
		put("爆", "3615");
		put("烁", "3617");
		put("炉", "3619");
		put("烂", "3620");
		put("爝", "3622");
		put("爟", "3623");
		put("焮", "3624");
		put("炖", "3625");
		put("焖", "3626");
		put("爨", "3627");
		put("烤", "3628");
		put("爪", "3629");
		put("争", "3630");
		put("爬", "3632");
		put("爰", "3633");
		put("为", "3634");
		put("爵", "3635");
		put("睑", "3636");
		put("父", "3637");
		put("爹", "3638");
		put("爷", "3639");
		put("爸", "3640");
		put("爻", "3641");
		put("爽", "3642");
		put("趼", "3644");
		put("爿", "3645");
		put("牂", "3647");
		put("牁", "3648");
		put("藠", "3650");
		put("片", "3651");
		put("版", "3652");
		put("牌", "3654");
		put("牒", "3655");
		put("牖", "3657");
		put("牍", "3658");
		put("牙", "3660");
		put("肼", "3661");
		put("牛", "3662");
		put("牝", "3663");
		put("牟", "3664");
		put("牡", "3665");
		put("牢", "3666");
		put("牣", "3667");
		put("牧", "3668");
		put("物", "3670");
		put("牯", "3672");
		put("牲", "3673");
		put("犋", "3674");
		put("牷", "3675");
		put("特", "3676");
		put("牵", "3677");
		put("犀", "3679");
		put("犁", "3680");
		put("犍", "3682");
		put("犒", "3683");
		put("荦", "3684");
		put("犊", "3685");
		put("牺", "3686");
		put("炼", "3550");
		put("煊", "3551");
		put("煌", "3552");
		put("煎", "3553");
		put("煮", "3554");
		put("炜", "3555");
		put("熙", "3556");
		put("煜", "3558");
		put("煞", "3559");
		put("茕", "3560");
		put("煤", "3561");
		put("焕", "3562");
		put("煦", "3563");
		put("照", "3564");
		put("烦", "3565");
		put("煨", "3566");
		put("炀", "3568");
		put("煽", "3569");
		put("熄", "3571");
		put("熊", "3574");
		put("熏", "3575");
		put("荧", "3576");
		put("熟", "3578");
		put("熔", "3579");
		put("熨", "3580");
		put("熬", "3581");
		put("热", "3583");
		put("熠", "3584");
		put("猢", "3724");
		put("猥", "3725");
		put("猩", "3726");
		put("猪", "3727");
		put("猫", "3728");
		put("猴", "3729");
		put("犹", "3730");
		put("猷", "3731");
		put("猔", "3732");
		put("猱", "3733");
		put("猺", "3734");
		put("狲", "3735");
		put("猾", "3736");
		put("猿", "3737");
		put("狱", "3739");
		put("狮", "3740");
		put("獍", "3741");
		put("獐", "3742");
		put("獒", "3743");
		put("獗", "3744");
		put("独", "3747");
		put("狯", "3748");
		put("獬", "3749");
		put("獠", "3750");
		put("狝", "3751");
		put("获", "3752");
		put("獯", "3753");
		put("犷", "3755");
		put("猎", "3756");
		put("兽", "3757");
		put("獭", "3758");
		put("献", "3759");
		put("狞", "3761");
		put("猡", "3762");
		put("玄", "3763");
		put("率", "3764");
		put("锔", "3767");
		put("玉", "3768");
		put("王", "3769");
		put("玕", "3771");
		put("玔", "3772");
		put("玖", "3773");
		put("玟", "3774");
		put("玠", "3775");
		put("玢", "3776");
		put("玦", "3777");
		put("珏", "3778");
		put("玩", "3779");
		put("玫", "3780");
		put("玲", "3781");
		put("玳", "3782");
		put("玷", "3783");
		put("珂", "3784");
		put("珈", "3786");
		put("珉", "3787");
		put("玻", "3788");
		put("珀", "3789");
		put("珊", "3790");
		put("珍", "3791");
		put("珞", "3794");
		put("珠", "3796");
		put("珙", "3797");
		put("珖", "3798");
		put("珥", "3799");
		put("珣", "3800");
		put("珩", "3801");
		put("珪", "3802");
		put("班", "3803");
		put("珧", "3804");
		put("珮", "3805");
		put("珽", "3806");
		put("现", "3807");
		put("球", "3808");
		put("琅", "3809");
		put("理", "3810");
		put("琇", "3811");
		put("琉", "3812");
		put("珶", "3813");
		put("琊", "3815");
		put("琕", "3816");
		put("琚", "3818");
		put("琛", "3819");
		put("琢", "3820");
		put("琤", "3821");
		put("琥", "3822");
		put("琦", "3823");
		put("琨", "3824");
		put("琪", "3825");
		put("犬", "3689");
		put("犯", "3690");
		put("犴", "3691");
		put("状", "3692");
		put("狂", "3693");
		put("狃", "3694");
		put("狄", "3695");
		put("狁", "3696");
		put("狎", "3697");
		put("狐", "3698");
		put("狗", "3699");
		put("狙", "3700");
		put("狡", "3701");
		put("狠", "3703");
		put("狩", "3704");
		put("狴", "3705");
		put("狸", "3706");
		put("狭", "3707");
		put("狼", "3708");
		put("狈", "3709");
		put("狻", "3710");
		put("狺", "3711");
		put("狷", "3712");
		put("狢", "3713");
		put("猁", "3714");
		put("猖", "3715");
		put("猗", "3716");
		put("狰", "3717");
		put("猛", "3718");
		put("猜", "3719");
		put("猝", "3720");
		put("猞", "3721");
		put("猊", "3722");
		put("璀", "3862");
		put("璃", "3863");
		put("璋", "3864");
		put("瑾", "3866");
		put("璆", "3868");
		put("琎", "3871");
		put("璇", "3872");
		put("璐", "3873");
		put("璜", "3874");
		put("玑", "3875");
		put("璘", "3876");
		put("璞", "3877");
		put("璟", "3878");
		put("璠", "3879");
		put("璧", "3880");
		put("璨", "3881");
		put("环", "3883");
		put("珰", "3885");
		put("玺", "3886");
		put("璲", "3887");
		put("瑸", "3888");
		put("玙", "3889");
		put("珑", "3891");
		put("璎", "3892");
		put("瓒", "3895");
		put("瓜", "3900");
		put("瓞", "3901");
		put("瓠", "3902");
		put("瓢", "3903");
		put("瓣", "3904");
		put("瓤", "3905");
	}

	static void init1() {
		put("瓦", "3907");
		put("瓮", "3908");
		put("瓴", "3909");
		put("瓶", "3910");
		put("瓷", "3911");
		put("瓻", "3912");
		put("甃", "3913");
		put("甄", "3914");
		put("瓿", "3916");
		put("瓯", "3917");
		put("甍", "3918");
		put("甑", "3920");
		put("甓", "3921");
		put("瓩", "3926");
		put("甘", "3927");
		put("甚", "3928");
		put("甜", "3929");
		put("咔", "3931");
		put("生", "3932");
		put("甡", "3933");
		put("产", "3934");
		put("甥", "3935");
		put("甦", "3936");
		put("用", "3938");
		put("甪", "3939");
		put("甫", "3940");
		put("甬", "3941");
		put("甯", "3942");
		put("甩", "3943");
		put("田", "3944");
		put("由", "3945");
		put("甲", "3946");
		put("申", "3947");
		put("男", "3948");
		put("甸", "3949");
		put("甾", "3951");
		put("畀", "3952");
		put("畋", "3953");
		put("界", "3954");
		put("畎", "3955");
		put("畏", "3956");
		put("町", "3957");
		put("畔", "3961");
		put("畚", "3962");
		put("畛", "3963");
		put("琮", "3827");
		put("琯", "3828");
		put("琳", "3829");
		put("琴", "3830");
		put("琶", "3831");
		put("琵", "3832");
		put("琬", "3834");
		put("琲", "3836");
		put("玮", "3837");
		put("瑕", "3838");
		put("瑙", "3839");
		put("瑚", "3840");
		put("瑛", "3841");
		put("瑜", "3842");
		put("瑞", "3843");
		put("瑟", "3844");
		put("瑁", "3847");
		put("瑗", "3850");
		put("琐", "3851");
		put("瑶", "3852");
		put("莹", "3853");
		put("玛", "3854");
		put("瑰", "3855");
		put("玱", "3856");
		put("瑱", "3859");
		put("疝", "3999");
		put("痠", "4000");
		put("疣", "4001");
		put("疤", "4002");
		put("疥", "4003");
		put("疫", "4004");
		put("痄", "4005");
		put("疲", "4006");
		put("疳", "4007");
		put("疴", "4008");
		put("疵", "4009");
		put("疸", "4010");
		put("疹", "4011");
		put("疼", "4012");
		put("疽", "4013");
		put("疾", "4014");
		put("病", "4016");
		put("症", "4017");
		put("痂", "4018");
		put("痊", "4019");
		put("疰", "4020");
		put("痍", "4021");
		put("痒", "4022");
		put("痔", "4023");
		put("痕", "4024");
		put("痧", "4025");
		put("痘", "4026");
		put("痛", "4027");
		put("痞", "4028");
		put("痣", "4029");
		put("痢", "4031");
		put("痱", "4032");
		put("痰", "4033");
		put("痳", "4034");
		put("痴", "4035");
		put("痹", "4036");
		put("痼", "4037");
		put("痿", "4039");
		put("瘀", "4040");
		put("瘁", "4041");
		put("疯", "4045");
		put("疡", "4046");
		put("瘝", "4048");
		put("瘐", "4049");
		put("痪", "4050");
		put("瘕", "4051");
		put("瘗", "4053");
		put("瘟", "4054");
		put("瘠", "4055");
		put("疮", "4056");
		put("瘢", "4057");
		put("瘤", "4058");
		put("瘥", "4059");
		put("瘦", "4060");
		put("疟", "4061");
		put("瘸", "4063");
		put("瘰", "4064");
		put("瘳", "4065");
		put("瘴", "4066");
		put("瘵", "4067");
		put("瘘", "4068");
		put("瘼", "4069");
		put("疗", "4070");
		put("癃", "4071");
		put("痨", "4072");
		put("痫", "4073");
		put("癌", "4074");
		put("癖", "4075");
		put("疠", "4078");
		put("瘩", "4079");
		put("癜", "4080");
		put("疖", "4081");
		put("瘪", "4085");
		put("瘿", "4086");
		put("疬", "4087");
		put("癞", "4088");
		put("癣", "4089");
		put("瘾", "4090");
		put("癯", "4091");
		put("痈", "4092");
		put("瘫", "4093");
		put("癫", "4094");
		put("疙", "4095");
		put("癶", "4096");
		put("癸", "4097");
		put("登", "4098");
		put("发", "4099");
		put("钶", "4100");
		put("白", "4101");
		put("畜", "3964");
		put("亩", "3965");
		put("留", "3966");
		put("毕", "3968");
		put("畤", "3969");
		put("略", "3970");
		put("畦", "3971");
		put("番", "3972");
		put("画", "3973");
		put("畬", "3974");
		put("畯", "3975");
		put("榃", "3978");
		put("畹", "3979");
		put("当", "3981");
		put("畸", "3982");
		put("畿", "3983");
		put("疆", "3984");
		put("畴", "3985");
		put("锎", "3987");
		put("疋", "3988");
		put("疏", "3990");
		put("疑", "3992");
		put("钪", "3993");
		put("疒", "3994");
		put("疔", "3995");
		put("痃", "3996");
		put("痉", "3997");
		put("疚", "3998");
		put("盍", "4137");
		put("盎", "4138");
		put("盒", "4139");
		put("盔", "4140");
		put("盛", "4141");
		put("盗", "4142");
		put("盝", "4143");
		put("盏", "4144");
		put("盟", "4145");
		put("尽", "4147");
		put("监", "4148");
		put("盘", "4149");
		put("盥", "4150");
		put("卢", "4151");
		put("盦", "4152");
		put("盪", "4155");
		put("盬", "4156");
		put("蛞", "4157");
		put("目", "4158");
		put("盲", "4159");
		put("直", "4160");
		put("相", "4161");
		put("盼", "4162");
		put("盾", "4163");
		put("省", "4164");
		put("眄", "4165");
		put("眇", "4166");
		put("眈", "4167");
		put("眉", "4168");
		put("盯", "4169");
		put("看", "4170");
		put("盻", "4171");
		put("眨", "4172");
		put("盱", "4173");
		put("睬", "4174");
		put("眛", "4175");
		put("真", "4176");
		put("眠", "4177");
		put("瞄", "4178");
		put("眦", "4180");
		put("眩", "4181");
		put("眙", "4182");
		put("岷", "1488");
		put("岸", "1489");
		put("岣", "1490");
		put("峒", "1491");
		put("峙", "1492");
		put("峋", "1493");
		put("峨", "1494");
		put("峭", "1495");
		put("峰", "1496");
		put("岛", "1497");
		put("峻", "1498");
		put("峡", "1499");
		put("峓", "1500");
		put("岘", "1501");
		put("峪", "1502");
		put("崃", "1503");
		put("崇", "1504");
		put("崎", "1505");
		put("岽", "1506");
		put("崑", "1507");
		put("崔", "1508");
		put("崖", "1509");
		put("崙", "1510");
		put("岗", "1511");
		put("崛", "1512");
		put("峥", "1513");
		put("崩", "1514");
		put("度", "1653");
		put("座", "1654");
		put("库", "1655");
		put("庭", "1656");
		put("庵", "1658");
		put("庶", "1659");
		put("康", "1660");
		put("庸", "1661");
		put("庾", "1662");
		put("厢", "1666");
		put("廑", "1668");
		put("廈", "1669");
		put("廉", "1670");
		put("廊", "1671");
		put("廋", "1672");
		put("厩", "1673");
		put("廓", "1674");
		put("廖", "1675");
		put("厨", "1676");
		put("廛", "1679");
		put("庙", "1680");
		put("庑", "1682");
		put("废", "1683");
		put("廨", "1685");
		put("廪", "1686");
		put("庐", "1687");
		put("厅", "1689");
		put("庞", "1690");
		put("锆", "1691");
		put("廴", "1692");
		put("延", "1693");
		put("廷", "1694");
		put("建", "1696");
		put("廾", "1699");
		put("弁", "1701");
		put("弄", "1702");
		put("弈", "1704");
		put("弊", "1705");
		put("茛", "1706");
		put("弋", "1707");
		put("弍", "1708");
		put("式", "1709");
		put("弑", "1710");
		put("瀔", "1711");
		put("弓", "1712");
		put("引", "1714");
		put("弗", "1715");
		put("弛", "1716");
		put("弟", "1717");
		put("弢", "1718");
		put("弦", "1720");
		put("弧", "1721");
		put("弩", "1722");
		put("弭", "1725");
		put("弱", "1726");
		put("张", "1728");
		put("强", "1730");
		put("弼", "1732");
		put("彀", "1733");
		put("弹", "1734");
		put("弥", "1736");
		put("弯", "1737");
		put("弘", "1738");
		put("", "1739");
		put("彖", "1740");
		put("彗", "1741");
		put("彘", "1742");
		put("彝", "1744");
		put("彟", "1745");
		put("臌", "1746");
		put("彡", "1747");
		put("形", "1748");
		put("彤", "1749");
		put("彦", "1750");
		put("彧", "1751");
		put("彩", "1752");
		put("彪", "1753");
		put("彬", "1755");
		put("幪", "1619");
		put("帮", "1620");
		put("帱", "1621");
		put("幛", "1625");
		put("干", "1626");
		put("平", "1627");
		put("年", "1628");
		put("并", "1629");
		put("幸", "1630");
		put("矸", "1632");
		put("幺", "1633");
		put("幻", "1634");
		put("幼", "1635");
		put("幽", "1636");
		put("酐", "1638");
		put("广", "1639");
		put("庀", "1640");
		put("庹", "1641");
		put("庇", "1642");
		put("床", "1643");
		put("庋", "1644");
		put("序", "1645");
		put("底", "1646");
		put("庖", "1647");
		put("店", "1648");
		put("庚", "1649");
		put("府", "1650");
		put("庠", "1651");
		put("庥", "1652");
		put("徭", "1791");
		put("微", "1792");
		put("徯", "1793");
		put("徵", "1794");
		put("德", "1795");
		put("彻", "1796");
		put("徼", "1797");
		put("徽", "1798");
		put("焓", "1799");
		put("心", "1800");
		put("必", "1801");
		put("忉", "1802");
		put("忌", "1803");
		put("忍", "1804");
		put("忒", "1805");
		put("忖", "1806");
		put("志", "1807");
		put("忘", "1808");
		put("忐", "1809");
		put("忑", "1810");
		put("忙", "1811");
		put("忝", "1812");
		put("忠", "1813");
		put("忡", "1814");
		put("恬", "1815");
		put("快", "1816");
		put("忭", "1817");
		put("忮", "1818");
		put("念", "1819");
		put("忱", "1820");
		put("忸", "1821");
		put("忤", "1822");
		put("忻", "1823");
		put("忽", "1824");
		put("忿", "1825");
		put("怍", "1826");
		put("怎", "1827");
		put("怏", "1828");
		put("怒", "1829");
		put("怕", "1830");
		put("怖", "1831");
		put("怗", "1832");
		put("怙", "1833");
		put("怛", "1834");
		put("思", "1835");
		put("怠", "1836");
		put("怡", "1837");
		put("急", "1838");
		put("怦", "1839");
		put("性", "1840");
		put("怨", "1841");
		put("怩", "1842");
		put("怪", "1843");
		put("怫", "1844");
		put("怯", "1845");
		put("怵", "1847");
		put("您", "1849");
		put("怔", "1850");
		put("恁", "1851");
		put("恂", "1852");
		put("恃", "1853");
		put("恒", "1854");
		put("惦", "1855");
		put("恍", "1857");
		put("恐", "1858");
		put("恕", "1859");
		put("恙", "1860");
		put("恚", "1861");
		put("恝", "1862");
		put("恢", "1863");
		put("恣", "1864");
		put("恤", "1865");
		put("恧", "1867");
		put("恨", "1868");
		put("恩", "1869");
		put("恪", "1870");
		put("恫", "1871");
		put("恭", "1872");
		put("息", "1873");
		put("恰", "1874");
		put("悃", "1875");
		put("悄", "1877");
		put("悦", "1878");
		put("悌", "1879");
		put("悍", "1880");
		put("悒", "1881");
		put("悔", "1882");
		put("悖", "1883");
		put("悚", "1884");
		put("悉", "1885");
		put("悛", "1886");
		put("悝", "1887");
		put("悮", "1888");
		put("悟", "1889");
		put("悠", "1890");
		put("患", "1891");
		put("彭", "1756");
		put("彰", "1757");
		put("影", "1758");
		put("彨", "1759");
		put("铪", "1760");
		put("彳", "1761");
		put("彷", "1762");
		put("役", "1763");
		put("彼", "1764");
		put("往", "1766");
		put("征", "1767");
		put("徂", "1768");
		put("待", "1769");
		put("徇", "1770");
		put("很", "1771");
		put("徉", "1772");
		put("徊", "1773");
		put("律", "1774");
		put("徐", "1776");
		put("径", "1777");
		put("徒", "1778");
		put("得", "1779");
		put("徘", "1780");
		put("徙", "1781");
		put("徜", "1782");
		put("从", "1783");
		put("徕", "1784");
		put("御", "1785");
		put("徨", "1787");
		put("复", "1788");
		put("循", "1789");
		put("徬", "1790");
		put("惶", "1929");
		put("惹", "1931");
		put("惺", "1932");
		put("恻", "1933");
		put("愀", "1934");
		put("愁", "1935");
		put("愆", "1936");
		put("愈", "1937");
		put("愉", "1938");
		put("愍", "1940");
		put("愎", "1941");
		put("意", "1942");
		put("愔", "1944");
		put("愕", "1945");
		put("愚", "1946");
		put("爱", "1947");
		put("惬", "1948");
		put("感", "1949");
		put("愠", "1950");
		put("愧", "1951");
		put("悫", "1952");
		put("愫", "1953");
		put("怆", "1955");
		put("恺", "1956");
		put("慎", "1957");
		put("忾", "1958");
		put("愿", "1959");
		put("恿", "1960");
		put("慈", "1964");
		put("慊", "1965");
		put("态", "1966");
		put("慌", "1967");
		put("慓", "1969");
		put("慕", "1970");
		put("惨", "1971");
		put("慝", "1973");
		put("惭", "1974");
		put("恸", "1975");
		put("慢", "1976");
		put("惯", "1977");
		put("慧", "1979");
		put("慨", "1980");
		put("怂", "1981");
		put("虑", "1982");
		put("慰", "1983");
		put("悭", "1984");
		put("慵", "1986");
		put("庆", "1987");
		put("慷", "1988");
		put("忧", "1992");
		put("惫", "1994");
		put("怜", "1995");
		put("憔", "1997");
		put("惮", "1998");
		put("憎", "1999");
		put("憝", "2000");
		put("愤", "2001");
		put("憧", "2002");
		put("憨", "2003");
		put("憩", "2004");
		put("憬", "2005");
		put("悯", "2006");
		put("怃", "2007");
		put("宪", "2009");
		put("忆", "2011");
		put("憾", "2013");
		put("懂", "2016");
		put("恳", "2017");
		put("懈", "2018");
		put("应", "2019");
		put("懊", "2020");
		put("懋", "2021");
		put("怿", "2022");
		put("懔", "2023");
		put("愦", "2025");
		put("怼", "2029");
		put("懑", "2030");
		put("悱", "1894");
		put("悲", "1896");
		put("悴", "1897");
		put("怅", "1898");
		put("闷", "1899");
		put("悸", "1900");
		put("悻", "1901");
		put("悼", "1902");
		put("情", "1906");
		put("惆", "1907");
		put("惇", "1908");
		put("惑", "1910");
		put("惓", "1911");
		put("惕", "1912");
		put("惘", "1913");
		put("惙", "1914");
		put("惚", "1915");
		put("惜", "1917");
		put("惟", "1919");
		put("惠", "1920");
		put("恶", "1921");
		put("悰", "1922");
		put("惋", "1923");
		put("惰", "1924");
		put("恼", "1925");
		put("恽", "1926");
		put("想", "1927");
		put("惴", "1928");
		put("戳", "2067");
		put("戮", "2068");
		put("战", "2069");
		put("戏", "2070");
		put("戴", "2071");
		put("戤", "2072");
		put("户", "2073");
		put("戾", "2074");
		put("房", "2075");
		put("所", "2076");
		put("戽", "2077");
		put("扁", "2078");
		put("扃", "2079");
		put("扆", "2081");
		put("扇", "2082");
		put("扈", "2083");
		put("扉", "2084");
		put("槲", "2086");
		put("手", "2087");
		put("才", "2088");
		put("扎", "2089");
		put("扑", "2090");
		put("扒", "2091");
		put("打", "2092");
		put("扔", "2093");
		put("托", "2094");
		put("扛", "2095");
		put("扠", "2096");
		put("拤", "2097");
		put("扣", "2099");
		put("扭", "2100");
		put("扮", "2101");
		put("扯", "2102");
		put("扳", "2104");
		put("扶", "2105");
		put("批", "2106");
		put("抵", "2107");
		put("扼", "2108");
		put("找", "2109");
		put("承", "2110");
		put("技", "2111");
		put("抄", "2113");
		put("抉", "2115");
		put("把", "2116");
		put("抑", "2117");
		put("抒", "2118");
		put("抓", "2119");
		put("投", "2121");
		put("抖", "2122");
		put("抗", "2123");
		put("折", "2124");
		put("抨", "2125");
		put("披", "2126");
		put("抬", "2127");
		put("抱", "2128");
		put("抿", "2129");
		put("抹", "2130");
		put("押", "2131");
		put("抽", "2132");
		put("拂", "2133");
		put("拄", "2134");
		put("拆", "2135");
		put("拇", "2136");
		put("担", "2137");
		put("拈", "2138");
		put("拉", "2139");
		put("拊", "2140");
		put("抛", "2141");
		put("拌", "2142");
		put("拍", "2143");
		put("拐", "2145");
		put("拎", "2146");
		put("拒", "2147");
		put("拓", "2148");
		put("拔", "2149");
		put("拖", "2151");
		put("拗", "2152");
		put("拘", "2153");
		put("拙", "2154");
		put("拚", "2155");
		put("招", "2156");
		put("拜", "2157");
		put("捂", "2158");
		put("拮", "2159");
		put("拭", "2160");
		put("括", "2161");
		put("拱", "2162");
		put("拯", "2163");
		put("拳", "2164");
		put("拴", "2165");
		put("拷", "2166");
		put("拽", "2167");
		put("拾", "2168");
		put("拿", "2169");
		put("懦", "2032");
		put("惩", "2033");
		put("懿", "2034");
		put("懵", "2035");
		put("懒", "2036");
		put("怀", "2037");
		put("悬", "2038");
		put("忏", "2039");
		put("惧", "2040");
		put("慑", "2042");
		put("恋", "2043");
		put("恹", "2044");
		put("戆", "2045");
		put("戈", "2047");
		put("戊", "2048");
		put("戌", "2049");
		put("戍", "2050");
		put("戎", "2051");
		put("成", "2052");
		put("我", "2053");
		put("戒", "2054");
		put("戋", "2055");
		put("戕", "2056");
		put("或", "2057");
		put("戚", "2058");
		put("戛", "2059");
		put("戟", "2060");
		put("戢", "2061");
		put("戥", "2062");
		put("戡", "2063");
		put("戬", "2065");
		put("截", "2066");
		put("捺", "2205");
		put("捭", "2206");
		put("据", "2207");
		put("捱", "2209");
		put("捶", "2211");
		put("捷", "2212");
		put("揍", "2213");
		put("捻", "2214");
		put("掀", "2216");
		put("扫", "2217");
		put("掇", "2218");
		put("授", "2219");
		put("掉", "2220");
		put("掊", "2221");
		put("掌", "2222");
		put("掏", "2223");
		put("掎", "2224");
		put("掐", "2225");
		put("排", "2226");
		put("掖", "2227");
		put("掘", "2228");
		put("掠", "2230");
		put("探", "2232");
		put("掣", "2233");
		put("接", "2234");
		put("控", "2235");
		put("推", "2236");
		put("掩", "2237");
		put("措", "2238");
		put("掬", "2239");
		put("掯", "2240");
		put("抡", "2241");
		put("拣", "2245");
		put("揄", "2246");
		put("揆", "2247");
		put("揉", "2248");
		put("描", "2249");
		put("捏", "2250");
		put("提", "2251");
		put("插", "2252");
		put("揖", "2253");
		put("扬", "2254");
		put("换", "2255");
		put("揠", "2257");
		put("搽", "2258");
		put("握", "2259");
		put("揣", "2260");
		put("揩", "2261");
		put("揪", "2262");
		put("揭", "2263");
		put("挥", "2264");
		put("援", "2266");
		put("掾", "2268");
		put("搞", "2269");
		put("搧", "2271");
		put("损", "2275");
		put("搏", "2276");
		put("搒", "2277");
		put("搓", "2278");
		put("搔", "2279");
		put("摇", "2280");
		put("捣", "2281");
		put("搜", "2282");
		put("搠", "2283");
		put("搦", "2286");
		put("搪", "2288");
		put("搬", "2289");
		put("搭", "2290");
		put("搴", "2291");
		put("摔", "2292");
		put("抢", "2293");
		put("搐", "2294");
		put("掺", "2296");
		put("撂", "2297");
		put("摘", "2298");
		put("搂", "2299");
		put("撑", "2300");
		put("摧", "2301");
		put("摩", "2302");
		put("摭", "2303");
		put("挚", "2304");
		put("抠", "2305");
		put("抟", "2306");
		put("摸", "2307");
		put("持", "2170");
		put("挂", "2171");
		put("指", "2172");
		put("挈", "2173");
		put("按", "2174");
		put("挎", "2175");
		put("挑", "2176");
		put("挖", "2177");
		put("拼", "2178");
		put("挨", "2179");
		put("挪", "2180");
		put("挫", "2181");
		put("振", "2182");
		put("捅", "2183");
		put("挹", "2184");
		put("挺", "2185");
		put("挼", "2186");
		put("挽", "2187");
		put("挟", "2188");
		put("捆", "2190");
		put("捉", "2191");
		put("捋", "2192");
		put("捌", "2193");
		put("捍", "2194");
		put("掂", "2195");
		put("捐", "2196");
		put("挣", "2197");
		put("捕", "2198");
		put("挲", "2199");
		put("捎", "2200");
		put("捧", "2201");
		put("捩", "2203");
		put("扪", "2204");
		put("擅", "2343");
		put("择", "2344");
		put("击", "2345");
		put("挡", "2346");
		put("操", "2347");
		put("擎", "2348");
		put("拧", "2349");
		put("擒", "2350");
		put("擗", "2352");
		put("擘", "2353");
		put("撷", "2355");
		put("搀", "2356");
		put("挤", "2357");
		put("擢", "2359");
		put("擦", "2361");
		put("拟", "2362");
		put("摈", "2363");
		put("搁", "2364");
		put("撵", "2365");
		put("擭", "2366");
		put("掷", "2367");
		put("扩", "2368");
		put("摆", "2369");
		put("擞", "2370");
		put("扰", "2371");
		put("攀", "2372");
		put("摅", "2373");
		put("拦", "2374");
		put("撄", "2375");
		put("攘", "2376");
		put("携", "2377");
		put("摄", "2378");
		put("拢", "2379");
		put("攒", "2380");
		put("挛", "2381");
		put("摊", "2382");
		put("搅", "2383");
		put("攫", "2384");
		put("揽", "2385");
		put("拃", "2387");
		put("支", "2388");
		put("塃", "2390");
		put("攴", "2391");
		put("收", "2392");
		put("攸", "2394");
		put("改", "2395");
		put("攻", "2396");
		put("放", "2397");
		put("政", "2398");
		put("故", "2399");
		put("效", "2400");
		put("敉", "2401");
		put("教", "2403");
		put("敏", "2404");
		put("救", "2405");
		put("敕", "2406");
		put("敖", "2407");
		put("败", "2408");
		put("敝", "2411");
		put("敞", "2412");
		put("敢", "2413");
		put("散", "2414");
		put("敦", "2415");
		put("敻", "2416");
		put("敬", "2417");
		put("敲", "2418");
		put("整", "2419");
		put("敌", "2420");
		put("敷", "2421");
		put("数", "2422");
		put("敛", "2425");
		put("毙", "2426");
		put("敩", "2427");
		put("文", "2429");
		put("斌", "2430");
		put("斐", "2431");
		put("斑", "2432");
		put("斓", "2433");
		put("蟥", "2434");
		put("斗", "2435");
		put("料", "2436");
		put("斛", "2437");
		put("斜", "2438");
		put("斝", "2439");
		put("斟", "2440");
		put("斡", "2441");
		put("斢", "2442");
		put("斤", "2443");
		put("斥", "2444");
		put("斧", "2445");
		put("摹", "2308");
		put("摺", "2309");
		put("掴", "2310");
		put("摒", "2311");
		put("撅", "2313");
		put("撩", "2316");
		put("撇", "2317");
		put("捞", "2318");
		put("撒", "2320");
		put("挠", "2321");
		put("撕", "2322");
		put("撙", "2323");
		put("撞", "2326");
		put("撤", "2327");
		put("拨", "2328");
		put("抚", "2329");
		put("播", "2330");
		put("撮", "2331");
		put("撰", "2332");
		put("撬", "2334");
		put("掼", "2335");
		put("攉", "2336");
		put("挞", "2337");
		put("撼", "2338");
		put("挝", "2339");
		put("拥", "2340");
		put("擂", "2341");
		put("掳", "2342");
		put("旦", "2481");
		put("旨", "2482");
		put("早", "2483");
		put("旬", "2484");
		put("旭", "2485");
		put("旰", "2486");
		put("旱", "2487");
		put("晒", "2488");
		put("旺", "2489");
		put("昌", "2490");
		put("昂", "2491");
		put("昆", "2492");
		put("昃", "2493");
		put("明", "2494");
		put("昏", "2495");
		put("易", "2496");
		put("昔", "2497");
		put("晗", "2498");
		put("昉", "2499");
		put("昕", "2500");
		put("昝", "2501");
		put("星", "2502");
		put("映", "2503");
		put("春", "2504");
		put("昧", "2505");
		put("昨", "2506");
		put("昭", "2507");
		put("是", "2508");
		put("昱", "2509");
		put("昴", "2510");
		put("昵", "2511");
		put("昶", "2512");
		put("晁", "2513");
		put("时", "2514");
		put("晃", "2515");
		put("晋", "2516");
		put("晌", "2517");
		put("晏", "2518");
		put("晚", "2519");
		put("晢", "2520");
		put("昼", "2521");
		put("晞", "2522");
		put("晡", "2523");
		put("晤", "2524");
		put("晨", "2525");
		put("晦", "2526");
		put("暎", "2527");
		put("普", "2528");
		put("景", "2529");
		put("晰", "2530");
		put("晳", "2531");
		put("晴", "2532");
		put("晶", "2533");
		put("晷", "2534");
		put("智", "2535");
		put("晬", "2536");
		put("暄", "2537");
		put("暇", "2538");
		put("暍", "2539");
		put("暑", "2540");
		put("暖", "2541");
		put("暗", "2542");
		put("旸", "2543");
		put("暝", "2544");
		put("畅", "2545");
		put("晕", "2546");
		put("晖", "2547");
		put("暂", "2548");
		put("旻", "2549");
		put("暮", "2550");
		put("暴", "2552");
		put("暹", "2554");
		put("暨", "2555");
		put("晓", "2556");
		put("暾", "2557");
		put("昙", "2560");
		put("曙", "2562");
		put("曛", "2564");
		put("曜", "2565");
		put("曝", "2566");
		put("旷", "2568");
		put("曦", "2569");
		put("昽", "2570");
		put("曩", "2571");
		put("昇", "2573");
		put("曰", "2574");
		put("曲", "2575");
		put("曳", "2576");
		put("更", "2577");
		put("曷", "2578");
		put("书", "2579");
		put("曹", "2580");
		put("曼", "2581");
		put("曾", "2582");
		put("替", "2583");
		put("斨", "2446");
		put("斩", "2447");
		put("斯", "2448");
		put("新", "2450");
		put("断", "2451");
		put("方", "2455");
		put("於", "2456");
		put("施", "2457");
		put("旎", "2459");
		put("旁", "2460");
		put("旂", "2461");
		put("旃", "2462");
		put("旄", "2463");
		put("旅", "2464");
		put("旆", "2465");
		put("旋", "2467");
		put("旌", "2468");
		put("族", "2469");
		put("旖", "2470");
		put("旒", "2471");
		put("旗", "2475");
		put("烩", "2476");
		put("无", "2477");
		put("既", "2478");
		put("日", "2480");
		put("杉", "2619");
		put("杌", "2620");
		put("李", "2621");
		put("杏", "2622");
		put("机", "2623");
		put("材", "2624");
		put("村", "2625");
		put("杓", "2626");
		put("杖", "2627");
		put("柈", "2628");
		put("杜", "2629");
		put("杞", "2630");
		put("束", "2631");
		put("梠", "2632");
		put("栓", "2633");
		put("杪", "2634");
		put("杭", "2635");
		put("柿", "2636");
		put("杯", "2637");
		put("杰", "2638");
		put("东", "2639");
		put("杲", "2640");
		put("杳", "2641");
		put("朿", "2642");
		put("杵", "2643");
		put("棵", "2644");
		put("杼", "2645");
		put("松", "2646");
		put("板", "2647");
		put("枉", "2648");
		put("析", "2649");
		put("枕", "2650");
		put("林", "2651");
		put("枙", "2652");
		put("枚", "2653");
		put("果", "2654");
		put("枝", "2655");
		put("枇", "2657");
		put("枋", "2658");
		put("枘", "2660");
		put("枯", "2661");
		put("枳", "2663");
		put("枵", "2664");
		put("架", "2665");
		put("枷", "2666");
		put("枸", "2667");
		put("柁", "2670");
		put("柄", "2671");
		put("柏", "2672");
		put("某", "2673");
		put("柑", "2674");
		put("柒", "2675");
		put("染", "2676");
		put("柔", "2677");
		put("柘", "2678");
		put("柙", "2679");
		put("柚", "2680");
		put("柜", "2681");
		put("柝", "2682");
		put("柞", "2683");
		put("柢", "2685");
		put("查", "2686");
		put("柬", "2687");
		put("柯", "2688");
		put("柮", "2689");
		put("柰", "2690");
		put("柱", "2691");
		put("柳", "2692");
		put("柴", "2693");
		put("栅", "2694");
		put("枰", "2695");
		put("樋", "2697");
		put("栗", "2698");
		put("校", "2699");
		put("栩", "2700");
		put("株", "2701");
		put("核", "2702");
		put("根", "2704");
		put("槾", "2705");
		put("格", "2706");
		put("栽", "2707");
		put("桀", "2708");
		put("桁", "2709");
		put("桂", "2710");
		put("桃", "2711");
		put("桅", "2712");
		put("框", "2713");
		put("案", "2714");
		put("桌", "2715");
		put("桎", "2716");
		put("桐", "2717");
		put("桑", "2718");
		put("桓", "2719");
		put("桔", "2720");
		put("桕", "2721");
		put("最", "2584");
		put("会", "2585");
		put("月", "2588");
		put("有", "2589");
		put("朋", "2590");
		put("服", "2591");
		put("朔", "2592");
		put("朐", "2593");
		put("朓", "2594");
		put("朕", "2596");
		put("朗", "2597");
		put("望", "2598");
		put("朝", "2600");
		put("期", "2601");
		put("朦", "2603");
		put("胧", "2604");
		put("耠", "2605");
		put("木", "2606");
		put("未", "2607");
		put("末", "2608");
		put("本", "2609");
		put("札", "2610");
		put("术", "2611");
		put("朱", "2612");
		put("朴", "2613");
		put("朵", "2614");
		put("朽", "2615");
		put("杆", "2616");
		put("杈", "2617");
		put("杷", "2618");
		put("弃", "2757");
		put("棉", "2758");
		put("棋", "2759");
		put("棍", "2760");
		put("棒", "2761");
		put("棕", "2762");
		put("枨", "2763");
		put("枣", "2764");
		put("棘", "2765");
		put("棚", "2766");
		put("栋", "2767");
		put("棠", "2768");
		put("棣", "2769");
		put("栈", "2770");
		put("棫", "2771");
		put("棬", "2772");
		put("森", "2773");
		put("棰", "2774");
		put("楦", "2775");
		put("棹", "2777");
		put("棺", "2778");
		put("椁", "2779");
		put("棻", "2780");
		put("棼", "2781");
		put("椅", "2783");
		put("植", "2784");
		put("椎", "2785");
		put("椒", "2786");
		put("棐", "2787");
		put("棓", "2788");
		put("棱", "2789");
		put("棨", "2790");
		put("椐", "2791");
		put("椓", "2792");
		put("椰", "2794");
		put("椹", "2795");
		put("椽", "2796");
		put("椿", "2797");
		put("杨", "2799");
		put("枫", "2800");
		put("楔", "2801");
		put("栉", "2802");
		put("楚", "2806");
		put("楞", "2807");
		put("楝", "2808");
		put("楠", "2809");
		put("榆", "2810");
		put("楢", "2811");
		put("楣", "2812");
		put("楫", "2813");
		put("业", "2814");
		put("楯", "2815");
		put("楮", "2816");
		put("极", "2817");
		put("楷", "2818");
		put("楹", "2819");
		put("槠", "2822");
		put("桢", "2823");
		put("楩", "2824");
		put("楸", "2825");
		put("榔", "2826");
		put("榕", "2827");
		put("榖", "2828");
		put("榘", "2829");
		put("榛", "2830");
		put("榜", "2831");
		put("榧", "2833");
		put("榨", "2834");
		put("榫", "2835");
		put("榭", "2836");
		put("荣", "2837");
		put("榱", "2838");
		put("榴", "2839");
		put("榻", "2840");
		put("榾", "2841");
		put("槁", "2842");
		put("槅", "2843");
		put("槊", "2844");
		put("构", "2845");
		put("槌", "2846");
		put("枪", "2847");
		put("槎", "2848");
		put("槐", "2849");
		put("榷", "2854");
		put("榼", "2855");
		put("樑", "2856");
		put("椠", "2858");
		put("栖", "2722");
		put("栲", "2723");
		put("栳", "2724");
		put("桄", "2725");
		put("桫", "2726");
		put("桴", "2727");
		put("桶", "2729");
		put("桷", "2730");
		put("梃", "2732");
		put("梁", "2733");
		put("梅", "2734");
		put("梆", "2735");
		put("梏", "2736");
		put("梓", "2737");
		put("栀", "2738");
		put("梗", "2739");
		put("条", "2742");
		put("枭", "2743");
		put("梢", "2744");
		put("梧", "2745");
		put("梨", "2746");
		put("梭", "2747");
		put("梯", "2748");
		put("梱", "2749");
		put("械", "2750");
		put("棁", "2751");
		put("梳", "2752");
		put("梵", "2753");
		put("桯", "2756");
		put("橡", "2895");
		put("横", "2897");
		put("樨", "2898");
		put("柠", "2899");
		put("椭", "2900");
		put("桦", "2901");
		put("檖", "2903");
		put("橘", "2904");
		put("檀", "2905");
		put("柽", "2906");
		put("檄", "2907");
		put("檐", "2908");
		put("档", "2909");
		put("桧", "2910");
		put("槚", "2911");
		put("檠", "2913");
		put("检", "2914");
		put("樯", "2915");
		put("檬", "2916");
		put("梼", "2917");
		put("槟", "2919");
		put("槛", "2920");
		put("橹", "2924");
		put("榈", "2925");
		put("椟", "2927");
		put("橼", "2928");
		put("栎", "2929");
		put("枥", "2931");
		put("榇", "2932");
		put("棂", "2933");
		put("栊", "2934");
		put("栏", "2936");
		put("樱", "2937");
		put("权", "2938");
		put("栾", "2940");
		put("榄", "2941");
		put("柩", "2943");
		put("欠", "2944");
		put("次", "2945");
		put("欣", "2946");
		put("欲", "2948");
		put("款", "2949");
		put("欷", "2950");
		put("欹", "2951");
		put("欺", "2952");
		put("钦", "2953");
		put("歃", "2955");
		put("歆", "2956");
		put("歇", "2957");
		put("歉", "2959");
		put("歌", "2960");
		put("欧", "2962");
		put("歔", "2963");
		put("歙", "2965");
		put("欤", "2968");
		put("欢", "2970");
		put("止", "2972");
		put("正", "2973");
		put("此", "2974");
		put("步", "2975");
		put("武", "2976");
		put("歪", "2977");
		put("歧", "2978");
		put("岁", "2979");
		put("历", "2980");
		put("归", "2981");
		put("钬", "2982");
		put("歹", "2983");
		put("死", "2984");
		put("殁", "2985");
		put("殂", "2987");
		put("殃", "2988");
		put("殄", "2989");
		put("殆", "2990");
		put("殉", "2991");
		put("殊", "2992");
		put("殍", "2993");
		put("殖", "2994");
		put("残", "2995");
		put("殕", "2996");
		put("殛", "2997");
		put("概", "2861");
		put("桨", "2862");
		put("槔", "2863");
		put("槽", "2864");
		put("槿", "2865");
		put("桩", "2866");
		put("乐", "2867");
		put("樊", "2868");
		put("楼", "2869");
		put("橦", "2870");
		put("标", "2871");
		put("枢", "2873");
		put("樟", "2874");
		put("模", "2875");
		put("样", "2876");
		put("槥", "2877");
		put("枞", "2878");
		put("樗", "2880");
		put("檩", "2882");
		put("樵", "2884");
		put("树", "2885");
		put("樽", "2886");
		put("樾", "2887");
		put("撖", "2888");
		put("桡", "2889");
		put("桥", "2890");
		put("橐", "2891");
		put("橙", "2892");
		put("橛", "2893");
		put("毯", "3034");
		put("毳", "3035");
		put("牦", "3036");
		put("氅", "3037");
		put("毹", "3039");
		put("氍", "3042");
		put("氏", "3044");
		put("氐", "3045");
		put("民", "3046");
		put("氓", "3047");
		put("氯", "3048");
		put("气", "3049");
		put("氛", "3050");
		put("氤", "3052");
		put("氲", "3053");
		put("浠", "3054");
		put("水", "3055");
		put("永", "3057");
		put("氾", "3058");
		put("汁", "3059");
		put("汀", "3060");
		put("求", "3061");
		put("汗", "3063");
		put("污", "3064");
		put("汛", "3065");
		put("汜", "3066");
		put("汝", "3067");
		put("江", "3068");
		put("池", "3069");
		put("汊", "3070");
		put("汐", "3071");
		put("汔", "3072");
		put("汕", "3073");
		put("汞", "3074");
		put("汨", "3075");
		put("汪", "3076");
		put("汰", "3077");
		put("汲", "3078");
		put("汴", "3079");
		put("汶", "3080");
		put("汹", "3081");
		put("汾", "3083");
		put("沁", "3084");
		put("沂", "3085");
		put("汽", "3086");
		put("沃", "3087");
		put("沈", "3088");
		put("沉", "3089");
		put("沌", "3090");
		put("沐", "3092");
		put("没", "3093");
		put("沔", "3094");
		put("沖", "3095");
		put("沙", "3097");
		put("沚", "3098");
		put("沛", "3099");
		put("沆", "3100");
		put("沓", "3101");
		put("汩", "3102");
		put("汭", "3103");
		put("沅", "3104");
		put("沫", "3106");
		put("沮", "3107");
		put("沱", "3108");
		put("河", "3109");
		put("沸", "3110");
		put("油", "3111");
		put("治", "3112");
		put("沼", "3113");
		put("沽", "3114");
		put("沾", "3115");
		put("沿", "3116");
		put("洺", "3117");
		put("泄", "3118");
		put("泵", "3119");
		put("泅", "3120");
		put("泔", "3121");
		put("泆", "3122");
		put("泉", "3123");
		put("泊", "3124");
		put("泌", "3125");
		put("泓", "3126");
		put("法", "3127");
		put("泗", "3128");
		put("洩", "3129");
		put("泛", "3131");
		put("泠", "3132");
		put("泡", "3133");
		put("波", "3134");
		put("泣", "3135");
		put("殒", "2998");
		put("殇", "2999");
		put("殚", "3000");
		put("殪", "3002");
		put("殓", "3003");
		put("殡", "3004");
		put("歼", "3005");
		put("镓", "3006");
		put("殳", "3007");
		put("段", "3008");
		put("殷", "3009");
		put("杀", "3010");
		put("殿", "3013");
		put("毁", "3014");
		put("毅", "3015");
		put("殴", "3016");
		put("椵", "3017");
		put("母", "3018");
		put("毋", "3019");
		put("每", "3020");
		put("毒", "3021");
		put("毓", "3022");
		put("比", "3024");
		put("毖", "3025");
		put("毗", "3026");
		put("鲣", "3028");
		put("毛", "3029");
		put("毡", "3030");
		put("毫", "3032");
		put("洸", "3171");
		put("活", "3172");
		put("洼", "3173");
		put("洽", "3174");
		put("派", "3175");
		put("洨", "3176");
		put("流", "3177");
		put("洙", "3178");
		put("洚", "3179");
		put("洹", "3180");
		put("浙", "3181");
		put("浚", "3182");
		put("浣", "3183");
		put("浦", "3184");
		put("浩", "3185");
		put("浪", "3186");
		put("浮", "3187");
		put("浴", "3188");
		put("海", "3189");
		put("浸", "3190");
		put("浃", "3191");
		put("浬", "3192");
		put("泾", "3193");
		put("消", "3194");
		put("涉", "3195");
		put("淳", "3196");
		put("涓", "3197");
		put("涔", "3198");
		put("涕", "3199");
		put("涞", "3200");
		put("涢", "3201");
		put("浡", "3202");
		put("浜", "3203");
		put("浥", "3204");
		put("涂", "3205");
		put("涅", "3206");
		put("湟", "3207");
		put("涑", "3208");
		put("涯", "3209");
		put("液", "3210");
		put("涵", "3211");
		put("涸", "3212");
		put("涿", "3214");
		put("淅", "3215");
		put("淆", "3216");
		put("淇", "3217");
		put("淋", "3218");
		put("淑", "3219");
		put("渊", "3220");
		put("淘", "3221");
		put("窄", "4504");
		put("窅", "4505");
		put("窆", "4506");
		put("窈", "4507");
		put("窒", "4510");
		put("窕", "4511");
		put("窖", "4512");
		put("窗", "4514");
		put("窘", "4515");
		put("祟", "4378");
		put("祠", "4379");
		put("祲", "4381");
		put("祥", "4382");
		put("祧", "4383");
		put("票", "4384");
		put("祭", "4385");
		put("祺", "4388");
		put("禄", "4389");
		put("禀", "4390");
		put("禁", "4391");
		put("祸", "4393");
		put("祯", "4394");
		put("福", "4395");
		put("禘", "4396");
		put("禊", "4397");
		put("祎", "4400");
		put("祃", "4401");
		put("禠", "4404");
		put("禧", "4406");
		put("禅", "4407");
		put("礼", "4409");
		put("祢", "4410");
		put("祷", "4411");
		put("禳", "4412");
		put("端", "4551");
		put("竞", "4552");
		put("铑", "4553");
		put("竹", "4554");
		put("竺", "4555");
		put("竽", "4556");
		put("竿", "4557");
		put("笄", "4558");
		put("笈", "4559");
		put("笊", "4560");
		put("笏", "4561");
		put("笑", "4562");
		put("笙", "4563");
		put("笛", "4564");
		put("笞", "4565");
		put("笤", "4566");
		put("笠", "4567");
		put("笥", "4568");
		put("符", "4569");
		put("笨", "4570");
		put("笋", "4571");
		put("笪", "4572");
		put("笫", "4573");
		put("第", "4574");
		put("笮", "4575");
		put("笆", "4576");
		put("笱", "4578");
		put("笳", "4579");
		put("筅", "4580");
		put("笔", "4581");
		put("筇", "4582");
		put("等", "4583");
		put("筊", "4584");
		put("筋", "4585");
		put("筌", "4586");
		put("筎", "4588");
		put("筏", "4589");
		put("筐", "4590");
		put("筑", "4591");
		put("筒", "4592");
		put("答", "4594");
		put("策", "4595");
		put("筠", "4596");
		put("筷", "4599");
		put("笕", "4600");
		put("筮", "4602");
		put("筲", "4603");
		put("筵", "4605");
		put("筱", "4607");
		put("笺", "4608");
		put("箍", "4611");
		put("箇", "4612");
		put("箔", "4613");
		put("箕", "4614");
		put("算", "4615");
		put("箝", "4617");
		put("管", "4619");
		put("筝", "4620");
		put("箐", "4621");
		put("箜", "4625");
		put("箬", "4627");
		put("箭", "4628");
		put("箱", "4630");
		put("箴", "4631");
		put("箸", "4632");
		put("节", "4634");
		put("篁", "4635");
		put("篆", "4637");
		put("篇", "4638");
		put("箧", "4640");
		put("篙", "4643");
		put("篚", "4644");
		put("篝", "4645");
		put("筿", "4646");
		put("篡", "4647");
		put("笃", "4648");
		put("筛", "4652");
		put("篦", "4653");
		put("窟", "4516");
		put("窠", "4517");
		put("窨", "4518");
		put("窝", "4519");
		put("窬", "4521");
		put("穷", "4522");
		put("窑", "4523");
		put("窳", "4524");
		put("窎", "4525");
		put("窭", "4526");
		put("窸", "4527");
		put("窣", "4528");
		put("窥", "4529");
		put("窾", "4530");
		put("窜", "4533");
		put("窍", "4534");
		put("窦", "4535");
		put("窃", "4537");
		put("蒗", "4538");
		put("立", "4539");
		put("竑", "4540");
		put("站", "4541");
		put("竟", "4544");
		put("章", "4545");
		put("竣", "4546");
		put("童", "4547");
		put("竦", "4548");
		put("竖", "4549");
		put("竭", "4550");
		put("簿", "4689");
		put("篮", "4691");
		put("筹", "4693");
		put("籍", "4694");
		put("籐", "4696");
		put("籀", "4699");
		put("箓", "4700");
		put("篯", "4702");
		put("箨", "4703");
		put("籁", "4704");
		put("笼", "4705");
		put("笾", "4710");
		put("箩", "4712");
		put("篱", "4713");
		put("篰", "4716");
		put("米", "4717");
		put("粉", "4720");
		put("粒", "4721");
		put("粕", "4722");
		put("粗", "4723");
		put("粘", "4724");
		put("粟", "4725");
		put("粢", "4726");
		put("粤", "4727");
		put("粥", "4728");
		put("籼", "4729");
		put("粱", "4731");
		put("粲", "4732");
		put("粹", "4733");
		put("粳", "4734");
		put("粼", "4736");
		put("精", "4737");
		put("糈", "4738");
		put("糊", "4739");
		put("糕", "4741");
		put("糇", "4742");
		put("糖", "4743");
		put("糗", "4744");
		put("糜", "4745");
		put("糁", "4746");
		put("粪", "4747");
		put("糟", "4748");
		put("糠", "4749");
		put("籽", "4750");
		put("糙", "4751");
		put("糯", "4754");
		put("粝", "4755");
		put("籴", "4756");
		put("粑", "4757");
		put("粜", "4759");
		put("", "4761");
		put("系", "4762");
		put("纠", "4763");
		put("纪", "4764");
		put("纣", "4765");
		put("约", "4766");
		put("红", "4767");
		put("纡", "4768");
		put("纥", "4769");
		put("纨", "4770");
		put("纫", "4771");
		put("紊", "4772");
		put("纹", "4773");
		put("缐", "4775");
		put("纴", "4777");
		put("纳", "4780");
		put("纽", "4781");
		put("纾", "4782");
		put("纯", "4783");
		put("纱", "4784");
		put("纮", "4785");
		put("纸", "4786");
		put("级", "4787");
		put("纷", "4788");
		put("纭", "4789");
		put("素", "4790");
		put("纺", "4791");
		put("篪", "4654");
		put("篲", "4655");
		put("筚", "4656");
		put("篷", "4659");
		put("箦", "4660");
		put("簕", "4661");
		put("簇", "4662");
		put("簋", "4663");
		put("篓", "4664");
		put("簌", "4665");
		put("簏", "4666");
		put("篌", "4667");
		put("篾", "4668");
		put("簃", "4669");
		put("簟", "4672");
		put("箪", "4673");
		put("簠", "4674");
		put("简", "4675");
		put("篑", "4676");
		put("簦", "4678");
		put("簧", "4679");
		put("簪", "4681");
		put("箫", "4682");
		put("簸", "4684");
		put("签", "4687");
		put("菊", "5468");
		put("菑", "5469");
		put("菖", "5471");
		put("菔", "5472");
		put("菘", "5473");
		put("菠", "5474");
		put("菜", "5475");
		put("菩", "5476");
		put("堇", "5477");
		put("华", "5478");
		put("菰", "5479");
		put("菱", "5480");
		put("菲", "5481");
		put("芑", "5344");
		put("芒", "5345");
		put("芙", "5346");
		put("芝", "5347");
		put("芟", "5348");
		put("芡", "5349");
		put("艿", "5350");
		put("芤", "5351");
		put("芧", "5352");
		put("芨", "5353");
		put("芥", "5354");
		put("芩", "5355");
		put("芪", "5356");
		put("芫", "5357");
		put("芬", "5358");
		put("芭", "5359");
		put("芮", "5360");
		put("芯", "5361");
		put("芰", "5362");
		put("花", "5363");
		put("芳", "5364");
		put("芷", "5365");
		put("芸", "5366");
		put("芹", "5367");
		put("刍", "5368");
		put("芼", "5369");
		put("芽", "5370");
		put("莜", "5371");
		put("芾", "5372");
		put("苑", "5373");
		put("茆", "5374");
		put("苒", "5375");
		put("苓", "5376");
		put("苔", "5377");
		put("苕", "5378");
		put("苇", "5517");
		put("葩", "5518");
		put("葫", "5519");
		put("葬", "5520");
		put("葭", "5521");
		put("药", "5522");
		put("葱", "5523");
		put("葳", "5524");
		put("葵", "5525");
		put("荤", "5526");
		put("葸", "5527");
		put("葺", "5528");
		put("蒂", "5530");
		put("葑", "5531");
		put("莳", "5535");
		put("蒙", "5536");
		put("蒜", "5537");
		put("蒡", "5538");
		put("蒞", "5539");
		put("蒟", "5540");
		put("蒨", "5541");
		put("蒲", "5543");
		put("蒸", "5544");
		put("蒹", "5545");
		put("蒺", "5546");
		put("苍", "5547");
		put("蒿", "5548");
		put("荪", "5549");
		put("蓁", "5550");
		put("蓂", "5551");
		put("蓄", "5552");
		put("蓆", "5553");
		put("蓉", "5554");
		put("蓊", "5555");
		put("盖", "5556");
		put("蓖", "5557");
		put("蓍", "5558");
		put("蓏", "5559");
		put("蓑", "5560");
		put("蓐", "5562");
		put("蓓", "5563");
		put("萏", "5564");
		put("蔌", "5565");
		put("蒯", "5566");
		put("蓬", "5570");
		put("莲", "5571");
		put("蓰", "5572");
		put("莼", "5573");
		put("蔻", "5575");
		put("蔫", "5576");
		put("芗", "5577");
		put("蓼", "5578");
		put("蓿", "5581");
		put("蔑", "5583");
		put("蕻", "5584");
		put("蔓", "5585");
		put("蔗", "5587");
		put("蔚", "5588");
		put("蒌", "5589");
		put("蔡", "5591");
		put("蒋", "5592");
		put("荫", "5593");
		put("蔴", "5594");
		put("茑", "5596");
		put("荜", "5598");
		put("蔽", "5599");
		put("蔬", "5600");
		put("蕺", "5601");
		put("蕃", "5603");
		put("蕉", "5604");
		put("蕊", "5605");
		put("荞", "5606");
		put("莸", "5607");
		put("蕖", "5608");
		put("荛", "5609");
		put("蕙", "5610");
		put("蕞", "5612");
		put("蒉", "5614");
		put("蕨", "5615");
		put("荡", "5616");
		put("芜", "5617");
		put("萧", "5618");
		put("蕰", "5619");
		put("莙", "5483");
		put("菹", "5484");
		put("菼", "5485");
		put("菽", "5486");
		put("萁", "5487");
		put("萃", "5488");
		put("萜", "5489");
		put("莱", "5490");
		put("萋", "5491");
		put("萌", "5492");
		put("萍", "5493");
		put("萎", "5494");
		put("萑", "5495");
		put("菡", "5496");
		put("菌", "5497");
		put("萩", "5498");
		put("苌", "5500");
		put("萼", "5501");
		put("萱", "5503");
		put("莴", "5504");
		put("萸", "5505");
		put("萹", "5506");
		put("落", "5507");
		put("葆", "5508");
		put("菸", "5510");
		put("著", "5511");
		put("葚", "5513");
		put("葛", "5514");
		put("葡", "5515");
		put("董", "5516");
	}

	static void init2() {
		put("藉", "5659");
		put("荩", "5660");
		put("藏", "5661");
		put("藐", "5662");
		put("蓝", "5663");
		put("藁", "5664");
		put("藕", "5665");
		put("藜", "5668");
		put("艺", "5669");
		put("藤", "5671");
		put("藩", "5672");
		put("薮", "5674");
		put("蔼", "5676");
		put("蔺", "5677");
		put("藻", "5679");
		put("藿", "5681");
		put("蕲", "5682");
		put("蘅", "5683");
		put("芦", "5684");
		put("苏", "5685");
		put("蕴", "5686");
		put("夔", "5688");
		put("萚", "5690");
		put("藓", "5691");
		put("蔹", "5692");
		put("茏", "5693");
		put("蘩", "5694");
		put("兰", "5695");
		put("蘧", "5698");
		put("蘸", "5699");
		put("萝", "5700");
		put("蘼", "5701");
		put("蘑", "5704");
		put("虍", "5705");
		put("虎", "5706");
		put("虐", "5707");
		put("虔", "5709");
		put("处", "5710");
		put("虚", "5711");
		put("虏", "5712");
		put("虞", "5713");
		put("号", "5714");
		put("虢", "5716");
		put("亏", "5719");
		put("醚", "5721");
		put("虫", "5722");
		put("虱", "5723");
		put("虻", "5724");
		put("虹", "5725");
		put("虺", "5726");
		put("蚓", "5727");
		put("蚍", "5728");
		put("蚕", "5729");
		put("蚊", "5730");
		put("蚋", "5731");
		put("蚌", "5732");
		put("蚣", "5733");
		put("蚤", "5734");
		put("蚧", "5735");
		put("蚩", "5736");
		put("蚪", "5737");
		put("蚨", "5738");
		put("蚯", "5739");
		put("蚰", "5740");
		put("蚱", "5741");
		put("蚜", "5742");
		put("蚶", "5743");
		put("蛄", "5745");
		put("蛀", "5746");
		put("蛆", "5747");
		put("蛇", "5748");
		put("蛉", "5749");
		put("蛑", "5750");
		put("蛋", "5751");
		put("蛙", "5752");
		put("蛛", "5753");
		put("蛟", "5754");
		put("蚴", "5755");
		put("蛤", "5756");
		put("蛩", "5757");
		put("薯", "5620");
		put("蒇", "5621");
		put("蕈", "5622");
		put("蕤", "5624");
		put("蓣", "5626");
		put("蕾", "5628");
		put("薢", "5630");
		put("薄", "5631");
		put("薅", "5632");
		put("薇", "5633");
		put("荟", "5634");
		put("蓟", "5636");
		put("蔷", "5638");
		put("薛", "5641");
		put("孽", "5642");
		put("薜", "5643");
		put("薨", "5645");
		put("萨", "5646");
		put("薪", "5647");
		put("薤", "5648");
		put("蘖", "5649");
		put("薏", "5650");
		put("薰", "5651");
		put("薳", "5652");
		put("薹", "5653");
		put("荠", "5654");
		put("蚀", "5793");
		put("蝗", "5794");
		put("蝘", "5795");
		put("蝙", "5796");
		put("蜅", "5797");
		put("猬", "5798");
		put("蝠", "5799");
		put("蜿", "5800");
		put("蝥", "5801");
		put("虾", "5802");
		put("蝮", "5803");
		put("蝶", "5805");
		put("蜷", "5806");
		put("蟮", "5807");
		put("蜻", "5808");
		put("蝌", "5809");
		put("蜾", "5811");
		put("蝣", "5812");
		put("蝻", "5813");
		put("蝴", "5814");
		put("蜗", "5815");
		put("融", "5816");
		put("蠓", "5817");
		put("蚂", "5818");
		put("螨", "5819");
		put("螟", "5820");
		put("萤", "5821");
		put("蚝", "5823");
		put("螃", "5824");
		put("蛳", "5825");
		put("螫", "5826");
		put("螺", "5828");
		put("蝼", "5829");
		put("螽", "5830");
		put("", "5831");
		put("蛰", "5832");
		put("蟊", "5833");
		put("螬", "5834");
		put("螭", "5835");
		put("螯", "5837");
		put("螳", "5838");
		put("蟀", "5839");
		put("蟆", "5840");
		put("蝈", "5841");
		put("蟋", "5842");
		put("虮", "5843");
		put("蟪", "5844");
		put("蟒", "5845");
		put("蟛", "5846");
		put("蟠", "5847");
		put("蝉", "5848");
		put("蟹", "5851");
		put("蚁", "5852");
		put("蟾", "5853");
		put("蛏", "5855");
		put("蠃", "5856");
		put("蚃", "5858");
		put("蝇", "5859");
		put("虿", "5860");
		put("蠕", "5862");
		put("蠖", "5863");
		put("蠛", "5864");
		put("蠡", "5867");
		put("蠢", "5868");
		put("蛎", "5869");
		put("蟏", "5870");
		put("蠹", "5871");
		put("蛊", "5872");
		put("蠲", "5873");
		put("蛮", "5875");
		put("虬", "5876");
		put("血", "5877");
		put("嘧", "5878");
		put("衄", "5880");
		put("衅", "5881");
		put("众", "5883");
		put("嚜", "5886");
		put("行", "5887");
		put("衍", "5888");
		put("衒", "5891");
		put("街", "5894");
		put("衙", "5895");
		put("蛱", "5758");
		put("蛭", "5759");
		put("蛾", "5760");
		put("蛘", "5761");
		put("蜂", "5762");
		put("蜃", "5763");
		put("蜇", "5764");
		put("蜈", "5765");
		put("蜓", "5767");
		put("蛸", "5768");
		put("蛹", "5769");
		put("蜕", "5770");
		put("蜀", "5771");
		put("蜉", "5772");
		put("蜍", "5773");
		put("蜊", "5774");
		put("蜘", "5776");
		put("蜚", "5777");
		put("蜜", "5778");
		put("蜡", "5779");
		put("蜣", "5780");
		put("蜩", "5781");
		put("蜥", "5782");
		put("蜒", "5783");
		put("蛔", "5784");
		put("蜴", "5785");
		put("蜞", "5786");
		put("螂", "5788");
		put("蜢", "5789");
		put("蜮", "5790");
		put("蝎", "5791");
		put("蛐", "5792");
		put("袱", "5931");
		put("裁", "5932");
		put("裂", "5933");
		put("袅", "5934");
		put("裎", "5935");
		put("裀", "5936");
		put("裒", "5938");
		put("裔", "5939");
		put("裕", "5940");
		put("裘", "5941");
		put("裙", "5942");
		put("补", "5943");
		put("装", "5944");
		put("裟", "5945");
		put("裨", "5947");
		put("裰", "5949");
		put("裱", "5950");
		put("裳", "5951");
		put("裴", "5952");
		put("裸", "5953");
		put("裼", "5954");
		put("裹", "5955");
		put("裾", "5957");
		put("褊", "5959");
		put("裈", "5960");
		put("褂", "5962");
		put("褪", "5963");
		put("褐", "5964");
		put("褓", "5965");
		put("袆", "5966");
		put("褙", "5967");
		put("褡", "5968");
		put("褚", "5969");
		put("褥", "5970");
		put("褫", "5972");
		put("褰", "5973");
		put("裤", "5974");
		put("褦", "5975");
		put("褶", "5977");
		put("亵", "5978");
		put("襄", "5980");
		put("衮", "5981");
		put("襁", "5982");
		put("袢", "5983");
		put("袄", "5984");
		put("褒", "5988");
		put("裣", "5990");
		put("裆", "5991");
		put("襟", "5992");
		put("襦", "5993");
		put("褴", "5996");
		put("褛", "5997");
		put("衬", "6000");
		put("袭", "6002");
		put("襶", "6004");
		put("襾", "6006");
		put("西", "6007");
		put("要", "6008");
		put("覃", "6009");
		put("覆", "6010");
		put("镎", "6014");
		put("见", "6015");
		put("规", "6016");
		put("觅", "6017");
		put("视", "6018");
		put("觇", "6019");
		put("觋", "6021");
		put("觎", "6022");
		put("亲", "6024");
		put("觊", "6025");
		put("觏", "6026");
		put("觐", "6027");
		put("觑", "6028");
		put("觃", "6029");
		put("觉", "6030");
		put("览", "6031");
		put("觌", "6032");
		put("觍", "6033");
		put("卫", "5898");
		put("衡", "5899");
		put("衢", "5900");
		put("衣", "5902");
		put("表", "5903");
		put("衫", "5904");
		put("衰", "5905");
		put("衲", "5906");
		put("衷", "5907");
		put("衩", "5908");
		put("衽", "5910");
		put("衾", "5911");
		put("衿", "5912");
		put("袁", "5913");
		put("袈", "5914");
		put("袋", "5915");
		put("袍", "5916");
		put("袒", "5917");
		put("袖", "5918");
		put("袜", "5920");
		put("袤", "5924");
		put("袪", "5925");
		put("被", "5926");
		put("袂", "5927");
		put("讧", "6069");
		put("讱", "6072");
		put("讹", "6073");
		put("讶", "6074");
		put("讼", "6075");
		put("诀", "6076");
		put("讷", "6077");
		put("访", "6078");
		put("许", "6079");
		put("设", "6080");
		put("讻", "6081");
		put("诉", "6083");
		put("诃", "6084");
		put("诊", "6085");
		put("证", "6086");
		put("訾", "6088");
		put("诂", "6089");
		put("诋", "6090");
		put("詈", "6091");
		put("讵", "6093");
		put("诈", "6094");
		put("诒", "6095");
		put("诏", "6096");
		put("评", "6097");
		put("诎", "6099");
		put("诅", "6100");
		put("词", "6101");
		put("诩", "6103");
		put("询", "6104");
		put("诣", "6105");
		put("试", "6107");
		put("诗", "6108");
		put("诧", "6109");
		put("诟", "6110");
		put("诡", "6111");
		put("诠", "6112");
		put("诘", "6113");
		put("话", "6114");
		put("该", "6115");
		put("详", "6116");
		put("诙", "6117");
		put("诖", "6118");
		put("诓", "6119");
		put("诔", "6120");
		put("诛", "6121");
		put("詹", "6124");
		put("认", "6126");
		put("诳", "6128");
		put("誓", "6129");
		put("诞", "6130");
		put("诱", "6131");
		put("诮", "6132");
		put("语", "6133");
		put("诚", "6134");
		put("诫", "6135");
		put("诬", "6136");
		put("误", "6137");
		put("诰", "6138");
		put("诵", "6139");
		put("诲", "6140");
		put("说", "6141");
		put("谁", "6142");
		put("课", "6143");
		put("谇", "6144");
		put("诽", "6145");
		put("谊", "6146");
		put("訚", "6147");
		put("调", "6148");
		put("谄", "6149");
		put("谆", "6150");
		put("谈", "6151");
		put("诿", "6152");
		put("请", "6153");
		put("诤", "6154");
		put("诹", "6155");
		put("谅", "6156");
		put("诼", "6157");
		put("论", "6158");
		put("谂", "6159");
		put("谀", "6161");
		put("谞", "6162");
		put("谝", "6163");
		put("谔", "6166");
		put("谛", "6167");
		put("谐", "6168");
		put("谏", "6169");
		put("谕", "6170");
		put("谘", "6171");
		put("观", "6034");
		put("氖", "6036");
		put("角", "6037");
		put("觔", "6038");
		put("觖", "6039");
		put("觚", "6040");
		put("解", "6043");
		put("觥", "6044");
		put("觫", "6046");
		put("觱", "6048");
		put("觳", "6049");
		put("觞", "6050");
		put("触", "6051");
		put("萘", "6055");
		put("言", "6056");
		put("订", "6057");
		put("讣", "6058");
		put("訇", "6059");
		put("计", "6060");
		put("讯", "6061");
		put("讨", "6062");
		put("讦", "6063");
		put("训", "6064");
		put("讪", "6065");
		put("讫", "6066");
		put("讬", "6067");
		put("记", "6068");
		put("谪", "6207");
		put("谬", "6208");
		put("讴", "6209");
		put("谨", "6210");
		put("谩", "6211");
		put("谲", "6216");
		put("讥", "6217");
		put("谮", "6220");
		put("识", "6221");
		put("谯", "6222");
		put("谭", "6223");
		put("谱", "6225");
		put("警", "6226");
		put("谵", "6228");
		put("譬", "6229");
		put("译", "6230");
		put("议", "6231");
		put("谴", "6232");
		put("护", "6233");
		put("诪", "6234");
		put("誉", "6235");
		put("读", "6236");
		put("谫", "6237");
		put("变", "6239");
		put("雠", "6241");
		put("詟", "6242");
		put("谗", "6243");
		put("谶", "6244");
		put("让", "6245");
		put("谠", "6249");
		put("谳", "6250");
		put("谷", "6253");
		put("豁", "6255");
		put("硇", "6257");
		put("豆", "6258");
		put("岂", "6259");
		put("豌", "6261");
		put("豉", "6262");
		put("豇", "6263");
		put("豕", "6269");
		put("豚", "6270");
		put("象", "6272");
		put("豢", "6273");
		put("豨", "6274");
		put("豪", "6275");
		put("豫", "6276");
		put("豭", "6278");
		put("豳", "6279");
		put("铌", "6281");
		put("豸", "6282");
		put("豹", "6283");
		put("豺", "6284");
		put("貂", "6285");
		put("貆", "6286");
		put("貅", "6287");
		put("貉", "6288");
		put("貊", "6289");
		put("貌", "6290");
		put("貔", "6293");
		put("脲", "6295");
		put("贝", "6296");
		put("贞", "6297");
		put("负", "6298");
		put("财", "6299");
		put("贡", "6300");
		put("贫", "6302");
		put("货", "6303");
		put("贪", "6304");
		put("贩", "6305");
		put("贯", "6306");
		put("责", "6307");
		put("贮", "6308");
		put("讳", "6172");
		put("谙", "6173");
		put("讽", "6174");
		put("诸", "6175");
		put("谚", "6176");
		put("谖", "6178");
		put("诺", "6179");
		put("谋", "6180");
		put("谒", "6181");
		put("谓", "6182");
		put("谍", "6183");
		put("谌", "6186");
		put("誊", "6187");
		put("诌", "6188");
		put("謇", "6190");
		put("谎", "6192");
		put("谜", "6193");
		put("谧", "6194");
		put("谑", "6195");
		put("谤", "6196");
		put("谦", "6197");
		put("谥", "6198");
		put("讲", "6199");
		put("谢", "6200");
		put("谣", "6202");
		put("谡", "6204");
		put("謦", "6205");
		put("谟", "6206");
		put("贱", "6345");
		put("赋", "6346");
		put("质", "6347");
		put("账", "6348");
		put("赌", "6350");
		put("赖", "6351");
		put("赗", "6352");
		put("赚", "6354");
		put("赙", "6355");
		put("购", "6356");
		put("赛", "6357");
		put("赜", "6358");
		put("贽", "6360");
		put("赘", "6361");
		put("赠", "6362");
		put("赞", "6363");
		put("赡", "6365");
		put("赢", "6366");
		put("赆", "6367");
		put("赃", "6368");
		put("赑", "6369");
		put("赎", "6370");
		put("赝", "6372");
		put("赣", "6373");
		put("赤", "6375");
		put("赦", "6376");
		put("赧", "6377");
		put("赫", "6378");
		put("赭", "6379");
		put("赪", "6380");
		put("钕", "6381");
		put("走", "6382");
		put("赳", "6383");
		put("赴", "6384");
		put("赶", "6385");
		put("起", "6386");
		put("趁", "6387");
		put("趄", "6388");
		put("超", "6389");
		put("越", "6390");
		put("赵", "6392");
		put("趣", "6393");
		put("趑", "6394");
		put("趋", "6395");
		put("趟", "6396");
		put("蒎", "6397");
		put("足", "6398");
		put("趵", "6399");
		put("趾", "6400");
		put("趺", "6402");
		put("跋", "6405");
		put("跌", "6407");
		put("跎", "6408");
		put("跏", "6409");
		put("跑", "6410");
		put("跖", "6412");
		put("跚", "6413");
		put("跛", "6414");
		put("距", "6415");
		put("跫", "6417");
		put("跟", "6418");
		put("跣", "6420");
		put("跨", "6421");
		put("跪", "6422");
		put("跬", "6423");
		put("路", "6424");
		put("踩", "6425");
		put("跳", "6426");
		put("跽", "6427");
		put("踉", "6428");
		put("统", "4827");
		put("丝", "4828");
		put("绛", "4829");
		put("绢", "4831");
		put("绑", "4834");
		put("绨", "4835");
		put("绣", "4836");
		put("绠", "4837");
		put("绥", "4840");
		put("经", "4842");
		put("绡", "4843");
		put("综", "4844");
		put("绿", "4845");
		put("绸", "4846");
		put("綦", "4847");
		put("线", "4848");
		put("绶", "4849");
		put("维", "4850");
		put("綮", "4851");
		put("绾", "4852");
		put("纲", "4854");
		put("绷", "4855");
		put("缀", "4856");
		put("纶", "4858");
		put("绺", "4859");
		put("绮", "4860");
		put("绽", "4861");
		put("绰", "4862");
		put("绫", "4866");
		put("缁", "4867");
		put("紧", "4868");
		put("绯", "4869");
		put("绻", "4870");
		put("绪", "4872");
		put("缄", "4873");
		put("缉", "4874");
		put("绵", "4875");
		put("缎", "4876");
		put("缔", "4877");
		put("缘", "4878");
		put("缌", "4880");
		put("编", "4882");
		put("缓", "4883");
		put("缅", "4884");
		put("纬", "4885");
		put("练", "4886");
		put("缊", "4888");
		put("缃", "4889");
		put("缗", "4891");
		put("缈", "4893");
		put("缇", "4895");
		put("萦", "4896");
		put("缙", "4897");
		put("缢", "4898");
		put("缒", "4899");
		put("绉", "4900");
		put("缞", "4901");
		put("缚", "4902");
		put("缜", "4903");
		put("缛", "4904");
		put("县", "4905");
		put("缣", "4906");
		put("繁", "4907");
		put("缟", "4908");
		put("缝", "4911");
		put("纵", "4912");
		put("缧", "4913");
		put("缦", "4915");
		put("絷", "4916");
		put("缕", "4917");
		put("缥", "4918");
		put("縻", "4919");
		put("总", "4920");
		put("绩", "4921");
		put("缫", "4923");
		put("缪", "4924");
		put("繇", "4925");
		put("缡", "4926");
		put("索", "4792");
		put("紫", "4793");
		put("缏", "4794");
		put("累", "4797");
		put("细", "4798");
		put("缩", "4799");
		put("绅", "4800");
		put("绍", "4801");
		put("绀", "4802");
		put("绋", "4803");
		put("绐", "4805");
		put("绌", "4806");
		put("终", "4807");
		put("组", "4809");
		put("绊", "4810");
		put("绂", "4811");
		put("结", "4814");
		put("绝", "4815");
		put("绔", "4818");
		put("绞", "4819");
		put("络", "4820");
		put("绚", "4821");
		put("给", "4822");
		put("绒", "4823");
		put("絮", "4825");
		put("绖", "4826");
		put("纛", "4966");
		put("缆", "4968");
		put("绦", "4969");
		put("缶", "4970");
		put("缸", "4971");
		put("缺", "4972");
		put("罄", "4977");
		put("罅", "4978");
		put("罂", "4981");
		put("罍", "4982");
		put("罐", "4984");
		put("网", "4986");
		put("罔", "4987");
		put("罕", "4988");
		put("罘", "4989");
		put("罟", "4992");
		put("罡", "4993");
		put("罩", "4996");
		put("罪", "4997");
		put("置", "4999");
		put("罚", "5000");
		put("罨", "5001");
		put("署", "5002");
		put("骂", "5006");
		put("罢", "5007");
		put("罹", "5008");
		put("罽", "5010");
		put("罗", "5012");
		put("罴", "5013");
		put("羁", "5015");
		put("鳓", "5016");
		put("羊", "5017");
		put("羌", "5018");
		put("美", "5019");
		put("羑", "5020");
		put("羔", "5021");
		put("羚", "5024");
		put("羝", "5025");
		put("羞", "5026");
		put("群", "5028");
		put("羡", "5029");
		put("义", "5030");
		put("羯", "5031");
		put("羲", "5032");
		put("羧", "5033");
		put("羸", "5035");
		put("羹", "5036");
		put("羼", "5037");
		put("羽", "5038");
		put("翀", "5039");
		put("翁", "5040");
		put("翅", "5041");
		put("翊", "5042");
		put("翌", "5043");
		put("翎", "5044");
		put("习", "5045");
		put("翔", "5046");
		put("翕", "5047");
		put("翟", "5049");
		put("翠", "5050");
		put("翡", "5051");
		put("翥", "5053");
		put("翦", "5054");
		put("翩", "5055");
		put("羿", "5056");
		put("翚", "5057");
		put("翮", "5058");
		put("翰", "5060");
		put("翳", "5061");
		put("翘", "5062");
		put("翱", "5063");
		put("翻", "5064");
		put("翼", "5065");
		put("织", "4930");
		put("缮", "4931");
		put("缭", "4933");
		put("绕", "4935");
		put("缯", "4938");
		put("绳", "4939");
		put("绘", "4940");
		put("茧", "4942");
		put("缰", "4943");
		put("缳", "4944");
		put("缴", "4945");
		put("绎", "4946");
		put("辫", "4947");
		put("继", "4949");
		put("纂", "4951");
		put("缤", "4952");
		put("缱", "4953");
		put("颣", "4955");
		put("缬", "4956");
		put("缵", "4957");
		put("续", "4958");
		put("纤", "4960");
		put("缠", "4961");
		put("纩", "4962");
		put("缨", "4964");
		put("耻", "5103");
		put("耽", "5104");
		put("耿", "5105");
		put("聃", "5106");
		put("聆", "5107");
		put("聊", "5108");
		put("聒", "5109");
		put("圣", "5110");
		put("聘", "5111");
		put("聚", "5112");
		put("闻", "5113");
		put("联", "5114");
		put("聪", "5115");
		put("声", "5116");
		put("耸", "5117");
		put("聩", "5118");
		put("聂", "5119");
		put("职", "5120");
		put("聋", "5122");
		put("啉", "5123");
		put("聿", "5124");
		put("肄", "5125");
		put("肃", "5126");
		put("肆", "5127");
		put("肇", "5128");
		put("呤", "5129");
		put("馏", "5130");
		put("肉", "5131");
		put("肋", "5132");
		put("肌", "5133");
		put("肓", "5134");
		put("肖", "5135");
		put("肘", "5136");
		put("肚", "5137");
		put("肛", "5138");
		put("肝", "5139");
		put("股", "5140");
		put("肢", "5141");
		put("肥", "5142");
		put("胺", "5143");
		put("肩", "5144");
		put("肫", "5145");
		put("肯", "5146");
		put("肱", "5147");
		put("育", "5148");
		put("肴", "5149");
		put("", "5150");
		put("肺", "5151");
		put("胃", "5152");
		put("胄", "5153");
		put("背", "5154");
		put("胂", "5155");
		put("胔", "5156");
		put("胍", "5157");
		put("胎", "5158");
		put("胖", "5160");
		put("胙", "5161");
		put("胚", "5162");
		put("胳", "5163");
		put("胝", "5164");
		put("胞", "5165");
		put("胰", "5166");
		put("脒", "5167");
		put("胭", "5168");
		put("胯", "5169");
		put("胡", "5170");
		put("胥", "5171");
		put("胸", "5172");
		put("胼", "5173");
		put("能", "5174");
		put("胾", "5175");
		put("脂", "5176");
		put("腺", "5177");
		put("胁", "5178");
		put("胱", "5180");
		put("脊", "5182");
		put("脚", "5183");
		put("脘", "5184");
		put("胫", "5185");
		put("脆", "5186");
		put("腈", "5187");
		put("朘", "5189");
		put("脩", "5190");
		put("膨", "5191");
		put("脱", "5192");
		put("脬", "5193");
		put("脯", "5194");
		put("胀", "5195");
		put("脾", "5196");
		put("腆", "5197");
		put("腊", "5198");
		put("腋", "5199");
		put("肾", "5200");
		put("腐", "5201");
		put("腑", "5202");
		put("腓", "5203");
		put("腔", "5204");
		put("腕", "5205");
		put("耀", "5069");
		put("喱", "5070");
		put("老", "5071");
		put("考", "5072");
		put("耄", "5073");
		put("者", "5074");
		put("耆", "5075");
		put("耋", "5077");
		put("而", "5079");
		put("耍", "5080");
		put("耐", "5082");
		put("唡", "5084");
		put("耒", "5085");
		put("耔", "5086");
		put("耕", "5087");
		put("耗", "5088");
		put("耘", "5089");
		put("耙", "5090");
		put("耜", "5091");
		put("耢", "5092");
		put("耩", "5095");
		put("耦", "5096");
		put("耨", "5097");
		put("耧", "5098");
		put("耰", "5099");
		put("钌", "5100");
		put("耳", "5101");
		put("耶", "5102");
		put("臀", "5241");
		put("臂", "5242");
		put("臆", "5244");
		put("脸", "5245");
		put("臊", "5246");
		put("脐", "5247");
		put("臃", "5250");
		put("胪", "5251");
		put("脏", "5253");
		put("脔", "5254");
		put("胤", "5255");
		put("臣", "5256");
		put("卧", "5257");
		put("臧", "5258");
		put("临", "5259");
		put("栌", "5260");
		put("自", "5261");
		put("臬", "5262");
		put("臭", "5263");
		put("镥", "5266");
		put("至", "5267");
		put("致", "5268");
		put("臻", "5271");
		put("氇", "5272");
		put("臼", "5273");
		put("臾", "5274");
		put("舁", "5275");
		put("舀", "5276");
		put("舂", "5277");
		put("舄", "5278");
		put("舅", "5279");
		put("与", "5280");
		put("兴", "5281");
		put("举", "5282");
		put("旧", "5283");
		put("摞", "5285");
		put("舌", "5286");
		put("舍", "5287");
		put("舐", "5288");
		put("舒", "5289");
		put("舘", "5290");
		put("", "5291");
		put("舛", "5292");
		put("舜", "5293");
		put("舞", "5294");
		put("镅", "5296");
		put("舟", "5297");
		put("舡", "5299");
		put("航", "5300");
		put("般", "5301");
		put("舫", "5302");
		put("舷", "5304");
		put("舵", "5305");
		put("舶", "5306");
		put("船", "5307");
		put("舸", "5308");
		put("舳", "5310");
		put("舴", "5311");
		put("艇", "5312");
		put("艋", "5313");
		put("艄", "5314");
		put("艘", "5318");
		put("舱", "5319");
		put("舣", "5320");
		put("艟", "5321");
		put("艨", "5322");
		put("舰", "5324");
		put("舻", "5325");
		put("酶", "5326");
		put("艮", "5327");
		put("良", "5328");
		put("艰", "5329");
		put("钔", "5330");
		put("色", "5331");
		put("艳", "5333");
		put("艸", "5335");
		put("艽", "5336");
		put("艾", "5337");
		put("芃", "5338");
		put("芄", "5339");
		put("芊", "5340");
		put("芋", "5341");
		put("芍", "5342");
		put("芎", "5343");
		put("腥", "5206");
		put("脑", "5207");
		put("腴", "5208");
		put("肿", "5209");
		put("腮", "5210");
		put("脿", "5211");
		put("腰", "5212");
		put("腱", "5213");
		put("肠", "5214");
		put("腹", "5215");
		put("腠", "5216");
		put("腿", "5217");
		put("膀", "5218");
		put("膂", "5219");
		put("腩", "5220");
		put("膏", "5221");
		put("肷", "5222");
		put("膈", "5224");
		put("膊", "5225");
		put("腘", "5226");
		put("肤", "5227");
		put("膛", "5228");
		put("膜", "5229");
		put("膝", "5230");
		put("胶", "5231");
		put("腻", "5232");
		put("膳", "5234");
		put("膺", "5235");
		put("膻", "5238");
		put("脍", "5239");
		put("脓", "5240");
		put("苗", "5379");
		put("苄", "5380");
		put("苛", "5381");
		put("苜", "5382");
		put("苞", "5383");
		put("苟", "5384");
		put("苣", "5385");
		put("苡", "5386");
		put("若", "5387");
		put("苦", "5388");
		put("苎", "5389");
		put("苫", "5390");
		put("英", "5391");
		put("苴", "5392");
		put("苹", "5393");
		put("苻", "5394");
		put("苾", "5396");
		put("茀", "5397");
		put("茁", "5398");
		put("茂", "5399");
		put("范", "5400");
		put("茄", "5401");
		put("茈", "5402");
		put("茅", "5403");
		put("茇", "5404");
		put("茉", "5406");
		put("茗", "5407");
		put("荔", "5408");
		put("茜", "5409");
		put("茨", "5412");
		put("茫", "5413");
		put("茭", "5414");
		put("茯", "5415");
		put("茱", "5416");
		put("兹", "5417");
		put("茴", "5418");
		put("茵", "5419");
		put("茶", "5420");
		put("茸", "5422");
		put("茹", "5423");
		put("荀", "5424");
		put("荃", "5425");
		put("萄", "5426");
		put("荆", "5427");
		put("荸", "5428");
		put("荇", "5429");
		put("草", "5430");
		put("荏", "5432");
		put("荐", "5433");
		put("荑", "5434");
		put("荒", "5435");
		put("茝", "5436");
		put("荄", "5437");
		put("荷", "5440");
		put("荻", "5441");
		put("荼", "5442");
		put("荽", "5443");
		put("莅", "5444");
		put("莎", "5446");
		put("莒", "5447");
		put("莓", "5448");
		put("茎", "5449");
		put("莘", "5450");
		put("莞", "5451");
		put("莠", "5452");
		put("荚", "5453");
		put("苋", "5454");
		put("莆", "5455");
		put("莨", "5456");
		put("莩", "5457");
		put("莪", "5458");
		put("莫", "5459");
		put("菟", "5460");
		put("莉", "5461");
		put("莽", "5462");
		put("菀", "5463");
		put("菁", "5464");
		put("菅", "5465");
		put("菇", "5466");
		put("眯", "4183");
		put("眵", "4184");
		put("眶", "4186");
		put("眷", "4187");
		put("眸", "4188");
		put("眺", "4189");
		put("眼", "4190");
		put("着", "4192");
		put("睨", "4194");
		put("睇", "4195");
		put("瞅", "4197");
		put("睚", "4199");
		put("睛", "4200");
		put("睁", "4201");
		put("睐", "4202");
		put("睾", "4203");
		put("睡", "4204");
		put("睢", "4205");
		put("督", "4206");
		put("睦", "4207");
		put("睫", "4208");
		put("瞌", "4209");
		put("睥", "4211");
		put("睹", "4212");
		put("睿", "4213");
		put("瞀", "4214");
		put("瞋", "4217");
		put("瞎", "4219");
		put("瞑", "4220");
		put("瞒", "4221");
		put("瞠", "4222");
		put("瞪", "4223");
		put("瞥", "4224");
		put("瞧", "4225");
		put("瞬", "4226");
		put("瞳", "4227");
		put("瞭", "4229");
		put("瞰", "4230");
		put("瞆", "4231");
		put("瞻", "4232");
		put("瞽", "4233");
		put("瞿", "4234");
		put("矍", "4236");
		put("眬", "4237");
		put("矗", "4238");
		put("百", "4102");
		put("皂", "4103");
		put("的", "4104");
		put("皆", "4105");
		put("皇", "4106");
		put("皈", "4107");
		put("皋", "4108");
		put("皎", "4109");
		put("皓", "4110");
		put("皖", "4111");
		put("皑", "4114");
		put("皤", "4115");
		put("皡", "4116");
		put("氪", "4121");
		put("皮", "4122");
		put("皴", "4123");
		put("皲", "4125");
		put("皱", "4126");
		put("醌", "4128");
		put("皿", "4129");
		put("盂", "4130");
		put("盅", "4132");
		put("盆", "4133");
		put("盈", "4134");
		put("益", "4135");
		put("破", "4275");
		put("砜", "4277");
		put("砧", "4278");
		put("砼", "4279");
		put("砰", "4280");
		put("研", "4282");
		put("硎", "4283");
		put("硁", "4284");
		put("硝", "4285");
		put("硖", "4286");
		put("砗", "4287");
		put("硫", "4288");
		put("硬", "4289");
		put("碘", "4290");
		put("砚", "4291");
		put("确", "4292");
		put("碍", "4293");
		put("碌", "4294");
		put("碎", "4295");
		put("硼", "4296");
		put("碗", "4297");
		put("碇", "4299");
		put("碔", "4300");
		put("碑", "4301");
		put("碓", "4302");
		put("碡", "4304");
		put("碉", "4306");
		put("砀", "4307");
		put("碟", "4308");
		put("碣", "4309");
		put("碧", "4310");
		put("硕", "4311");
		put("磜", "4312");
		put("碰", "4314");
		put("码", "4316");
		put("碾", "4317");
		put("磁", "4318");
		put("磅", "4319");
		put("磊", "4320");
		put("礅", "4321");
		put("磋", "4322");
		put("磐", "4323");
		put("磙", "4324");
		put("磔", "4325");
		put("磕", "4326");
		put("磉", "4327");
		put("硙", "4328");
		put("碴", "4329");
		put("硵", "4330");
		put("砖", "4331");
		put("碛", "4332");
		put("磨", "4333");
		put("磬", "4334");
		put("矶", "4335");
		put("磲", "4336");
		put("磺", "4337");
		put("硗", "4338");
		put("礁", "4339");
		put("磷", "4340");
		put("础", "4342");
		put("矾", "4345");
		put("礳", "4346");
		put("磴", "4347");
		put("矿", "4349");
		put("砺", "4350");
		put("砾", "4351");
		put("砻", "4352");
		put("礴", "4353");
		put("碱", "4354");
		put("示", "4355");
		put("礽", "4356");
		put("社", "4357");
		put("祀", "4358");
		put("祁", "4359");
		put("祆", "4360");
		put("祇", "4361");
		put("祈", "4362");
		put("祉", "4363");
		put("祊", "4366");
		put("祐", "4368");
		put("祓", "4369");
		put("祖", "4371");
		put("祚", "4373");
		put("祛", "4374");
		put("祜", "4375");
		put("祝", "4376");
		put("神", "4377");
		put("瞩", "4241");
		put("眭", "4242");
		put("矛", "4243");
		put("矜", "4244");
		put("矞", "4245");
		put("砬", "4246");
		put("矢", "4247");
		put("矣", "4248");
		put("知", "4249");
		put("矧", "4250");
		put("矩", "4251");
		put("短", "4252");
		put("矮", "4253");
		put("矫", "4255");
		put("鑞", "4257");
		put("石", "4258");
		put("矼", "4260");
		put("砆", "4261");
		put("矻", "4262");
		put("砂", "4263");
		put("砉", "4264");
		put("砌", "4265");
		put("砍", "4266");
		put("砑", "4267");
		put("砒", "4268");
		put("砭", "4269");
		put("砟", "4270");
		put("砸", "4271");
		put("砥", "4272");
		put("斫", "4273");
		put("砦", "4274");
		put("铼", "4414");
		put("禸", "4415");
		put("禹", "4416");
		put("禺", "4417");
		put("离", "4418");
		put("禽", "4419");
		put("镧", "4420");
		put("禾", "4421");
		put("秃", "4422");
		put("秀", "4423");
		put("私", "4424");
		put("秉", "4426");
		put("秆", "4427");
		put("秋", "4428");
		put("种", "4429");
		put("科", "4430");
		put("秕", "4431");
		put("秒", "4432");
		put("秘", "4434");
		put("租", "4436");
		put("秣", "4438");
		put("秤", "4439");
		put("秦", "4440");
		put("秧", "4441");
		put("秩", "4442");
		put("秫", "4443");
		put("秭", "4444");
		put("秸", "4447");
		put("移", "4448");
		put("稀", "4449");
		put("稂", "4450");
		put("税", "4451");
		put("程", "4453");
		put("稍", "4455");
		put("稔", "4457");
		put("稗", "4458");
		put("稞", "4459");
		put("稚", "4460");
		put("稠", "4464");
		put("稹", "4466");
		put("称", "4468");
		put("稷", "4469");
		put("稻", "4470");
		put("稼", "4471");
		put("稽", "4472");
		put("稿", "4473");
		put("穄", "4475");
		put("穆", "4476");
		put("稣", "4479");
		put("积", "4480");
		put("颖", "4481");
		put("穗", "4482");
		put("穑", "4484");
		put("秽", "4486");
		put("秾", "4487");
		put("稳", "4489");
		put("穭", "4491");
		put("穰", "4492");
		put("罱", "4493");
		put("穴", "4494");
		put("究", "4496");
		put("穸", "4497");
		put("穹", "4498");
		put("突", "4499");
		put("空", "4500");
		put("穿", "4502");
		put("窀", "4503");
		put("伣", "8044");
		put("俅", "8045");
		put("伡", "8046");
		put("侴", "8051");
		put("倢", "8053");
		put("倴", "8055");
		put("倔", "8056");
		put("偲", "8059");
		put("偬", "8064");
		put("偌", "8067");
		put("僬", "8072");
		put("儡", "8079");
		put("傩", "8080");
		put("党", "8093");
		put("凃", "8094");
		put("减", "8096");
		put("冨", "8099");
		put("呙", "8139");
		put("呗", "8142");
		put("唑", "8143");
		put("啗", "8145");
		put("唪", "8146");
		put("唼", "8147");
		put("啐", "8148");
		put("啍", "8149");
		put("唶", "8150");
		put("唝", "8156");
		put("啯", "8165");
		put("嘌", "8166");
		put("唠", "8167");
		put("噙", "8168");
		put("呒", "8172");
		put("哒", "8174");
		put("嚆", "8178");
		put("嚭", "8181");
		put("呖", "8182");
		put("噜", "8183");
		put("喾", "8185");
		put("啃", "8186");
		put("囝", "8188");
		put("囡", "8189");
		put("圊", "8191");
		put("圯", "8197");
		put("圪", "8198");
		put("壳", "8199");
		put("坻", "8201");
		put("坿", "8205");
		put("垇", "8206");
		put("垞", "8208");
		put("垤", "8209");
		put("垚", "8210");
		put("垌", "8211");
		put("垟", "8213");
		put("垕", "8214");
		put("埂", "8217");
		put("垸", "8219");
		put("埌", "8222");
		put("埕", "8224");
		put("堉", "8230");
		put("堎", "8231");
		put("堀", "8232");
		put("堌", "8233");
		put("塓", "8237");
		put("堍", "8239");
		put("埚", "8241");
		put("剞", "8104");
		put("劂", "8105");
		put("刿", "8106");
		put("勋", "8113");
		put("匆", "8116");
		put("吋", "8120");
		put("吱", "8123");
		put("呎", "8125");
		put("听", "8126");
		put("咚", "8131");
		put("咥", "8134");
		put("哆", "8135");
		put("咻", "8136");
		put("姘", "8280");
		put("娒", "8281");
		put("婥", "8289");
		put("媄", "8292");
		put("媞", "8293");
		put("媮", "8295");
		put("婿", "8296");
		put("婳", "8299");
		put("娴", "8300");
		put("妩", "8301");
		put("嫒", "8302");
		put("孃", "8304");
		put("孖", "8311");
		put("屲", "8320");
		put("屴", "8321");
		put("屼", "8322");
		put("岙", "8324");
		put("岍", "8325");
		put("岜", "8326");
		put("岨", "8327");
		put("岬", "8328");
		put("岞", "8329");
		put("峃", "8330");
		put("峇", "8331");
		put("峱", "8335");
		put("崞", "8338");
		put("崮", "8339");
		put("崽", "8342");
		put("崴", "8343");
		put("峣", "8344");
		put("嶓", "8348");
		put("崄", "8349");
		put("岿", "8354");
		put("巯", "8360");
		put("幞", "8363");
		put("庄", "8369");
		put("廒", "8373");
		put("庼", "8375");
		put("塬", "8242");
		put("塱", "8243");
		put("塝", "8244");
		put("堽", "8245");
		put("塥", "8247");
		put("墕", "8250");
		put("垱", "8255");
		put("墙", "8259");
		put("墒", "8262");
		put("垅", "8263");
		put("夼", "8268");
		put("扡", "8421");
		put("扦", "8422");
		put("抃", "8423");
		put("掰", "8428");
		put("拶", "8429");
		put("捃", "8432");
		put("挜", "8436");
		put("掮", "8438");
		put("揸", "8439");
		put("揎", "8445");
		put("揶", "8448");
		put("搢", "8452");
		put("揾", "8453");
		put("搡", "8459");
		put("摷", "8464");
		put("搿", "8466");
		put("挦", "8469");
		put("揿", "8471");
		put("掸", "8473");
		put("掱", "8479");
		put("敜", "8489");
		put("敫", "8490");
		put("斏", "8494");
		put("昀", "8502");
		put("昊", "8504");
		put("晟", "8508");
		put("晧", "8510");
		put("晾", "8512");
		put("暋", "8513");
		put("暌", "8514");
		put("异", "8381");
		put("弌", "8382");
		put("弎", "8383");
		put("甙", "8384");
		put("忪", "8388");
		put("桋", "8557");
		put("栒", "8558");
		put("栟", "8559");
		put("桬", "8560");
		put("枧", "8565");
		put("棑", "8566");
		put("棸", "8569");
		put("椑", "8572");
		put("楙", "8574");
		put("楀", "8575");
		put("楂", "8576");
		put("椝", "8584");
		put("椫", "8586");
		put("橇", "8587");
		put("檎", "8589");
		put("桪", "8591");
		put("枟", "8592");
		put("槜", "8593");
		put("檑", "8594");
		put("檴", "8596");
		put("橱", "8599");
		put("橥", "8601");
		put("榉", "8603");
		put("毚", "8632");
		put("毵", "8633");
		put("氝", "8635");
		put("氟", "8636");
		put("氨", "8637");
		put("氧", "8638");
		put("氥", "8639");
		put("氦", "8640");
		put("氢", "8641");
		put("氰", "8642");
		put("氮", "8644");
		put("氩", "8645");
		put("氕", "8646");
		put("氘", "8647");
		put("氚", "8648");
		put("晔", "8518");
		put("暧", "8519");
		put("杠", "8530");
		put("杅", "8533");
		put("杻", "8537");
		put("栝", "8552");
		put("渌", "8692");
		put("淼", "8693");
		put("渲", "8694");
		put("溏", "8697");
		put("溴", "8700");
		put("滉", "8701");
		put("滏", "8702");
		put("漷", "8708");
		put("澉", "8709");
		put("沵", "8712");
		put("滢", "8714");
		put("瀌", "8715");
		put("灺", "8731");
		put("炅", "8732");
		put("烱", "8741");
		put("焯", "8743");
		put("焱", "8746");
		put("熳", "8759");
		put("燋", "8761");
		put("烨", "8763");
		put("牮", "8781");
		put("犇", "8786");
		put("犄", "8787");
		put("氿", "8659");
		put("汆", "8660");
		put("沄", "8661");
		put("沇", "8662");
		put("洴", "8671");
		put("涌", "8673");
		put("浤", "8677");
		put("浼", "8680");
		put("淖", "8681");
		put("淠", "8685");
		put("沨", "8688");
		put("珲", "8829");
		put("瑄", "8830");
		put("瑂", "8831");
		put("瑔", "8832");
		put("瑢", "8833");
		put("琏", "8834");
		put("瑭", "8835");
		put("璩", "8840");
		put("瑷", "8842");
		put("甏", "8853");
		put("畈", "8858");
		put("疃", "8859");
		put("疢", "8864");
		put("痤", "8871");
		put("瘃", "8873");
		put("瘄", "8875");
		put("瘈", "8876");
		put("瘌", "8877");
		put("瘭", "8880");
		put("瘅", "8883");
		put("癒", "8884");
		put("癿", "8895");
		put("盹", "8901");
		put("眚", "8904");
		put("眢", "8905");
		put("睽", "8915");
		put("瞟", "8918");
		put("瞫", "8919");
		put("矬", "8926");
		put("矽", "8928");
		put("砢", "8931");
		put("狨", "8799");
		put("猆", "8805");
		put("猃", "8811");
		put("猕", "8813");
		put("獾", "8814");
		put("玥", "8819");
		put("珺", "8823");
		put("琼", "8825");
		put("琷", "8826");
		put("琰", "8827");
		put("珐", "8828");
		put("礞", "8968");
		put("硷", "8969");
		put("硒", "8970");
		put("禚", "8981");
		put("禤", "8983");
		put("稖", "8993");
		put("窿", "9002");
		put("竜", "9005");
		put("筘", "9010");
		put("篥", "9018");
		put("簉", "9020");
		put("簰", "9023");
		put("籝", "9024");
		put("粞", "9035");
		put("粦", "9036");
		put("粮", "9037");
		put("粽", "9040");
		put("糅", "9041");
		put("踊", "6429");
		put("踏", "6431");
		put("践", "6432");
		put("踔", "6433");
		put("踖", "6434");
		put("踝", "6435");
		put("踞", "6436");
		put("踢", "6437");
		put("踣", "6438");
		put("踧", "6439");
		put("踡", "6441");
		put("踟", "6442");
		put("踮", "6443");
		put("踵", "6446");
		put("踹", "6447");
		put("贰", "6310");
		put("贵", "6311");
		put("贬", "6312");
		put("贷", "6313");
		put("买", "6314");
		put("贶", "6315");
		put("费", "6316");
		put("贴", "6317");
		put("贻", "6318");
		put("贸", "6319");
		put("贺", "6320");
		put("贲", "6321");
		put("贳", "6322");
		put("赂", "6323");
		put("赁", "6324");
		put("贿", "6325");
		put("赅", "6326");
		put("资", "6327");
		put("贾", "6328");
		put("贼", "6329");
		put("赈", "6331");
		put("赊", "6332");
		put("宾", "6333");
		put("赇", "6334");
		put("赒", "6335");
		put("赉", "6336");
		put("赐", "6337");
		put("赏", "6339");
		put("赍", "6340");
		put("赔", "6341");
		put("赓", "6342");
		put("贤", "6343");
		put("卖", "6344");
		put("躅", "6483");
		put("躇", "6484");
		put("趸", "6485");
		put("踌", "6486");
		put("跻", "6487");
		put("躐", "6489");
		put("踯", "6490");
		put("踬", "6492");
		put("跹", "6494");
		put("蹑", "6495");
		put("躏", "6497");
		put("蹦", "6498");
		put("身", "6500");
		put("躬", "6501");
		put("躭", "6502");
		put("躲", "6503");
		put("躯", "6504");
		put("躺", "6507");
		put("车", "6508");
		put("轧", "6509");
		put("轨", "6510");
		put("军", "6511");
		put("轩", "6513");
		put("轫", "6514");
		put("轭", "6515");
		put("软", "6516");
		put("轱", "6517");
		put("轸", "6518");
		put("轴", "6519");
		put("轵", "6520");
		put("轷", "6521");
		put("轶", "6522");
		put("轺", "6523");
		put("轼", "6524");
		put("较", "6525");
		put("辂", "6526");
		put("载", "6528");
		put("轾", "6531");
		put("辄", "6532");
		put("辅", "6534");
		put("轻", "6535");
		put("辆", "6538");
		put("辎", "6539");
		put("辉", "6540");
		put("辍", "6541");
		put("辇", "6542");
		put("辈", "6543");
		put("轮", "6544");
		put("辋", "6545");
		put("辊", "6547");
		put("辑", "6549");
		put("辏", "6550");
		put("输", "6551");
		put("辐", "6553");
		put("辗", "6558");
		put("舆", "6559");
		put("毂", "6560");
		put("辖", "6561");
		put("辕", "6562");
		put("辘", "6564");
		put("转", "6567");
		put("辙", "6568");
		put("轿", "6569");
		put("轰", "6575");
		put("辔", "6576");
		put("轹", "6577");
		put("轳", "6578");
		put("辛", "6580");
		put("辜", "6581");
		put("辟", "6582");
		put("辣", "6584");
		put("蹓", "6448");
		put("踽", "6449");
		put("蹁", "6450");
		put("蹂", "6451");
		put("蹄", "6452");
		put("踱", "6453");
		put("蹀", "6454");
		put("蹐", "6455");
		put("蹇", "6456");
		put("蹈", "6457");
		put("蹉", "6458");
		put("蹊", "6459");
		put("跃", "6460");
		put("跄", "6461");
		put("跸", "6462");
		put("蹙", "6464");
		put("蹒", "6466");
		put("蹯", "6471");
		put("蹲", "6472");
		put("蹴", "6473");
		put("蹶", "6474");
		put("跷", "6475");
		put("蹬", "6478");
		put("蹭", "6479");
		put("蹰", "6480");
		put("躁", "6481");
		put("迺", "6621");
		put("退", "6622");
		put("送", "6623");
		put("适", "6624");
		put("逃", "6625");
		put("逅", "6626");
		put("逆", "6627");
		put("逋", "6628");
		put("逯", "6629");
		put("逍", "6630");
		put("透", "6631");
		put("逐", "6632");
		put("逑", "6633");
		put("途", "6634");
		put("迳", "6635");
		put("逖", "6636");
		put("逗", "6637");
		put("这", "6638");
		put("通", "6639");
		put("逛", "6640");
		put("逝", "6641");
		put("逞", "6642");
		put("速", "6643");
		put("造", "6644");
		put("逡", "6645");
		put("逢", "6646");
		put("连", "6647");
		put("逭", "6648");
		put("逮", "6649");
		put("进", "6651");
		put("逵", "6652");
		put("逶", "6653");
		put("逸", "6654");
		put("逼", "6656");
		put("遇", "6657");
		put("遁", "6658");
		put("遂", "6659");
		put("遄", "6660");
		put("逾", "6661");
		put("运", "6663");
		put("遍", "6664");
		put("过", "6665");
		put("遏", "6666");
		put("遐", "6667");
		put("遑", "6668");
		put("遒", "6669");
		put("道", "6670");
		put("达", "6671");
		put("违", "6672");
		put("遘", "6673");
		put("遥", "6674");
		put("遛", "6675");
		put("逊", "6676");
		put("递", "6677");
		put("远", "6678");
		put("遣", "6680");
		put("遢", "6681");
		put("遨", "6683");
		put("遭", "6685");
		put("遮", "6686");
		put("迟", "6688");
		put("遴", "6689");
		put("遵", "6690");
		put("迁", "6692");
		put("选", "6693");
		put("遗", "6695");
		put("辽", "6697");
		put("遽", "6698");
		put("避", "6699");
		put("邀", "6700");
		put("迈", "6701");
		put("邂", "6702");
		put("还", "6703");
		put("迩", "6705");
		put("邃", "6706");
		put("邈", "6707");
		put("边", "6708");
		put("逻", "6709");
		put("逦", "6710");
		put("疱", "6711");
		put("邑", "6712");
		put("邛", "6713");
		put("邗", "6714");
		put("邙", "6715");
		put("邕", "6716");
		put("邢", "6717");
		put("那", "6719");
		put("邡", "6720");
		put("邦", "6721");
		put("邨", "6722");
		put("邪", "6723");
		put("办", "6586");
		put("辨", "6587");
		put("辞", "6588");
		put("辩", "6589");
		put("耪", "6590");
		put("辰", "6591");
		put("辱", "6592");
		put("农", "6593");
		put("狍", "6594");
		put("辵", "6595");
		put("迂", "6596");
		put("迄", "6597");
		put("迅", "6598");
		put("迎", "6601");
		put("近", "6602");
		put("迓", "6603");
		put("返", "6604");
		put("迢", "6606");
		put("迤", "6607");
		put("迥", "6608");
		put("迦", "6609");
		put("迨", "6610");
		put("迪", "6611");
		put("迫", "6612");
		put("迭", "6613");
		put("逄", "6614");
		put("述", "6615");
		put("迷", "6617");
		put("迸", "6618");
		put("迹", "6619");
		put("追", "6620");
		put("鄂", "6759");
		put("邹", "6760");
		put("鄄", "6761");
		put("邬", "6762");
		put("乡", "6763");
		put("鄌", "6764");
		put("郧", "6765");
		put("鄙", "6766");
		put("鄢", "6768");
		put("鄞", "6769");
		put("鄠", "6770");
		put("鄜", "6771");
		put("邓", "6772");
		put("郑", "6774");
		put("邻", "6775");
		put("鄱", "6776");
		put("邺", "6777");
		put("郐", "6778");
		put("郸", "6779");
		put("鄯", "6781");
		put("邝", "6782");
		put("酂", "6783");
		put("酃", "6784");
		put("酆", "6785");
		put("郦", "6786");
		put("郝", "6787");
		put("酉", "6788");
		put("酊", "6789");
		put("酋", "6790");
		put("酌", "6791");
		put("配", "6792");
		put("酎", "6793");
		put("酒", "6794");
		put("酗", "6796");
		put("酢", "6797");
		put("酥", "6798");
		put("酣", "6799");
		put("酤", "6800");
		put("酡", "6801");
		put("酩", "6802");
		put("酪", "6803");
		put("酬", "6804");
		put("酲", "6805");
		put("酵", "6806");
		put("酷", "6807");
		put("酸", "6808");
		put("酴", "6809");
		put("酹", "6810");
		put("酞", "6811");
		put("醅", "6813");
	}

	static void init3() {
		put("醇", "6815");
		put("醉", "6816");
		put("醋", "6818");
		put("醍", "6819");
		put("醐", "6820");
		put("醒", "6821");
		put("酝", "6824");
		put("醢", "6826");
		put("醪", "6828");
		put("医", "6829");
		put("酱", "6830");
		put("醮", "6831");
		put("醯", "6832");
		put("醴", "6833");
		put("醵", "6835");
		put("醺", "6836");
		put("酿", "6840");
		put("锫", "6844");
		put("釆", "6845");
		put("采", "6846");
		put("释", "6847");
		put("釉", "6848");
		put("里", "6849");
		put("重", "6850");
		put("野", "6851");
		put("量", "6852");
		put("锌", "6854");
		put("金", "6855");
		put("钊", "6856");
		put("钉", "6857");
		put("釜", "6858");
		put("针", "6859");
		put("钓", "6860");
		put("钏", "6861");
		put("邠", "6724");
		put("邯", "6725");
		put("邱", "6726");
		put("邴", "6728");
		put("邳", "6729");
		put("邵", "6730");
		put("邶", "6731");
		put("邸", "6732");
		put("邰", "6733");
		put("邽", "6734");
		put("郁", "6735");
		put("郅", "6736");
		put("郇", "6737");
		put("郊", "6738");
		put("郃", "6740");
		put("郗", "6741");
		put("郄", "6742");
		put("郛", "6744");
		put("郎", "6745");
		put("郡", "6746");
		put("郢", "6747");
		put("郤", "6748");
		put("郜", "6750");
		put("郏", "6751");
		put("部", "6752");
		put("郭", "6753");
		put("郴", "6754");
		put("邮", "6755");
		put("郯", "6756");
		put("都", "6757");
		put("郿", "6758");
		put("铣", "6897");
		put("铨", "6898");
		put("铢", "6899");
		put("铭", "6900");
		put("铦", "6901");
		put("衔", "6902");
		put("铫", "6903");
		put("锐", "6904");
		put("钾", "6905");
		put("销", "6906");
		put("锈", "6907");
		put("锉", "6908");
		put("铹", "6909");
		put("铤", "6910");
		put("铗", "6911");
		put("锋", "6912");
		put("锓", "6913");
		put("铀", "6914");
		put("锄", "6915");
		put("铓", "6916");
		put("铺", "6917");
		put("铻", "6918");
		put("锯", "6920");
		put("钢", "6921");
		put("录", "6922");
		put("锥", "6923");
		put("锟", "6924");
		put("锤", "6925");
		put("锱", "6926");
		put("铮", "6927");
		put("锭", "6928");
		put("钱", "6929");
		put("锦", "6930");
		put("锚", "6931");
		put("锡", "6932");
		put("锢", "6933");
		put("错", "6934");
		put("锛", "6935");
		put("锜", "6936");
		put("锅", "6938");
		put("锻", "6939");
		put("铋", "6940");
		put("铵", "6941");
		put("锹", "6942");
		put("键", "6943");
		put("锺", "6945");
		put("锴", "6946");
		put("镀", "6947");
		put("锷", "6948");
		put("锽", "6949");
		put("锸", "6950");
		put("锲", "6951");
		put("锾", "6953");
		put("锁", "6956");
		put("锘", "6959");
		put("镃", "6961");
		put("铠", "6963");
		put("镐", "6964");
		put("镒", "6965");
		put("镇", "6966");
		put("镑", "6967");
		put("镞", "6968");
		put("链", "6969");
		put("镝", "6970");
		put("鏖", "6971");
		put("铿", "6972");
		put("锵", "6973");
		put("镗", "6974");
		put("镜", "6975");
		put("镘", "6976");
		put("镖", "6977");
		put("镛", "6978");
		put("镂", "6979");
		put("铲", "6980");
		put("锑", "6982");
		put("錾", "6983");
		put("铙", "6984");
		put("铧", "6985");
		put("铝", "6986");
		put("镪", "6987");
		put("钟", "6988");
		put("镫", "6989");
		put("镌", "6990");
		put("镰", "6991");
		put("镯", "6992");
		put("铁", "6993");
		put("镮", "6994");
		put("铎", "6995");
		put("镍", "6996");
		put("铛", "6997");
		put("铸", "6999");
		put("钙", "6862");
		put("钗", "6865");
		put("钯", "6867");
		put("钝", "6868");
		put("钩", "6869");
		put("钤", "6870");
		put("钠", "6871");
		put("钞", "6872");
		put("钮", "6873");
		put("钧", "6874");
		put("铃", "6875");
		put("钹", "6876");
		put("钰", "6877");
		put("钿", "6879");
		put("钜", "6880");
		put("铉", "6881");
		put("钳", "6883");
		put("铅", "6884");
		put("钺", "6885");
		put("钵", "6886");
		put("铇", "6888");
		put("铏", "6889");
		put("铰", "6890");
		put("钲", "6891");
		put("银", "6892");
		put("铳", "6893");
		put("铜", "6894");
		put("钴", "6895");
		put("铂", "6896");
		put("间", "7035");
		put("闵", "7036");
		put("闸", "7037");
		put("阂", "7039");
		put("阁", "7041");
		put("阀", "7042");
		put("闺", "7043");
		put("闽", "7044");
		put("阃", "7045");
		put("阆", "7046");
		put("闾", "7047");
		put("阅", "7048");
		put("阊", "7049");
		put("阉", "7050");
		put("阎", "7051");
		put("阏", "7052");
		put("阍", "7053");
		put("阈", "7054");
		put("阒", "7055");
		put("闱", "7057");
		put("阔", "7059");
		put("阕", "7060");
		put("阑", "7061");
		put("阇", "7062");
		put("阗", "7063");
		put("阌", "7064");
		put("闿", "7065");
		put("阖", "7066");
		put("阙", "7067");
		put("闯", "7068");
		put("关", "7070");
		put("阓", "7072");
		put("阐", "7073");
		put("阚", "7074");
		put("阛", "7076");
		put("闼", "7077");
		put("阜", "7079");
		put("阝", "7080");
		put("阡", "7082");
		put("阮", "7086");
		put("阱", "7088");
		put("防", "7089");
		put("阪", "7090");
		put("阻", "7091");
		put("阼", "7092");
		put("阿", "7093");
		put("陀", "7094");
		put("陂", "7095");
		put("附", "7096");
		put("陋", "7097");
		put("限", "7098");
		put("陌", "7099");
		put("降", "7100");
		put("陔", "7101");
		put("陉", "7102");
		put("陛", "7103");
		put("陕", "7104");
		put("陞", "7105");
		put("陟", "7106");
		put("陡", "7107");
		put("院", "7108");
		put("阵", "7109");
		put("除", "7110");
		put("陪", "7111");
		put("陬", "7112");
		put("阴", "7113");
		put("陲", "7114");
		put("陈", "7115");
		put("陴", "7116");
		put("陵", "7117");
		put("陶", "7118");
		put("陷", "7119");
		put("陆", "7120");
		put("阳", "7122");
		put("隍", "7124");
		put("隅", "7126");
		put("隆", "7127");
		put("隈", "7128");
		put("陧", "7129");
		put("队", "7130");
		put("隋", "7131");
		put("阶", "7132");
		put("隔", "7133");
		put("陨", "7134");
		put("隗", "7136");
		put("隘", "7137");
		put("镬", "7000");
		put("镔", "7001");
		put("鉴", "7003");
		put("钨", "7005");
		put("铄", "7007");
		put("镳", "7009");
		put("钥", "7011");
		put("镭", "7012");
		put("镶", "7013");
		put("镊", "7015");
		put("", "7016");
		put("锣", "7017");
		put("銮", "7019");
		put("凿", "7020");
		put("锰", "7021");
		put("长", "7022");
		put("蜱", "7023");
		put("门", "7024");
		put("闩", "7025");
		put("闪", "7026");
		put("闫", "7027");
		put("闭", "7028");
		put("闬", "7029");
		put("开", "7030");
		put("闳", "7031");
		put("闰", "7032");
		put("闲", "7033");
		put("虽", "7173");
		put("雘", "7174");
		put("双", "7175");
		put("雏", "7176");
		put("杂", "7177");
		put("难", "7181");
		put("钷", "7182");
		put("雨", "7183");
		put("雩", "7184");
		put("雪", "7185");
		put("雯", "7186");
		put("零", "7190");
		put("雷", "7191");
		put("雹", "7192");
		put("电", "7193");
		put("需", "7194");
		put("霂", "7195");
		put("霄", "7197");
		put("霈", "7198");
		put("霉", "7199");
		put("霆", "7200");
		put("震", "7201");
		put("霍", "7202");
		put("霎", "7203");
		put("霏", "7204");
		put("霓", "7206");
		put("霖", "7207");
		put("霜", "7208");
		put("霞", "7209");
		put("雾", "7212");
		put("霰", "7213");
		put("霪", "7214");
		put("露", "7216");
		put("霸", "7218");
		put("霹", "7219");
		put("霁", "7221");
		put("霾", "7223");
		put("霭", "7224");
		put("雳", "7225");
		put("叇", "7226");
		put("灵", "7227");
		put("叆", "7228");
		put("青", "7230");
		put("靖", "7231");
		put("靓", "7232");
		put("靛", "7233");
		put("静", "7234");
		put("镤", "7235");
		put("非", "7236");
		put("靠", "7237");
		put("靡", "7238");
		put("氆", "7239");
		put("面", "7240");
		put("靥", "7243");
		put("钎", "7244");
		put("革", "7245");
		put("靳", "7246");
		put("靴", "7247");
		put("鞅", "7248");
		put("靶", "7249");
		put("靰", "7250");
		put("鞍", "7254");
		put("巩", "7255");
		put("鞋", "7256");
		put("鞘", "7258");
		put("鞡", "7260");
		put("鞠", "7263");
		put("鞫", "7265");
		put("鞭", "7267");
		put("鞮", "7268");
		put("鞲", "7271");
		put("鞴", "7272");
		put("鞑", "7275");
		put("隙", "7138");
		put("际", "7139");
		put("障", "7140");
		put("隧", "7143");
		put("险", "7145");
		put("隰", "7147");
		put("隐", "7148");
		put("隳", "7149");
		put("陇", "7150");
		put("随", "7151");
		put("隶", "7152");
		put("苤", "7154");
		put("隹", "7155");
		put("隻", "7156");
		put("隼", "7157");
		put("雀", "7158");
		put("雁", "7159");
		put("雄", "7160");
		put("雅", "7161");
		put("集", "7162");
		put("雇", "7163");
		put("雉", "7164");
		put("隽", "7165");
		put("雌", "7166");
		put("雍", "7167");
		put("雎", "7168");
		put("雒", "7170");
		put("雕", "7171");
		put("雟", "7172");
		put("顺", "7311");
		put("须", "7312");
		put("颂", "7313");
		put("颃", "7314");
		put("预", "7315");
		put("顽", "7316");
		put("颁", "7317");
		put("颔", "7318");
		put("顿", "7319");
		put("顼", "7320");
		put("颀", "7322");
		put("颇", "7324");
		put("领", "7325");
		put("颉", "7327");
		put("颐", "7328");
		put("颚", "7329");
		put("颏", "7331");
		put("头", "7333");
		put("颒", "7334");
		put("颊", "7335");
		put("颕", "7336");
		put("颈", "7338");
		put("颓", "7339");
		put("频", "7340");
		put("颗", "7341");
		put("题", "7344");
		put("额", "7345");
		put("颜", "7346");
		put("颛", "7348");
		put("颡", "7350");
		put("颠", "7351");
		put("类", "7352");
		put("颟", "7354");
		put("颢", "7355");
		put("顾", "7357");
		put("颤", "7358");
		put("显", "7359");
		put("颦", "7360");
		put("颅", "7361");
		put("颧", "7362");
		put("粬", "7363");
		put("风", "7364");
		put("飒", "7366");
		put("飓", "7367");
		put("飔", "7368");
		put("飏", "7370");
		put("飖", "7371");
		put("飕", "7372");
		put("飘", "7373");
		put("飙", "7374");
		put("飗", "7375");
		put("飞", "7378");
		put("醛", "7379");
		put("食", "7380");
		put("饥", "7382");
		put("饤", "7383");
		put("飧", "7385");
		put("饨", "7386");
		put("饪", "7387");
		put("饫", "7388");
		put("饬", "7389");
		put("饮", "7390");
		put("饭", "7391");
		put("饴", "7392");
		put("饲", "7393");
		put("饱", "7394");
		put("饰", "7395");
		put("饸", "7396");
		put("饹", "7397");
		put("饺", "7398");
		put("饼", "7399");
		put("饻", "7400");
		put("饷", "7401");
		put("养", "7402");
		put("饵", "7403");
		put("餐", "7404");
		put("饾", "7405");
		put("馁", "7407");
		put("饿", "7408");
		put("馂", "7410");
		put("馀", "7411");
		put("馄", "7412");
		put("鞯", "7277");
		put("羟", "7278");
		put("韦", "7279");
		put("韨", "7280");
		put("韩", "7281");
		put("韧", "7282");
		put("韪", "7289");
		put("韬", "7290");
		put("韫", "7291");
		put("炝", "7294");
		put("韭", "7295");
		put("音", "7299");
		put("韶", "7300");
		put("韵", "7301");
		put("苘", "7305");
		put("页", "7306");
		put("顶", "7307");
		put("顷", "7308");
		put("项", "7309");
		put("顸", "7310");
		put("香", "7449");
		put("馥", "7450");
		put("馨", "7451");
		put("蛲", "7455");
		put("马", "7456");
		put("驭", "7457");
		put("冯", "7458");
		put("驰", "7459");
		put("驯", "7460");
		put("驮", "7461");
		put("驲", "7462");
		put("驳", "7463");
		put("驻", "7465");
		put("驽", "7466");
		put("驹", "7467");
		put("驾", "7468");
		put("骀", "7469");
		put("驸", "7470");
		put("驶", "7471");
		put("驼", "7474");
		put("驷", "7475");
		put("骈", "7478");
		put("骇", "7480");
		put("骆", "7482");
		put("骃", "7484");
		put("骎", "7485");
		put("骏", "7486");
		put("骔", "7487");
		put("骋", "7488");
		put("骓", "7489");
		put("骍", "7490");
		put("骑", "7494");
		put("骐", "7496");
		put("骛", "7497");
		put("骗", "7499");
		put("骢", "7500");
		put("骞", "7505");
		put("腾", "7506");
		put("驺", "7507");
		put("骟", "7509");
		put("骚", "7510");
		put("骝", "7511");
		put("骡", "7512");
		put("蓦", "7513");
		put("骜", "7514");
		put("骖", "7515");
		put("骠", "7516");
		put("驱", "7517");
		put("骅", "7520");
		put("骁", "7522");
		put("骄", "7524");
		put("骕", "7525");
		put("验", "7526");
		put("惊", "7528");
		put("驿", "7531");
		put("骤", "7532");
		put("驴", "7533");
		put("骧", "7534");
		put("骥", "7535");
		put("骊", "7537");
		put("骨", "7539");
		put("骶", "7541");
		put("肮", "7542");
		put("骱", "7543");
		put("骰", "7544");
		put("骷", "7545");
		put("骸", "7546");
		put("骼", "7547");
		put("髀", "7549");
		put("髅", "7551");
		put("馅", "7414");
		put("饯", "7415");
		put("饽", "7416");
		put("馍", "7418");
		put("馆", "7419");
		put("餮", "7421");
		put("饧", "7424");
		put("饩", "7426");
		put("馒", "7429");
		put("馑", "7431");
		put("馈", "7432");
		put("馔", "7433");
		put("饶", "7437");
		put("饔", "7438");
		put("饕", "7439");
		put("飨", "7440");
		put("餍", "7441");
		put("馋", "7443");
		put("首", "7445");
		put("馘", "7446");
		put("馗", "7447");
		put("炔", "7448");
		put("鬟", "7587");
		put("鬣", "7588");
		put("鬓", "7589");
		put("鬥", "7591");
		put("阋", "7594");
		put("阄", "7596");
		put("铯", "7597");
		put("鬯", "7598");
		put("鬱", "7599");
		put("锶", "7600");
		put("鬲", "7601");
		put("鬻", "7605");
		put("鬼", "7607");
		put("魁", "7608");
		put("魂", "7609");
		put("魃", "7610");
		put("魄", "7611");
		put("魅", "7613");
		put("魏", "7614");
		put("魈", "7617");
		put("魍", "7618");
		put("魉", "7619");
		put("魑", "7620");
		put("魔", "7621");
		put("魇", "7623");
		put("唢", "7624");
		put("鱼", "7625");
		put("鱿", "7626");
		put("鲁", "7627");
		put("鲖", "7628");
		put("鲂", "7629");
		put("鲇", "7630");
		put("鲋", "7631");
		put("鲐", "7632");
		put("鲊", "7633");
		put("鲑", "7634");
		put("鲔", "7635");
		put("鲘", "7636");
		put("鲍", "7637");
		put("鲞", "7638");
		put("鲜", "7639");
		put("鲛", "7640");
		put("鲧", "7641");
		put("鲤", "7642");
		put("鲠", "7643");
		put("鲨", "7644");
		put("鲭", "7645");
		put("鲵", "7646");
		put("鲳", "7647");
		put("鲓", "7648");
		put("鲲", "7649");
		put("鲸", "7650");
		put("鲫", "7651");
		put("鲩", "7652");
		put("鳊", "7653");
		put("鲽", "7654");
		put("鳅", "7655");
		put("鲡", "7657");
		put("鳃", "7658");
		put("鳏", "7659");
		put("鳒", "7660");
		put("鲥", "7661");
		put("鳍", "7662");
		put("鳌", "7663");
		put("鲶", "7664");
		put("鳗", "7665");
		put("鳀", "7666");
		put("鳖", "7667");
		put("鳝", "7668");
		put("鲟", "7669");
		put("鲎", "7670");
		put("鲙", "7671");
		put("鳣", "7672");
		put("鳞", "7673");
		put("鳔", "7674");
		put("鳜", "7675");
		put("鳘", "7676");
		put("鲮", "7677");
		put("鲈", "7678");
		put("鲄", "7679");
		put("鸟", "7680");
		put("鸠", "7682");
		put("凫", "7683");
		put("凤", "7685");
		put("鸣", "7686");
		put("鸢", "7687");
		put("鸱", "7689");
		put("髑", "7553");
		put("体", "7555");
		put("髋", "7556");
		put("髓", "7557");
		put("鞣", "7558");
		put("高", "7559");
		put("铷", "7560");
		put("髟", "7561");
		put("髡", "7562");
		put("髦", "7565");
		put("髫", "7567");
		put("髭", "7568");
		put("髯", "7570");
		put("鬃", "7571");
		put("髻", "7573");
		put("髹", "7574");
		put("鬈", "7576");
		put("鬐", "7580");
		put("鬘", "7583");
		put("鹗", "7725");
		put("鹜", "7726");
		put("莺", "7727");
		put("鹤", "7729");
		put("鹡", "7731");
		put("鹚", "7732");
		put("鹒", "7733");
		put("鸧", "7735");
		put("鹘", "7737");
		put("鹣", "7738");
		put("鹢", "7739");
		put("鹞", "7740");
		put("鸡", "7741");
		put("鸥", "7743");
		put("鹧", "7744");
		put("鹥", "7746");
		put("鸷", "7747");
		put("鸶", "7748");
		put("鹫", "7750");
		put("鹰", "7751");
		put("鹭", "7752");
		put("鹪", "7753");
		put("鹩", "7754");
		put("鹬", "7756");
		put("鹯", "7757");
		put("鸴", "7759");
		put("鸬", "7760");
		put("鹦", "7761");
		put("鸾", "7762");
		put("鹳", "7764");
		put("鹂", "7765");
		put("卤", "7767");
		put("盐", "7770");
		put("鹾", "7771");
		put("熵", "7772");
		put("鹿", "7773");
		put("麂", "7774");
		put("铈", "7775");
		put("麀", "7776");
		put("麇", "7778");
		put("麈", "7779");
		put("麋", "7780");
		put("麑", "7783");
		put("麒", "7784");
		put("麓", "7785");
		put("丽", "7787");
		put("麝", "7790");
		put("麟", "7792");
		put("涮", "7795");
		put("麦", "7796");
		put("麴", "7800");
		put("蒴", "7801");
		put("麻", "7802");
		put("麽", "7803");
		put("麾", "7804");
		put("黄", "7806");
		put("黉", "7808");
		put("铊", "7809");
		put("黍", "7810");
		put("黏", "7811");
		put("黎", "7812");
		put("溚", "7814");
		put("黑", "7815");
		put("黔", "7816");
		put("默", "7817");
		put("黛", "7818");
		put("黜", "7819");
		put("点", "7820");
		put("黝", "7821");
		put("黠", "7822");
		put("黥", "7823");
		put("黧", "7824");
		put("黯", "7826");
		put("鸩", "7690");
		put("鸨", "7691");
		put("鸦", "7693");
		put("鸳", "7696");
		put("鸮", "7697");
		put("鸪", "7698");
		put("鸯", "7699");
		put("鸭", "7700");
		put("鸰", "7701");
		put("鸲", "7702");
		put("鸿", "7703");
		put("鸽", "7704");
		put("鸺", "7707");
		put("鹈", "7708");
		put("鹅", "7709");
		put("鹃", "7710");
		put("鹄", "7711");
		put("鹁", "7712");
		put("鹉", "7713");
		put("鹆", "7714");
		put("鹌", "7715");
		put("鹊", "7717");
		put("鹑", "7718");
		put("鹏", "7720");
		put("鹍", "7721");
		put("鹙", "7723");
		put("鹕", "7724");
		put("鼷", "7863");
		put("醣", "7864");
		put("鼻", "7865");
		put("鼾", "7867");
		put("铽", "7870");
		put("齐", "7871");
		put("斋", "7872");
		put("齑", "7874");
		put("菾", "7875");
		put("齿", "7876");
		put("龀", "7877");
		put("龁", "7878");
		put("龂", "7879");
		put("龃", "7880");
		put("龄", "7881");
		put("龆", "7882");
		put("龈", "7883");
		put("龉", "7886");
		put("龊", "7887");
		put("龋", "7889");
		put("腭", "7890");
		put("龌", "7891");
		put("龙", "7893");
		put("厐", "7894");
		put("龚", "7895");
		put("龛", "7896");
		put("舔", "7897");
		put("龟", "7898");
		put("烃", "7899");
		put("龠", "7900");
		put("楟", "7903");
		put("酮", "7904");
		put("茼", "7905");
		put("烔", "7906");
		put("钍", "7907");
		put("佤", "7908");
		put("烷", "7909");
		put("烯", "7910");
		put("螅", "7911");
		put("氙", "7912");
		put("酰", "7913");
		put("鳕", "7914");
		put("荨", "7915");
		put("垭", "7916");
		put("铱", "7917");
		put("钇", "7918");
		put("镱", "7919");
		put("铟", "7920");
		put("铕", "7921");
		put("鳟", "7923");
		put("蟑", "7924");
		put("礃", "7925");
		put("锗", "7926");
		put("酯", "7927");
		put("黡", "7829");
		put("黩", "7830");
		put("傣", "7831");
		put("黹", "7832");
		put("黻", "7833");
		put("黼", "7834");
		put("钛", "7835");
		put("黾", "7836");
		put("鼋", "7837");
		put("鼌", "7839");
		put("鼍", "7842");
		put("钽", "7843");
		put("鼎", "7844");
		put("鼐", "7845");
		put("鼒", "7847");
		put("鼓", "7849");
		put("鼗", "7851");
		put("鼙", "7853");
		put("羰", "7856");
		put("鼠", "7857");
		put("鼢", "7858");
		put("鼬", "7860");
		put("鼯", "7861");
		put("鼹", "7862");
		put("万", "8001");
		put("丘", "8002");
		put("乒", "8003");
		put("乓", "8004");
		put("甭", "8005");
		put("孬", "8006");
		put("么", "8010");
		put("乩", "8012");
		put("亍", "8015");
		put("亓", "8016");
		put("亘", "8017");
		put("仂", "8021");
		put("仉", "8022");
		put("伃", "8024");
		put("伽", "8026");
		put("伫", "8028");
		put("佼", "8032");
		put("佴", "8034");
		put("佫", "8035");
		put("侢", "8037");
		put("邘", "9386");
		put("郈", "9390");
		put("郚", "9392");
		put("郫", "9395");
		put("郓", "9398");
		put("郾", "9401");
		put("鄚", "9409");
		put("鄣", "9410");
		put("醑", "9428");
		put("酾", "9432");
		put("酽", "9433");
		put("钋", "9437");
		put("钐", "9438");
		put("钒", "9440");
		put("钑", "9442");
		put("钫", "9443");
		put("钭", "9445");
		put("钘", "9446");
		put("铍", "9448");
		put("钻", "9449");
		put("铆", "9450");
		put("钼", "9453");
		put("铐", "9456");
		put("铬", "9459");
		put("鋆", "9462");
		put("锂", "9465");
		put("锒", "9467");
		put("钡", "9468");
		put("铔", "9472");
		put("锨", "9475");
		put("锬", "9477");
		put("锖", "9478");
		put("锠", "9480");
		put("锞", "9481");
		put("鍪", "9482");
		put("蹼", "9349");
		put("躞", "9351");
		put("轲", "9363");
		put("辚", "9368");
		put("迮", "9374");
		put("邋", "9380");
		put("闹", "9527");
		put("阽", "9532");
		put("靔", "9552");
		put("靺", "9555");
		put("靼", "9557");
		put("鞨", "9560");
		put("頲", "9576");
		put("颌", "9577");
		put("", "9581");
		put("饦", "9594");
		put("馊", "9600");
		put("馎", "9601");
		put("馃", "9606");
		put("驵", "9615");
		put("铡", "9484");
		put("镁", "9485");
		put("钖", "9486");
		put("锳", "9487");
		put("镏", "9490");
		put("镉", "9493");
		put("铩", "9494");
		put("鏊", "9495");
		put("蓥", "9496");
		put("鎏", "9497");
		put("镦", "9503");
		put("锏", "9505");
		put("镨", "9506");
		put("镣", "9507");
		put("镡", "9509");
		put("镢", "9510");
		put("锧", "9514");
		put("鑫", "9515");
		put("鲴", "9657");
		put("鳁", "9658");
		put("鳎", "9660");
		put("鳢", "9661");
		put("鲚", "9662");
		put("鳙", "9663");
		put("鳠", "9664");
		put("鲿", "9665");
		put("鳛", "9667");
		put("鸵", "9676");
		put("鹓", "9677");
		put("鹖", "9679");
		put("鹔", "9681");
		put("鹴", "9684");
		put("麸", "9690");
		put("黟", "9694");
		put("ㄅ", "9720");
		put("ㄆ", "9721");
		put("ㄇ", "9722");
		put("ㄈ", "9723");
		put("ㄉ", "9724");
		put("ㄊ", "9725");
		put("ㄋ", "9726");
		put("ㄌ", "9727");
		put("ㄍ", "9728");
		put("ㄎ", "9729");
		put("ㄏ", "9730");
		put("ㄐ", "9731");
		put("ㄑ", "9732");
		put("ㄒ", "9733");
		put("ㄓ", "9734");
		put("ㄔ", "9735");
		put("ㄕ", "9736");
		put("ㄖ", "9737");
		put("ㄗ", "9738");
		put("ㄘ", "9739");
		put("ㄙ", "9740");
		put("ㄧ", "9741");
		put("ㄨ", "9742");
		put("ㄩ", "9743");
		put("ㄚ", "9744");
		put("ㄛ", "9745");
		put("ㄜ", "9746");
		put("ㄝ", "9747");
		put("ㄞ", "9748");
		put("ㄟ", "9749");
		put("ㄠ", "9750");
		put("ㄡ", "9751");
		put("ㄢ", "9752");
		put("ㄣ", "9753");
		put("ㄤ", "9754");
		put("ㄥ", "9755");
		put("ㄦ", "9756");
		put("骦", "9622");
		put("骉", "9623");
		put("髁", "9629");
		put("鲀", "9639");
		put("鲅", "9641");
		put("鲒", "9643");
		put("鲰", "9646");
		put("鲻", "9647");
		put("鲷", "9648");
		put("鲗", "9649");
		put("鳇", "9650");
		put("鲝", "9651");
		put("鳆", "9652");
		put("鳄", "9653");
		put("鲢", "9654");
		put("鲦", "9655");
		put("鲱", "9656");
		put("Я", "9795");
		put("Ъ", "9796");
		put("Ы", "9797");
		put("Ь", "9798");
		put("Й", "9799");
		put("Ａ", "9874");
		put("Ｂ", "9875");
		put("Ｃ", "9876");
		put("Ｄ", "9877");
		put("Ｅ", "9878");
		put("Ｆ", "9879");
		put("Ｇ", "9880");
		put("Ｈ", "9881");
		put("Ｉ", "9882");
		put("Ｊ", "9883");
		put("Ｋ", "9884");
		put("Ｌ", "9885");
		put("Ｍ", "9886");
		put("Ｎ", "9887");
		put("Ｏ", "9888");
		put("Ｐ", "9889");
		put("Ｑ", "9890");
		put("Ｒ", "9891");
		put("Ｓ", "9892");
		put("Ｔ", "9893");
		put("Ｕ", "9894");
		put("Ｖ", "9895");
		put("Ｗ", "9896");
		put("Ｘ", "9897");
		put("А", "9768");
		put("Б", "9769");
		put("В", "9770");
		put("Г", "9771");
		put("Д", "9772");
		put("Е", "9773");
		put("Ж", "9774");
		put("З", "9775");
		put("И", "9776");
		put("К", "9777");
		put("Л", "9778");
		put("М", "9779");
		put("Н", "9780");
		put("О", "9781");
		put("П", "9782");
		put("Р", "9783");
		put("С", "9784");
		put("Т", "9785");
		put("У", "9786");
		put("Ф", "9787");
		put("Х", "9788");
		put("Ц", "9789");
		put("Ч", "9790");
		put("Ш", "9791");
		put("Щ", "9792");
		put("Э", "9793");
		put("Ю", "9794");
		put("Ⅰ", "9941");
		put("Ⅱ", "9942");
		put("Ⅲ", "9943");
		put("Ⅳ", "9944");
		put("Ⅴ", "9945");
		put("Ⅵ", "9946");
		put("Ⅶ", "9947");
		put("Ⅷ", "9948");
		put("Ⅸ", "9949");
		put("Ⅹ", "9950");
		put("Ω", "9959");
		put("０", "9960");
		put("１", "9961");
		put("２", "9962");
		put("３", "9963");
		put("４", "9964");
		put("５", "9965");
		put("６", "9966");
		put("７", "9967");
		put("８", "9968");
		put("９", "9969");
		put("／", "9970");
		put("＋", "9971");
		put("－", "9972");
		put("×", "9973");
		put("÷", "9974");
		put("。", "9975");
		put("，", "9976");
		put("、", "9977");
		put("·", "9978");
		put("；", "9979");
		put("：", "9980");
		put("？", "9981");
		put("！", "9982");
		put("＝", "9983");
		put("‘", "9984");
		put("’", "9985");
		put("“", "9986");
		put("”", "9987");
		put("（", "9988");
		put("）", "9989");
		put("━", "9990");
		put("", "9991");
		put("·", "9992");
		put("·", "9993");
		put("—", "9994");
		put("—", "9995");
		put("《", "9996");
		put("》", "9997");
		put("□", "9998");
		put("", "9999");
		put("Ｙ", "9898");
		put("Ｚ", "9899");
		put("糌", "9048");
		put("纼", "9053");
		put("纰", "9055");
		put("绁", "9057");
		put("绲", "9064");
		put("缂", "9065");
		put("绬", "9069");
		put("砣", "8932");
		put("砠", "8933");
		put("砝", "8935");
		put("砷", "8938");
		put("硐", "8943");
		put("硅", "8944");
		put("硭", "8947");
		put("硪", "8948");
		put("碚", "8951");
		put("碳", "8955");
		put("碶", "8956");
		put("磡", "8962");
		put("硚", "8965");
		put("聱", "9107");
		put("肪", "9112");
		put("胛", "9113");
		put("脉", "9115");
		put("胆", "9116");
		put("脞", "9120");
		put("脖", "9126");
		put("腌", "9131");
		put("膘", "9136");
		put("臑", "9140");
		put("膑", "9141");
		put("舢", "9152");
		put("舨", "9153");
		put("舺", "9154");
		put("茓", "9166");
		put("芘", "9167");
		put("荅", "9171");
		put("茬", "9173");
		put("莦", "9174");
		put("荝", "9181");
		put("葠", "9185");
		put("蓪", "9186");
		put("苁", "9190");
		put("蓷", "9191");
		put("薸", "9195");
		put("缑", "9071");
		put("罾", "9088");
		put("芈", "9090");
		put("蠭", "9244");
		put("裢", "9260");
		put("襞", "9268");
		put("觜", "9278");
		put("诇", "9287");
		put("诨", "9294");
		put("谉", "9300");
		put("谰", "9301");
		put("豮", "9317");
		put("贠", "9322");
		put("趱", "9328");
		put("跤", "9332");
		put("踦", "9337");
		put("踪", "9338");
		put("蹩", "9345");
		put("虒", "9213");
		put("蚬", "9223");
		put("蝓", "9227");
		put("蝤", "9230");
		put("螓", "9231");
		put("螗", "9236");
		put("螵", "9237");
		put("蟢", "9240");
		put("蠋", "9241");
		put("蛴", "9242");
	}

	public static String encode(String value) {
		if (value != null)
			return m.get(value.trim());
		return null;
	}

	public static String decodeLine(String str, StringBuilder sb) {
		if (str == null || str == "")
			return null;
		Matcher matcher = pattern.matcher(str);
		int offset = 0;
		while (matcher.find()) {
			int start = matcher.start();
			int end = matcher.end();
			String mstr = matcher.group().trim();
			if (offset < start) {
				sb.append(str.substring(offset, start));
			}
			sb.append(encode(mstr));
			offset = end;
		}
		if (offset < str.length()) {
			sb.append(str.substring(offset));
		}
		return sb.toString();
	}

	public static String encodeLine(String str, StringBuilder sb)
			throws IOException {
		if (str == null || str == "")
			return null;
		char[] conlneby = IOUtils.toCharArray(
				IOUtils.toInputStream(str, "utf-8"), "utf-8");
		for (int i = 0; i < conlneby.length; i++) {
			String con = String.valueOf(conlneby[i]);
			String code = encode(con);
			if (code == null || code == "") {
				sb.append(con);
			} else {
				sb.append(code).append(' ');
			}

		}

		return sb.toString();
	}

	public static String encodeMsg(String msg) throws IOException {
		List<String> conlst = IOUtils.readLines(
				IOUtils.toInputStream(msg, "utf-8"), "utf-8");
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < conlst.size(); i++) {
			encodeLine(conlst.get(i), sb);
			if (i != (conlst.size() - 1)) {
				sb.append(LineSeparator);
			}
		}
		return sb.toString();
	}

	public static String decodeMsg(String msg) throws IOException {
		List<String> conlst = IOUtils.readLines(
				IOUtils.toInputStream(msg, "utf-8"), "utf-8");
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < conlst.size(); i++) {
			decodeLine(conlst.get(i), sb);
			if (i != (conlst.size() - 1)) {
				sb.append(LineSeparator);
			}
		}
		return sb.toString();
	}

	public static void main(String[] args) {
		try {
			System.out.println(encodeMsg("/中国/富强-2018 davi/s"));
			System.out.println(encodeMsg("45'ppp代门,."));
//			System.out.println(decodeMsg("1420 2494 "));
			
			String a="现在就可看出iiiiiiiooooooopp";
			String str = "";

			char[] conlneby = IOUtils.toCharArray(
					IOUtils.toInputStream(a, "utf-8"), "utf-8");
			for (int i = 0; i < 5; i++) {
				String con = String.valueOf(conlneby[i]);
					str+=conlneby[i];
			}
//			System.out.println(str);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}